@@ -1,30 +1,39 @@
 /*******************************************************************************
- * Copyright (c) 2005 Contributors.
+ * Copyright (c) 2005, 2017 Contributors.
  * All rights reserved.
  * This program and the accompanying materials are made available
  * under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution and is available at
  * http://eclipse.org/legal/epl-v10.html
- *
- * Contributors:
- *   Alexandre Vasseur         initial implementation
- *   David Knibb		       weaving context enhancments
- *   John Kew (vmware)         caching hook
  *******************************************************************************/
 package org.aspectj.weaver.loadtime;
 
-import java.io.*;
-import java.lang.reflect.InvocationTargetException;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.lang.reflect.Field;
 import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.security.ProtectionDomain;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Properties;
+import java.util.Set;
+import java.util.StringTokenizer;
 
 import org.aspectj.bridge.AbortException;
 import org.aspectj.bridge.Constants;
 import org.aspectj.bridge.MessageUtil;
 import org.aspectj.util.LangUtil;
+import org.aspectj.weaver.IUnwovenClassFile;
 import org.aspectj.weaver.Lint;
 import org.aspectj.weaver.Lint.Kind;
 import org.aspectj.weaver.ResolvedType;
@@ -39,13 +48,20 @@ import org.aspectj.weaver.loadtime.definition.DocumentParser;
 import org.aspectj.weaver.ltw.LTWWorld;
 import org.aspectj.weaver.patterns.PatternParser;
 import org.aspectj.weaver.patterns.TypePattern;
-import org.aspectj.weaver.tools.*;
+import org.aspectj.weaver.tools.GeneratedClassHandler;
+import org.aspectj.weaver.tools.Trace;
+import org.aspectj.weaver.tools.TraceFactory;
+import org.aspectj.weaver.tools.WeavingAdaptor;
 import org.aspectj.weaver.tools.cache.WeavedClassCache;
 
+import sun.misc.Unsafe;
+
 /**
  * @author Alexandre Vasseur
  * @author Andy Clement
  * @author Abraham Nevado
+ * @author David Knibb
+ * @author John Kew
  */
 public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 
@@ -53,8 +69,8 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 
 	private boolean initialized;
 
-	private List<TypePattern> m_dumpTypePattern = new ArrayList<TypePattern>();
-	private boolean m_dumpBefore = false;
+	private List<TypePattern> dumpTypePattern = new ArrayList<TypePattern>();
+	private boolean dumpBefore = false;
 	private boolean dumpDirPerClassloader = false;
 
 	private boolean hasExcludes = false;
@@ -67,14 +83,14 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 
 	private boolean hasIncludes = false;
 	private List<TypePattern> includeTypePattern = new ArrayList<TypePattern>();
-	private List<String> m_includeStartsWith = new ArrayList<String>();
+	private List<String> includeStartsWith = new ArrayList<String>();
 	private List<String> includeExactName = new ArrayList<String>();
 	private boolean includeStar = false;
 
-	private List<TypePattern> m_aspectExcludeTypePattern = new ArrayList<TypePattern>();
-	private List<String> m_aspectExcludeStartsWith = new ArrayList<String>();
-	private List<TypePattern> m_aspectIncludeTypePattern = new ArrayList<TypePattern>();
-	private List<String> m_aspectIncludeStartsWith = new ArrayList<String>();
+	private List<TypePattern> aspectExcludeTypePattern = new ArrayList<TypePattern>();
+	private List<String> aspectExcludeStartsWith = new ArrayList<String>();
+	private List<TypePattern> aspectIncludeTypePattern = new ArrayList<TypePattern>();
+	private List<String> aspectIncludeStartsWith = new ArrayList<String>();
 
 	private StringBuffer namespace;
 	private IWeavingContext weavingContext;
@@ -99,6 +115,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	 * 
 	 * @deprecated
 	 */
+	@Deprecated
 	public ClassLoaderWeavingAdaptor(final ClassLoader deprecatedLoader, final IWeavingContext deprecatedContext) {
 		super();
 		if (trace.isTraceEnabled()) {
@@ -120,6 +137,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		 * Callback when we need to define a Closure in the JVM
 		 * 
 		 */
+		@Override
 		public void acceptClass (String name, byte[] originalBytes, byte[] wovenBytes) {
 			try {
 				if (shouldDump(name.replace('/', '.'), false)) {
@@ -153,7 +171,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 
 		this.generatedClassHandler = new SimpleGeneratedClassHandler(classLoader);
 
-		List definitions = weavingContext.getDefinitions(classLoader, this);
+		List<Definition> definitions = weavingContext.getDefinitions(classLoader, this);
 		if (definitions.isEmpty()) {
 			disable(); // TODO maw Needed to ensure messages are flushed
 			if (trace.isTraceEnabled()) {
@@ -232,7 +250,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 				String file = System.getProperty("aj5.def", null);
 				if (file != null) {
 					info("using (-Daj5.def) " + file);
-					definitions.add(DocumentParser.parse((new File(file)).toURL()));
+					definitions.add(DocumentParser.parse((new File(file)).toURI().toURL()));
 				}
 			}
 
@@ -252,7 +270,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 						if (!configFile.exists()) {
 							warn("configuration does not exist: " + nextDefinition);
 						} else {
-							definitions.add(DocumentParser.parse(configFile.toURL()));
+							definitions.add(DocumentParser.parse(configFile.toURI().toURL()));
 						}
 					} catch (MalformedURLException mue) {
 						error("malformed definition url: " + nextDefinition);
@@ -414,10 +432,10 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		for (Definition definition : definitions) {
 			for (String exclude : definition.getAspectExcludePatterns()) {
 				TypePattern excludePattern = new PatternParser(exclude).parseTypePattern();
-				m_aspectExcludeTypePattern.add(excludePattern);
+				aspectExcludeTypePattern.add(excludePattern);
 				fastMatchInfo = looksLikeStartsWith(exclude);
 				if (fastMatchInfo != null) {
-					m_aspectExcludeStartsWith.add(fastMatchInfo);
+					aspectExcludeStartsWith.add(fastMatchInfo);
 				}
 			}
 		}
@@ -428,10 +446,10 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		for (Definition definition : definitions) {
 			for (String include : definition.getAspectIncludePatterns()) {
 				TypePattern includePattern = new PatternParser(include).parseTypePattern();
-				m_aspectIncludeTypePattern.add(includePattern);
+				aspectIncludeTypePattern.add(includePattern);
 				fastMatchInfo = looksLikeStartsWith(include);
 				if (fastMatchInfo != null) {
-					m_aspectIncludeStartsWith.add(fastMatchInfo);
+					aspectIncludeStartsWith.add(fastMatchInfo);
 				}
 			}
 		}
@@ -586,7 +604,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 				String include = iterator1.next();
 				fastMatchInfo = looksLikeStartsWith(include);
 				if (fastMatchInfo != null) {
-					m_includeStartsWith.add(fastMatchInfo);
+					includeStartsWith.add(fastMatchInfo);
 				} else if (include.equals("*")) {
 					includeStar = true;
 				} else if ((fastMatchInfo = looksLikeExactName(include)) != null) {
@@ -723,10 +741,10 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 			for (Iterator<String> iterator1 = definition.getDumpPatterns().iterator(); iterator1.hasNext();) {
 				String dump = iterator1.next();
 				TypePattern pattern = new PatternParser(dump).parseTypePattern();
-				m_dumpTypePattern.add(pattern);
+				dumpTypePattern.add(pattern);
 			}
 			if (definition.shouldDumpBefore()) {
-				m_dumpBefore = true;
+				dumpBefore = true;
 			}
 			if (definition.createDumpDirPerClassloader()) {
 				dumpDirPerClassloader = true;
@@ -811,9 +829,9 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 				}
 			}
 			boolean fastAccept = false;// defaults to false if no fast include
-			for (int i = 0; i < m_includeStartsWith.size(); i++) {
+			for (int i = 0; i < includeStartsWith.size(); i++) {
 				didSomeIncludeMatching = true;
-				fastAccept = fastClassName.startsWith(m_includeStartsWith.get(i));
+				fastAccept = fastClassName.startsWith(includeStartsWith.get(i));
 				if (fastAccept) {
 					return true;
 				}
@@ -849,9 +867,9 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 					}
 				}
 			}
-			for (int i = 0; i < m_includeStartsWith.size(); i++) {
+			for (int i = 0; i < includeStartsWith.size(); i++) {
 				didSomeIncludeMatching = true;
-				boolean fastaccept = fastClassName.startsWith(m_includeStartsWith.get(i));
+				boolean fastaccept = fastClassName.startsWith(includeStartsWith.get(i));
 				if (fastaccept) {
 					return true;
 				}
@@ -874,21 +892,21 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	// this can be nice but very dangerous as well to change that
 	private boolean acceptAspect(String aspectClassName) {
 		// avoid ResolvedType if not needed
-		if (m_aspectExcludeTypePattern.isEmpty() && m_aspectIncludeTypePattern.isEmpty()) {
+		if (aspectExcludeTypePattern.isEmpty() && aspectIncludeTypePattern.isEmpty()) {
 			return true;
 		}
 
 		// still try to avoid ResolvedType if we have simple patterns
 		// EXCLUDE: if one match then reject
 		String fastClassName = aspectClassName.replace('/', '.').replace('.', '$');
-		for (int i = 0; i < m_aspectExcludeStartsWith.size(); i++) {
-			if (fastClassName.startsWith(m_aspectExcludeStartsWith.get(i))) {
+		for (int i = 0; i < aspectExcludeStartsWith.size(); i++) {
+			if (fastClassName.startsWith(aspectExcludeStartsWith.get(i))) {
 				return false;
 			}
 		}
 		// INCLUDE: if one match then accept
-		for (int i = 0; i < m_aspectIncludeStartsWith.size(); i++) {
-			if (fastClassName.startsWith(m_aspectIncludeStartsWith.get(i))) {
+		for (int i = 0; i < aspectIncludeStartsWith.size(); i++) {
+			if (fastClassName.startsWith(aspectIncludeStartsWith.get(i))) {
 				return true;
 			}
 		}
@@ -896,8 +914,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		// needs further analysis
 		ResolvedType classInfo = weaver.getWorld().resolve(UnresolvedType.forName(aspectClassName), true);
 		// exclude are "AND"ed
-		for (Iterator iterator = m_aspectExcludeTypePattern.iterator(); iterator.hasNext();) {
-			TypePattern typePattern = (TypePattern) iterator.next();
+		for (TypePattern typePattern: aspectExcludeTypePattern) {
 			if (typePattern.matchesStatically(classInfo)) {
 				// exclude match - skip
 				return false;
@@ -905,8 +922,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		}
 		// include are "OR"ed
 		boolean accept = true;// defaults to true if no include
-		for (Iterator iterator = m_aspectIncludeTypePattern.iterator(); iterator.hasNext();) {
-			TypePattern typePattern = (TypePattern) iterator.next();
+		for (TypePattern typePattern: aspectIncludeTypePattern) {
 			accept = typePattern.matchesStatically(classInfo);
 			if (accept) {
 				break;
@@ -919,19 +935,19 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	@Override
 	protected boolean shouldDump(String className, boolean before) {
 		// Don't dump before weaving unless asked to
-		if (before && !m_dumpBefore) {
+		if (before && !dumpBefore) {
 			return false;
 		}
 
 		// avoid ResolvedType if not needed
-		if (m_dumpTypePattern.isEmpty()) {
+		if (dumpTypePattern.isEmpty()) {
 			return false;
 		}
 
 		// TODO AV - optimize for className.startWith only
 		ResolvedType classInfo = weaver.getWorld().resolve(UnresolvedType.forName(className), true);
 		// dump
-		for (Iterator<TypePattern> iterator = m_dumpTypePattern.iterator(); iterator.hasNext();) {
+		for (Iterator<TypePattern> iterator = dumpTypePattern.iterator(); iterator.hasNext();) {
 			TypePattern typePattern = iterator.next();
 			if (typePattern.matchesStatically(classInfo)) {
 				// dump match
@@ -990,73 +1006,183 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	 */
 	public void flushGeneratedClasses() {
 		// System.err.println("? ClassLoaderWeavingAdaptor.flushGeneratedClasses() generatedClasses=" + generatedClasses);
-		generatedClasses = new HashMap();
+		generatedClasses = new HashMap<String, IUnwovenClassFile>();
 	}
 
-	private Method defineClassMethod;
-	private Method defineClassWithProtectionDomainMethod;
+	/**
+	 * Remove generated classes based on the supplied className. This will
+	 * remove any entries related to this name - so the class itself plus
+	 * and inner classes.
+	 * @param className a slashed classname (e.g. com/foo/Bar)
+	 */
+	public void flushGeneratedClassesFor(String className) {
+		try {
+			String dottedClassName = className.replace('/', '.');
+			String dottedClassNameDollar = dottedClassName+"$"; // to pickup inner classes
+			Iterator<Map.Entry<String, IUnwovenClassFile>> iter = generatedClasses.entrySet().iterator();
+			while (iter.hasNext()) {
+				Entry<String, IUnwovenClassFile> next = iter.next();
+				String existingGeneratedName = next.getKey();
+				if (existingGeneratedName.equals(dottedClassName) ||
+						existingGeneratedName.startsWith(dottedClassNameDollar)) {
+					iter.remove();
+				}
+			}
+		} catch (Throwable t) {
+			new RuntimeException("Unexpected problem tidying up generated classes for "+className,t).printStackTrace();
+		}
+	}
 
-	private void defineClass(ClassLoader loader, String name, byte[] bytes) {
+	private Unsafe unsafe;
+
+	private Unsafe getUnsafe() throws NoSuchFieldException, IllegalAccessException {
+		if (unsafe == null) {
+	        Field theUnsafeField = Unsafe.class.getDeclaredField("theUnsafe");
+	        theUnsafeField.setAccessible(true);
+	        return (Unsafe) theUnsafeField.get(null);
+		}
+		return unsafe;
+    }
+	
+	private static Method bindTo_Method, invokeWithArguments_Method = null;
+	private static Object defineClassMethodHandle = null;
+	
+	private static Boolean initializedForJava11 = false;
+	
+	// In order to let this code compile on earlier versions of Java (8), use reflection to discover the elements
+	// we need to define classes.
+	private static synchronized void initializeForJava11() {
+		if (initializedForJava11) return;
+		try {
+			// MethodType defineClassMethodType = MethodType.methodType(Class.class, new Class[]{String.class, byte[].class, int.class, int.class});
+			Class<?> methodType_Class = Class.forName("java.lang.invoke.MethodType");
+			Method methodTypeMethodOnMethodTypeClass = methodType_Class.getDeclaredMethod("methodType", Class.class,Class[].class);
+			methodTypeMethodOnMethodTypeClass.setAccessible(true);
+			Object defineClassMethodType = methodTypeMethodOnMethodTypeClass.invoke(null, Class.class, new Class[] {String.class,byte[].class,int.class,int.class});
+
+			// MethodHandles.Lookup methodHandlesLookup = MethodHandles.lookup();
+			Class<?> methodHandles_Class = Class.forName("java.lang.invoke.MethodHandles");
+			Method lookupMethodOnMethodHandlesClass = methodHandles_Class.getDeclaredMethod("lookup");
+			lookupMethodOnMethodHandlesClass.setAccessible(true);			
+			Object methodHandlesLookup = lookupMethodOnMethodHandlesClass.invoke(null);
+			
+			// MethodHandles.Lookup lookup = MethodHandles.privateLookupIn(ClassLoader.class, baseLookup);
+			Class<?> methodHandlesLookup_Class = Class.forName("java.lang.invoke.MethodHandles$Lookup");
+			Method privateLookupMethodOnMethodHandlesClass = methodHandles_Class.getDeclaredMethod("privateLookupIn",Class.class,methodHandlesLookup_Class);
+			privateLookupMethodOnMethodHandlesClass.setAccessible(true);
+			Object lookup = privateLookupMethodOnMethodHandlesClass.invoke(null, ClassLoader.class, methodHandlesLookup);
+			
+			// MethodHandle defineClassMethodHandle = lookup.findVirtual(ClassLoader.class, "defineClass", defineClassMethodType);
+			Method findVirtual_Method = methodHandlesLookup_Class.getDeclaredMethod("findVirtual", Class.class,String.class,methodType_Class);
+			findVirtual_Method.setAccessible(true);
+			defineClassMethodHandle = findVirtual_Method.invoke(lookup, ClassLoader.class, "defineClass",defineClassMethodType);
+
+			// clazz = defineClassMethodHandle.bindTo(loader).invokeWithArguments(name, bytes, 0, bytes.length);
+			Class<?> methodHandle_Class = Class.forName("java.lang.invoke.MethodHandle");
+			bindTo_Method = methodHandle_Class.getDeclaredMethod("bindTo", Object.class);
+			invokeWithArguments_Method = methodHandle_Class.getDeclaredMethod("invokeWithArguments",Object[].class);
+			
+			initializedForJava11 = true;
+		} catch (Exception e) {
+			e.printStackTrace();
+		}
+	}
+	
+	private void defineClass(ClassLoader loader, String name, byte[] bytes, ProtectionDomain protectionDomain) {
 		if (trace.isTraceEnabled()) {
 			trace.enter("defineClass", this, new Object[] { loader, name, bytes });
 		}
 		Object clazz = null;
 		debug("generating class '" + name + "'");
-
-		try {
-			if (defineClassMethod == null) {
-				defineClassMethod = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] { String.class,
-						bytes.getClass(), int.class, int.class });
+		if (LangUtil.is11VMOrGreater()) {
+			try {
+				if (!initializedForJava11) {
+					initializeForJava11();
+				}
+				// Do this: clazz = defineClassMethodHandle.bindTo(loader).invokeWithArguments(name, bytes, 0, bytes.length);
+				Object o = bindTo_Method.invoke(defineClassMethodHandle,loader);
+				clazz = invokeWithArguments_Method.invoke(o, new Object[] {new Object[] {name, bytes, 0, bytes.length}});
+			    
+			} catch (Throwable t) {
+				t.printStackTrace(System.err);
+				warn("define generated class failed", t);
 			}
-			defineClassMethod.setAccessible(true);
-			clazz = defineClassMethod.invoke(loader, new Object[] { name, bytes, new Integer(0), new Integer(bytes.length) });
-		} catch (InvocationTargetException e) {
-			if (e.getTargetException() instanceof LinkageError) {
-				warn("define generated class failed", e.getTargetException());
+		} else {
+			try {
+				if (defineClassMethod == null) {
+					synchronized (lock) {
+						getUnsafe();
+						defineClassMethod = 
+								Unsafe.class.getDeclaredMethod("defineClass", String.class,byte[].class,Integer.TYPE,Integer.TYPE, ClassLoader.class,ProtectionDomain.class);
+					}
+				}
+				defineClassMethod.setAccessible(true);
+				clazz = defineClassMethod.invoke(getUnsafe(), name,bytes,0,bytes.length,loader,protectionDomain);
+			} catch (LinkageError le) {
+				le.printStackTrace();
+				// likely thrown due to defining something that already exists?
+				// Old comments from before moving to Unsafe.defineClass():
 				// is already defined (happens for X$ajcMightHaveAspect interfaces since aspects are reweaved)
 				// TODO maw I don't think this is OK and
-			} else {
-				warn("define generated class failed", e.getTargetException());
+			} catch (Exception e) {
+				e.printStackTrace(System.err);
+				warn("define generated class failed", e);
 			}
-		} catch (Exception e) {
-			warn("define generated class failed", e);
 		}
 
 		if (trace.isTraceEnabled()) {
 			trace.exit("defineClass", clazz);
 		}
 	}
+	static Method defineClassMethod;
+	private static String lock = "lock";
+	
+	
+//    /*
+//    This method is equivalent to the following code but use reflection to compile on Java 7:
+//     MethodHandles.Lookup baseLookup = MethodHandles.lookup();
+//    MethodHandles.Lookup lookup = MethodHandles.privateLookupIn(ClassLoader.class, baseLookup);
+//    MethodHandle defineClassMethodHandle = lookup.findVirtual(ClassLoader.class, "defineClass", defineClassMethodType);
+//    handle.bindTo(classLoader).invokeWithArguments(className, classBytes, 0, classBytes.length));
+// */
+//@Override
+//@SuppressWarnings("unchecked")
+//public <T> Class<T> defineClass(ClassLoader classLoader, String className, byte[] classBytes) {
+//    Object baseLookup = methodHandlesLookup.invoke(null);
+//    Object lookup = methodHandlesPrivateLookupIn.invoke(null, ClassLoader.class, baseLookup);
+//    MethodHandle defineClassMethodHandle = (MethodHandle) lookupFindVirtual.invoke(lookup, ClassLoader.class, "defineClass", defineClassMethodType);
+//    try {
+//        return Cast.uncheckedCast(defineClassMethodHandle.bindTo(classLoader).invokeWithArguments(className, classBytes, 0, classBytes.length));
+//    } catch (Throwable throwable) {
+//        throw new RuntimeException(throwable);
+//        return (Class) defineClassMethodHandle.bindTo(classLoader).invokeWithArguments(className, classBytes, 0, classBytes.length);
+//    } catch (Throwable e) {
+//        throw new RuntimeException(e);
+//    }
+//}
 
-	private void defineClass(ClassLoader loader, String name, byte[] bytes, ProtectionDomain protectionDomain) {
-		if (trace.isTraceEnabled()) {
-			trace.enter("defineClass", this, new Object[] { loader, name, bytes, protectionDomain });
-		}
-		Object clazz = null;
-		debug("generating class '" + name + "'");
-
-		try {
-			// System.out.println(">> Defining with protection domain " + name + " pd=" + protectionDomain);
-			if (defineClassWithProtectionDomainMethod == null) {
-				defineClassWithProtectionDomainMethod = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] {
-						String.class, bytes.getClass(), int.class, int.class, ProtectionDomain.class });
-			}
-			defineClassWithProtectionDomainMethod.setAccessible(true);
-			clazz = defineClassWithProtectionDomainMethod.invoke(loader, new Object[] { name, bytes, Integer.valueOf(0),
-					new Integer(bytes.length), protectionDomain });
-		} catch (InvocationTargetException e) {
-			if (e.getTargetException() instanceof LinkageError) {
-				warn("define generated class failed", e.getTargetException());
-				// is already defined (happens for X$ajcMightHaveAspect interfaces since aspects are reweaved)
-				// TODO maw I don't think this is OK and
-			} else {
-				warn("define generated class failed", e.getTargetException());
-			}
-		} catch (Exception e) {
-			warn("define generated class failed", e);
-		}
-
-		if (trace.isTraceEnabled()) {
-			trace.exit("defineClass", clazz);
-		}
+	private void defineClass(ClassLoader loader, String name, byte[] bytes){
+		defineClass(loader,name,bytes,null);//, ProtectionDomain protectionDomain) {
 	}
+//		if (trace.isTraceEnabled()) {
+//			trace.enter("defineClass", this, new Object[] { loader, name, bytes, protectionDomain });
+//		}
+//		Object clazz = null;
+//		debug("generating class '" + name + "'");
+//		try {
+//			getUnsafe().defineClass(name, bytes, 0, bytes.length, loader, protectionDomain);
+//		} catch (LinkageError le) {
+//			// likely thrown due to defining something that already exists?
+//			// Old comments from before moving to Unsafe.defineClass():
+//			// is already defined (happens for X$ajcMightHaveAspect interfaces since aspects are reweaved)
+//			// TODO maw I don't think this is OK and
+//		} catch (Exception e) {
+//			warn("define generated class failed", e);
+//		}
+//
+//		if (trace.isTraceEnabled()) {
+//			trace.exit("defineClass", clazz);
+//		}
+//	}
+
 }
\ No newline at end of file
