@@ -9,13 +9,10 @@
  * Contributors: 
  *     PARC     initial implementation 
  * ******************************************************************/
-
 package org.aspectj.weaver;
 
 import java.lang.reflect.Modifier;
 
-//import org.aspectj.weaver.ResolvedType.Name;
-
 /**
  * The AjcMemberMaker is responsible for creating the representations of methods/fields/etc that are placed in both aspects and
  * affected target types. It uses the NameMangler class to create the actual names that will be used.
@@ -69,12 +66,12 @@ public class AjcMemberMaker {
 	}
 
 	public static ResolvedMember perCflowField(UnresolvedType declaringType) {
-		return new ResolvedMemberImpl(Member.FIELD, declaringType, PUBLIC_STATIC_FINAL, NameMangler.PERCFLOW_FIELD_NAME,
+		return new ResolvedMemberImpl(Member.FIELD, declaringType, PUBLIC_STATIC, NameMangler.PERCFLOW_FIELD_NAME,
 				CFLOW_STACK_TYPE.getSignature());
 	}
 
 	public static ResolvedMember perSingletonField(UnresolvedType declaringType) {
-		return new ResolvedMemberImpl(Member.FIELD, declaringType, PUBLIC_STATIC_FINAL, NameMangler.PERSINGLETON_FIELD_NAME,
+		return new ResolvedMemberImpl(Member.FIELD, declaringType, PUBLIC_STATIC, NameMangler.PERSINGLETON_FIELD_NAME,
 				declaringType.getSignature());
 	}
 
