@@ -376,10 +376,16 @@ public class ReferenceType extends ResolvedType {
 		if (this.isFinal() || other.isFinal()) {
 			return false;
 		}
+		
+		// 20170927: What is the block of code for? It mentions jls5.5 which isn't on this topic (old version of jls?)
+		// Some possible references: http://docs.oracle.com/javase/specs/jls/se9/jls9.pdf  5.1.6 (narrowing reference conversion)
+		// On Java 9 the test GenericsTests.testAfterReturningWithWildcardVar will fail because this code below
+		// used to find Set and List were the same, but now finds they are not. (so it doesn't put out the unchecked
+		// conversion message). However the code "List l = (List)someSet;" still compiles on 9 - so is this code bogus?
+		
 		// ??? needs to be Methods, not just declared methods? JLS 5.5 unclear
 		ResolvedMember[] a = getDeclaredMethods();
-		ResolvedMember[] b = other.getDeclaredMethods(); // ??? is this cast
-		// always safe
+		ResolvedMember[] b = other.getDeclaredMethods();
 		for (int ai = 0, alen = a.length; ai < alen; ai++) {
 			for (int bi = 0, blen = b.length; bi < blen; bi++) {
 				if (!b[bi].isCompatibleWith(a[ai])) {
@@ -1161,6 +1167,7 @@ public class ReferenceType extends ResolvedType {
 		annotations = null;
 		annotationTypes = null;
 		newSuperclass = null;
+		bits = 0; // clears the hierarchy complete tag (amongst other things)
 		newInterfaces = null;
 		typeVariables = null;
 		parameterizedInterfaces.clear();
@@ -1168,7 +1175,7 @@ public class ReferenceType extends ResolvedType {
 		if (getDelegate() != null) {
 			delegate.ensureConsistent();
 		}
-		if (isRawType()) {
+		if (isParameterizedOrRawType()) {
 			ReferenceType genericType = getGenericType();
 			if (genericType != null) {
 				genericType.ensureConsistent();
