@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2010 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -472,4 +475,40 @@ public interface IClasspathEntry {
 	 * @deprecated Use {@link JavaCore#getResolvedClasspathEntry(IClasspathEntry)} instead
 	 */
 	IClasspathEntry getResolvedEntry();
+
+	/**
+	 * This is a convience method, that returns <code>true</code> if the extra attributes contain an attribute whose name
+	 * is {@link IClasspathAttribute#TEST} and whose value is 'true'.
+	 * 
+	 * @see #getExtraAttributes()
+	 * @see IClasspathAttribute#TEST
+	 * @return <code>true</code>, if if the extra attributes contain a attribute whose name is
+	 *         {@link IClasspathAttribute#TEST} and whose value is 'true'.
+	 * @since 3.14
+	 */
+	default public boolean isTest() {
+		for (IClasspathAttribute attribute : getExtraAttributes()) {
+			if (IClasspathAttribute.TEST.equals(attribute.getName()) && "true".equals(attribute.getValue())) //$NON-NLS-1$
+				return true;
+		}
+		return false;
+	}
+
+	/**
+	 * This is a convience method, that returns <code>true</code> if the extra attributes contain an attribute whose name
+	 * is {@link IClasspathAttribute#WITHOUT_TEST_CODE} and whose value is 'true'.
+	 * 
+	 * @see #getExtraAttributes()
+	 * @see IClasspathAttribute#WITHOUT_TEST_CODE
+	 * @return <code>true</code>, if if the extra attributes contain a attribute whose name is
+	 *         {@link IClasspathAttribute#WITHOUT_TEST_CODE} and whose value is 'true'.
+	 * @since 3.14
+	 */
+	default public boolean isWithoutTestCode() {
+		for (IClasspathAttribute attribute : getExtraAttributes()) {
+			if (IClasspathAttribute.WITHOUT_TEST_CODE.equals(attribute.getName()) && "true".equals(attribute.getValue())) //$NON-NLS-1$
+				return true;
+		}
+		return false;
+	}
 }
