@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -344,6 +347,7 @@ void discardWorkingCopy() throws JavaModelException;
  * @return the found elements in this compilation unit that correspond to the given element
  * @since 3.0
  */
+@Override
 IJavaElement[] findElements(IJavaElement element);
 /**
  * Finds the working copy for this compilation unit, given a {@link WorkingCopyOwner}.
@@ -550,6 +554,7 @@ public boolean hasResourceChanged();
  * @return true if this element is a working copy, false otherwise
  * @since 3.0
  */
+@Override
 boolean isWorkingCopy();
 
 /**
@@ -785,5 +790,6 @@ CompilationUnit reconcile(int astLevel, int reconcileFlags, WorkingCopyOwner own
  * </ul>
  * @since 3.0
  */
+@Override
 void restore() throws JavaModelException;
 }
