@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -17,6 +20,7 @@
 package org.aspectj.org.eclipse.jdt.core;
 
 import java.util.Map;
+import java.util.Set;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
@@ -62,7 +66,6 @@ import org.aspectj.org.eclipse.jdt.core.eval.IEvaluationContext;
  * @see IClasspathEntry
  * @noimplement This interface is not intended to be implemented by clients.
  */
-@SuppressWarnings("rawtypes")
 public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 
 	/**
@@ -240,6 +243,23 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 * @since 2.1
 	 */
 	IPackageFragmentRoot[] findPackageFragmentRoots(IClasspathEntry entry);
+
+	/**
+	 * In a Java 9 project, a classpath entry can be filtered using a {@link IClasspathAttribute#LIMIT_MODULES} attribute,
+	 * otherwise for an unnamed module a default set of roots is used as defined in JEP 261.
+	 * In both cases {@link IJavaProject#findPackageFragmentRoots(IClasspathEntry)} will not contain all roots physically
+	 * present in the container.
+	 * <p>
+	 * This API can be used to bypass any filter and get really all roots to which the given entry is resolved.
+	 * </p>
+	 * 
+	 * @param entry a classpath entry of this Java project
+	 * @return the unfiltered array of package fragment roots to which the classpath entry resolves
+	 * @see #findPackageFragmentRoots(IClasspathEntry)
+	 * @since 3.14
+	 */
+	IPackageFragmentRoot[] findUnfilteredPackageFragmentRoots(IClasspathEntry entry);
+
 	/**
 	 * Returns the first type (excluding secondary types) found following this project's
 	 * classpath with the given fully qualified name or <code>null</code> if none is found.
@@ -407,6 +427,20 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 */
 	IType findType(String packageName, String typeQualifiedName, WorkingCopyOwner owner, IProgressMonitor progressMonitor) throws JavaModelException;
 
+	/**
+	 * Finds the first module with the given name found following this project's module path.
+	 * If the returned module descriptor is part of a compilation unit, its owner is the given owner.
+	 * @param moduleName the given module name
+	 * @param owner the owner of the returned module descriptor's compilation unit
+	 * 
+	 * @exception JavaModelException if this project does not exist or if an
+	 *		exception occurs while accessing its corresponding resource
+	 * @return the first module found following this project's module path
+	 * with the given name or <code>null</code> if none is found
+	 * @since 3.14
+	 */
+	IModuleDescription findModule(String moduleName, WorkingCopyOwner owner) throws JavaModelException;
+
 	/**
 	 * Returns all of the existing package fragment roots that exist
 	 * on the classpath, in the order they are defined by the classpath.
@@ -467,7 +501,7 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 * @see JavaCore#getDefaultOptions()
 	 * @since 2.1
 	 */
-	Map getOptions(boolean inheritJavaCoreOptions);
+	Map<String, String> getOptions(boolean inheritJavaCoreOptions);
 
 	/**
 	 * Returns the default output location for this project as a workspace-
@@ -576,6 +610,23 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 */
 	IProject getProject();
 
+	/**
+	 * Returns the <code>IModuleDescription</code> this project represents or 
+	 * null if the Java project doesn't represent any named module. A Java 
+	 * project is said to represent a module if any of its source package 
+	 * fragment roots (see {@link IPackageFragmentRoot#K_SOURCE}) contains a 
+	 * valid Java module descriptor, or if one of its classpath entries
+	 * has a valid {@link IClasspathAttribute#PATCH_MODULE} attribute.
+	 * In the latter case the corresponding module description of the
+	 * location referenced by that classpath entry is returned.
+	 * 
+	 * @return the <code>IModule</code> this project represents.
+	 * @exception JavaModelException if this element does not exist or if an
+	 *		exception occurs while accessing its corresponding resource
+	 * @since 3.14
+	 */
+	IModuleDescription getModuleDescription() throws JavaModelException;
+
 	/**
 	 * Returns the raw classpath for the project, as a list of classpath
 	 * entries. This corresponds to the exact set of entries which were assigned
@@ -913,7 +964,7 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 * @see JavaCore#getDefaultOptions()
 	 * @since 2.1
 	 */
-	void setOptions(Map newOptions);
+	void setOptions(Map<String, String> newOptions);
 
 	/**
 	 * Sets the default output location of this project to the location
@@ -1160,4 +1211,30 @@ public interface IJavaProject extends IParent, IJavaElement, IOpenable {
 	 */
 	void setRawClasspath(IClasspathEntry[] entries, IPath outputLocation, IProgressMonitor monitor)
 		throws JavaModelException;
+
+	/**
+	 * Returns the classpath entry that refers to the given path or <code>null</code> if there is no reference to the
+	 * path.
+	 * 
+	 * @param path
+	 *            IPath
+	 * @return the classpath entry or <code>null</null>.
+	 * @throws JavaModelException
+	 * @since 3.14
+	 */
+	IClasspathEntry getClasspathEntryFor(IPath path) throws JavaModelException;
+
+	/**
+	 * When compiling test code in a modular project that has non-source classpath entries which don't have the
+	 * {@link IClasspathAttribute#MODULE} set, the module is assumed to read the unnamed module (which is useful for
+	 * test-only dependencies that should not be mentioned in the module-info.java). When executing test code that was
+	 * compiled like this, corresponding "--add-reads" options need to be passed to the java runtime. This method
+	 * returns the list of modules on the project's classpath for which this is the case.
+	 * 
+	 * @return the set of module names
+	 * @throws JavaModelException
+	 *             when access to the classpath or module description of the given project fails.
+	 * @since 3.14
+	 */
+	Set<String> determineModulesOfProjectsWithNonEmptyClasspath() throws JavaModelException;
 }
