@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -68,6 +71,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 	 * @throws BuildException if anything wrong happen during the compilation
 	 * @return boolean true if the compilation is ok, false otherwise
 	 */
+	@Override
 	public boolean execute() throws BuildException {
 		this.attributes.log(AntAdapterMessages.getString("ant.jdtadapter.info.usingJDTCompiler"), Project.MSG_VERBOSE); //$NON-NLS-1$
 		Commandline cmd = setupJavacCommand();
@@ -84,13 +88,14 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 			}
 			return resultValue;
 		} catch (ClassNotFoundException cnfe) {
-			throw new BuildException(AntAdapterMessages.getString("ant.jdtadapter.error.cannotFindJDTCompiler")); //$NON-NLS-1$
+			throw new BuildException(AntAdapterMessages.getString("ant.jdtadapter.error.cannotFindJDTCompiler"), cnfe); //$NON-NLS-1$
 		} catch (Exception ex) {
 			throw new BuildException(ex);
 		}
 	}
 
 
+	@Override
 	protected Commandline setupJavacCommand() throws BuildException {
 		Commandline cmd = new Commandline();
 		this.customDefaultOptions = new CompilerOptions().getMap();
@@ -305,6 +310,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 		String source = this.attributes.getSource();
 		if (source != null) {
 			this.customDefaultOptions.put(CompilerOptions.OPTION_Source, source);
+			this.customDefaultOptions.put(CompilerOptions.OPTION_Compliance, source);
 		}
 
 		if (compilerArgs != null) {
@@ -509,6 +515,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 	 *
 	 * @param cmd the given command line
 	 */
+	@Override
 	protected void logAndAddFilesToCompile(Commandline cmd) {
 		this.attributes.log("Compilation " + cmd.describeArguments(), //$NON-NLS-1$
 				Project.MSG_VERBOSE);
@@ -518,7 +525,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 			niceSourceList.append("s"); //$NON-NLS-1$
 		}
 		niceSourceList.append(" to be compiled:"); //$NON-NLS-1$
-		niceSourceList.append(lSep);
+		niceSourceList.append(System.lineSeparator());
 
 		String[] encodedFiles = null, encodedDirs = null;
 		int encodedFilesLength = 0, encodedDirsLength = 0;
@@ -534,6 +541,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 			//we need the directories sorted, longest first,since sub directories can
 			//override encodings for their parent directories
 			Comparator comparator = new Comparator() {
+				@Override
 				public int compare(Object o1, Object o2) {
 					return ((String) o2).length() - ((String) o1).length();
 				}
@@ -569,7 +577,7 @@ public class JDTCompilerAdapter extends DefaultCompilerAdapter {
 				}
 			}
 			cmd.createArgument().setValue(arg);
-			niceSourceList.append("    " + arg + lSep); //$NON-NLS-1$
+			niceSourceList.append("    " + arg + System.lineSeparator()); //$NON-NLS-1$
 		}
 
 		this.attributes.log(niceSourceList.toString(), Project.MSG_VERBOSE);
