@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -72,7 +75,8 @@ public class ToolFactory {
 	 * @see #createCodeFormatter(Map, int)
 	 * @since 3.3
 	 */
-	public static final int M_FORMAT_NEW = new Integer(0).intValue();
+	// Supposed to be a non-compile time constant
+	public static final int M_FORMAT_NEW = Integer.valueOf(0).intValue();
 
 	/**
 	 * This mode is used for formatting existing code when all formatter options should be used.
@@ -85,7 +89,8 @@ public class ToolFactory {
 	 * @see #createCodeFormatter(Map, int)
 	 * @since 3.3
 	 */
-	public static final int M_FORMAT_EXISTING = new Integer(1).intValue();
+	// Supposed to be a non-compile time constant
+	public static final int M_FORMAT_EXISTING = Integer.valueOf(1).intValue();
 
 	/**
 	 * Create an instance of a code formatter. A code formatter implementation can be contributed via the
@@ -436,6 +441,7 @@ public class ToolFactory {
 	 * @see org.aspectj.org.eclipse.jdt.core.compiler.IScanner
 	 * @see #createScanner(boolean, boolean, boolean, String, String)
 	 */
+	@SuppressWarnings("javadoc") // references deprecated TokenNameIdentifier
 	public static IScanner createScanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean assertMode, boolean recordLineSeparator){
 		// use default workspace compliance
 		long complianceLevelValue = CompilerOptions.versionToJdkLevel(JavaCore.getOption(JavaCore.COMPILER_COMPLIANCE));
@@ -490,6 +496,7 @@ public class ToolFactory {
 	 * @see #createScanner(boolean, boolean, boolean, String, String)
 	 * @since 3.0
 	 */
+	@SuppressWarnings("javadoc") // references deprecated TokenNameIdentifier
 	public static IScanner createScanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean recordLineSeparator, String sourceLevel) {
 		// use default workspace compliance
 		long complianceLevelValue = CompilerOptions.versionToJdkLevel(JavaCore.getOption(JavaCore.COMPILER_COMPLIANCE));
@@ -544,6 +551,7 @@ public class ToolFactory {
 	 *
 	 * @since 3.1
 	 */
+	@SuppressWarnings("javadoc") // references deprecated TokenNameIdentifier
 	public static IScanner createScanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean recordLineSeparator, String sourceLevel, String complianceLevel) {
 		PublicScanner scanner = null;
 		long sourceLevelValue = CompilerOptions.versionToJdkLevel(sourceLevel);
