@@ -1,9 +1,13 @@
+// AspectJ
 /*******************************************************************************
- * Copyright (c) 2004, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2004, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -20,6 +24,7 @@ import java.util.List;
 import java.util.Map;
 
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.SubMonitor;
 import org.aspectj.org.eclipse.jdt.core.IClassFile;
 import org.aspectj.org.eclipse.jdt.core.ICompilationUnit;
 import org.aspectj.org.eclipse.jdt.core.IJavaElement;
@@ -34,6 +39,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclarat
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ConstructorDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ExplicitConstructorCall;
 import org.aspectj.org.eclipse.jdt.internal.compiler.batch.Main;
+import org.aspectj.org.eclipse.jdt.internal.compiler.batch.FileSystem.Classpath;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveryScanner;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveryScannerData;
@@ -90,7 +96,7 @@ import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
  * @since 3.0
  * @noinstantiate This class is not intended to be instantiated by clients.
  */
-@SuppressWarnings({ "rawtypes", "unchecked" })
+@SuppressWarnings({"rawtypes"})
 public class ASTParser {
 
     // AspectJ Extension start 
@@ -179,7 +185,7 @@ public class ASTParser {
 	/**
 	 * Compiler options. Defaults to JavaCore.getOptions().
 	 */
-	private Map compilerOptions;
+	private Map<String, String> compilerOptions;
 	
     /**
 	 * The focal point for a partial AST request.
@@ -262,7 +268,10 @@ public class ASTParser {
 			case AST.JLS2_INTERNAL:
 			case AST.JLS3_INTERNAL:
 			case AST.JLS4_INTERNAL:
-			case AST.JLS8:
+			case AST.JLS8_INTERNAL:
+			case AST.JLS9_INTERNAL:
+			case AST.JLS10_INTERNAL:
+			case AST.JLS11_INTERNAL:
 				break;
 			default:
 				throw new IllegalArgumentException();
@@ -271,9 +280,9 @@ public class ASTParser {
 		initializeDefaults();
 	}
 
-	private List getClasspath() throws IllegalStateException {
+	private List<Classpath> getClasspath() throws IllegalStateException {
 		Main main = new Main(new PrintWriter(System.out), new PrintWriter(System.err), false/*systemExit*/, null/*options*/, null/*progress*/);
-		ArrayList allClasspaths = new ArrayList();
+		ArrayList<Classpath> allClasspaths = new ArrayList<Classpath>();
 		try {
 			if ((this.bits & CompilationUnitResolver.INCLUDE_RUNNING_VM_BOOTCLASSPATH) != 0) {
 				org.aspectj.org.eclipse.jdt.internal.compiler.util.Util.collectRunningVMBootclasspath(allClasspaths);
@@ -298,7 +307,7 @@ public class ASTParser {
 				throw new IllegalStateException("invalid environment settings"); //$NON-NLS-1$
 			}
 		} catch (IllegalArgumentException e) {
-			throw new IllegalStateException("invalid environment settings"); //$NON-NLS-1$
+			throw new IllegalStateException("invalid environment settings", e); //$NON-NLS-1$
 		}
 		return allClasspaths;
 	}
@@ -410,12 +419,12 @@ public class ASTParser {
 	 * value type: <code>String</code>), or <code>null</code>
 	 * to set it back to the default
 	 */
-	public void setCompilerOptions(Map options) {
+	public void setCompilerOptions(Map<String, String> options) {
 		if (options == null) {
 			options = JavaCore.getOptions();
 		} else {
 			// copy client's options so as to not do any side effect on them
-			options = new HashMap(options);
+			options = new HashMap<>(options);
 		}
 		options.remove(JavaCore.COMPILER_TASK_TAGS); // no need to parse task tags
 		this.compilerOptions = options;
@@ -690,7 +699,7 @@ public class ASTParser {
 		this.rawSource = null;
 		if (source != null) {
 			this.project = source.getJavaProject();
-			Map options = this.project.getOptions(true);
+			Map<String, String> options = this.project.getOptions(true);
 			options.remove(JavaCore.COMPILER_TASK_TAGS); // no need to parse task tags
 			this.compilerOptions = options;
 		}
@@ -783,6 +792,10 @@ public class ASTParser {
 	 * {@link JavaCore#getJavaLikeExtensions() Java-like extensions}
 	 * and match the name of the main (public) class or interface declared in the source.</p>
 	 *
+	 * <p>
+	 * For compilation of a module-info.java file (since Java 9), the name of the compilation unit must be supplied.
+	 * Otherwise, module-info.java will be compiled as an ordinary Java file resulting in compilation errors.</p>
+	 *
 	 * <p>This name must represent the full path of the unit inside the given project. For example, if the source
 	 * declares a public class named "Foo" in a project "P" where the source folder is the project itself, the name
 	 * of the compilation unit must be "/P/Foo.java".
@@ -822,7 +835,7 @@ public class ASTParser {
 	public void setProject(IJavaProject project) {
 		this.project = project;
 		if (project != null) {
-			Map options = project.getOptions(true);
+			Map<String, String> options = project.getOptions(true);
 			options.remove(JavaCore.COMPILER_TASK_TAGS); // no need to parse task tags
 			this.compilerOptions = options;
 		}
@@ -834,6 +847,9 @@ public class ASTParser {
 	 * A successful call to this method returns all settings to their
 	 * default values so the object is ready to be reused.
 	 * </p>
+	 * <p>For identifying a module-info.java file as a special file instead of an ordinary 
+	 * Java file (Since Java 9), a call to this should be preceded by a call to 
+	 * {@link #setUnitName(String)} that sets the unit name as module-info.java</p>
 	 *
 	 * @param monitor the progress monitor used to report progress and request cancellation,
 	 *   or <code>null</code> if none
@@ -844,17 +860,16 @@ public class ASTParser {
 	 * are insufficient, contradictory, or otherwise unsupported
 	 */
 	public ASTNode createAST(IProgressMonitor monitor) {
+		SubMonitor subMonitor = SubMonitor.convert(monitor, 1);
 		ASTNode result = null;
-		if (monitor != null) monitor.beginTask("", 1); //$NON-NLS-1$
 		try {
 			if (this.rawSource == null && this.typeRoot == null) {
 				throw new IllegalStateException("source not specified"); //$NON-NLS-1$
 			}
-			result = internalCreateAST(monitor);
+			result = internalCreateAST(subMonitor.split(1));
 		} finally {
 			// reset to defaults to allow reuse (and avoid leaking)
 			initializeDefaults();
-			if (monitor != null) monitor.done();
 		}
 		return result;
 	}
@@ -1173,19 +1188,27 @@ public class ASTParser {
 							}
 							PackageFragment packageFragment = (PackageFragment) this.typeRoot.getParent();
 							BinaryType type = (BinaryType) this.typeRoot.findPrimaryType();
-							IBinaryType binaryType = (IBinaryType) type.getElementInfo();
-							// file name is used to recreate the Java element, so it has to be the toplevel .class file name
-							char[] fileName = binaryType.getFileName();
-							int firstDollar = CharOperation.indexOf('$', fileName);
-							if (firstDollar != -1) {
-								char[] suffix = SuffixConstants.SUFFIX_class;
-								int suffixLength = suffix.length;
-								char[] newFileName = new char[firstDollar + suffixLength];
-								System.arraycopy(fileName, 0, newFileName, 0, firstDollar);
-								System.arraycopy(suffix, 0, newFileName, firstDollar, suffixLength);
-								fileName = newFileName;
+							String fileNameString = null;
+							if (type != null) {
+								IBinaryType binaryType = (IBinaryType) type.getElementInfo();
+								// file name is used to recreate the Java element, so it has to be the toplevel .class file name
+								char[] fileName = binaryType.getFileName();
+
+								int firstDollar = CharOperation.indexOf('$', fileName);
+								if (firstDollar != -1) {
+									char[] suffix = SuffixConstants.SUFFIX_class;
+									int suffixLength = suffix.length;
+									char[] newFileName = new char[firstDollar + suffixLength];
+									System.arraycopy(fileName, 0, newFileName, 0, firstDollar);
+									System.arraycopy(suffix, 0, newFileName, firstDollar, suffixLength);
+									fileName = newFileName;
+								}
+								fileNameString = new String(fileName);
+							} else {
+								// assumed to be "module-info.class" (which has no type):
+								fileNameString = this.typeRoot.getElementName();
 							}
-							sourceUnit = new BasicCompilationUnit(sourceString.toCharArray(), Util.toCharArrays(packageFragment.names), new String(fileName), this.project);
+							sourceUnit = new BasicCompilationUnit(sourceString.toCharArray(), Util.toCharArrays(packageFragment.names), fileNameString, this.typeRoot);
 						} catch(JavaModelException e) {
 							// an error occured accessing the java element
 							StringWriter stringWriter = new StringWriter();
