@@ -1,9 +1,13 @@
+// AspectJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,9 +27,11 @@ import java.util.Map;
 
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.OperationCanceledException;
+import org.eclipse.core.runtime.SubMonitor;
 import org.aspectj.org.eclipse.jdt.core.ICompilationUnit;
 import org.aspectj.org.eclipse.jdt.core.IJavaElement;
 import org.aspectj.org.eclipse.jdt.core.IJavaProject;
+import org.aspectj.org.eclipse.jdt.core.JavaCore;
 import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.core.WorkingCopyOwner;
 import org.aspectj.org.eclipse.jdt.core.compiler.CategorizedProblem;
@@ -39,6 +45,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.IProblemFactory;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.batch.FileSystem.Classpath;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRestriction;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.INameEnvironment;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.ISourceType;
@@ -56,6 +63,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.util.HashtableOfObjectToInt
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Messages;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
 import org.aspectj.org.eclipse.jdt.internal.core.BinaryMember;
+import org.aspectj.org.eclipse.jdt.internal.core.BinaryModule;
 import org.aspectj.org.eclipse.jdt.internal.core.CancelableNameEnvironment;
 import org.aspectj.org.eclipse.jdt.internal.core.CancelableProblemFactory;
 import org.aspectj.org.eclipse.jdt.internal.core.INameEnvironmentWithProgress;
@@ -166,6 +174,7 @@ class CompilationUnitResolver extends Compiler {
 	/*
 	 * Add additional source types
 	 */
+	@Override
 	public void accept(ISourceType[] sourceTypes, PackageBinding packageBinding, AccessRestriction accessRestriction) {
 		// Need to reparse the entire source of the compilation unit so as to get source positions
 		// (case of processing a source that was not known by beginToCompile (e.g. when asking to createBinding))
@@ -173,6 +182,7 @@ class CompilationUnitResolver extends Compiler {
 		accept((org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit) sourceType.getHandle().getCompilationUnit(), accessRestriction);
 	}
 	
+	@Override
 	public synchronized void accept(org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit, AccessRestriction accessRestriction) {
 		super.accept(sourceUnit, accessRestriction);
 	}
@@ -206,6 +216,9 @@ class CompilationUnitResolver extends Compiler {
 							new String(sourceUnit.getFileName())
 						}));
 				}
+				if (this.parser instanceof CommentRecorderParser) {
+					((CommentRecorderParser) this.parser).resetComments();
+				}
 				// diet parsing for large collection of units
 				if (this.totalUnits < this.parseThreshold) {
 					parsedUnit = this.parser.parse(sourceUnit, unitResult);
@@ -247,6 +260,8 @@ class CompilationUnitResolver extends Compiler {
 			} else {
 				char[] key = resolver.hasTypeName()
 					? resolver.getKey().toCharArray() // binary binding
+					: resolver.hasModuleName()
+					    ? resolver.moduleName()
 					: CharOperation.concatWith(resolver.compoundName(), '.'); // package binding or base type binding
 				this.requestedKeys.put(key, resolver);
 			}
@@ -285,6 +300,20 @@ class CompilationUnitResolver extends Compiler {
 		// new code:
 		AST ast = ASTParser.getAST(apiLevel);
 		// End AspectJ Extension
+				String sourceModeSetting = (String) options.get(JavaCore.COMPILER_SOURCE);
+		long sourceLevel = CompilerOptions.versionToJdkLevel(sourceModeSetting);
+		if (sourceLevel == 0) {
+			// unknown sourceModeSetting
+			sourceLevel = ClassFileConstants.JDK1_3;
+		}
+		ast.scanner.sourceLevel = sourceLevel;
+		String compliance = (String) options.get(JavaCore.COMPILER_COMPLIANCE);
+		long complianceLevel = CompilerOptions.versionToJdkLevel(compliance);
+		if (complianceLevel == 0) {
+			// unknown sourceModeSetting
+			complianceLevel = sourceLevel;
+		}
+		ast.scanner.complianceLevel = complianceLevel;
 		ast.setDefaultNodeFlag(ASTNode.ORIGINAL);
 		CompilationUnit compilationUnit = null;
 		// AspectJ Extension - use the factory
@@ -325,12 +354,15 @@ class CompilationUnitResolver extends Compiler {
 
 		// passes the initial set of files to the batch oracle (to avoid finding more than once the same units when case insensitive match)
 		return new IErrorHandlingPolicy() {
+			@Override
 			public boolean stopOnFirstError() {
 				return false;
 			}
+			@Override
 			public boolean proceedOnErrors() {
 				return false; // stop if there are some errors
 			}
+			@Override
 			public boolean ignoreAllErrors() {
 				return false;
 			}
@@ -342,18 +374,18 @@ class CompilationUnitResolver extends Compiler {
 	 */
 	protected static ICompilerRequestor getRequestor() {
 		return new ICompilerRequestor() {
+			@Override
 			public void acceptResult(CompilationResult compilationResult) {
 				// do nothing
 			}
 		};
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.Compiler#initializeParser()
-	 */
+	@Override
 	public void initializeParser() {
 		this.parser = new CommentRecorderParser(this.problemReporter, false);
 	}
+	@Override
 	public void process(CompilationUnitDeclaration unit, int i) {
 		// don't resolve a second time the same unit (this would create the same binding twice)
 		char[] fileName = unit.compilationResult.getFileName();
@@ -363,6 +395,7 @@ class CompilationUnitResolver extends Compiler {
 	/*
 	 * Compiler crash recovery in case of unexpected runtime exceptions
 	 */
+	@Override
 	protected void handleInternalException(
 			Throwable internalException,
 			CompilationUnitDeclaration unit,
@@ -376,6 +409,7 @@ class CompilationUnitResolver extends Compiler {
 	/*
 	 * Compiler recovery in case of internal AbortCompilation event
 	 */
+	@Override
 	protected void handleInternalException(
 			AbortCompilation abortException,
 			CompilationUnitDeclaration unit) {
@@ -388,7 +422,6 @@ class CompilationUnitResolver extends Compiler {
 	}
 
 	public static void parse(ICompilationUnit[] compilationUnits, ASTRequestor astRequestor, int apiLevel, Map options, int flags, IProgressMonitor monitor) {
-		try {
 			CompilerOptions compilerOptions = new CompilerOptions(options);
 			compilerOptions.ignoreMethodBodies = (flags & ICompilationUnit.IGNORE_METHOD_BODIES) != 0;
 			Parser parser = new CommentRecorderParser(
@@ -398,10 +431,12 @@ class CompilationUnitResolver extends Compiler {
 						new DefaultProblemFactory()),
 				false);
 			int unitLength = compilationUnits.length;
-			if (monitor != null) monitor.beginTask("", unitLength); //$NON-NLS-1$
+		SubMonitor subMonitor = SubMonitor.convert(monitor);
 			for (int i = 0; i < unitLength; i++) {
+			subMonitor.setWorkRemaining(unitLength - i);
 				org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit = (org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit) compilationUnits[i];
-				CompilationResult compilationResult = new CompilationResult(sourceUnit, 0, 0, compilerOptions.maxProblemsPerUnit);
+			CompilationResult compilationResult = new CompilationResult(sourceUnit, 0, 0,
+					compilerOptions.maxProblemsPerUnit);
 				CompilationUnitDeclaration compilationUnitDeclaration = parser.dietParse(sourceUnit, compilationResult);
 
 				if (compilationUnitDeclaration.ignoreMethodBodies) {
@@ -420,16 +455,13 @@ class CompilationUnitResolver extends Compiler {
 				}
 
 				// convert AST
-				CompilationUnit node = convert(compilationUnitDeclaration, parser.scanner.getSource(), apiLevel, options, false/*don't resolve binding*/, null/*no owner needed*/, null/*no binding table needed*/, flags /* flags */, monitor, true);
+			CompilationUnit node = convert(compilationUnitDeclaration, parser.scanner.getSource(), apiLevel, options,
+					false/* don't resolve binding */, null/* no owner needed */, null/* no binding table needed */,
+					flags /* flags */, subMonitor.split(1), true);
 				node.setTypeRoot(compilationUnits[i]);
 
 				// accept AST
 				astRequestor.acceptAST(compilationUnits[i], node);
-
-				if (monitor != null) monitor.worked(1);
-			}
-		} finally {
-			if (monitor != null) monitor.done();
 		}
 	}
 	public static void parse(
@@ -440,7 +472,6 @@ class CompilationUnitResolver extends Compiler {
 			Map options,
 			int flags,
 			IProgressMonitor monitor) {
-		try {
 			CompilerOptions compilerOptions = new CompilerOptions(options);
 			compilerOptions.ignoreMethodBodies = (flags & ICompilationUnit.IGNORE_METHOD_BODIES) != 0;
 			Parser parser = new CommentRecorderParser(
@@ -450,8 +481,9 @@ class CompilationUnitResolver extends Compiler {
 						new DefaultProblemFactory()),
 				false);
 			int unitLength = sourceUnits.length;
-			if (monitor != null) monitor.beginTask("", unitLength); //$NON-NLS-1$
+		SubMonitor subMonitor = SubMonitor.convert(monitor, unitLength);
 			for (int i = 0; i < unitLength; i++) {
+			SubMonitor iterationMonitor = subMonitor.split(1);
 				char[] contents = null;
 				String encoding = encodings != null ? encodings[i] : null;
 				try {
@@ -485,16 +517,13 @@ class CompilationUnitResolver extends Compiler {
 				}
 
 				// convert AST
-				CompilationUnit node = convert(compilationUnitDeclaration, parser.scanner.getSource(), apiLevel, options, false/*don't resolve binding*/, null/*no owner needed*/, null/*no binding table needed*/, flags /* flags */, monitor, true);
+			CompilationUnit node = convert(compilationUnitDeclaration, parser.scanner.getSource(), apiLevel, options,
+					false/* don't resolve binding */, null/* no owner needed */, null/* no binding table needed */,
+					flags /* flags */, iterationMonitor, true);
 				node.setTypeRoot(null);
 
 				// accept AST
 				astRequestor.acceptAST(sourceUnits[i], node);
-
-				if (monitor != null) monitor.worked(1);
-			}
-		} finally {
-			if (monitor != null) monitor.done();
 		}
 	}
 	public static CompilationUnitDeclaration parse(
@@ -578,12 +607,10 @@ class CompilationUnitResolver extends Compiler {
 		CancelableNameEnvironment environment = null;
 		CancelableProblemFactory problemFactory = null;
 		try {
-			if (monitor != null) {
 				int amountOfWork = (compilationUnits.length + bindingKeys.length) * 2; // 1 for beginToCompile, 1 for resolve
-				monitor.beginTask("", amountOfWork); //$NON-NLS-1$
-			}
-			environment = new CancelableNameEnvironment(((JavaProject) javaProject), owner, monitor);
-			problemFactory = new CancelableProblemFactory(monitor);
+			SubMonitor subMonitor = SubMonitor.convert(monitor, amountOfWork);
+			environment = new CancelableNameEnvironment(((JavaProject) javaProject), owner, subMonitor);
+			problemFactory = new CancelableProblemFactory(subMonitor);
 			CompilerOptions compilerOptions = getCompilerOptions(options, (flags & ICompilationUnit.ENABLE_STATEMENTS_RECOVERY) != 0);
 			compilerOptions.ignoreMethodBodies = (flags & ICompilationUnit.IGNORE_METHOD_BODIES) != 0;
 			CompilationUnitResolver resolver =
@@ -593,7 +620,7 @@ class CompilationUnitResolver extends Compiler {
 					compilerOptions,
 					getRequestor(),
 					problemFactory,
-					monitor,
+					subMonitor,
 					javaProject != null);
 			resolver.resolve(compilationUnits, bindingKeys, requestor, apiLevel, options, owner, flags);
 			if (NameLookup.VERBOSE) {
@@ -604,7 +631,6 @@ class CompilationUnitResolver extends Compiler {
 			// project doesn't exist -> simple parse without resolving
 			parse(compilationUnits, requestor, apiLevel, options, flags, monitor);
 		} finally {
-			if (monitor != null) monitor.done();
 			if (environment != null) {
 				environment.setMonitor(null); // don't hold a reference to this external object
 			}
@@ -627,14 +653,12 @@ class CompilationUnitResolver extends Compiler {
 			INameEnvironmentWithProgress environment = null;
 			CancelableProblemFactory problemFactory = null;
 			try {
-				if (monitor != null) {
 					int amountOfWork = (sourceUnits.length + bindingKeys.length) * 2; // 1 for beginToCompile, 1 for resolve
-					monitor.beginTask("", amountOfWork); //$NON-NLS-1$
-				}
+				SubMonitor subMonitor = SubMonitor.convert(monitor, amountOfWork);
 				Classpath[] allEntries = new Classpath[classpaths.size()];
 				classpaths.toArray(allEntries);
-				environment = new NameEnvironmentWithProgress(allEntries, null, monitor);
-				problemFactory = new CancelableProblemFactory(monitor);
+				environment = new NameEnvironmentWithProgress(allEntries, null, subMonitor);
+				problemFactory = new CancelableProblemFactory(subMonitor);
 				CompilerOptions compilerOptions = getCompilerOptions(options, (flags & ICompilationUnit.ENABLE_STATEMENTS_RECOVERY) != 0);
 				compilerOptions.ignoreMethodBodies = (flags & ICompilationUnit.IGNORE_METHOD_BODIES) != 0;
 				CompilationUnitResolver resolver =
@@ -644,7 +668,7 @@ class CompilationUnitResolver extends Compiler {
 						compilerOptions,
 						getRequestor(),
 						problemFactory,
-						monitor,
+						subMonitor,
 						false);
 				resolver.resolve(sourceUnits, encodings, bindingKeys, requestor, apiLevel, options, flags);
 				if (NameLookup.VERBOSE && (environment instanceof CancelableNameEnvironment)) {
@@ -653,7 +677,6 @@ class CompilationUnitResolver extends Compiler {
 					System.out.println(Thread.currentThread() + " TIME SPENT in NameLoopkup#seekTypesInBinaryPackage: " + cancelableNameEnvironment.nameLookup.timeSpentInSeekTypesInBinaryPackage + "ms");  //$NON-NLS-1$ //$NON-NLS-2$
 				}
 			} finally {
-				if (monitor != null) monitor.done();
 				if (environment != null) {
 					environment.setMonitor(null); // don't hold a reference to this external object
 				}
@@ -775,11 +798,13 @@ class CompilationUnitResolver extends Compiler {
 						key = ((LocalVariable) element).getKey(true/*open to get resolved info*/);
 					else if (element instanceof org.aspectj.org.eclipse.jdt.internal.core.TypeParameter)
 						key = ((org.aspectj.org.eclipse.jdt.internal.core.TypeParameter) element).getKey(true/*open to get resolved info*/);
+					else if (element instanceof BinaryModule)
+						key = ((BinaryModule) element).getKey(true);
 					else
 						throw new IllegalArgumentException(element + " has an unexpected type"); //$NON-NLS-1$
 					binaryElementPositions.put(key, i);
 				} catch (JavaModelException e) {
-					throw new IllegalArgumentException(element + " does not exist"); //$NON-NLS-1$
+					throw new IllegalArgumentException(element + " does not exist", e); //$NON-NLS-1$
 				}
 			}
 		}
@@ -792,6 +817,7 @@ class CompilationUnitResolver extends Compiler {
 
 		class Requestor extends ASTRequestor {
 			IBinding[] bindings = new IBinding[length];
+			@Override
 			public void acceptAST(ICompilationUnit source, CompilationUnit ast) {
 				// TODO (jerome) optimize to visit the AST only once
 				IntArrayList intList = (IntArrayList) sourceElementPositions.get(source);
@@ -802,11 +828,12 @@ class CompilationUnitResolver extends Compiler {
 					try {
 						finder.search();
 					} catch (JavaModelException e) {
-						throw new IllegalArgumentException(element + " does not exist"); //$NON-NLS-1$
+						throw new IllegalArgumentException(element + " does not exist", e); //$NON-NLS-1$
 					}
 					this.bindings[index] = finder.foundBinding;
 				}
 			}
+			@Override
 			public void acceptBinding(String bindingKey, IBinding binding) {
 				int index = binaryElementPositions.get(bindingKey);
 				this.bindings[index] = binding;
@@ -1018,6 +1045,11 @@ class CompilationUnitResolver extends Compiler {
 				}
 				sourceUnits[count++] = new org.aspectj.org.eclipse.jdt.internal.compiler.batch.CompilationUnit(contents, sourceUnitPath, encoding);
 			}
+			if (count < length) {
+				org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit[] newArray = new org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit[count];
+				System.arraycopy(sourceUnits, 0, newArray, 0, count);
+				sourceUnits = newArray;
+			}
 			beginToCompile(sourceUnits, bindingKeys);
 			// process all units (some more could be injected in the loop by the lookup environment)
 			for (int i = 0; i < this.totalUnits; i++) {
@@ -1224,6 +1256,9 @@ class CompilationUnitResolver extends Compiler {
 			}
 
 			if (unit.scope != null) {
+				CompilationUnitDeclaration previousUnit = this.lookupEnvironment.unitBeingCompleted;
+				this.lookupEnvironment.unitBeingCompleted = unit;
+				try {
 				// fault in fields & methods
 				unit.scope.faultInTypes();
 				if (unit.scope != null && verifyMethods) {
@@ -1242,6 +1277,9 @@ class CompilationUnitResolver extends Compiler {
 
 				// finalize problems (suppressWarnings)
 				unit.finalizeProblems();
+				} finally {
+					this.lookupEnvironment.unitBeingCompleted = previousUnit; // paranoia, always null in org.aspectj.org.eclipse.jdt.core.tests.dom.RunAllTests
+				}
 			}
 			if (this.unitsToProcess != null) this.unitsToProcess[0] = null; // release reference to processed unit declaration
 			this.requestor.acceptResult(unit.compilationResult.tagAsAccepted());
@@ -1268,6 +1306,7 @@ class CompilationUnitResolver extends Compiler {
 	/*
 	 * Internal API used to resolve a given compilation unit. Can run a subset of the compilation process
 	 */
+	@Override
 	public CompilationUnitDeclaration resolve(
 			org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit,
 			boolean verifyMethods,
@@ -1299,6 +1338,7 @@ class CompilationUnitResolver extends Compiler {
 	/*
 	 * Internal API used to resolve a given compilation unit. Can run a subset of the compilation process
 	 */
+	@Override
 	public CompilationUnitDeclaration resolve(
 			CompilationUnitDeclaration unit,
 			org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit sourceUnit,
