@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -14,7 +17,7 @@
 package org.aspectj.org.eclipse.jdt.core.dom;
 
 /**
- * A type binding represents fully-resolved type. There are a number of
+ * A type binding represents a fully-resolved type. There are a number of
  * different kinds of type bindings:
  * <ul>
  * <li>a class - represents the class declaration;
@@ -276,6 +279,27 @@ public interface ITypeBinding extends IBinding {
 	 */
 	public IBinding getDeclaringMember();
 
+	/**
+	 * Returns the binding of the module associated with the package in which
+	 * this type is declared.
+	 *
+	 * <p>The module of a recovered type reference binding is the module
+	 * associated with the package returned by {@link #getPackage()} ie either
+	 * the module associated with the package of the enclosing type, or, 
+	 * if the type name is the name of a {@linkplain AST#resolveWellKnownType(String) 
+	 * well-known type},  the module associated with the package of the matching 
+	 * well-known type.</p>
+	 *
+	 * @return the binding of the module associated with the package
+	 * in which this class, interface, enum, or annotation type is declared, 
+	 * or <code>null</code> if this type binding represents a primitive type, an array type, 
+	 * the null type, a type variable, a wildcard type or a capture binding.
+	 *
+	 * @since 3.14
+	 */
+	public default IModuleBinding getModule() {
+		return null;
+	}
 	/**
 	 * Returns the dimensionality of this array type, or <code>0</code> if this
 	 * is not an array type binding.
@@ -289,6 +313,9 @@ public interface ITypeBinding extends IBinding {
 	 * Returns the binding representing the element type of this array type,
 	 * or <code>null</code> if this is not an array type binding. The element
 	 * type of an array type is never itself an array type.
+	 * 
+	 * To get the type annotations on dimensions, clients should repeatedly
+	 *  call getComponentType() and get the type annotations from there. 
 	 *
 	 * @return the element type binding, or <code>null</code> if this is
 	 *   not an array type
@@ -385,6 +412,7 @@ public interface ITypeBinding extends IBinding {
 	 * if this type does not represent a class, an interface, an enum, an annotation
 	 * type or a recovered type.
 	 */
+	@Override
 	public int getModifiers();
 
 	/**
@@ -432,6 +460,7 @@ public interface ITypeBinding extends IBinding {
 	 * or the empty string if it has none
 	 * @see #getQualifiedName()
 	 */
+	@Override
 	public String getName();
 
 	/**
@@ -553,6 +582,9 @@ public interface ITypeBinding extends IBinding {
 	 * <li>java.lang.String</li>
 	 * </ul>
 	 * </p>
+	 * To get the type annotations on dimensions, clients should repeatedly call 
+	 * {@link #getComponentType()} and get the type annotations from there. 
+
 	 * @return type annotations specified on this type reference, or an empty array if
 	 * no type use annotations are found.
 	 * @see #getTypeDeclaration()
@@ -584,8 +616,8 @@ public interface ITypeBinding extends IBinding {
 	public ITypeBinding[] getTypeArguments();
 
 	/**
-	 * Returns the upper type bounds of this type variable, wildcard, or capture. If the
-	 * variable, wildcard, or capture had no explicit bound, then it returns an empty list.
+	 * Returns the upper type bounds of this type variable, wildcard, capture, or intersectionType. 
+	 * If the variable, wildcard, or capture had no explicit bound, then it returns an empty list.
      * <p>
      * Note that per construction, it can only contain one class or array type,
      * at most, and then it is located in first position.
@@ -595,11 +627,12 @@ public interface ITypeBinding extends IBinding {
      * binding, e.g. <code>capture-of ? extends Object[]</code>
      * </p>
 	 *
-	 * @return the list of upper bounds for this type variable, wildcard, or capture,
+	 * @return the list of upper bounds for this type variable, wildcard, capture, or intersection type
      * or otherwise the empty list
      * @see #isTypeVariable()
      * @see #isWildcardType()
 	 * @see #isCapture()
+	 * @see #isIntersectionType()
 	 * @since 3.1
 	 */
 	public ITypeBinding[] getTypeBounds();
@@ -818,6 +851,28 @@ public interface ITypeBinding extends IBinding {
 	 */
 	public boolean isInterface();
 
+	/**
+	 * Returns whether this type binding represents an intersection binding.
+	 * <p>
+	 * Intersection types can be derived from type parameter bounds and cast
+	 * expressions; they also arise in the processes of capture conversion 
+	 * and least upper bound computation as specified in section 4.9 of 
+	 * <em>The Java Language Specification, Java SE 8 Edition</em> (JLS8).
+	 * </p>
+	 * <p>
+	 * All the types in the intersection type can be accessed using
+	 * {@link #getTypeBounds()}. Wildcard types with more than one
+	 * bound will also be reported as intersection type. To check whether this
+	 * is a wildcard type, use {@link #isWildcardType()}.
+	 * </p>
+	 * @return <code>true</code> if this type binding is an intersecting type,
+	 *   and <code>false</code> otherwise
+	 * @see #getTypeBounds()
+	 * @see ITypeBinding#isWildcardType()
+	 * @since 3.12
+	 */
+	public boolean isIntersectionType();
+
 	/**
 	 * Returns whether this type binding represents a local class.
 	 * <p>
