@@ -1,18 +1,24 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2015 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
+ *     Stephan Herrmann - Contribution for
+ *								Bug 429813 - [1.8][dom ast] IMethodBinding#getJavaElement() should return IMethod for lambda
  *******************************************************************************/
 
 package org.aspectj.org.eclipse.jdt.core.dom;
 
 import org.aspectj.org.eclipse.jdt.core.IJavaElement;
 import org.aspectj.org.eclipse.jdt.core.JavaCore;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ExtraCompilerModifiers;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
@@ -34,24 +40,25 @@ class MethodBinding implements IMethodBinding {
 		Modifier.ABSTRACT | Modifier.STATIC | Modifier.FINAL | Modifier.SYNCHRONIZED | Modifier.NATIVE |
 		Modifier.STRICTFP | Modifier.DEFAULT;
 	private static final ITypeBinding[] NO_TYPE_BINDINGS = new ITypeBinding[0];
-	private org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding binding;
-	private BindingResolver resolver;
-	private ITypeBinding[] parameterTypes;
-	private ITypeBinding[] exceptionTypes;
-	private String name;
-	private ITypeBinding declaringClass;
-	private ITypeBinding returnType;
-	private String key;
-	private ITypeBinding[] typeParameters;
-	private ITypeBinding[] typeArguments;
-	private IAnnotationBinding[] annotations;
-	private IAnnotationBinding[][] parameterAnnotations;
+	protected org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding binding;
+	protected BindingResolver resolver;
+	private volatile ITypeBinding[] parameterTypes;
+	private volatile ITypeBinding[] exceptionTypes;
+	private volatile String name;
+	private volatile ITypeBinding declaringClass;
+	private volatile ITypeBinding returnType;
+	private volatile String key;
+	private volatile ITypeBinding[] typeParameters;
+	private volatile ITypeBinding[] typeArguments;
+	private volatile IAnnotationBinding[] annotations;
+	private volatile IAnnotationBinding[][] parameterAnnotations;
 
 	MethodBinding(BindingResolver resolver, org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding binding) {
 		this.resolver = resolver;
 		this.binding = binding;
 	}
 
+	@Override
 	public boolean isAnnotationMember() {
 		return getDeclaringClass().isAnnotation();
 	}
@@ -59,6 +66,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#isConstructor()
 	 */
+	@Override
 	public boolean isConstructor() {
 		return this.binding.isConstructor();
 	}
@@ -67,6 +75,7 @@ class MethodBinding implements IMethodBinding {
 	 * @see IMethodBinding#isDefaultConstructor()
 	 * @since 3.0
 	 */
+	@Override
 	public boolean isDefaultConstructor() {
 		final ReferenceBinding declaringClassBinding = this.binding.declaringClass;
 		if (declaringClassBinding.isRawType()) {
@@ -85,6 +94,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#getName()
 	 */
+	@Override
 	public String getName() {
 		if (this.name == null) {
 			if (this.binding.isConstructor()) {
@@ -96,6 +106,7 @@ class MethodBinding implements IMethodBinding {
 		return this.name;
 	}
 
+	@Override
 	public IAnnotationBinding[] getAnnotations() {
 		if (this.annotations != null) {
 			return this.annotations;
@@ -107,6 +118,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#getDeclaringClass()
 	 */
+	@Override
 	public ITypeBinding getDeclaringClass() {
 		if (this.declaringClass == null) {
 			this.declaringClass = this.resolver.getTypeBinding(this.binding.declaringClass);
@@ -114,6 +126,12 @@ class MethodBinding implements IMethodBinding {
 		return this.declaringClass;
 	}
 
+	@Override
+	public IBinding getDeclaringMember() {
+		return null;
+	}
+
+	@Override
 	public IAnnotationBinding[] getParameterAnnotations(int index) {
 		if (getParameterTypes() == NO_TYPE_BINDINGS) {
 			return AnnotationBinding.NoAnnotations;
@@ -147,6 +165,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#getParameterTypes()
 	 */
+	@Override
 	public ITypeBinding[] getParameterTypes() {
 		if (this.parameterTypes != null) {
 			return this.parameterTypes;
@@ -181,12 +200,14 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#getDeclaredReceiverType()
 	 */
+	@Override
 	public ITypeBinding getDeclaredReceiverType() {
 		return this.resolver.getTypeBinding(this.binding.receiver);
 	}
 	/**
 	 * @see IMethodBinding#getReturnType()
 	 */
+	@Override
 	public ITypeBinding getReturnType() {
 		if (this.returnType == null) {
 			this.returnType = this.resolver.getTypeBinding(this.binding.returnType);
@@ -226,6 +247,7 @@ class MethodBinding implements IMethodBinding {
 		return AnnotationBinding.NoAnnotations;
 	}
 
+	@Override
 	public Object getDefaultValue() {
 		if (isAnnotationMember())
 			return MemberValuePairBinding.buildDOMValue(this.binding.getDefaultValue(), this.resolver);
@@ -235,6 +257,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#getExceptionTypes()
 	 */
+	@Override
 	public ITypeBinding[] getExceptionTypes() {
 		if (this.exceptionTypes != null) {
 			return this.exceptionTypes;
@@ -255,6 +278,7 @@ class MethodBinding implements IMethodBinding {
 		return this.exceptionTypes = exTypes;
 	}
 
+	@Override
 	public IJavaElement getJavaElement() {
 		JavaElement element = getUnresolvedJavaElement();
 		if (element == null)
@@ -279,6 +303,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#getKind()
 	 */
+	@Override
 	public int getKind() {
 		return IBinding.METHOD;
 	}
@@ -286,6 +311,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#getModifiers()
 	 */
+	@Override
 	public int getModifiers() {
 		return this.binding.getAccessFlags() & VALID_MODIFIERS;
 	}
@@ -293,6 +319,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#isDeprecated()
 	 */
+	@Override
 	public boolean isDeprecated() {
 		return this.binding.isDeprecated();
 	}
@@ -300,6 +327,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#isRecovered()
 	 */
+	@Override
 	public boolean isRecovered() {
 		return false;
 	}
@@ -307,6 +335,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#isSynthetic()
 	 */
+	@Override
 	public boolean isSynthetic() {
 		return this.binding.isSynthetic();
 	}
@@ -315,6 +344,7 @@ class MethodBinding implements IMethodBinding {
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#isVarargs()
 	 * @since 3.1
 	 */
+	@Override
 	public boolean isVarargs() {
 		return this.binding.isVarargs();
 	}
@@ -322,6 +352,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IBinding#getKey()
 	 */
+	@Override
 	public String getKey() {
 		if (this.key == null) {
 			this.key = new String(this.binding.computeUniqueKey());
@@ -333,6 +364,7 @@ class MethodBinding implements IMethodBinding {
 	 * @see IBinding#isEqualTo(IBinding)
 	 * @since 3.1
 	 */
+	@Override
 	public boolean isEqualTo(IBinding other) {
 		if (other == this) {
 			// identical binding - equal (key or no key)
@@ -352,6 +384,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#getTypeParameters()
 	 */
+	@Override
 	public ITypeBinding[] getTypeParameters() {
 		if (this.typeParameters != null) {
 			return this.typeParameters;
@@ -376,6 +409,7 @@ class MethodBinding implements IMethodBinding {
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#isGenericMethod()
 	 * @since 3.1
 	 */
+	@Override
 	public boolean isGenericMethod() {
 		// equivalent to return getTypeParameters().length > 0;
 		if (this.typeParameters != null) {
@@ -388,6 +422,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#getTypeArguments()
 	 */
+	@Override
 	public ITypeBinding[] getTypeArguments() {
 		if (this.typeArguments != null) {
 			return this.typeArguments;
@@ -415,6 +450,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#isParameterizedMethod()
 	 */
+	@Override
 	public boolean isParameterizedMethod() {
 		return (this.binding instanceof ParameterizedGenericMethodBinding)
 			&& !((ParameterizedGenericMethodBinding) this.binding).isRaw;
@@ -423,11 +459,13 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#isRawMethod()
 	 */
+	@Override
 	public boolean isRawMethod() {
 		return (this.binding instanceof ParameterizedGenericMethodBinding)
 			&& ((ParameterizedGenericMethodBinding) this.binding).isRaw;
 	}
 
+	@Override
 	public boolean isSubsignature(IMethodBinding otherMethod) {
 		try {
 			LookupEnvironment lookupEnvironment = this.resolver.lookupEnvironment();
@@ -443,6 +481,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.IMethodBinding#getMethodDeclaration()
 	 */
+	@Override
 	public IMethodBinding getMethodDeclaration() {
 		return this.resolver.getMethodBinding(this.binding.original());
 	}
@@ -450,6 +489,7 @@ class MethodBinding implements IMethodBinding {
 	/**
 	 * @see IMethodBinding#overrides(IMethodBinding)
 	 */
+	@Override
 	public boolean overrides(IMethodBinding otherMethod) {
 			LookupEnvironment lookupEnvironment = this.resolver.lookupEnvironment();
 			return lookupEnvironment != null
@@ -460,7 +500,76 @@ class MethodBinding implements IMethodBinding {
 	 * For debugging purpose only.
 	 * @see java.lang.Object#toString()
 	 */
+	@Override
 	public String toString() {
 		return this.binding.toString();
 	}
+
+	/*
+	 * Method binding representing a lambda expression.
+	 * Most properties are read from the SAM descriptor,
+	 * but key, parameter types, and annotations are taken from the lambda implementation.
+	 * Additionally we store the declaring member (see #getDeclaringMember()).
+	 */
+	static class LambdaMethod extends MethodBinding {
+
+		private MethodBinding implementation;
+		private IBinding declaringMember;
+
+		public LambdaMethod(DefaultBindingResolver resolver,
+							org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding lambdaDescriptor,
+							org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding implementation,
+							IBinding declaringMember)
+		{
+			super(resolver, lambdaDescriptor);
+			this.implementation = new MethodBinding(resolver, implementation);
+			this.declaringMember = declaringMember;
+		}
+
+		/**
+		 * @see IBinding#getModifiers()
+		 */
+		@Override
+		public int getModifiers() {
+			return super.getModifiers() & ~ClassFileConstants.AccAbstract;
+		}
+
+		/**
+		 * @see IBinding#getKey()
+		 */
+		@Override
+		public String getKey() {
+			return this.implementation.getKey();
+		}
+
+		@Override
+		public ITypeBinding[] getParameterTypes() {
+			return this.implementation.getParameterTypes();
+		}
+
+		@Override
+		public IAnnotationBinding[] getParameterAnnotations(int paramIndex) {
+			return this.implementation.getParameterAnnotations(paramIndex);
+		}
+
+		@Override
+		public IAnnotationBinding[] getAnnotations() {
+			return this.implementation.getAnnotations();
+		}
+
+		@Override
+		public IBinding getDeclaringMember() {
+			return this.declaringMember;
+		}
+
+		@Override
+		public IMethodBinding getMethodDeclaration() {
+			return this.resolver.getMethodBinding(this.binding);
+		}
+
+		@Override
+		public String toString() {
+			return super.toString().replace("public abstract ", "public ");  //$NON-NLS-1$//$NON-NLS-2$
+		}
+	}
 }
