@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2007, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2007, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -20,6 +23,7 @@ import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.CompilationUnitScope;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SuffixConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
@@ -70,59 +74,43 @@ class RecoveredTypeBinding implements ITypeBinding {
 		this.resolver = resolver;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#createArrayType(int)
-	 */
+	@Override
 	public ITypeBinding createArrayType(int dims) {
 		return this.resolver.getTypeBinding(this, dims);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getBinaryName()
-	 */
+	@Override
 	public String getBinaryName() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getBound()
-	 */
+	@Override
 	public ITypeBinding getBound() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getGenericTypeOfWildcardType()
-	 */
+	@Override
 	public ITypeBinding getGenericTypeOfWildcardType() {
 		return null;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getRank()
-	 */
+	@Override
 	public int getRank() {
 		return -1;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getComponentType()
-	 */
+	@Override
 	public ITypeBinding getComponentType() {
 		if (this.dimensions == 0) return null;
 		return this.resolver.getTypeBinding(this, -1);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaredFields()
-	 */
+	@Override
 	public IVariableBinding[] getDeclaredFields() {
 		return TypeBinding.NO_VARIABLE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaredMethods()
-	 */
+	@Override
 	public IMethodBinding[] getDeclaredMethods() {
 		return TypeBinding.NO_METHOD_BINDINGS;
 	}
@@ -131,49 +119,37 @@ class RecoveredTypeBinding implements ITypeBinding {
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaredModifiers()
 	 * @deprecated Use ITypeBinding#getModifiers() instead
 	 */
+	@Override
 	public int getDeclaredModifiers() {
 		return 0;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaredTypes()
-	 */
+	@Override
 	public ITypeBinding[] getDeclaredTypes() {
 		return TypeBinding.NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaringClass()
-	 */
+	@Override
 	public ITypeBinding getDeclaringClass() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaringMethod()
-	 */
+	@Override
 	public IMethodBinding getDeclaringMethod() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDeclaringMember()
-	 */
 	@Override
 	public IBinding getDeclaringMember() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getDimensions()
-	 */
+	@Override
 	public int getDimensions() {
 		return this.dimensions;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getElementType()
-	 */
+	@Override
 	public ITypeBinding getElementType() {
 		if (this.binding != null) {
 			if (this.binding.isArrayType()) {
@@ -195,38 +171,27 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return null;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getErasure()
-	 */
+	@Override
 	public ITypeBinding getErasure() {
 		return this;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getFunctionalInterfaceMethod
-	 */
 	@Override
 	public IMethodBinding getFunctionalInterfaceMethod() {
 		return null;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getInterfaces()
-	 */
+	@Override
 	public ITypeBinding[] getInterfaces() {
 		return TypeBinding.NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getModifiers()
-	 */
+	@Override
 	public int getModifiers() {
 		return Modifier.NONE;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getName()
-	 */
+	@Override
 	public String getName() {
 		char[] brackets = new char[this.dimensions * 2];
 		for (int i = this.dimensions * 2 - 1; i >= 0; i -= 2) {
@@ -249,9 +214,29 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return getTypeNameFrom(getType());
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getPackage()
-	 */
+	@Override
+	public IModuleBinding getModule() {
+		if (this.binding != null) {
+			switch (this.binding.kind()) {
+				case Binding.BASE_TYPE :
+				case Binding.ARRAY_TYPE :
+				case Binding.TYPE_PARAMETER : // includes capture scenario
+				case Binding.WILDCARD_TYPE :
+				case Binding.INTERSECTION_TYPE:
+					return null;
+			}
+			return getModule(this.binding.getPackage());
+		}
+		CompilationUnitScope scope = this.resolver.scope();
+		return scope != null ? getModule(scope.getCurrentPackage()) : null;
+	}
+
+	private IModuleBinding getModule(PackageBinding pBinding) {
+		IPackageBinding packageBinding = this.resolver.getPackageBinding(pBinding);
+		return packageBinding != null ? packageBinding.getModule() : null;
+	}
+
+	@Override
 	public IPackageBinding getPackage() {
 		if (this.binding != null) {
 			switch (this.binding.kind()) {
@@ -275,9 +260,7 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getQualifiedName()
-	 */
+	@Override
 	public String getQualifiedName() {
 		ReferenceBinding referenceBinding = getReferenceBinding();
 		if (referenceBinding != null) {
@@ -311,9 +294,7 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getSuperclass()
-	 */
+	@Override
 	public ITypeBinding getSuperclass() {
 		if (getQualifiedName().equals("java.lang.Object")) {	//$NON-NLS-1$
 			return null;
@@ -321,9 +302,7 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return this.resolver.resolveWellKnownType("java.lang.Object"); //$NON-NLS-1$
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeArguments()
-	 */
+	@Override
 	public ITypeBinding[] getTypeArguments() {
 		if (this.binding != null) {
 			return this.typeArguments = TypeBinding.NO_TYPE_BINDINGS;
@@ -353,58 +332,42 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return this.typeArguments = TypeBinding.NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeBounds()
-	 */
+	@Override
 	public ITypeBinding[] getTypeBounds() {
 		return TypeBinding.NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeDeclaration()
-	 */
+	@Override
 	public ITypeBinding getTypeDeclaration() {
 		return this;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeParameters()
-	 */
+	@Override
 	public ITypeBinding[] getTypeParameters() {
 		return TypeBinding.NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getWildcard()
-	 */
+	@Override
 	public ITypeBinding getWildcard() {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isAnnotation()
-	 */
+	@Override
 	public boolean isAnnotation() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isAnonymous()
-	 */
+	@Override
 	public boolean isAnonymous() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isArray()
-	 */
+	@Override
 	public boolean isArray() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isAssignmentCompatible(org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding)
-	 */
+	@Override
 	public boolean isAssignmentCompatible(ITypeBinding typeBinding) {
 		if ("java.lang.Object".equals(typeBinding.getQualifiedName())) { //$NON-NLS-1$
 			return true;
@@ -413,16 +376,12 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return isEqualTo(typeBinding);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isCapture()
-	 */
+	@Override
 	public boolean isCapture() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isCastCompatible(org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding)
-	 */
+	@Override
 	public boolean isCastCompatible(ITypeBinding typeBinding) {
 		if ("java.lang.Object".equals(typeBinding.getQualifiedName())) { //$NON-NLS-1$
 			return true;
@@ -431,72 +390,57 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return isEqualTo(typeBinding);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isClass()
-	 */
+	@Override
 	public boolean isClass() {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isEnum()
-	 */
+	@Override
 	public boolean isEnum() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isFromSource()
-	 */
+	@Override
 	public boolean isFromSource() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isGenericType()
-	 */
+	@Override
 	public boolean isGenericType() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isInterface()
-	 */
+	@Override
 	public boolean isInterface() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isLocal()
-	 */
+	@Override
+	public boolean isIntersectionType() {
+		return false;
+	}
+
+	@Override
 	public boolean isLocal() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isMember()
-	 */
+	@Override
 	public boolean isMember() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isNested()
-	 */
+	@Override
 	public boolean isNested() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isNullType()
-	 */
+	@Override
 	public boolean isNullType() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isParameterizedType()
-	 */
+	@Override
 	public boolean isParameterizedType() {
 		if (this.innerTypeBinding != null) {
 			return this.innerTypeBinding.isParameterizedType();
@@ -507,23 +451,17 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isPrimitive()
-	 */
+	@Override
 	public boolean isPrimitive() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isRawType()
-	 */
+	@Override
 	public boolean isRawType() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isSubTypeCompatible(org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding)
-	 */
+	@Override
 	public boolean isSubTypeCompatible(ITypeBinding typeBinding) {
 		if ("java.lang.Object".equals(typeBinding.getQualifiedName())) { //$NON-NLS-1$
 			return true;
@@ -532,44 +470,32 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return isEqualTo(typeBinding);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isTopLevel()
-	 */
+	@Override
 	public boolean isTopLevel() {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isTypeVariable()
-	 */
+	@Override
 	public boolean isTypeVariable() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isUpperbound()
-	 */
+	@Override
 	public boolean isUpperbound() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isWildcardType()
-	 */
+	@Override
 	public boolean isWildcardType() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#getAnnotations()
-	 */
+	@Override
 	public IAnnotationBinding[] getAnnotations() {
 		return AnnotationBinding.NoAnnotations;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#getJavaElement()
-	 */
+	@Override
 	public IJavaElement getJavaElement() {
 		IPackageBinding packageBinding = getPackage();
 		if (packageBinding != null) {
@@ -582,9 +508,7 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#getKey()
-	 */
+	@Override
 	public String getKey() {
 		StringBuffer buffer = new StringBuffer();
 		buffer.append("Recovered#"); //$NON-NLS-1$
@@ -618,38 +542,28 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return String.valueOf(buffer);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#getKind()
-	 */
+	@Override
 	public int getKind() {
 		return IBinding.TYPE;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#isDeprecated()
-	 */
+	@Override
 	public boolean isDeprecated() {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#isEqualTo(org.aspectj.org.eclipse.jdt.core.dom.IBinding)
-	 */
+	@Override
 	public boolean isEqualTo(IBinding other) {
 		if (!other.isRecovered() || other.getKind() != IBinding.TYPE) return false;
 		return getKey().equals(other.getKey());
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#isRecovered()
-	 */
+	@Override
 	public boolean isRecovered() {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.IBinding#isSynthetic()
-	 */
+	@Override
 	public boolean isSynthetic() {
 		return false;
 	}
@@ -725,6 +639,7 @@ class RecoveredTypeBinding implements ITypeBinding {
 		return null; // should not happen
 	}
 
+	@Override
 	public IAnnotationBinding[] getTypeAnnotations() {
 		return AnnotationBinding.NoAnnotations;
 	}
