@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -90,6 +93,7 @@ class TypeBinding implements ITypeBinding {
 		this.prototype = (TypeBinding) (compilerPrototype == null || compilerPrototype == binding ? null : resolver.getTypeBinding(compilerPrototype)); //$IDENTITY-COMPARISON$
 	}
 
+	@Override
 	public ITypeBinding createArrayType(int dimension) {
 		int realDimensions = dimension;
 		realDimensions += getDimensions();
@@ -99,6 +103,7 @@ class TypeBinding implements ITypeBinding {
 		return this.resolver.resolveArrayType(this, dimension);
 	}
 
+	@Override
 	public IAnnotationBinding[] getAnnotations() {
 		if (this.prototype != null) {
 			return this.prototype.getAnnotations();
@@ -144,10 +149,7 @@ class TypeBinding implements ITypeBinding {
 		return AnnotationBinding.NoAnnotations;
 	}
 
-	/*
-	 * @see ITypeBinding#getBinaryName()
-	 * @since 3.0
-	 */
+	@Override
 	public String getBinaryName() {
 		if (this.binding.isCapture()) {
 			return null; // no binary name for capture binding
@@ -184,9 +186,7 @@ class TypeBinding implements ITypeBinding {
 		return new String(dotSeparated);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getBound()
-	 */
+	@Override
 	public ITypeBinding getBound() {
 		switch (this.binding.kind()) {
 			case Binding.WILDCARD_TYPE :
@@ -200,9 +200,7 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getGenericTypeOfWildcardType()
-	 */
+	@Override
 	public ITypeBinding getGenericTypeOfWildcardType() {
 		switch (this.binding.kind()) {
 			case Binding.WILDCARD_TYPE :
@@ -215,9 +213,7 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getRank()
-	 */
+	@Override
 	public int getRank() {
 		switch (this.binding.kind()) {
 			case Binding.WILDCARD_TYPE :
@@ -229,9 +225,7 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 	
-	/*
-	 * @see ITypeBinding#getComponentType()
-	 */
+	@Override
 	public ITypeBinding getComponentType() {
 		if (!isArray()) {
 			return null;
@@ -240,9 +234,7 @@ class TypeBinding implements ITypeBinding {
 		return this.resolver.getTypeBinding(arrayBinding.elementsType());
 	}
 
-	/*
-	 * @see ITypeBinding#getDeclaredFields()
-	 */
+	@Override
 	public synchronized IVariableBinding[] getDeclaredFields() {
 		if (this.prototype != null) {
 			return this.prototype.getDeclaredFields();
@@ -286,9 +278,7 @@ class TypeBinding implements ITypeBinding {
 		return this.fields = NO_VARIABLE_BINDINGS;
 	}
 
-	/*
-	 * @see ITypeBinding#getDeclaredMethods()
-	 */
+	@Override
 	public synchronized IMethodBinding[] getDeclaredMethods() {
 		if (this.prototype != null) {
 			return this.prototype.getDeclaredMethods();
@@ -338,13 +328,12 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#getDeclaredModifiers()
 	 * @deprecated Use ITypeBinding#getModifiers() instead
 	 */
+	@Override
 	public int getDeclaredModifiers() {
 		return getModifiers();
 	}
 
-	/*
-	 * @see ITypeBinding#getDeclaredTypes()
-	 */
+	@Override
 	public synchronized ITypeBinding[] getDeclaredTypes() { // should not deflect to prototype.
 		if (this.members != null) {
 			return this.members;
@@ -377,9 +366,7 @@ class TypeBinding implements ITypeBinding {
 		return this.members = NO_TYPE_BINDINGS;
 	}
 
-	/*
-	 * @see ITypeBinding#getDeclaringMethod()
-	 */
+	@Override
 	public synchronized IMethodBinding getDeclaringMethod() {
 		if (this.binding instanceof org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding) {
 			org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding localTypeBinding = (org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LocalTypeBinding) this.binding;
@@ -415,9 +402,7 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/*
-	 * @see ITypeBinding#getDeclaringClass()
-	 */
+	@Override
 	public synchronized ITypeBinding getDeclaringClass() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -451,15 +436,20 @@ class TypeBinding implements ITypeBinding {
 		}
 		return null;
 	}
-
+	@Override
+	public IModuleBinding getModule() {
+		if (this.binding instanceof ReferenceBinding && !this.binding.isTypeVariable()) {
+			IPackageBinding packageBinding = this.resolver.getPackageBinding(((ReferenceBinding) this.binding).getPackage());
+			return packageBinding != null ? packageBinding.getModule() : null;
+		}
+		return null;
+	}
 	@Override
 	public IBinding getDeclaringMember() {
 		return null;
 	}
 
-	/*
-	 * @see ITypeBinding#getDimensions()
-	 */
+	@Override
 	public int getDimensions() {
 		if (!isArray()) {
 			return 0;
@@ -468,9 +458,7 @@ class TypeBinding implements ITypeBinding {
 		return arrayBinding.dimensions;
 	}
 
-	/*
-	 * @see ITypeBinding#getElementType()
-	 */
+	@Override
 	public ITypeBinding getElementType() {
 		if (!isArray()) {
 			return null;
@@ -479,25 +467,18 @@ class TypeBinding implements ITypeBinding {
 		return this.resolver.getTypeBinding(arrayBinding.leafComponentType);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeDeclaration()
-	 */
+	@Override
 	public ITypeBinding getTypeDeclaration() {
 		if (this.binding instanceof ParameterizedTypeBinding)
 			return this.resolver.getTypeBinding(((ParameterizedTypeBinding)this.binding).genericType());
 		return this.resolver.getTypeBinding(this.binding.unannotated());
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getErasure()
-	 */
+	@Override
 	public ITypeBinding getErasure() {
 		return this.resolver.getTypeBinding(this.binding.erasure());
 	}
 	
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getFunctionalInterfaceMethod
-	 */
 	@Override
 	public IMethodBinding getFunctionalInterfaceMethod() {
 		Scope scope = this.resolver.scope();
@@ -509,6 +490,7 @@ class TypeBinding implements ITypeBinding {
 		return this.resolver.getMethodBinding(sam);
 	}
 
+	@Override
 	public synchronized ITypeBinding[] getInterfaces() {
 		if (this.prototype != null) {
 			return this.prototype.getInterfaces();
@@ -554,6 +536,20 @@ class TypeBinding implements ITypeBinding {
 		return this.interfaces = NO_TYPE_BINDINGS;
 	}
 
+	private ITypeBinding[] getIntersectingTypes() {
+		ITypeBinding[] intersectionBindings = TypeBinding.NO_TYPE_BINDINGS;
+		if (this.binding instanceof IntersectionTypeBinding18) {
+			ReferenceBinding[] intersectingTypes = this.binding.getIntersectingTypes();
+			int l = intersectingTypes.length;
+			intersectionBindings = new ITypeBinding[l];
+			for (int i = 0; i < l; ++i) {
+				intersectionBindings[i] = this.resolver.getTypeBinding(intersectingTypes[i]);
+			}
+		}
+		return intersectionBindings;
+	}
+
+	@Override
 	public IJavaElement getJavaElement() {
 		JavaElement element = getUnresolvedJavaElement();
 		if (element != null)
@@ -590,9 +586,7 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/*
-	 * @see IBinding#getKey()
-	 */
+	@Override
 	public String getKey() {
 		if (this.key == null) {
 			this.key = new String(this.binding.computeUniqueKey());
@@ -600,16 +594,12 @@ class TypeBinding implements ITypeBinding {
 		return this.key;
 	}
 
-	/*
-	 * @see IBinding#getKind()
-	 */
+	@Override
 	public int getKind() {
 		return IBinding.TYPE;
 	}
 
-	/*
-	 * @see IBinding#getModifiers()
-	 */
+	@Override
 	public int getModifiers() {
 		if (isClass()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -638,6 +628,7 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 
+	@Override
 	public String getName() {
 		StringBuffer buffer;
 		switch (this.binding.kind()) {
@@ -718,9 +709,7 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 
-	/*
-	 * @see ITypeBinding#getPackage()
-	 */
+	@Override
 	public IPackageBinding getPackage() {
 		switch (this.binding.kind()) {
 			case Binding.BASE_TYPE :
@@ -738,6 +727,7 @@ class TypeBinding implements ITypeBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getQualifiedName()
 	 */
+	@Override
 	public String getQualifiedName() {
 		StringBuffer buffer;
 		switch (this.binding.kind()) {
@@ -850,9 +840,7 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 
-	/*
-	 * @see ITypeBinding#getSuperclass()
-	 */
+	@Override
 	public synchronized ITypeBinding getSuperclass() {
 		if (this.binding == null)
 			return null;
@@ -883,9 +871,7 @@ class TypeBinding implements ITypeBinding {
 		return this.resolver.getTypeBinding(superclass);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeArguments()
-	 */
+	@Override
 	public ITypeBinding[] getTypeArguments() {
 		if (this.prototype != null) {
 			return this.prototype.getTypeArguments();
@@ -910,9 +896,7 @@ class TypeBinding implements ITypeBinding {
 		return this.typeArguments = NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeBounds()
-	 */
+	@Override
 	public ITypeBinding[] getTypeBounds() {
 		if (this.prototype != null) {
 			return this.prototype.getTypeBounds();
@@ -926,6 +910,8 @@ class TypeBinding implements ITypeBinding {
 		} else if (this.binding instanceof WildcardBinding) {
 			WildcardBinding wildcardBinding = (WildcardBinding) this.binding;
 			typeVariableBinding = wildcardBinding.typeVariable();
+		} else if (this.binding instanceof IntersectionTypeBinding18) {
+			return this.bounds = getIntersectingTypes();
 		}
 		if (typeVariableBinding != null) {
 			ReferenceBinding varSuperclass = typeVariableBinding.superclass();
@@ -976,9 +962,7 @@ class TypeBinding implements ITypeBinding {
 		return this.bounds = NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getTypeParameters()
-	 */
+	@Override
 	public ITypeBinding[] getTypeParameters() {
 		if (this.prototype != null) {
 			return this.prototype.getTypeParameters();
@@ -1007,10 +991,7 @@ class TypeBinding implements ITypeBinding {
 		return this.typeParameters = NO_TYPE_BINDINGS;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#getWildcard()
-	 * @since 3.1
-	 */
+	@Override
 	public ITypeBinding getWildcard() {
 		if (this.binding instanceof CaptureBinding) {
 			CaptureBinding captureBinding = (CaptureBinding) this.binding;
@@ -1019,10 +1000,7 @@ class TypeBinding implements ITypeBinding {
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isGenericType()
-	 * @since 3.1
-	 */
+	@Override
 	public boolean isGenericType() {
 		// equivalent to return getTypeParameters().length > 0;
 		if (isRawType()) {
@@ -1032,16 +1010,12 @@ class TypeBinding implements ITypeBinding {
 		return (typeVariableBindings != null && typeVariableBindings.length > 0);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isAnnotation()
-	 */
+	@Override
 	public boolean isAnnotation() {
 		return this.binding.isAnnotationType();
 	}
 
-	/*
-	 * @see ITypeBinding#isAnonymous()
-	 */
+	@Override
 	public boolean isAnonymous() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1050,16 +1024,12 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isArray()
-	 */
+	@Override
 	public boolean isArray() {
 		return this.binding.isArrayType();
 	}
 
-	/* (non-Javadoc)
-	 * @see ITypeBinding#isAssignmentCompatible(ITypeBinding)
-	 */
+	@Override
 	public boolean isAssignmentCompatible(ITypeBinding type) {
 		try {
 			if (this == type) return true; //$IDENTITY-COMPARISON$
@@ -1075,16 +1045,12 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see ITypeBinding#isCapture()
-	 */
+	@Override
 	public boolean isCapture() {
 		return this.binding.isCapture();
 	}
 
-	/* (non-Javadoc)
-	 * @see ITypeBinding#isCastCompatible(ITypeBinding)
-	 */
+	@Override
 	public boolean isCastCompatible(ITypeBinding type) {
 		try {
 			Scope scope = this.resolver.scope();
@@ -1101,9 +1067,7 @@ class TypeBinding implements ITypeBinding {
 		}
 	}
 
-	/*
-	 * @see ITypeBinding#isClass()
-	 */
+	@Override
 	public boolean isClass() {
 		switch (this.binding.kind()) {
 			case Binding.TYPE_PARAMETER :
@@ -1114,9 +1078,7 @@ class TypeBinding implements ITypeBinding {
 		return this.binding.isClass();
 	}
 
-	/*
-	 * @see IBinding#isDeprecated()
-	 */
+	@Override
 	public boolean isDeprecated() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1125,17 +1087,12 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see ITypeBinding#isEnum()
-	 */
+	@Override
 	public boolean isEnum() {
 		return this.binding.isEnum();
 	}
 
-	/*
-	 * @see IBinding#isEqualTo(Binding)
-	 * @since 3.1
-	 */
+	@Override
 	public boolean isEqualTo(IBinding other) {
 		if (other == this) {
 			// identical binding - equal (key or no key)
@@ -1156,9 +1113,7 @@ class TypeBinding implements ITypeBinding {
 		return BindingComparator.isEqual(this.binding, otherBinding);
 	}
 
-	/*
-	 * @see ITypeBinding#isFromSource()
-	 */
+	@Override
 	public boolean isFromSource() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1200,9 +1155,7 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isInterface()
-	 */
+	@Override
 	public boolean isInterface() {
 		switch (this.binding.kind()) {
 			case Binding.TYPE_PARAMETER :
@@ -1213,9 +1166,13 @@ class TypeBinding implements ITypeBinding {
 		return this.binding.isInterface();
 	}
 
-	/*
-	 * @see ITypeBinding#isLocal()
-	 */
+	@Override
+	public boolean isIntersectionType() {
+		int kind = this.binding.kind();
+		return kind == Binding.INTERSECTION_TYPE18 || kind == Binding.INTERSECTION_TYPE;
+	}
+
+	@Override
 	public boolean isLocal() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1224,9 +1181,7 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isMember()
-	 */
+	@Override
 	public boolean isMember() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1235,9 +1190,7 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isNested()
-	 */
+	@Override
 	public boolean isNested() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1249,41 +1202,32 @@ class TypeBinding implements ITypeBinding {
 	/**
 	 * @see ITypeBinding#isNullType()
 	 */
+	@Override
 	public boolean isNullType() {
 		return this.binding == org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding.NULL;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isParameterizedType()
-	 */
+	@Override
 	public boolean isParameterizedType() {
 		return this.binding.isParameterizedTypeWithActualArguments();
 	}
 
-	/*
-	 * @see ITypeBinding#isPrimitive()
-	 */
+	@Override
 	public boolean isPrimitive() {
 		return !isNullType() && this.binding.isBaseType();
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isRawType()
-	 */
+	@Override
 	public boolean isRawType() {
 		return this.binding.isRawType();
 	}
 
-	/* (non-Javadoc)
-	 * @see IBinding#isRecovered()
-	 */
+	@Override
 	public boolean isRecovered() {
 		return (this.binding.tagBits & TagBits.HasMissingType) != 0;
 	}
 
-	/* (non-Javadoc)
-	 * @see ITypeBinding#isSubTypeCompatible(ITypeBinding)
-	 */
+	@Override
 	public boolean isSubTypeCompatible(ITypeBinding type) {
 		try {
 			if (this == type) return true; //$IDENTITY-COMPARISON$
@@ -1302,13 +1246,12 @@ class TypeBinding implements ITypeBinding {
 	/**
 	 * @see IBinding#isSynthetic()
 	 */
+	@Override
 	public boolean isSynthetic() {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isTopLevel()
-	 */
+	@Override
 	public boolean isTopLevel() {
 		if (isClass() || isInterface() || isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
@@ -1317,16 +1260,12 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/*
-	 * @see ITypeBinding#isTypeVariable()
-	 */
+	@Override
 	public boolean isTypeVariable() {
 		return this.binding.isTypeVariable() && !this.binding.isCapture();
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isUpperbound()
-	 */
+	@Override
 	public boolean isUpperbound() {
 		switch (this.binding.kind()) {
 			case Binding.WILDCARD_TYPE :
@@ -1337,9 +1276,7 @@ class TypeBinding implements ITypeBinding {
 		return false;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.core.dom.ITypeBinding#isWildcardType()
-	 */
+	@Override
 	public boolean isWildcardType() {
 		return this.binding.isWildcard();
 	}
@@ -1348,13 +1285,12 @@ class TypeBinding implements ITypeBinding {
 	 * For debugging purpose only.
 	 * @see java.lang.Object#toString()
 	 */
+	@Override
 	public String toString() {
 		return this.binding.toString();
 	}
 
-	/*
-	 * @see ITypeBinding#getTypeUseAnnotations()
-	 */
+	@Override
 	public IAnnotationBinding[] getTypeAnnotations() {
 		if (this.typeAnnotations != null) {
 			return this.typeAnnotations;
