@@ -1,13 +1,18 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     John Glassmyer <jogl@google.com> - import group sorting is broken - https://bugs.eclipse.org/430303
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 
 package org.aspectj.org.eclipse.jdt.core.dom.rewrite;
@@ -22,8 +27,7 @@ import java.util.Set;
 
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.NullProgressMonitor;
-import org.eclipse.core.runtime.SubProgressMonitor;
+import org.eclipse.core.runtime.SubMonitor;
 import org.aspectj.org.eclipse.jdt.core.Flags;
 import org.aspectj.org.eclipse.jdt.core.ICompilationUnit;
 import org.aspectj.org.eclipse.jdt.core.IImportDeclaration;
@@ -66,8 +70,8 @@ import org.aspectj.org.eclipse.jdt.core.dom.StringLiteral;
 import org.aspectj.org.eclipse.jdt.core.dom.Type;
 import org.aspectj.org.eclipse.jdt.core.dom.TypeLiteral;
 import org.aspectj.org.eclipse.jdt.core.dom.WildcardType;
-import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.imports.ImportRewriteConfiguration;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.imports.ImportRewriteAnalyzer;
+import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.imports.ImportRewriteConfiguration;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.imports.ImportRewriteConfiguration.ImplicitImportIdentification;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.imports.ImportRewriteConfiguration.ImportContainerSorting;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Messages;
@@ -100,6 +104,134 @@ import org.eclipse.text.edits.TextEdit;
  */
 @SuppressWarnings({ "rawtypes", "unchecked" })
 public final class ImportRewrite {
+	/**
+	 * Used to determine how a type will be used, so that unwanted annotations can be filtered,
+	 * which is in particular relevant for avoiding redundant null annotations in the scope of {@code @NonNullByDefault}. 
+	 * This enum is a superset of org.aspectj.org.eclipse.jdt.annotation.DefaultLocation, and roughly corresponds
+	 * to the classification of type locations as introduced by JSR 308.
+	 *
+	 * @since 3.13
+	 */
+	public enum TypeLocation {
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.PARAMETER
+		 * 
+		 * @since 3.13
+		 */
+		PARAMETER,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.RETURN_TYPE
+		 * 
+		 * @since 3.13
+		 */
+		RETURN_TYPE,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.FIELD
+		 * 
+		 * @since 3.13
+		 */
+		FIELD,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.TYPE_PARAMETER
+		 * 
+		 * @since 3.13
+		 */
+		TYPE_PARAMETER,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.TYPE_BOUND
+		 * 
+		 * @since 3.13
+		 */
+		TYPE_BOUND,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.TYPE_ARGUMENT
+		 * 
+		 * @since 3.13
+		 */
+		TYPE_ARGUMENT,
+
+		/**
+		 * see org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.ARRAY_CONTENTS
+		 * 
+		 * @since 3.13
+		 */
+		ARRAY_CONTENTS,
+
+		/**
+		 * The special value {@link #LOCAL_VARIABLE} is used for local variables: their nullness is determines by flow analysis,
+		 * so top level nullness annotations are usually not needed for local variables (unless their type is a free
+		 * type variable). Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		LOCAL_VARIABLE,
+
+		/**
+		 * The special value {@link #CAST} is used for casts.
+		 * Casts are never affected by {@code @NonNullByDefault}
+		 * Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		CAST,
+		
+		/**
+		 * The special value {@link #INSTANCEOF} is used for {@code instanceof} expressions.
+		 * Null annotations are not supported in this location.
+		 * Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		INSTANCEOF,
+
+		/**
+		 * The special value {@link #NEW} is used for {@code new} expressions (object allocations). 
+		 * Null annotations are not supported in this location.
+		 * Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		NEW,
+
+		/**
+		 * The special value {@link #RECEIVER} is used for the receiver type in a method declaration or method reference.
+		 * Null annotations are not supported in this location.
+		 * Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		RECEIVER,
+
+		/**
+		 * The special value {@link #EXCEPTION} is used for exception types in catch and throws declarations, which are
+		 * implicitly non-null. Does not correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		EXCEPTION,
+
+		/**
+		 * The special value {@link #OTHER} is used for locations where type annotations are illegal, like type literals
+		 * (X.class), annotations, or as scope for static field accesses. Does not correspond to a value in
+		 * org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		OTHER,
+
+		/**
+		 * The special value {@link #UNKNOWN} is used for invocations that don't specify the intended type usage. Does not
+		 * correspond to a value in org.aspectj.org.eclipse.jdt.annotation.DefaultLocation.
+		 * 
+		 * @since 3.13
+		 */
+		UNKNOWN,
+	}
 
 	/**
 	 * A {@link ImportRewrite.ImportRewriteContext} can optionally be used in e.g. {@link ImportRewrite#addImport(String, ImportRewrite.ImportRewriteContext)} to
@@ -166,11 +298,26 @@ public final class ImportRewrite {
 		 * {@link #RES_NAME_CONFLICT}, or {@link #RES_NAME_UNKNOWN_NEEDS_EXPLICIT_IMPORT}.
 		 */
 		public abstract int findInContext(String qualifier, String name, int kind);
+
+		/**
+		 * This method can be overridden in subclasses to remove annotations that are redundant, e.g. because of NonNullByDefault.
+		 * @param annotations The array of type annotations to be filtered.
+		 * @param location Specifies how the type being annotated will be used.
+		 * @param type the type being annotated
+		 * @return Returns the annotations that should be added to the type.
+		 * @since 3.13
+		 */
+		public IAnnotationBinding[] removeRedundantTypeAnnotations(IAnnotationBinding[] annotations, TypeLocation location, ITypeBinding type) {
+			return annotations;
+		}
 	}
 
 	private static final char STATIC_PREFIX= 's';
 	private static final char NORMAL_PREFIX= 'n';
 
+	/** @deprecated using deprecated code */
+	private static final int JLS8_INTERNAL = AST.JLS8;
+
 	private final ImportRewriteContext defaultContext;
 
 	private final ICompilationUnit compilationUnit;
@@ -292,14 +439,15 @@ public final class ImportRewrite {
 		this.useContextToFilterImplicitImports = false;
 
 		this.defaultContext= new ImportRewriteContext() {
+			@Override
 			public int findInContext(String qualifier, String name, int kind) {
 				return findInImports(qualifier, name, kind);
 			}
 		};
-		this.addedImports= new ArrayList<String>();
-		this.removedImports= new ArrayList<String>();
-		this.typeExplicitSimpleNames = new HashSet<String>();
-		this.staticExplicitSimpleNames = new HashSet<String>();
+		this.addedImports= new ArrayList<>();
+		this.removedImports= new ArrayList<>();
+		this.typeExplicitSimpleNames = new HashSet<>();
+		this.staticExplicitSimpleNames = new HashSet<>();
 		this.createdImports= null;
 		this.createdStaticImports= null;
 
@@ -528,7 +676,7 @@ public final class ImportRewrite {
 	 * @since 3.10
 	 */
 	public Annotation addAnnotation(IAnnotationBinding annotation, AST ast, ImportRewriteContext context) {
-		Type type = addImport(annotation.getAnnotationType(), ast, context);
+		Type type = addImport(annotation.getAnnotationType(), ast, context, TypeLocation.OTHER);
 		Name name;
 		if (type instanceof SimpleType) {
 			SimpleType simpleType = (SimpleType) type;
@@ -826,7 +974,7 @@ public final class ImportRewrite {
 	 * or else qualified names if an import conflict prevented an import.
 	 */
 	public Type addImport(ITypeBinding binding, AST ast) {
-		return addImport(binding, ast, this.defaultContext);
+		return addImport(binding, ast, this.defaultContext, TypeLocation.UNKNOWN);
 	}
 
 	/**
@@ -851,10 +999,37 @@ public final class ImportRewrite {
 	 * or else qualified names if an import conflict prevented an import.
 	 */
 	public Type addImport(ITypeBinding binding, AST ast, ImportRewriteContext context) {
+		return addImport(binding, ast, context, TypeLocation.UNKNOWN);
+	}
+
+	/**
+	 * Adds a new import to the rewriter's record and returns a {@link Type} that can be used
+	 * in the code. The type binding can be an array binding, type variable or wildcard.
+	 * If the binding is a generic type, the type parameters are ignored. For parameterized types, also the type
+	 * arguments are processed and imports added if necessary. Anonymous types inside type arguments are normalized to their base type, wildcard
+	 * of wildcards are ignored. If type annotations or type arguments are present at any point, the import is added up to that point and 
+	 * the type is retained from that point with type annotations and type arguments
+	 * 	<p>
+ 	 * No imports are added for types that are already known. If a import for a type is recorded to be removed, this record is discarded instead.
+	 * </p>
+	 * <p>
+	 * The content of the compilation unit itself is actually not modified
+	 * in any way by this method; rather, the rewriter just records that a new import has been added.
+	 * </p>
+	 * @param binding the signature of the type to be added.
+	 * @param ast the AST to create the returned type for.
+	 * @param context an optional context that knows about types visible in the current scope or <code>null</code>
+	 * to use the default context only using the available imports.
+	 * @param location where the type will be used. This information is needed to filter unwanted (nullness) type annotations.
+	 * @return a type node for the given type binding. Type names are simple names if an import could be used,
+	 * or else qualified names if an import conflict prevented an import.
+	 * @since 3.13
+	 */
+	public Type addImport(ITypeBinding binding, AST ast, ImportRewriteContext context, TypeLocation location) {	
 		ITypeBinding bindingPoint = checkAnnotationAndGenerics(binding);
-		Type type = internalAddImport(bindingPoint == null ? binding : bindingPoint, ast, context, null, /* getBase */ true);
+		Type type = internalAddImport(bindingPoint == null ? binding : bindingPoint, ast, context, null, /* getBase */ true, bindingPoint != null && !bindingPoint.equals(binding) ? TypeLocation.OTHER : location);
 		if (bindingPoint != null && !bindingPoint.equals(binding)) {
-			type = buildType(binding, bindingPoint, ast, context, type);
+			type = buildType(binding, bindingPoint, ast, context, type, location);
 		}
 		return type;
 	}
@@ -1008,7 +1183,7 @@ public final class ImportRewrite {
 			context= this.defaultContext;
 		}
 		int kind= isField ? ImportRewriteContext.KIND_STATIC_FIELD : ImportRewriteContext.KIND_STATIC_METHOD;
-		this.importsKindMap.put(key, new Integer(kind));
+		this.importsKindMap.put(key, Integer.valueOf(kind));
 		int res= context.findInContext(declaringTypeName, simpleName, kind);
 		if (res == ImportRewriteContext.RES_NAME_CONFLICT) {
 			return key;
@@ -1129,61 +1304,55 @@ public final class ImportRewrite {
 	 * @throws CoreException the exception is thrown if the rewrite fails.
 	 */
 	public final TextEdit rewriteImports(IProgressMonitor monitor) throws CoreException {
-		if (monitor == null) {
-			monitor= new NullProgressMonitor();
+
+		SubMonitor subMonitor = SubMonitor.convert(monitor,
+				Messages.bind(Messages.importRewrite_processDescription), 2);
+		if (!hasRecordedChanges()) {
+			this.createdImports= CharOperation.NO_STRINGS;
+			this.createdStaticImports= CharOperation.NO_STRINGS;
+			return new MultiTextEdit();
 		}
 
-		try {
-			monitor.beginTask(Messages.bind(Messages.importRewrite_processDescription), 2);
-			if (!hasRecordedChanges()) {
-				this.createdImports= CharOperation.NO_STRINGS;
-				this.createdStaticImports= CharOperation.NO_STRINGS;
-				return new MultiTextEdit();
-			}
-
-			CompilationUnit usedAstRoot= this.astRoot;
-			if (usedAstRoot == null) {
-				ASTParser parser= ASTParser.newParser(AST.JLS8);
-				parser.setSource(this.compilationUnit);
-				parser.setFocalPosition(0); // reduced AST
-				parser.setResolveBindings(false);
-				usedAstRoot= (CompilationUnit) parser.createAST(new SubProgressMonitor(monitor, 1));
-			}
-
-			ImportRewriteConfiguration config= buildImportRewriteConfiguration();
-
-			ImportRewriteAnalyzer computer=
-				new ImportRewriteAnalyzer(this.compilationUnit, usedAstRoot, config);
-
-			for (String addedImport : this.addedImports) {
-				boolean isStatic = STATIC_PREFIX == addedImport.charAt(0);
-				String qualifiedName = addedImport.substring(1);
-				computer.addImport(isStatic, qualifiedName);
-			}
-
-			for (String removedImport : this.removedImports) {
-				boolean isStatic = STATIC_PREFIX == removedImport.charAt(0);
-				String qualifiedName = removedImport.substring(1);
-				computer.removeImport(isStatic, qualifiedName);
-			}
-
-			for (String typeExplicitSimpleName : this.typeExplicitSimpleNames) {
-				computer.requireExplicitImport(false, typeExplicitSimpleName);
-			}
-
-			for (String staticExplicitSimpleName : this.staticExplicitSimpleNames) {
-				computer.requireExplicitImport(true, staticExplicitSimpleName);
-			}
-
-			ImportRewriteAnalyzer.RewriteResult result= computer.analyzeRewrite(new SubProgressMonitor(monitor, 1));
-
-			this.createdImports= result.getCreatedImports();
-			this.createdStaticImports= result.getCreatedStaticImports();
-
-			return result.getTextEdit();
-		} finally {
-			monitor.done();
+		CompilationUnit usedAstRoot= this.astRoot;
+		if (usedAstRoot == null) {
+			ASTParser parser= ASTParser.newParser(AST.JLS11);
+			parser.setSource(this.compilationUnit);
+			parser.setFocalPosition(0); // reduced AST
+			parser.setResolveBindings(false);
+			usedAstRoot= (CompilationUnit) parser.createAST(subMonitor.split(1));
 		}
+
+		ImportRewriteConfiguration config= buildImportRewriteConfiguration();
+
+		ImportRewriteAnalyzer computer=
+			new ImportRewriteAnalyzer(this.compilationUnit, usedAstRoot, config);
+
+		for (String addedImport : this.addedImports) {
+			boolean isStatic = STATIC_PREFIX == addedImport.charAt(0);
+			String qualifiedName = addedImport.substring(1);
+			computer.addImport(isStatic, qualifiedName);
+		}
+	
+		for (String removedImport : this.removedImports) {
+			boolean isStatic = STATIC_PREFIX == removedImport.charAt(0);
+			String qualifiedName = removedImport.substring(1);
+			computer.removeImport(isStatic, qualifiedName);
+		}
+
+		for (String typeExplicitSimpleName : this.typeExplicitSimpleNames) {
+			computer.requireExplicitImport(false, typeExplicitSimpleName);
+		}
+
+		for (String staticExplicitSimpleName : this.staticExplicitSimpleNames) {
+			computer.requireExplicitImport(true, staticExplicitSimpleName);
+		}
+
+		ImportRewriteAnalyzer.RewriteResult result= computer.analyzeRewrite(subMonitor.split(1));
+
+		this.createdImports= result.getCreatedImports();
+		this.createdStaticImports= result.getCreatedStaticImports();
+
+		return result.getTextEdit();
 	}
 
 	private ImportRewriteConfiguration buildImportRewriteConfiguration() {
@@ -1287,7 +1456,7 @@ public final class ImportRewrite {
 		if (imports == null) {
 			return CharOperation.NO_STRINGS;
 		}
-		List<String> res= new ArrayList<String>();
+		List<String> res= new ArrayList<>();
 		for (String curr : imports) {
 			if (prefix == curr.charAt(0)) {
 				res.add(curr.substring(1));
@@ -1296,28 +1465,32 @@ public final class ImportRewrite {
 		return res.toArray(new String[res.size()]);
 	}
 
-	private void annotateList(List annotations, IAnnotationBinding [] annotationBindings, AST ast, ImportRewriteContext context) {
+	private void annotateList(List annotations, IAnnotationBinding [] annotationBindings, AST ast,
+						ImportRewriteContext context, TypeLocation location, ITypeBinding type) {
+		if (context == null)
+			context= this.defaultContext;
+		annotationBindings = context.removeRedundantTypeAnnotations(annotationBindings, location, type);
 		for (int i = 0; i< annotationBindings.length; i++) {
 			Annotation annotation = addAnnotation(annotationBindings[i], ast, context);
 			if (annotation != null) annotations.add(annotation);
 		}
 	}
 
-	private Type annotateType(ITypeBinding binding, AST ast, ImportRewriteContext context, Type type) {
+	private Type annotateType(ITypeBinding binding, AST ast, ImportRewriteContext context, Type type, TypeLocation location) {
 		IAnnotationBinding [] annotationBindings = binding.getTypeAnnotations();
 		if (annotationBindings != null && annotationBindings.length > 0 && type instanceof AnnotatableType) {
-			annotateList(((AnnotatableType) type).annotations(), annotationBindings, ast, context);
+			annotateList(((AnnotatableType) type).annotations(), annotationBindings, ast, context, location, binding);
 		}
 		return type;
 	}
 
-	private Type buildType(ITypeBinding binding, ITypeBinding bindingPoint, AST ast, ImportRewriteContext context, Type qualifier) {
+	private Type buildType(ITypeBinding binding, ITypeBinding bindingPoint, AST ast, ImportRewriteContext context, Type qualifier, TypeLocation location) {
 		if (binding.equals(bindingPoint)) {
 			return qualifier;
 		}
 		// build the type recursively from left to right
-		Type type = binding.isMember() ? buildType(binding.getDeclaringClass(), bindingPoint, ast, context, qualifier) : null;
-		type = internalAddImport(binding, ast, context, type, false);
+		Type type = binding.isMember() ? buildType(binding.getDeclaringClass(), bindingPoint, ast, context, qualifier, TypeLocation.OTHER) : null;
+		type = internalAddImport(binding, ast, context, type, false, location);
 		return type;
 	}
 
@@ -1339,7 +1512,7 @@ public final class ImportRewrite {
 		return bindingPoint;
 	}
 
-	private Type createBaseType(AST ast, ImportRewriteContext context, ITypeBinding normalizedBinding) {
+	private Type createBaseType(AST ast, ImportRewriteContext context, ITypeBinding normalizedBinding, TypeLocation location) {
 		Type type;
 		IAnnotationBinding annotationBinding [] = normalizedBinding.getTypeAnnotations();
 		boolean annotsPresent = annotationBinding != null && annotationBinding.length > 0;
@@ -1356,22 +1529,23 @@ public final class ImportRewrite {
 			} else {
 				type = ast.newSimpleType(ast.newName(res));
 			}
-			annotateList(((AnnotatableType) type).annotations(), annotationBinding, ast, context);
+			annotateList(((AnnotatableType) type).annotations(), annotationBinding, ast, context, location, normalizedBinding);
 		} else {
 			type = ast.newSimpleType(ast.newName(res));
 		}
 		return type;
 	}
 
-	private Type getArrayType(Type elementType, AST ast, ImportRewriteContext context, ITypeBinding normalizedBinding) {
+	private Type getArrayType(Type elementType, AST ast, ImportRewriteContext context, ITypeBinding normalizedBinding, TypeLocation location) {
 		int noDimensions = normalizedBinding.getDimensions();
 		ArrayType arrayType = ast.newArrayType(elementType, noDimensions);
-		if (ast.apiLevel() >= AST.JLS8) {
+		if (ast.apiLevel() >= JLS8_INTERNAL) {
 			for (int i = 0; i < noDimensions; i++) {
 				IAnnotationBinding[] typeAnnotations = normalizedBinding.getTypeAnnotations();
 				if (typeAnnotations.length > 0) {
 					Dimension dimension = (Dimension) arrayType.dimensions().get(i);
-					annotateList(dimension.annotations(), typeAnnotations, ast, context);
+					annotateList(dimension.annotations(), typeAnnotations, ast, context,
+							i == 0 ? location : TypeLocation.ARRAY_CONTENTS, normalizedBinding);
 				}
 				normalizedBinding = normalizedBinding.getComponentType();
 			}
@@ -1379,7 +1553,7 @@ public final class ImportRewrite {
 		return arrayType;
 	}
 
-	private Type internalAddImport(ITypeBinding binding, AST ast, ImportRewriteContext context, Type currentType, boolean getBase) {
+	private Type internalAddImport(ITypeBinding binding, AST ast, ImportRewriteContext context, Type currentType, boolean getBase, TypeLocation location) {
 		Type type = null;
 		ITypeBinding normalizedBinding = null;
 		
@@ -1389,7 +1563,7 @@ public final class ImportRewrite {
 		} else {
 			normalizedBinding= normalizeTypeBinding(binding);
 			if (normalizedBinding == null) {
-				type = ast.newSimpleType(ast.newSimpleName("invalid")); //$NON-NLS-1$
+				 return ast.newSimpleType(ast.newSimpleName("invalid")); //$NON-NLS-1$
 			} else if (normalizedBinding.isTypeVariable()) {
 					// no import
 				type = ast.newSimpleType(ast.newSimpleName(binding.getName()));
@@ -1397,26 +1571,26 @@ public final class ImportRewrite {
 				WildcardType wcType= ast.newWildcardType();
 				ITypeBinding bound= normalizedBinding.getBound();
 				if (bound != null && !bound.isWildcardType() && !bound.isCapture()) { // bug 96942
-					Type boundType= addImport(bound, ast, context);
+					Type boundType= addImport(bound, ast, context, TypeLocation.TYPE_BOUND);
 					wcType.setBound(boundType, normalizedBinding.isUpperbound());
 				}
 				type = wcType;
 			} else if (normalizedBinding.isArray()) {
-				Type elementType= addImport(normalizedBinding.getElementType(), ast, context);
-				type = getArrayType(elementType, ast, context, normalizedBinding);
+				Type elementType= addImport(normalizedBinding.getElementType(), ast, context, TypeLocation.ARRAY_CONTENTS);
+				type = getArrayType(elementType, ast, context, normalizedBinding, location);
 			}
 		}
 
 		if (type != null) {
-			return annotateType(normalizedBinding, ast, context, type);
+			return annotateType(normalizedBinding, ast, context, type, location);
 		}
 
 		if (getBase) {
-			type = createBaseType(ast, context, normalizedBinding);
+			type = createBaseType(ast, context, normalizedBinding, location);
 		} else  {
 			type = currentType != null ? (Type) ast.newQualifiedType(currentType, ast.newSimpleName(getRawName(normalizedBinding))) : 
 				ast.newSimpleType(ast.newName(getRawName(normalizedBinding)));
-			type = annotateType(normalizedBinding, ast, context, type);
+			type = annotateType(normalizedBinding, ast, context, type, location);
 		}
 
 		ITypeBinding[] typeArguments = normalizedBinding.getTypeArguments();
@@ -1428,7 +1602,7 @@ public final class ImportRewrite {
 				if (containsNestedCapture(curr, false)) { // see bug 103044
 					arguments.add(ast.newWildcardType());
 				} else {
-					arguments.add(addImport(curr, ast, context));
+					arguments.add(addImport(curr, ast, context, TypeLocation.TYPE_ARGUMENT));
 				}
 			}
 			type = paramType;
@@ -1448,7 +1622,7 @@ public final class ImportRewrite {
 			return result;
 		} else if (value instanceof ITypeBinding) {
 			TypeLiteral result = ast.newTypeLiteral();
-			result.setType(addImport((ITypeBinding) value, ast, context));
+			result.setType(addImport((ITypeBinding) value, ast, context, TypeLocation.OTHER));
 			return result;
 		} else if (value instanceof String) {
 			StringLiteral result = ast.newStringLiteral();
@@ -1459,7 +1633,7 @@ public final class ImportRewrite {
 
 			FieldAccess result = ast.newFieldAccess();
 			result.setName(ast.newSimpleName(variable.getName()));
-			Type type = addImport(variable.getType(), ast, context);
+			Type type = addImport(variable.getType(), ast, context, TypeLocation.OTHER); // ignore annotations, type will discarded
 			Name name;
 			if (type instanceof SimpleType) {
 				SimpleType simpleType = (SimpleType) type;
