@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -60,6 +63,7 @@ public class SearchEngine {
 		/**
 		 * @see org.aspectj.org.eclipse.jdt.core.search.SearchRequestor#acceptSearchMatch(org.aspectj.org.eclipse.jdt.core.search.SearchMatch)
 		 */
+		@Override
 		public void acceptSearchMatch(SearchMatch match) throws CoreException {
 			this.resultCollector.accept(
 				match.getResource(),
@@ -72,12 +76,14 @@ public class SearchEngine {
 		/**
 		 * @see org.aspectj.org.eclipse.jdt.core.search.SearchRequestor#beginReporting()
 		 */
+		@Override
 		public void beginReporting() {
 			this.resultCollector.aboutToStart();
 		}
 		/**
 		 * @see org.aspectj.org.eclipse.jdt.core.search.SearchRequestor#endReporting()
 		 */
+		@Override
 		public void endReporting() {
 			this.resultCollector.done();
 		}
@@ -92,6 +98,7 @@ public class SearchEngine {
 		TypeNameRequestorAdapter(ITypeNameRequestor requestor) {
 			this.nameRequestor = requestor;
 		}
+		@Override
 		public void acceptType(int modifiers, char[] packageName, char[] simpleTypeName, char[][] enclosingTypeNames, String path, AccessRestriction access) {
 			if (Flags.isInterface(modifiers)) {
 				this.nameRequestor.acceptInterface(packageName, simpleTypeName, enclosingTypeNames, path);
@@ -288,6 +295,31 @@ public class SearchEngine {
 		return BasicSearchEngine.createJavaSearchScope(elements, includeReferencedProjects);
 	}
 
+	/**
+	 * Returns a Java search scope limited to the given Java elements.
+	 * The Java elements resulting from a search with this scope will
+	 * be children of the given elements.
+	 *
+	 * <p>If an element is an {@link IJavaProject}, then the project's source folders,
+	 * its jars (external and internal) and - if specified - its referenced projects
+	 * (with their source folders and jars, recursively) will be included.</p>
+	 * <p>If an element is an {@link IPackageFragmentRoot}, then only the package fragments of
+	 * this package fragment root will be included.</p>
+	 * <p>If an element is an {@link IPackageFragment}, then only the compilation unit and class
+	 * files of this package fragment will be included. Subpackages will NOT be
+	 * included.</p>
+	 *
+	 * @param excludeTestCode if true, test code we be excluded
+	 * @param elements the Java elements the scope is limited to
+	 * @param includeReferencedProjects a flag indicating if referenced projects must be
+	 * 									 recursively included
+	 * @return a new Java search scope
+	 * @since 3.14
+	 */
+	public static IJavaSearchScope createJavaSearchScope(boolean excludeTestCode, IJavaElement[] elements, boolean includeReferencedProjects) {
+		return BasicSearchEngine.createJavaSearchScope(excludeTestCode, elements, includeReferencedProjects);
+	}
+
 	/**
 	 * Returns a Java search scope limited to the given Java elements.
 	 * The Java elements resulting from a search with this scope will
@@ -325,6 +357,44 @@ public class SearchEngine {
 		return BasicSearchEngine.createJavaSearchScope(elements, includeMask);
 	}
 
+	/**
+	 * Returns a Java search scope limited to the given Java elements.
+	 * The Java elements resulting from a search with this scope will
+	 * be children of the given elements.
+	 *
+	 * <p>If an element is an IJavaProject, then it includes:</p>
+	 * <ul>
+	 * <li>its source folders if {@link IJavaSearchScope#SOURCES} is specified,</li>
+	 * <li>its application libraries (internal and external jars, class folders that are on the raw classpath,
+	 *   or the ones that are coming from a classpath path variable,
+	 *   or the ones that are coming from a classpath container with the K_APPLICATION kind)
+	 *   if {@link IJavaSearchScope#APPLICATION_LIBRARIES} is specified</li>
+	 * <li>its system libraries (internal and external jars, class folders that are coming from an
+	 *   IClasspathContainer with the K_SYSTEM kind)
+	 *   if {@link IJavaSearchScope#SYSTEM_LIBRARIES} is specified</li>
+	 * <li>its referenced projects (with their source folders and jars, recursively)
+	 *   if {@link IJavaSearchScope#REFERENCED_PROJECTS} is specified.</li>
+	 * </ul>
+	 * <p>If an element is an {@link IPackageFragmentRoot}, then only the package fragments of
+	 * this package fragment root will be included.</p>
+	 * <p>If an element is an {@link IPackageFragment}, then only the compilation unit and class
+	 * files of this package fragment will be included. Subpackages will NOT be
+	 * included.</p>
+	 *
+	 * @param excludeTestCode
+	 * @param elements the Java elements the scope is limited to
+	 * @param includeMask the bit-wise OR of all include types of interest
+	 * @return a new Java search scope
+	 * @see IJavaSearchScope#SOURCES
+	 * @see IJavaSearchScope#APPLICATION_LIBRARIES
+	 * @see IJavaSearchScope#SYSTEM_LIBRARIES
+	 * @see IJavaSearchScope#REFERENCED_PROJECTS
+	 * @since 3.14
+	 */
+	public static IJavaSearchScope createJavaSearchScope(boolean excludeTestCode, IJavaElement[] elements, int includeMask) {
+		return BasicSearchEngine.createJavaSearchScope(excludeTestCode, elements, includeMask);
+	}
+
 	/**
 	 * Returns a search pattern that combines the given two patterns into a "or" pattern.
 	 * The search result will match either the left pattern or the right pattern.
@@ -434,6 +504,22 @@ public class SearchEngine {
 		return BasicSearchEngine.createTypeNameMatch(type, modifiers);
 	}
 
+	/**
+	 * Create a method name match on a given method with specific modifiers.
+	 * <p>
+	 * NOTE: this is a first-cut version and might have noticeable delay times in some cases. 
+	 * Targeted for optimization during 4.7. It is recommended to contact JDT Team if 
+	 * anyone plans to use this.
+	 * </p>
+	 * @param method The Java model handle of the method
+	 * @param modifiers Modifiers of the method
+	 * @return A non-null match on the given method.
+	 * @since 3.12
+	 */
+	public static MethodNameMatch createMethodNameMatch(IMethod method, int modifiers) {
+		return BasicSearchEngine.createMethodNameMatch(method, modifiers);
+	}
+
 	/**
 	 * Returns a Java search scope with the workspace as the only limit.
 	 *
@@ -584,6 +670,274 @@ public class SearchEngine {
 		this.basicEngine.search(pattern, participants, scope, requestor, monitor);
 	}
 
+	/**
+	 * Searches for all method declarations in the given scope. Accepted matches will be returned by
+	 * {@link MethodNameRequestor#acceptMethod}.
+	 * <p>
+	 * NOTE: this is a first-cut version and might have noticeable delay times in some cases. 
+	 * Targeted for optimization during 4.7. It is recommended to contact JDT Team if 
+	 * anyone plans to use this.
+	 * </p>
+	 * 
+	 * @param packageName the full name of the package of the searched types, or a prefix for this
+	 *						package, or a wild-carded string for this package.
+	 *						May be <code>null</code>, then any package name is accepted.
+	 * @param pkgMatchRule match rule for package.
+	 * @param declaringQualification Qualification of the declaring type.
+	 * @param declQualificationMatchRule match rule for declaring qualifier of parent of the type.
+	 * @param declaringSimpleName simple name of the declaring type.
+	 * @param declSimpleNameMatchRule match rule for the simple name of the enclosing type.
+	 * @param methodName the method name searched for.
+	 * @param methodMatchRule match rule for the method name.
+	 * @param scope the scope to search in
+	 * @param nameRequestor the requestor that collects the results of the search.
+	 * @param waitingPolicy one of
+	 * <ul>
+	 *		<li>{@link IJavaSearchConstants#FORCE_IMMEDIATE_SEARCH} if the search should start immediately</li>
+	 *		<li>{@link IJavaSearchConstants#CANCEL_IF_NOT_READY_TO_SEARCH} if the search should be cancelled if the
+	 *			underlying indexer has not finished indexing the workspace</li>
+	 *		<li>{@link IJavaSearchConstants#WAIT_UNTIL_READY_TO_SEARCH} if the search should wait for the
+	 *			underlying indexer to finish indexing the workspace</li>
+	 * </ul>
+	 * @param progressMonitor the progress monitor to report progress to, or <code>null</code> if no progress
+	 *							monitor is provided
+	 * @exception JavaModelException if the search failed.
+	 * 
+	 * @since 3.12
+	 */
+	public void searchAllMethodNames(
+			final char[] packageName,
+			final int pkgMatchRule,
+			final char[] declaringQualification,
+			final int declQualificationMatchRule,
+			final char[] declaringSimpleName,
+			final int declSimpleNameMatchRule,
+			final char[] methodName,
+			final int methodMatchRule,
+			IJavaSearchScope scope,
+			final MethodNameRequestor nameRequestor,
+			int waitingPolicy,
+			IProgressMonitor progressMonitor)  throws JavaModelException {
+		MethodNameRequestorWrapper requestorWrapper = new MethodNameRequestorWrapper(nameRequestor);
+		this.basicEngine.searchAllMethodNames(
+				packageName, pkgMatchRule, 
+				declaringQualification, declQualificationMatchRule, 
+				declaringSimpleName, declSimpleNameMatchRule, 
+				methodName, methodMatchRule, 
+				scope, requestorWrapper, 
+				waitingPolicy, progressMonitor);
+	}
+
+	/**
+	 * Searches for all method declarations in the given scope. 
+	 * <p>
+	 * Provided {@link MethodNameMatchRequestor} requestor will collect the {@link MethodNameMatch}
+	 * matches found during the search.
+
+	 * </p>
+	 * <p>
+	 * NOTE: this is a first-cut version and might have noticeable delay times in some cases. 
+	 * Targeted for optimization during 4.7. It is recommended to contact JDT Team if 
+	 * anyone plans to use this.
+	 * </p>
+	 * 
+	 * @param packageName the full name of the package of the searched types, or a prefix for this
+	 *						package, or a wild-carded string for this package.
+	 *						May be <code>null</code>, then any package name is accepted.
+	 * @param pkgMatchRule match rule for package.
+	 * @param declaringQualification Qualification of the declaring type.
+	 * @param declQualificationMatchRule match rule for declaring qualifier of parent of the type.
+	 * @param declaringSimpleName simple name of the declaring type.
+	 * @param declSimpleNameMatchRule match rule for the simple name of the enclosing type.
+	 * @param methodName the method name searched for.
+	 * @param methodMatchRule match rule for the method name.
+	 * @param scope the scope to search in
+	 * @param nameRequestor the {@link MethodNameMatchRequestor}
+	 * @param waitingPolicy one of
+	 * <ul>
+	 *		<li>{@link IJavaSearchConstants#FORCE_IMMEDIATE_SEARCH} if the search should start immediately</li>
+	 *		<li>{@link IJavaSearchConstants#CANCEL_IF_NOT_READY_TO_SEARCH} if the search should be cancelled if the
+	 *			underlying indexer has not finished indexing the workspace</li>
+	 *		<li>{@link IJavaSearchConstants#WAIT_UNTIL_READY_TO_SEARCH} if the search should wait for the
+	 *			underlying indexer to finish indexing the workspace</li>
+	 * </ul>
+	 * @param progressMonitor the progress monitor to report progress to, or <code>null</code> if no progress
+	 *							monitor is provided
+	 * @exception JavaModelException if the search failed.
+	 * 
+	 * @since 3.12
+	 */
+	public void searchAllMethodNames(
+			final char[] packageName,
+			final int pkgMatchRule,
+			final char[] declaringQualification,
+			final int declQualificationMatchRule,
+			final char[] declaringSimpleName,
+			final int declSimpleNameMatchRule,
+			final char[] methodName,
+			final int methodMatchRule,
+			IJavaSearchScope scope,
+			final MethodNameMatchRequestor nameRequestor,
+			int waitingPolicy,
+			IProgressMonitor progressMonitor)  throws JavaModelException {
+		MethodNameMatchRequestorWrapper requestorWrapper = new MethodNameMatchRequestorWrapper(nameRequestor, scope);
+		this.basicEngine.searchAllMethodNames(
+				packageName, pkgMatchRule,
+				declaringQualification, declQualificationMatchRule,
+				declaringSimpleName, declSimpleNameMatchRule,
+				methodName, methodMatchRule,
+				scope, requestorWrapper, 
+				waitingPolicy, progressMonitor);
+	}
+
+	/**
+	 * Searches for all method declarations in the given scope. Accepted matches will be returned by
+	 * {@link MethodNameRequestor#acceptMethod}.
+	 * <p>
+	 * NOTE: this is a first-cut version and might have noticeable delay times in some cases. 
+	 * Targeted for optimization during 4.7. It is recommended to contact JDT Team if 
+	 * anyone plans to use this.
+	 * </p>
+	 * 
+	 * @param qualifier qualifier including package name and qualified type name
+	 *	May be <code>null</code>, then any qualifier name is accepted.
+	 * @param qualifierMatchRule match rule for the qualifier and can be one of
+	 * 	 * <ul>
+	 *		<li>{@link SearchPattern#R_EXACT_MATCH} if the package name and type
+	 *			name are the full names of the types of the searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PREFIX_MATCH} if the package name and type
+	 *			name are prefixes of the names of the types of searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PATTERN_MATCH} if the package name and
+	 *			type name contain wild-cards.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_MATCH} if the package name and types are
+	 *			camel case of the package and type of searched methods.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_SAME_PART_COUNT_MATCH}
+	 *			if the package name and type names are camel case with same part count of the 
+	 *			package and types of searched methods.</li>
+	 * </ul>
+	 * @param methodName the method name searched for.
+	 * @param methodMatchRule match rule for the method name and can be one of
+	 * <ul>
+	 *		<li>{@link SearchPattern#R_EXACT_MATCH} if the method name searched
+	 *			is exact.</li>
+	 *		<li>{@link SearchPattern#R_PREFIX_MATCH} if method
+	 *			name is prefix of the names of the searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PATTERN_MATCH} if the method name 
+	 *			contains wild-cards.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_MATCH} if the method name is a
+	 *			camel case of the searched method name.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_SAME_PART_COUNT_MATCH}
+	 *			if the method name is a camel case with same part count of the searched
+	 *			methods name.</li>
+	 * </ul>
+	 * @param scope the scope to search in
+	 * @param nameRequestor the requestor that collects the results of the search.
+	 * @param waitingPolicy one of
+	 * <ul>
+	 *		<li>{@link IJavaSearchConstants#FORCE_IMMEDIATE_SEARCH} if the search should start immediately</li>
+	 *		<li>{@link IJavaSearchConstants#CANCEL_IF_NOT_READY_TO_SEARCH} if the search should be cancelled if the
+	 *			underlying indexer has not finished indexing the workspace</li>
+	 *		<li>{@link IJavaSearchConstants#WAIT_UNTIL_READY_TO_SEARCH} if the search should wait for the
+	 *			underlying indexer to finish indexing the workspace</li>
+	 * </ul>
+	 * @param progressMonitor the progress monitor to report progress to, or <code>null</code> if no progress
+	 *							monitor is provided
+	 * @exception JavaModelException if the search failed.
+	 * 
+	 * @since 3.12
+	 */
+	public void searchAllMethodNames(
+			final char[] qualifier,
+			final int qualifierMatchRule,
+			final char[] methodName,
+			final int methodMatchRule,
+			IJavaSearchScope scope,
+			final MethodNameRequestor nameRequestor,
+			int waitingPolicy,
+			IProgressMonitor progressMonitor)  throws JavaModelException {
+		MethodNameRequestorWrapper requestorWrapper = new MethodNameRequestorWrapper(nameRequestor);
+		this.basicEngine.searchAllMethodNames(
+				qualifier, qualifierMatchRule, 
+				methodName, methodMatchRule, 
+				scope, requestorWrapper, 
+				waitingPolicy, progressMonitor);
+	}
+
+	/**
+	 * Searches for all method declarations in the given scope. 
+	 * <p>
+	 * Provided {@link MethodNameMatchRequestor} requestor will collect the {@link MethodNameMatch}
+	 * matches found during the search.
+	 * </p>
+	 * <p>
+	 * NOTE: this is a first-cut version and might have noticeable delay times in some cases. 
+	 * Targeted for optimization during 4.7. It is recommended to contact JDT Team if 
+	 * anyone plans to use this.
+	 * </p>
+	 * 
+	 * @param qualifier qualifier including package name and qualified type name
+	 *	May be <code>null</code>, then any qualifier name is accepted.
+	 * @param qualifierMatchRule match rule for the qualifier and can be one of
+	 * 	 * <ul>
+	 *		<li>{@link SearchPattern#R_EXACT_MATCH} if the package name and type
+	 *			name are the full names of the types of the searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PREFIX_MATCH} if the package name and type
+	 *			name are prefixes of the names of the types of searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PATTERN_MATCH} if the package name and
+	 *			type name contain wild-cards.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_MATCH} if the package name and types are
+	 *			camel case of the package and type of searched methods.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_SAME_PART_COUNT_MATCH}
+	 *			if the package name and type names are camel case with same part count of the 
+	 *			package and types of searched methods.</li>
+	 * </ul>
+	 * @param methodName the method name searched for.
+	 * @param methodMatchRule match rule for the method name and can be one of
+	 * <ul>
+	 *		<li>{@link SearchPattern#R_EXACT_MATCH} if the method name searched
+	 *			is exact.</li>
+	 *		<li>{@link SearchPattern#R_PREFIX_MATCH} if method
+	 *			name is prefix of the names of the searched methods.</li>
+	 *		<li>{@link SearchPattern#R_PATTERN_MATCH} if the method name 
+	 *			contains wild-cards.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_MATCH} if the method name is a
+	 *			camel case of the searched method name.</li>
+	 *		<li>{@link SearchPattern#R_CAMELCASE_SAME_PART_COUNT_MATCH}
+	 *			if the method name is a camel case with same part count of the searched
+	 *			methods name.</li>
+	 * </ul>
+	 * @param scope the scope to search in
+	 * @param nameRequestor the {@link MethodNameMatchRequestor}
+	 * @param waitingPolicy one of
+	 * <ul>
+	 *		<li>{@link IJavaSearchConstants#FORCE_IMMEDIATE_SEARCH} if the search should start immediately</li>
+	 *		<li>{@link IJavaSearchConstants#CANCEL_IF_NOT_READY_TO_SEARCH} if the search should be cancelled if the
+	 *			underlying indexer has not finished indexing the workspace</li>
+	 *		<li>{@link IJavaSearchConstants#WAIT_UNTIL_READY_TO_SEARCH} if the search should wait for the
+	 *			underlying indexer to finish indexing the workspace</li>
+	 * </ul>
+	 * @param progressMonitor the progress monitor to report progress to, or <code>null</code> if no progress
+	 *							monitor is provided
+	 * @exception JavaModelException if the search failed.
+	 * 
+	 * @since 3.12
+	 */
+	public void searchAllMethodNames(
+			final char[] qualifier,
+			final int qualifierMatchRule,
+			final char[] methodName,
+			final int methodMatchRule,
+			IJavaSearchScope scope,
+			final MethodNameMatchRequestor nameRequestor,
+			int waitingPolicy,
+			IProgressMonitor progressMonitor)  throws JavaModelException {
+		MethodNameMatchRequestorWrapper requestorWrapper = new MethodNameMatchRequestorWrapper(nameRequestor, scope);
+		this.basicEngine.searchAllMethodNames(
+				qualifier, qualifierMatchRule,
+				methodName, methodMatchRule,
+				scope, requestorWrapper, 
+				waitingPolicy, progressMonitor);
+	}
 	/**
 	 * Searches for all top-level types and member types in the given scope.
 	 * The search can be selecting specific types (given a package exact full name or
