@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2015 GK Software AG.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2015, 2016 GK Software AG.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Stephan Herrmann - initial API and implementation
@@ -185,7 +188,7 @@ public final class ExternalAnnotationUtil {
 				return null;
 		}
 		
-		annotationPath = annotationPath.append(binaryTypeName).addFileExtension(ExternalAnnotationProvider.ANNOTION_FILE_EXTENSION);
+		annotationPath = annotationPath.append(binaryTypeName).addFileExtension(ExternalAnnotationProvider.ANNOTATION_FILE_EXTENSION);
 		return workspaceRoot.getFile(annotationPath);
 	}
 
@@ -294,7 +297,7 @@ public final class ExternalAnnotationUtil {
 						newContent.append('\n');
 						continue;
 					}
-					if (!Character.isJavaIdentifierStart(line.charAt(0))) {
+					if (!Character.isJavaIdentifierStart(line.charAt(0)) && line.charAt(0) != '<') {
 						newContent.append(line).append('\n');
 						continue;
 					}
