@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -113,6 +116,16 @@ public interface IClassFileReader {
 	 */
 	IInnerClassesAttribute getInnerClassesAttribute();
 
+	/**
+	 * Answer back the nest members attribute of this .class file, null if none.
+	 *
+	 * @return the nest members attribute of this .class file, null if none
+	 * @since 3.16
+	 */
+	default INestMembersAttribute getNestMembersAttribute() {
+		return null;
+	}
+
 	/**
 	 * Answer back the array of method infos of this .class file,
 	 * an empty array if none.
