@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -64,6 +67,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Do not parse comment if completion location is not included.
 	 */
+	@Override
 	public boolean checkDeprecation(int commentPtr) {
 		boolean isDeprecated = false;
 
@@ -77,7 +81,6 @@ public class CompletionJavadocParser extends JavadocParser {
 				System.out.println("COMPLETION in Javadoc:"); //$NON-NLS-1$
 			}
 			completionScanner.completionIdentifier = null;
-			this.firstTagPosition = 1;
 			super.checkDeprecation(commentPtr);
 		} else {
 			if (this.sourceParser.scanner.commentTagStarts[commentPtr] != 0) {
@@ -94,14 +97,17 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Replace stored Javadoc node with specific completion one.
 	 */
+	@Override
 	protected boolean commentParse() {
 		this.docComment = new CompletionJavadoc(this.javadocStart, this.javadocEnd);
+		this.firstTagPosition = 1; // bug 429340: completion parser needs to parse the whole comment
 		return super.commentParse();
 	}
 
 	/*
 	 * Create argument expression. If it includes completion location, create and store completion node.
 	 */
+	@Override
 	protected Object createArgumentReference(char[] name, int dim, boolean isVarargs, Object typeRef, long[] dimPositions, long argNamePos) throws InvalidInputException {
 		// Create argument as we may need it after
 		char[] argName = name==null ? CharOperation.NO_CHAR : name;
@@ -130,6 +136,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Create field reference. If it includes completion location, create and store completion node.
 	 */
+	@Override
 	protected Object createFieldReference(Object receiver) throws InvalidInputException {
 		int refStart = (int) (this.identifierPositionStack[0] >>> 32);
 		int refEnd = (int) this.identifierPositionStack[0];
@@ -157,6 +164,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	 * If so, create method reference and store it.
 	 * Otherwise return null as we do not need this reference.
 	 */
+	@Override
 	protected Object createMethodReference(Object receiver, List arguments) throws InvalidInputException {
 		int memberPtr = this.identifierLengthStack[0] - 1; // may be > 0 for inner class constructor reference
 		int refStart = (int) (this.identifierPositionStack[memberPtr] >>> 32);
@@ -189,6 +197,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Create type reference. If it includes completion location, create and store completion node.
 	 */
+	@Override
 	protected Object createTypeReference(int primitiveToken) {
 		// Need to create type ref in case it was needed by members
 		int nbIdentifiers = this.identifierLengthStack[this.identifierLengthPtr];
@@ -286,6 +295,8 @@ public class CompletionJavadocParser extends JavadocParser {
 	 */
 	private void initLevelTags() {
 		int level = ((int)(this.complianceLevel >>> 16)) - ClassFileConstants.MAJOR_VERSION_1_1 + 1;
+		if ( level >= BLOCK_TAGS_LENGTH)
+			return; // To support future JDKs
 		// Init block tags
 		this.levelTags[BLOCK_IDX] = new char[BLOCK_ALL_TAGS_LENGTH][];
 		this.levelTagsLength[BLOCK_IDX] = 0;
@@ -312,6 +323,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Parse argument in @see tag method reference
 	 */
+	@Override
 	protected Object parseArguments(Object receiver) throws InvalidInputException {
 
 		if (this.tagSourceStart>this.cursorLocation) {
@@ -458,6 +470,7 @@ public class CompletionJavadocParser extends JavadocParser {
 		throw new InvalidInputException();
 	}
 
+		@Override
 		protected boolean parseParam() throws InvalidInputException {
 			int startPosition = this.index;
 			int endPosition = this.index;
@@ -535,9 +548,7 @@ public class CompletionJavadocParser extends JavadocParser {
 			return valid;
 		}
 
-	/* (non-Javadoc)
-		 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseReference()
-		 */
+		@Override
 		protected boolean parseReference() throws InvalidInputException {
 			boolean completed = this.completionNode != null;
 			boolean valid = super.parseReference();
@@ -547,9 +558,7 @@ public class CompletionJavadocParser extends JavadocParser {
 			return valid;
 		}
 
-	/*(non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseTag(int)
-	 */
+	@Override
 	protected boolean parseTag(int previousPosition) throws InvalidInputException {
 		int startPosition = this.inlineTagStarted ? this.inlineTagStart : previousPosition;
 		boolean newLine = !this.lineStarted;
@@ -573,9 +582,7 @@ public class CompletionJavadocParser extends JavadocParser {
 		return valid;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseThrows()
-	 */
+	@Override
 	protected boolean parseThrows() {
 		try {
 			Object typeRef = parseQualifiedName(true);
@@ -592,6 +599,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Push param name reference. If it includes completion location, create and store completion node.
 	 */
+	@Override
 	protected boolean pushParamName(boolean isTypeParam) {
 		if (super.pushParamName(isTypeParam)) {
 			Expression expression = (Expression) this.astStack[this.astPtr];
@@ -617,6 +625,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	 *
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushText(int, int)
 	 */
+	@Override
 	protected void pushText(int start, int end) {
 		if (start <= this.cursorLocation && this.cursorLocation <= end) {
 			this.scanner.resetTo(start, end);
@@ -782,9 +791,7 @@ public class CompletionJavadocParser extends JavadocParser {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#readToken()
-	 */
+	@Override
 	protected int readToken() throws InvalidInputException {
 		int token = super.readToken();
 		if (token == TerminalTokens.TokenNameIdentifier && this.scanner.currentPosition == this.scanner.startPosition) {
@@ -797,6 +804,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Recover syntax on invalid qualified name.
 	 */
+	@Override
 	protected Object syntaxRecoverQualifiedName(int primitiveToken) throws InvalidInputException {
 		if (this.cursorLocation == ((int)this.identifierPositionStack[this.identifierPtr])) {
 			// special case of completion just before the dot.
@@ -898,6 +906,7 @@ public class CompletionJavadocParser extends JavadocParser {
 	/*
 	 * Store completion node into doc comment.
 	 */
+	@Override
 	protected void updateDocComment() {
 		super.updateDocComment();
 		if (this.completionNode instanceof Expression) {
@@ -907,9 +916,7 @@ public class CompletionJavadocParser extends JavadocParser {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#verifySpaceOrEndComment()
-	 */
+	@Override
 	protected boolean verifySpaceOrEndComment() {
 		CompletionScanner completionScanner = (CompletionScanner) this.scanner;
 		if (completionScanner.completionIdentifier != null && completionScanner.completedIdentifierStart <= this.cursorLocation && this.cursorLocation <= completionScanner.completedIdentifierEnd) {
