@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -18,6 +21,7 @@ package org.aspectj.org.eclipse.jdt.internal.codeassist.impl;
 import java.util.HashSet;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.InvalidInputException;
+import org.aspectj.org.eclipse.jdt.internal.compiler.CompilationResult;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.AbstractVariableDeclaration;
@@ -35,7 +39,10 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.ast.LambdaExpression;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.MessageSend;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.MethodDeclaration;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ModuleDeclaration;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ModuleReference;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.NameReference;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.RequiresStatement;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Statement;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.SuperReference;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
@@ -46,6 +53,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ExtraCompilerModifie
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.Parser;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredBlock;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredElement;
+import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredExportsStatement;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredField;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredInitializer;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredLocalVariable;
@@ -55,7 +63,6 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.RecoveredUnit;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
-@SuppressWarnings({"rawtypes"})
 public abstract class AssistParser extends Parser {
 	public ASTNode assistNode;
 	public boolean isOrphanCompletionNode;
@@ -96,7 +103,8 @@ public abstract class AssistParser extends Parser {
 	protected static final int K_ATTRIBUTE_VALUE_DELIMITER = ASSIST_PARSER + 5; // whether we are inside a annotation attribute valuer
 	protected static final int K_ENUM_CONSTANT_DELIMITER = ASSIST_PARSER + 6; // whether we are inside a field initializer
 	protected static final int K_LAMBDA_EXPRESSION_DELIMITER = ASSIST_PARSER + 7; // whether we are inside a lambda expression
-	
+	protected static final int K_MODULE_INFO_DELIMITER = ASSIST_PARSER + 8; // whether we are inside a module info declaration
+
 	// selector constants
 	protected static final int THIS_CONSTRUCTOR = -1;
 	protected static final int SUPER_CONSTRUCTOR = -2;
@@ -104,14 +112,14 @@ public abstract class AssistParser extends Parser {
 	// enum constant constants
 	protected static final int NO_BODY = 0;
 	protected static final int WITH_BODY = 1;
-	
+
 	protected static final int EXPRESSION_BODY = 0;
 	protected static final int BLOCK_BODY = 1;
 
 	protected boolean isFirst = false;
 
 	public AssistParser snapShot;
-	private static final int[] RECOVERY_TOKENS = new int [] { TokenNameSEMICOLON, TokenNameRPAREN,};
+	protected static final int[] RECOVERY_TOKENS = { TokenNameSEMICOLON, TokenNameRPAREN, TokenNameRBRACE, TokenNameRBRACKET};
 
 
 public AssistParser(ProblemReporter problemReporter) {
@@ -124,6 +132,7 @@ public AssistParser(ProblemReporter problemReporter) {
 
 public abstract char[] assistIdentifier();
 
+@Override
 public void copyState(Parser from) {
 	
 	super.copyState(from);
@@ -173,6 +182,7 @@ public int bodyEnd(Initializer initializer){
  * Build initial recovery state.
  * Recovery state is inferred from the current state of the parser (reduced node stack).
  */
+@Override
 public RecoveredElement buildInitialRecoveryState(){
 	/* recovery in unit structure */
 	if (this.referenceContext instanceof CompilationUnitDeclaration){
@@ -180,6 +190,7 @@ public RecoveredElement buildInitialRecoveryState(){
 		flushAssistState();
 		flushElementStack();
 		this.snapShot = null;
+		initModuleInfo(element);
 		return element;
 	}
 
@@ -223,9 +234,6 @@ public RecoveredElement buildInitialRecoveryState(){
 	ASTNode node = null, lastNode = null;
 	for (int i = 0; i <= this.astPtr; i++, lastNode = node) {
 		node = this.astStack[i];
-		if(node instanceof ForeachStatement && ((ForeachStatement)node).action == null) {
-			node = ((ForeachStatement)node).elementVariable;
-		}
 		/* check for intermediate block creation, so recovery can properly close them afterwards */
 		int nodeStart = node.sourceStart;
 		for (int j = blockIndex; j <= this.realBlockPtr; j++){
@@ -326,6 +334,9 @@ public RecoveredElement buildInitialRecoveryState(){
 					element.add(stmt, 0);
 					this.lastCheckPoint = stmt.sourceEnd + 1;
 					this.isOrphanCompletionNode = false;
+				} else if ((stmt instanceof ForeachStatement) && ((ForeachStatement) stmt).action == null) {
+					element = element.add(stmt, 0);
+					this.lastCheckPoint = stmt.sourceEnd + 1;
 				}
 			}
 			continue;
@@ -366,30 +377,53 @@ public RecoveredElement buildInitialRecoveryState(){
 
 	return element;
 }
+
+private void initModuleInfo(RecoveredElement element) {
+	if (element  instanceof RecoveredUnit) {
+		RecoveredUnit unit = (RecoveredUnit) element;
+		if (unit.unitDeclaration.isModuleInfo()) {
+			ASTNode node = null;
+			int i = 0;
+			for (; i <= this.astPtr; i++) {
+				if ((node = this.astStack[i]) instanceof ModuleDeclaration) {
+					unit.add((ModuleDeclaration) node, this.bracketDepth); 
+					break;
+				}
+			}
+		}
+	}
+}
+@Override
 protected void consumeAnnotationTypeDeclarationHeader() {
 	super.consumeAnnotationTypeDeclarationHeader();
 	pushOnElementStack(K_TYPE_DELIMITER);
 }
+@Override
 protected void consumeClassBodyDeclaration() {
 	popElement(K_METHOD_DELIMITER);
 	super.consumeClassBodyDeclaration();
 }
+@Override
 protected void consumeClassBodyopt() {
 	super.consumeClassBodyopt();
 	popElement(K_SELECTOR);
 }
+@Override
 protected void consumeClassHeader() {
 	super.consumeClassHeader();
 	pushOnElementStack(K_TYPE_DELIMITER);
 }
+@Override
 protected void consumeConstructorBody() {
 	super.consumeConstructorBody();
 	popElement(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeConstructorHeader() {
 	super.consumeConstructorHeader();
 	pushOnElementStack(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeEnhancedForStatementHeaderInit(boolean hasModifiers) {
 	super.consumeEnhancedForStatementHeaderInit(hasModifiers);
 
@@ -399,15 +433,18 @@ protected void consumeEnhancedForStatementHeaderInit(boolean hasModifiers) {
 		this.currentElement = this.currentElement.add(localDecl, 0);
 	}
 }
+@Override
 protected void consumeEnterAnonymousClassBody(boolean qualified) {
 	super.consumeEnterAnonymousClassBody(qualified);
 	popElement(K_SELECTOR);
 	pushOnElementStack(K_TYPE_DELIMITER);
 }
+@Override
 protected void consumeEnterMemberValue() {
 	super.consumeEnterMemberValue();
 	pushOnElementStack(K_ATTRIBUTE_VALUE_DELIMITER, this.identifierPtr);
 }
+@Override
 protected void consumeEnumConstantHeader() {
 	if(this.currentToken == TokenNameLBRACE) {
 		popElement(K_ENUM_CONSTANT_DELIMITER);
@@ -416,30 +453,38 @@ protected void consumeEnumConstantHeader() {
 		pushOnElementStack(K_TYPE_DELIMITER);
 	}
 	super.consumeEnumConstantHeader();
+	if (triggerRecoveryUponLambdaClosure((Statement) this.astStack[this.astPtr], true) && this.currentElement != null)
+		this.restartRecovery = true;
 }
+@Override
 protected void consumeEnumConstantHeaderName() {
 	super.consumeEnumConstantHeaderName();
 	pushOnElementStack(K_ENUM_CONSTANT_DELIMITER);
 }
+@Override
 protected void consumeEnumConstantWithClassBody() {
 	popElement(K_TYPE_DELIMITER);
 	popElement(K_FIELD_INITIALIZER_DELIMITER);
 	popElement(K_ENUM_CONSTANT_DELIMITER);
 	super.consumeEnumConstantWithClassBody();
 }
+@Override
 protected void consumeEnumConstantNoClassBody() {
 	popElement(K_ENUM_CONSTANT_DELIMITER);
 	super.consumeEnumConstantNoClassBody();
 }
+@Override
 protected void consumeEnumHeader() {
 	super.consumeEnumHeader();
 	pushOnElementStack(K_TYPE_DELIMITER);
 }
+@Override
 protected void consumeExitMemberValue() {
 	super.consumeExitMemberValue();
 	popElement(K_ATTRIBUTE_VALUE_DELIMITER);
 }
 
+@Override
 protected void consumeExplicitConstructorInvocation(int flag, int recFlag) {
 	super.consumeExplicitConstructorInvocation(flag, recFlag);
 	popElement(K_SELECTOR);
@@ -474,7 +519,7 @@ protected boolean triggerRecoveryUponLambdaClosure(Statement statement, boolean
 			return false;
 		}
 	}
-	
+
 	if (lambdaClosed && this.currentElement != null && !(this.currentElement instanceof RecoveredField)) {
 		if (!(statement instanceof AbstractVariableDeclaration)) { // added already as part of standard recovery since these contribute a name to the scope prevailing at the cursor.
 			/* See if CompletionParser.attachOrphanCompletionNode has already added bits and pieces of AST to the recovery tree. If so, we want to
@@ -482,31 +527,42 @@ protected boolean triggerRecoveryUponLambdaClosure(Statement statement, boolean
 			   art/precedent in the Java 7 world to this: Search for recoveredBlock.statements[--recoveredBlock.statementCount] = null;
 			   See also that this concern does not arise in the case of field/local initialization since the initializer is replaced with full tree by consumeExitVariableWithInitialization.
 			*/
-			RecoveredBlock recoveredBlock = (RecoveredBlock) (this.currentElement instanceof RecoveredBlock ? this.currentElement : 
-				(this.currentElement.parent instanceof RecoveredBlock) ? this.currentElement.parent : 
-					this.currentElement instanceof RecoveredMethod ? ((RecoveredMethod) this.currentElement).methodBody : null);
-			if (recoveredBlock != null) {
-				RecoveredStatement recoveredStatement = recoveredBlock.statementCount > 0 ? recoveredBlock.statements[recoveredBlock.statementCount - 1] : null;
-				ASTNode parseTree = recoveredStatement != null ? recoveredStatement.updatedStatement(0, new HashSet()) : null;
-				if (parseTree != null) {
-					if ((parseTree.sourceStart == 0 || parseTree.sourceEnd == 0) || (parseTree.sourceStart >= statementStart && parseTree.sourceEnd <= statementEnd)) {
-						recoveredBlock.statements[recoveredBlock.statementCount - 1] = new RecoveredStatement(statement, recoveredBlock, 0);
-						statement = null;
-					} else if (recoveredStatement instanceof RecoveredLocalVariable && statement instanceof Expression) {
-						RecoveredLocalVariable local = (RecoveredLocalVariable) recoveredStatement;
-						if (local.localDeclaration != null && local.localDeclaration.initialization != null) {
-							if ((local.localDeclaration.initialization.sourceStart == 0 || local.localDeclaration.initialization.sourceEnd == 0) || 
-							        (local.localDeclaration.initialization.sourceStart >= statementStart && local.localDeclaration.initialization.sourceEnd <= statementEnd) ){
-								local.localDeclaration.initialization = (Expression) statement;
-								local.localDeclaration.declarationSourceEnd = statement.sourceEnd;
-								local.localDeclaration.declarationEnd = statement.sourceEnd;
-								statement = null;
+			/*
+			 * All the above comments will not work if the assist node is buried deeper. This happens when there the 
+			 * lambda was part of a complex statement, such as it was one of the arguments to a method invocation. In which case,
+			 * we start from the topmost recovery element and look for assist nodes. If the operation is successful, the method
+			 * replaceAssistStatement() returns null. Else, it returns the original statement, thus falling back to replacing the
+			 * last statement in the current recovered element.
+			 */
+			statement = replaceAssistStatement(this.currentElement.topElement(),
+					this.assistNodeParent(), statementStart, statementEnd, statement);
+			
+			if (statement != null) {
+				RecoveredBlock recoveredBlock = (RecoveredBlock) (this.currentElement instanceof RecoveredBlock ? this.currentElement : 
+					(this.currentElement.parent instanceof RecoveredBlock) ? this.currentElement.parent : 
+						this.currentElement instanceof RecoveredMethod ? ((RecoveredMethod) this.currentElement).methodBody : null);
+				if (recoveredBlock != null) {
+					RecoveredStatement recoveredStatement = recoveredBlock.statementCount > 0 ? recoveredBlock.statements[recoveredBlock.statementCount - 1] : null;
+					ASTNode parseTree = recoveredStatement != null ? recoveredStatement.updatedStatement(0, new HashSet<TypeDeclaration>()) : null;
+					if (parseTree != null) {
+						if ((parseTree.sourceStart == 0 || parseTree.sourceEnd == 0) || (parseTree.sourceStart >= statementStart && parseTree.sourceEnd <= statementEnd)) {
+							recoveredBlock.statements[recoveredBlock.statementCount - 1] = new RecoveredStatement(statement, recoveredBlock, 0);
+							statement = null;
+						} else if (recoveredStatement instanceof RecoveredLocalVariable && statement instanceof Expression) {
+							RecoveredLocalVariable local = (RecoveredLocalVariable) recoveredStatement;
+							if (local.localDeclaration != null && local.localDeclaration.initialization != null) {
+								if ((local.localDeclaration.initialization.sourceStart == 0 || local.localDeclaration.initialization.sourceEnd == 0) || 
+								        (local.localDeclaration.initialization.sourceStart >= statementStart && local.localDeclaration.initialization.sourceEnd <= statementEnd) ){
+									local.localDeclaration.initialization = (Expression) statement;
+									local.localDeclaration.declarationSourceEnd = statement.sourceEnd;
+									local.localDeclaration.declarationEnd = statement.sourceEnd;
+									statement = null;
+								}
 							}
 						}
 					}
 				}
 			}
-			
 			if (statement != null) {
 				while (this.currentElement != null) {
 					ASTNode tree = this.currentElement.parseTree();
@@ -522,6 +578,38 @@ protected boolean triggerRecoveryUponLambdaClosure(Statement statement, boolean
 	this.snapShot = null;
 	return lambdaClosed;
 }
+public Statement replaceAssistStatement(RecoveredElement top, ASTNode assistParent, int start, int end, Statement stmt) {
+	if (top == null) return null;
+	if (top instanceof RecoveredBlock) {
+		RecoveredBlock blk = (RecoveredBlock) top;
+		RecoveredStatement[] statements = blk.statements;
+		boolean found = false;
+		if (statements != null) {
+			for(int i = 0; i < statements.length; i++) {
+				if (statements[i] == null) break;
+				ASTNode node = statements[i].parseTree();
+				if ((node.sourceStart >= start && node.sourceEnd <= end)) {
+					if (!found) {
+						statements[i] = new RecoveredStatement(stmt, blk, 0);
+						found = true;
+						blk.statementCount = i + 1;
+					} else {
+						// Proceed to wipe out the nodes after the assist node. Most likely
+						// these are wrapped inside the just-closed-lambda. Even otherwise,
+						// they are not much useful for completion.
+						statements[i] = null;
+					}
+				}
+			}
+			if (found) return null;
+		}
+	} else if (top instanceof RecoveredMethod) {
+		stmt = replaceAssistStatement(((RecoveredMethod) top).methodBody, assistParent, start, end, stmt);
+	} else if (top instanceof RecoveredInitializer) {
+		stmt = replaceAssistStatement(((RecoveredInitializer) top).initializerBody, assistParent, start, end, stmt);
+	}
+	return stmt;
+}
 protected ASTNode assistNodeParent() {
 	return null;
 }
@@ -529,20 +617,24 @@ protected ASTNode enclosingNode() {
 	return null;
 }
 
+@Override
 protected boolean isAssistParser() {
 	return true;
 }
+@Override
 protected void consumeBlockStatement() {
 	super.consumeBlockStatement();
 	if (triggerRecoveryUponLambdaClosure((Statement) this.astStack[this.astPtr], true) && this.currentElement != null)
 		this.restartRecovery = true;
 }
+@Override
 protected void consumeBlockStatements() {
 	super.consumeBlockStatements();
 	if (triggerRecoveryUponLambdaClosure((Statement) this.astStack[this.astPtr], true) && this.currentElement != null) {
 		this.restartRecovery = true;
 	}
 }
+@Override
 protected void consumeFieldDeclaration() {
 	super.consumeFieldDeclaration();
 	if (triggerRecoveryUponLambdaClosure((Statement) this.astStack[this.astPtr], true)) {
@@ -552,6 +644,7 @@ protected void consumeFieldDeclaration() {
 			this.restartRecovery = true;
 	}
 }
+@Override
 protected void consumeForceNoDiet() {
 	super.consumeForceNoDiet();
 	// if we are not in a method (i.e. we are not in a local variable initializer)
@@ -570,6 +663,7 @@ protected void consumeForceNoDiet() {
 
 	}
 }
+@Override
 protected void consumeInterfaceHeader() {
 	super.consumeInterfaceHeader();
 	pushOnElementStack(K_TYPE_DELIMITER);
@@ -580,20 +674,24 @@ protected void consumeNestedLambda() {
 	LambdaExpression lexp = (LambdaExpression) this.astStack[this.astPtr];
 	pushOnElementStack(K_LAMBDA_EXPRESSION_DELIMITER, EXPRESSION_BODY, lexp);
 }
+@Override
 protected void consumeMethodBody() {
 	super.consumeMethodBody();
 	popElement(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeMethodDeclaration(boolean isNotAbstract, boolean isDefaultMethod) {
 	if (!isNotAbstract) {
 		popElement(K_METHOD_DELIMITER);
 	}
 	super.consumeMethodDeclaration(isNotAbstract, isDefaultMethod);
 }
+@Override
 protected void consumeMethodHeader() {
 	super.consumeMethodHeader();
 	pushOnElementStack(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeMethodInvocationName() {
 	super.consumeMethodInvocationName();
 	popElement(K_SELECTOR);
@@ -602,6 +700,7 @@ protected void consumeMethodInvocationName() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
 protected void consumeMethodInvocationNameWithTypeArguments() {
 	super.consumeMethodInvocationNameWithTypeArguments();
 	popElement(K_SELECTOR);
@@ -610,6 +709,7 @@ protected void consumeMethodInvocationNameWithTypeArguments() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
 protected void consumeMethodInvocationPrimary() {
 	super.consumeMethodInvocationPrimary();
 	popElement(K_SELECTOR);
@@ -618,6 +718,7 @@ protected void consumeMethodInvocationPrimary() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
 protected void consumeMethodInvocationPrimaryWithTypeArguments() {
 	super.consumeMethodInvocationPrimaryWithTypeArguments();
 	popElement(K_SELECTOR);
@@ -626,6 +727,7 @@ protected void consumeMethodInvocationPrimaryWithTypeArguments() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
 protected void consumeMethodInvocationSuper() {
 	super.consumeMethodInvocationSuper();
 	popElement(K_SELECTOR);
@@ -634,6 +736,7 @@ protected void consumeMethodInvocationSuper() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
 protected void consumeMethodInvocationSuperWithTypeArguments() {
 	super.consumeMethodInvocationSuperWithTypeArguments();
 	popElement(K_SELECTOR);
@@ -642,10 +745,65 @@ protected void consumeMethodInvocationSuperWithTypeArguments() {
 		this.lastCheckPoint = messageSend.sourceEnd + 1;
 	}
 }
+@Override
+protected void consumeModuleHeader() {
+	pushOnElementStack(K_MODULE_INFO_DELIMITER);
+	// ModuleHeader ::= 'module' Name
+	/* build an ImportRef build from the last name
+	stored in the identifier stack. */
+
+	int index;
+
+	/* no need to take action if not inside assist identifiers */
+	if ((index = indexOfAssistIdentifier()) < 0) {
+		super.consumeModuleHeader();
+		return;
+	}
+	/* retrieve identifiers subset and whole positions, the assist node positions
+	should include the entire replaced source. */
+	int length = this.identifierLengthStack[this.identifierLengthPtr];
+	char[][] subset = identifierSubSet(index+1); // include the assistIdentifier
+	this.identifierLengthPtr--;
+	this.identifierPtr -= length;
+	long[] positions = new long[length];
+	System.arraycopy(
+			this.identifierPositionStack,
+			this.identifierPtr + 1,
+			positions,
+			0,
+			length);
+	ModuleDeclaration typeDecl = createAssistModuleDeclaration(this.compilationUnit.compilationResult, subset, positions);
+
+	this.compilationUnit.moduleDeclaration = typeDecl;
+	this.assistNode = typeDecl;
+	this.lastCheckPoint = typeDecl.sourceEnd + 1;
+
+	//compute the declaration source too
+	typeDecl.declarationSourceStart = this.intStack[this.intPtr--];
+
+	typeDecl.bodyStart = typeDecl.sourceEnd + 1;
+	pushOnAstStack(typeDecl);
+
+	this.listLength = 0; // will be updated when reading super-interfaces
+	// recovery
+	if (this.currentElement != null){
+		this.lastCheckPoint = typeDecl.bodyStart;
+		this.currentElement = this.currentElement.add(typeDecl, 0);
+		this.lastIgnoredToken = -1;
+	}
+}
+
+@Override
+protected void consumeModuleDeclaration() {
+	super.consumeModuleDeclaration();
+	popElement(K_MODULE_INFO_DELIMITER);
+}
+@Override
 protected void consumeNestedMethod() {
 	super.consumeNestedMethod();
 	if(!isInsideMethod()) pushOnElementStack(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeOpenBlock() {
 	// OpenBlock ::= $empty
 	super.consumeOpenBlock();
@@ -688,6 +846,7 @@ protected void consumeOpenFakeBlock() {
 	}
 	this.blockStarts[this.realBlockPtr] = -this.scanner.startPosition;
 }
+@Override
 protected void consumePackageDeclarationName() {
 	// PackageDeclarationName ::= 'package' Name
 	/* build an ImportRef build from the last name
@@ -736,6 +895,7 @@ protected void consumePackageDeclarationName() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+@Override
 protected void consumePackageDeclarationNameWithModifiers() {
 	// PackageDeclarationName ::= Modifiers 'package' PushRealModifiers Name
 	/* build an ImportRef build from the last name
@@ -795,6 +955,7 @@ protected void consumePackageDeclarationNameWithModifiers() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+@Override
 protected void consumeRestoreDiet() {
 	super.consumeRestoreDiet();
 	// if we are not in a method (i.e. we were not in a local variable initializer)
@@ -804,6 +965,7 @@ protected void consumeRestoreDiet() {
 		popElement(K_FIELD_INITIALIZER_DELIMITER);
 	}
 }
+@Override
 protected void consumeSingleStaticImportDeclarationName() {
 	// SingleTypeImportDeclarationName ::= 'import' 'static' Name
 	/* push an ImportRef build from the last name
@@ -855,6 +1017,108 @@ protected void consumeSingleStaticImportDeclarationName() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+@Override
+protected void consumeSinglePkgName() {
+	int index;
+	/* no need to take action if not inside assist identifiers */
+	if ((index = indexOfAssistIdentifier()) < 0) {
+		super.consumeSinglePkgName();
+		return;
+	}
+	/* retrieve identifiers subset and whole positions, the assist node positions
+	should include the entire replaced source. */
+	int length = this.identifierLengthStack[this.identifierLengthPtr];
+	char[][] subset = identifierSubSet(index+1); // include the assistIdentifier
+	this.identifierLengthPtr--;
+	this.identifierPtr -= length;
+	long[] positions = new long[length];
+	System.arraycopy(
+			this.identifierPositionStack,
+			this.identifierPtr + 1,
+			positions,
+			0,
+			length);
+
+	/* build specific assist node on import statement */
+	ImportReference reference = createAssistPackageVisibilityReference(subset, positions);
+	this.assistNode = reference;
+	this.lastCheckPoint = reference.sourceEnd + 1;
+
+	pushOnAstStack(reference);
+
+	if (this.currentToken == TokenNameSEMICOLON) {
+		reference.declarationSourceEnd = this.scanner.currentPosition - 1;
+	} else {
+		reference.declarationSourceEnd = (int) positions[length-1];
+	}
+}
+@Override
+protected void consumeSingleTargetModuleName() {
+	int index;
+	/* no need to take action if not inside assist identifiers */
+	if ((index = indexOfAssistIdentifier()) < 0) {
+		super.consumeSingleTargetModuleName();
+		return;
+	}
+
+	/* build specific assist node on targetted exports statement */
+	ModuleReference reference = createAssistModuleReference(index);
+	this.assistNode = reference;
+	this.lastCheckPoint = reference.sourceEnd + 1;
+	pushOnAstStack(reference);
+
+	// recovery - TBD
+	if (this.currentElement instanceof RecoveredExportsStatement){
+		// TODO
+		this.lastCheckPoint = reference.sourceEnd+1;
+		this.currentElement = ((RecoveredExportsStatement) this.currentElement).add(reference, 0);
+		this.lastIgnoredToken = -1;
+		//this.restartRecovery = true; // used to avoid branching back into the regular automaton
+	}
+
+}
+@Override
+protected void consumeSingleRequiresModuleName() {
+
+	int index = indexOfAssistIdentifier();
+	/* no need to take action if not inside assist identifiers */
+	if (index < 0) {
+		super.consumeSingleRequiresModuleName();
+		return;
+	}
+
+	/* build specific assist node on requires statement */
+	ModuleReference reference = createAssistModuleReference(index);
+	this.assistNode = reference;
+	this.lastCheckPoint = reference.sourceEnd + 1;
+	RequiresStatement req = new RequiresStatement(reference);
+	if (this.currentToken == TokenNameSEMICOLON){
+		req.declarationSourceEnd = this.scanner.currentPosition - 1;
+	} else {
+		req.declarationSourceEnd = reference.sourceEnd;
+	}
+	req.sourceStart = req.declarationSourceStart;
+	req.declarationEnd = req.declarationSourceEnd;
+	req.modifiersSourceStart = this.intStack[this.intPtr--];
+	req.modifiers |= this.intStack[this.intPtr--];
+	req.declarationSourceStart = this.intStack[this.intPtr--];
+	if (req.modifiersSourceStart >= 0) {
+		req.declarationSourceStart = req.modifiersSourceStart;
+	}
+	req.sourceEnd = reference.sourceEnd;
+	pushOnAstStack(req);
+
+	// recovery TBD
+
+	if (this.currentElement != null){
+		this.lastCheckPoint = req.declarationSourceEnd + 1;
+		this.currentElement = this.currentElement.add(req, 0);
+		this.lastIgnoredToken = -1;
+	}
+
+}
+
+@Override
 protected void consumeSingleTypeImportDeclarationName() {
 	// SingleTypeImportDeclarationName ::= 'import' Name
 	/* push an ImportRef build from the last name
@@ -906,6 +1170,7 @@ protected void consumeSingleTypeImportDeclarationName() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+@Override
 protected void consumeStaticImportOnDemandDeclarationName() {
 	// TypeImportOnDemandDeclarationName ::= 'import' 'static' Name '.' '*'
 	/* push an ImportRef build from the last name
@@ -960,14 +1225,31 @@ protected void consumeStaticImportOnDemandDeclarationName() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+@Override
 protected void consumeStaticInitializer() {
 	super.consumeStaticInitializer();
 	popElement(K_METHOD_DELIMITER);
 }
+@Override
 protected void consumeStaticOnly() {
 	super.consumeStaticOnly();
 	pushOnElementStack(K_METHOD_DELIMITER);
 }
+private void adjustBracket(int token) {
+	switch (token) {
+		case TokenNameLPAREN :
+		case TokenNameLBRACE:
+		case TokenNameLBRACKET:
+			this.bracketDepth++;
+			break;
+		case TokenNameRBRACE:
+		case TokenNameRBRACKET:
+		case TokenNameRPAREN:
+			this.bracketDepth--;
+			break;
+	}
+}
+@Override
 protected void consumeToken(int token) {
 	super.consumeToken(token);
 
@@ -977,10 +1259,10 @@ protected void consumeToken(int token) {
 	}
 	// register message send selector only if inside a method or if looking at a field initializer
 	// and if the current token is an open parenthesis
-	if (isInsideMethod() || isInsideFieldInitialization() || isInsideAttributeValue()) {
+	if (isInsideMethod() || isInsideFieldInitialization() || isInsideAttributeValue() || isInsideEnumConstantnitialization()) {
+		adjustBracket(token);
 		switch (token) {
 			case TokenNameLPAREN :
-				this.bracketDepth++;
 				switch (this.previousToken) {
 					case TokenNameIdentifier:
 						this.pushOnElementStack(K_SELECTOR, this.identifierPtr);
@@ -1005,21 +1287,10 @@ protected void consumeToken(int token) {
 					popElement(K_LAMBDA_EXPRESSION_DELIMITER);
 					pushOnElementStack(K_LAMBDA_EXPRESSION_DELIMITER, BLOCK_BODY, this.previousObjectInfo);
 				}
-				this.bracketDepth++;
-				break;
-			case TokenNameLBRACKET:
-				this.bracketDepth++;
-				break;
-			case TokenNameRBRACE:
-				this.bracketDepth--;
-				break;
-			case TokenNameRBRACKET:
-				this.bracketDepth--;
-				break;
-			case TokenNameRPAREN:
-				this.bracketDepth--;
 				break;
 		}
+	} else if (isInsideModuleInfo()) { 
+		adjustBracket(token);
 	} else {
 		switch (token) {
 			case TokenNameRBRACE :
@@ -1034,6 +1305,7 @@ protected void consumeToken(int token) {
 		this.previousIdentifierPtr = this.identifierPtr;
 	}
 }
+@Override
 protected void consumeTypeImportOnDemandDeclarationName() {
 	// TypeImportOnDemandDeclarationName ::= 'import' Name '.' '*'
 	/* push an ImportRef build from the last name
@@ -1088,7 +1360,11 @@ protected void consumeTypeImportOnDemandDeclarationName() {
 		this.restartRecovery = true; // used to avoid branching back into the regular automaton
 	}
 }
+
+// TODO : Change to ExportsReference/PackageReference once we have the new compiler ast.node
+public abstract ImportReference createAssistPackageVisibilityReference(char[][] tokens, long[] positions);
 public abstract ImportReference createAssistImportReference(char[][] tokens, long[] positions, int mod);
+public abstract ModuleReference createAssistModuleReference(int index);
 public abstract ImportReference createAssistPackageReference(char[][] tokens, long[] positions);
 public abstract NameReference createQualifiedAssistNameReference(char[][] previousIdentifiers, char[] assistName, long[] positions);
 public abstract TypeReference createQualifiedAssistTypeReference(char[][] previousIdentifiers, char[] assistName, long[] positions);
@@ -1096,6 +1372,7 @@ public abstract TypeReference createParameterizedQualifiedAssistTypeReference(ch
 public abstract NameReference createSingleAssistNameReference(char[] assistName, long position);
 public abstract TypeReference createSingleAssistTypeReference(char[] assistName, long position);
 public abstract TypeReference createParameterizedSingleAssistTypeReference(TypeReference[] typeArguments, char[] assistName, long position);
+public abstract ModuleDeclaration createAssistModuleDeclaration(CompilationResult compilationResult, char[][] tokens, long[] positions);
 /*
  * Flush parser/scanner state regarding to code assist
  */
@@ -1117,6 +1394,7 @@ protected void flushElementStack() {
 /*
  * Build specific type reference nodes in case the cursor is located inside the type reference
  */
+@Override
 protected TypeReference getTypeReference(int dim) {
 
 	int index;
@@ -1273,6 +1551,7 @@ protected TypeReference getAssistTypeReferenceForGenericType(int dim, int identi
  * qualified name reference, then create a CompletionOnQualifiedNameReference
  * instead.
  */
+@Override
 protected NameReference getUnspecifiedReferenceOptimized() {
 
 	int completionIndex;
@@ -1314,18 +1593,22 @@ protected NameReference getUnspecifiedReferenceOptimized() {
 	this.lastCheckPoint = reference.sourceEnd + 1;
 	return reference;
 }
+@Override
 public void goForBlockStatementsopt() {
 	super.goForBlockStatementsopt();
 	this.isFirst = true;
 }
+@Override
 public void goForHeaders(){
 	super.goForHeaders();
 	this.isFirst = true;
 }
+@Override
 public void goForCompilationUnit(){
 	super.goForCompilationUnit();
 	this.isFirst = true;
 }
+@Override
 public void goForBlockStatementsOrCatchHeader() {
 	super.goForBlockStatementsOrCatchHeader();
 	this.isFirst = true;
@@ -1383,6 +1666,7 @@ protected int indexOfAssistIdentifier(boolean useGenericsStack){
 	// none of the awaiting identifiers is the completion one
 	return -1;
 }
+@Override
 public void initialize() {
 	super.initialize();
 	flushAssistState();
@@ -1390,6 +1674,7 @@ public void initialize() {
 	this.previousIdentifierPtr = -1;
 	this.bracketDepth = 0;
 }
+@Override
 public void initialize(boolean parsingCompilationUnit) {
 	super.initialize(parsingCompilationUnit);
 	flushAssistState();
@@ -1397,6 +1682,7 @@ public void initialize(boolean parsingCompilationUnit) {
 	this.previousIdentifierPtr = -1;
 	this.bracketDepth = 0;
 }
+@Override
 public abstract void initializeScanner();
 protected boolean isIndirectlyInsideFieldInitialization(){
 	int i = this.elementPtr;
@@ -1407,6 +1693,15 @@ protected boolean isIndirectlyInsideFieldInitialization(){
 	}
 	return false;
 }
+protected boolean isIndirectlyInsideEnumConstantnitialization(){
+	int i = this.elementPtr;
+	while(i > -1) {
+		if(this.elementKindStack[i] == K_ENUM_CONSTANT_DELIMITER)
+			return true;
+		i--;
+	}
+	return false;
+}
 protected boolean isIndirectlyInsideMethod(){
 	int i = this.elementPtr;
 	while(i > -1) {
@@ -1416,6 +1711,7 @@ protected boolean isIndirectlyInsideMethod(){
 	}
 	return false;
 }
+@Override
 protected boolean isIndirectlyInsideLambdaExpression(){
 	int i = this.elementPtr;
 	while (i > -1) {
@@ -1462,7 +1758,36 @@ protected boolean isInsideFieldInitialization(){
 		switch (this.elementKindStack[i]) {
 			case K_TYPE_DELIMITER : return false;
 			case K_METHOD_DELIMITER : return false;
-			case K_FIELD_INITIALIZER_DELIMITER : return true;
+			case K_FIELD_INITIALIZER_DELIMITER : 
+				return true;
+		}
+		i--;
+	}
+	return false;
+}
+protected boolean isInsideEnumConstantnitialization(){
+	int i = this.elementPtr;
+	while(i > -1) {
+		switch (this.elementKindStack[i]) {
+			case K_TYPE_DELIMITER : return false;
+			case K_METHOD_DELIMITER : return false;
+			case K_ENUM_CONSTANT_DELIMITER :
+				return true;
+		}
+		i--;
+	}
+	return false;
+}
+protected boolean isInsideModuleInfo(){
+	int i = this.elementPtr;
+	while(i > -1) {
+		switch (this.elementKindStack[i]) {
+			case K_TYPE_DELIMITER : 
+			case K_METHOD_DELIMITER :
+			case K_FIELD_INITIALIZER_DELIMITER : 
+				return false;
+			case K_MODULE_INFO_DELIMITER:
+				return true;
 		}
 		i--;
 	}
@@ -1646,8 +1971,6 @@ public void parseBlockStatements(MethodDeclaration md, CompilationUnitDeclaratio
 
 	//convert bugs into parse error
 
-	if (md.isAbstract())
-		return;
 	if (md.isNative())
 		return;
 	if ((md.modifiers & ExtraCompilerModifiers.AccSemicolonBody) != 0)
@@ -1698,7 +2021,7 @@ public void parseBlockStatements(MethodDeclaration md, CompilationUnitDeclaratio
 
 }
 
-// the name is a misnomer, we allow "pop"s not just at the TOS. Lambda wants to be sticky till fully reduced, however we do want other elements poppped at the right point, so ... 
+// the name is a misnomer, we allow "pop"s not just at the TOS. Lambda wants to be sticky till fully reduced, however we do want other elements popped at the right point, so ... 
 protected void popElement(int kind) {
 	
 	if (this.elementPtr < 0)
@@ -1708,7 +2031,7 @@ protected void popElement(int kind) {
 
 	if (kind != K_LAMBDA_EXPRESSION_DELIMITER) {
 		while (this.elementKindStack[stackPointer] == K_LAMBDA_EXPRESSION_DELIMITER) {
-			stackPointer --;
+			if (--stackPointer < 0) break;
 		}
 	}
 	if (stackPointer < 0 || this.elementKindStack[stackPointer] != kind)
@@ -1749,6 +2072,7 @@ protected void popUntilElement(int kind){
 /*
  * Prepares the state of the parser to go for BlockStatements.
  */
+@Override
 protected void prepareForBlockStatements() {
 	this.nestedMethod[this.nestedType = 0] = 1;
 	this.variablesCounter[this.nestedType] = 0;
@@ -1819,6 +2143,7 @@ protected void pushOnElementStack(int kind, int info, Object objectInfo){
 	this.elementInfoStack[this.elementPtr] = info;
 	this.elementObjectInfoStack[this.elementPtr] = objectInfo;
 }
+@Override
 public void recoveryExitFromVariable() {
 	if(this.currentElement != null && this.currentElement instanceof RecoveredField
 		&& !(this.currentElement instanceof RecoveredInitializer)) {
@@ -1831,6 +2156,7 @@ public void recoveryExitFromVariable() {
 		super.recoveryExitFromVariable();
 	}
 }
+@Override
 public void recoveryTokenCheck() {
 	RecoveredElement oldElement = this.currentElement;
 	switch (this.currentToken) {
@@ -1907,9 +2233,12 @@ protected int fallBackToSpringForward(Statement unused) {
 			
 	// If triggered fake EOF at completion site, see if the real next token would have passed muster.
 	if (this.currentToken == TokenNameEOF) {
-		if (this.scanner.eofPosition < this.scanner.source.length) {
+		int extendedEnd = this.scanner.source.length;
+		if (this.referenceContext instanceof AbstractMethodDeclaration)
+			extendedEnd = ((AbstractMethodDeclaration) this.referenceContext).bodyEnd; // no use parsing beyond the method's body end
+		if (this.scanner.eofPosition < extendedEnd) {
 			shouldStackAssistNode();
-			this.scanner.eofPosition = this.scanner.source.length;
+			this.scanner.eofPosition = extendedEnd;
 			nextToken = getNextToken();
 			if (automatonWillShift(nextToken, automatonState)) {
 				this.currentToken = nextToken;
@@ -1953,6 +2282,7 @@ protected int fallBackToSpringForward(Statement unused) {
  * Move checkpoint location, reset internal stacks and
  * decide which grammar goal is activated.
  */
+@Override
 protected int resumeAfterRecovery() {
 	if (requireExtendedRecovery()) {
 		if (this.unstackedAct == ERROR_ACTION) {
@@ -2020,8 +2350,15 @@ protected int resumeAfterRecovery() {
 			goForBlockStatementsopt();
 		} else {
 			prepareForHeaders();
+			if (this.referenceContext instanceof CompilationUnitDeclaration) {
+				CompilationUnitDeclaration unit = (CompilationUnitDeclaration) this.referenceContext;
+				if (unit.isModuleInfo()) {
+					pushOnElementStack(K_MODULE_INFO_DELIMITER);
+				}
+			}
 			goForHeaders();
 			this.diet = true; // passed this point, will not consider method bodies
+			this.dietInt = 0;
 		}
 		return RESTART;
 	}
