@@ -1,9 +1,13 @@
+// AspectJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -17,6 +21,7 @@ import org.aspectj.org.eclipse.jdt.core.compiler.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.*;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.*;
@@ -106,7 +111,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 	public Compiler(
 			INameEnvironment environment,
 			IErrorHandlingPolicy policy,
-			Map settings,
+			Map<String, String> settings,
 			final ICompilerRequestor requestor,
 			IProblemFactory problemFactory) {
 		this(environment, policy, new CompilerOptions(settings), requestor, problemFactory, null /* printwriter */, null /* progress */);
@@ -273,6 +278,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 			this.requestor = requestor;
 		} else {
 			this.requestor = new ICompilerRequestor(){
+				@Override
 				public void acceptResult(CompilationResult result){
 					if (DebugRequestor.isActive()){
 						DebugRequestor.acceptDebugResult(result);
@@ -291,6 +297,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 	/**
 	 * Add an additional binary type
 	 */
+	@Override
 	public void accept(IBinaryType binaryType, PackageBinding packageBinding, AccessRestriction accessRestriction) {
 		if (this.options.verbose) {
 			this.out.println(
@@ -298,13 +305,15 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 //			new Exception("TRACE BINARY").printStackTrace(System.out);
 //		    System.out.println();
 		}
-		this.lookupEnvironment.createBinaryTypeFrom(binaryType, packageBinding, accessRestriction);
+		LookupEnvironment env = packageBinding.environment;
+		env.createBinaryTypeFrom(binaryType, packageBinding, accessRestriction);
 	}
 
 	/**
 	 * Add an additional compilation unit into the loop
 	 *  ->  build compilation unit declarations, their bindings and record their results.
 	 */
+	@Override
 	public void accept(ICompilationUnit sourceUnit, AccessRestriction accessRestriction) {
 		// Switch the current policy and compilation result for this unit to the requested one.
 		CompilationResult unitResult =
@@ -348,6 +357,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 	/**
 	 * Add additional source types
 	 */
+	@Override
 	public void accept(ISourceType[] sourceTypes, PackageBinding packageBinding, AccessRestriction accessRestriction) {
 		this.problemReporter.abortDueToInternalError(
 			Messages.bind(Messages.abort_againstSourceModel, new String[] { String.valueOf(sourceTypes[0].getName()), String.valueOf(sourceTypes[0].getFileName()) }));
@@ -413,25 +423,33 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 		}
 	}
 
+	public void compile(ICompilationUnit[] sourceUnits) {
+		compile(sourceUnits, false);
+	}
 	/**
 	 * General API
 	 * -> compile each of supplied files
 	 * -> recompile any required types for which we have an incomplete principle structure
 	 */
-	public void compile(ICompilationUnit[] sourceUnits) {
+	private void compile(ICompilationUnit[] sourceUnits, boolean lastRound) {
 		this.stats.startTime = System.currentTimeMillis();
 		try {
 			// build and record parsed units
 			reportProgress(Messages.compilation_beginningToCompile);
 
+			if (this.options.complianceLevel >= ClassFileConstants.JDK9) {
+				// in Java 9 the compiler must never ask the oracle for a module that is contained in the input units:
+				sortModuleDeclarationsFirst(sourceUnits);
+			}
 			if (this.annotationProcessorManager == null) {
 				beginToCompile(sourceUnits);
 			} else {
 				ICompilationUnit[] originalUnits = sourceUnits.clone(); // remember source units in case a source type collision occurs
 				try {
 					beginToCompile(sourceUnits);
-
+					if (!lastRound) {
 						processAnnotations();
+					}
 					if (!this.options.generateClassFiles) {
 						// -proc:only was set on the command line
 						return;
@@ -449,13 +467,13 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 					System.arraycopy(originalUnits, 0, combinedUnits, 0, originalLength);
 					System.arraycopy(e.newAnnotationProcessorUnits, 0, combinedUnits, originalLength, newProcessedLength);
 					this.annotationProcessorStartIndex  = originalLength;
-					compile(combinedUnits);
+					compile(combinedUnits, e.isLastRound);
 					return;
 				}
 			}
 			// Restore the problems before the results are processed and cleaned up.
 			restoreAptProblems();
-			processCompiledUnits(0);
+			processCompiledUnits(0, lastRound);
 		} catch (AbortCompilation e) {
 			this.handleInternalException(e, null);
 		}
@@ -470,6 +488,18 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 		}
 	}
 
+	private void sortModuleDeclarationsFirst(ICompilationUnit[] sourceUnits) {
+		Arrays.sort(sourceUnits, (u1, u2) -> {
+			char[] fn1 = u1.getFileName();
+			char[] fn2 = u2.getFileName();
+			boolean isMod1 = CharOperation.endsWith(fn1, TypeConstants.MODULE_INFO_FILE_NAME) || CharOperation.endsWith(fn1, TypeConstants.MODULE_INFO_CLASS_NAME);
+			boolean isMod2 = CharOperation.endsWith(fn2, TypeConstants.MODULE_INFO_FILE_NAME) || CharOperation.endsWith(fn2, TypeConstants.MODULE_INFO_CLASS_NAME);
+			if (isMod1 == isMod2)
+				return 0;
+			return isMod1 ? -1 : 1;
+		});
+	}
+
 	class APTProblem {
 		CategorizedProblem problem;
 		ReferenceContext context;
@@ -478,11 +508,11 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 			this.context = context;
 		}
 	}
-
+	
 	protected void backupAptProblems() {
 		if (this.unitsToProcess == null) return;
-		for (CompilationUnitDeclaration unitDecl : this.unitsToProcess) {
-			if (unitDecl == null) continue;
+		for (int i = 0; i < this.totalUnits; i++) {
+			CompilationUnitDeclaration unitDecl = this.unitsToProcess[i];
 			CompilationResult result = unitDecl.compilationResult;
 			if (result != null && result.hasErrors()) {
 				CategorizedProblem[] errors = result.getErrors();
@@ -510,11 +540,12 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 	
 	protected void restoreAptProblems() {
 		if (this.unitsToProcess != null && this.aptProblems!= null) {
-			for (CompilationUnitDeclaration unit : this.unitsToProcess) {
-				APTProblem[] problems = this.aptProblems.get(new String(unit.getFileName()));
+			for (int i = 0; i < this.totalUnits; i++) {
+				CompilationUnitDeclaration unitDecl = this.unitsToProcess[i];
+				APTProblem[] problems = this.aptProblems.get(new String(unitDecl.getFileName()));
 				if (problems != null) {
 					for (APTProblem problem : problems) {
-						unit.compilationResult.record(problem.problem, problem.context);
+						unitDecl.compilationResult.record(problem.problem, problem.context);
 					}
 				}
 			}
@@ -522,7 +553,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 		this.aptProblems = null; // No need for this.
 	}
 
-	protected void processCompiledUnits(int startingIndex) throws java.lang.Error {
+	protected void processCompiledUnits(int startingIndex, boolean lastRound) throws java.lang.Error {
 		CompilationUnitDeclaration unit = null;
 		ProcessTaskManager processingTask = null;
 		try {
@@ -544,14 +575,12 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 								}));
 						process(unit, i);
 					} finally {
-						// cleanup compilation unit result, but only if not annotation processed.
+						// cleanup compilation unit result
 						// if (this.annotationProcessorManager == null || shouldCleanup(i))
 						// unit.cleanUp(); // AspectJ Extension - moved to afterProcessing
 					}
 					// AspectJ Extension
-					// 					if (this.annotationProcessorManager == null) {
 					// this.unitsToProcess[i] = null; // release reference to processed unit declaration
-					// }
 					// AspectJ Extension end
 					reportWorked(1, i);
 					this.stats.lineCount += unit.compilationResult.lineSeparatorPositions.length;
@@ -599,15 +628,17 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 							}));
 				}
 			}
-			if (this.annotationProcessorManager != null && this.totalUnits > this.annotationProcessorStartIndex) {
-				int backup = this.annotationProcessorStartIndex;
-				int prevUnits = this.totalUnits;
-				processAnnotations();
-				// Clean up the units that were left out previously for annotation processing.
-				for (int i = backup; i < prevUnits; i++) {
-					this.unitsToProcess[i].cleanUp();
+			if (!lastRound) {
+				if (this.annotationProcessorManager != null && this.totalUnits > this.annotationProcessorStartIndex) {
+					int backup = this.annotationProcessorStartIndex;
+					int prevUnits = this.totalUnits;
+					processAnnotations();
+					// Clean up the units that were left out previously for annotation processing.
+					for (int i = backup; i < prevUnits; i++) {
+						this.unitsToProcess[i].cleanUp();
+					}
+					processCompiledUnits(backup, lastRound);
 				}
-				processCompiledUnits(backup);
 			}
 		} catch (AbortCompilation e) {
 			this.handleInternalException(e, unit);
@@ -969,6 +1000,7 @@ public class Compiler implements ITypeRequestor, ProblemSeverities {
 				this.lookupEnvironment.isProcessingAnnotations = true;
 				internalBeginToCompile(newUnits, newUnitSize);
 			} catch (SourceTypeCollisionException e) {
+				e.isLastRound = true;
 				e.newAnnotationProcessorUnits = newProcessedUnits;
 				throw e;
 			} finally {
