@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2005, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
- * 
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ *
  * Contributors:
  *    IBM Corporation - initial API and implementation
  *    IBM Corporation - Fix for bug 328575
@@ -51,7 +54,7 @@ public class RoundEnvImpl implements RoundEnvironment
 		_isLastRound = isLastRound;
 		_units = units;
 		_factory = _processingEnv.getFactory();
-		
+
 		// Discover the annotations that will be passed to Processor.process()
 		AnnotationDiscoveryVisitor visitor = new AnnotationDiscoveryVisitor(_processingEnv);
 		if (_units != null) {
@@ -74,7 +77,7 @@ public class RoundEnvImpl implements RoundEnvironment
 			}
 			AnnotationBinding[] annotationBindings = Factory.getPackedAnnotationBindings(referenceBinding.getAnnotations());
 			for (AnnotationBinding annotationBinding : annotationBindings) {
-				TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType()); 
+				TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType());
 				Element element = _factory.newElement(referenceBinding);
 				_annoToUnit.put(anno, element);
 			}
@@ -82,7 +85,7 @@ public class RoundEnvImpl implements RoundEnvironment
 			for (FieldBinding fieldBinding : fieldBindings) {
 				annotationBindings = Factory.getPackedAnnotationBindings(fieldBinding.getAnnotations());
 				for (AnnotationBinding annotationBinding : annotationBindings) {
-					TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType()); 
+					TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType());
 					Element element = _factory.newElement(fieldBinding);
 					_annoToUnit.put(anno, element);
 				}
@@ -91,7 +94,7 @@ public class RoundEnvImpl implements RoundEnvironment
 			for (MethodBinding methodBinding : methodBindings) {
 				annotationBindings = Factory.getPackedAnnotationBindings(methodBinding.getAnnotations());
 				for (AnnotationBinding annotationBinding : annotationBindings) {
-					TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType()); 
+					TypeElement anno = (TypeElement)_factory.newElement(annotationBinding.getAnnotationType());
 					Element element = _factory.newElement(methodBinding);
 					_annoToUnit.put(anno, element);
 				}
@@ -120,7 +123,7 @@ public class RoundEnvImpl implements RoundEnvironment
 
 	/**
 	 * From the set of root elements and their enclosed elements, return the subset that are annotated
-	 * with {@code a}.  If {@code a} is annotated with the {@link java.lang.annotation.Inherited} 
+	 * with {@code a}.  If {@code a} is annotated with the {@link java.lang.annotation.Inherited}
 	 * annotation, include those elements that inherit the annotation from their superclasses.
 	 * Note that {@link java.lang.annotation.Inherited} only applies to classes (i.e. TypeElements).
 	 */
@@ -132,7 +135,7 @@ public class RoundEnvImpl implements RoundEnvironment
 		}
 		Binding annoBinding = ((TypeElementImpl)a)._binding;
 		if (0 != (annoBinding.getAnnotationTagBits() & TagBits.AnnotationInherited)) {
-			Set<Element> annotatedElements = new HashSet<Element>(_annoToUnit.getValues(a));
+			Set<Element> annotatedElements = new HashSet<>(_annoToUnit.getValues(a));
 			// For all other root elements that are TypeElements, and for their recursively enclosed
 			// types, add each element if it has a superclass are annotated with 'a'
 			ReferenceBinding annoTypeBinding = (ReferenceBinding) annoBinding;
@@ -144,7 +147,7 @@ public class RoundEnvImpl implements RoundEnvironment
 		}
 		return Collections.unmodifiableSet(_annoToUnit.getValues(a));
 	}
-	
+
 	/**
 	 * For every type in types that is a class and that is annotated with anno, either directly or by inheritance,
 	 * add that type to result.  Recursively descend on each types's child classes as well.
@@ -162,7 +165,7 @@ public class RoundEnvImpl implements RoundEnvironment
 			addAnnotatedElements(anno, element, result);
 		}
 	}
-	
+
 	/**
 	 * Check whether an element has a superclass that is annotated with an @Inherited annotation.
 	 * @param element must be a class (not an interface, enum, etc.).
@@ -185,7 +188,7 @@ public class RoundEnvImpl implements RoundEnvironment
 		} while (null != (searchedElement = searchedElement.superclass()));
 		return false;
 	}
-	
+
 	@Override
 	public Set<? extends Element> getElementsAnnotatedWith(Class<? extends Annotation> a)
 	{
@@ -208,8 +211,13 @@ public class RoundEnvImpl implements RoundEnvironment
 			return Collections.emptySet();
 		}
 		if (_rootElements == null) {
-			Set<Element> elements = new HashSet<Element>(_units.length);
+			Set<Element> elements = new HashSet<>(_units.length);
 			for (CompilationUnitDeclaration unit : _units) {
+				if (unit.moduleDeclaration != null && unit.moduleDeclaration.binding != null) {
+					Element m = _factory.newElement(unit.moduleDeclaration.binding);
+					elements.add(m);
+					continue;
+				}
 				if (null == unit.scope || null == unit.scope.topLevelTypes)
 					continue;
 				for (SourceTypeBinding binding : unit.scope.topLevelTypes) {
