@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2006, 2013 BEA Systems, Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2006, 2017 BEA Systems, Inc. and others
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *    wharley@bea.com - initial API and implementation
  *    IBM Corporation - Fix for bug 341494
@@ -27,6 +30,7 @@ import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import javax.lang.model.SourceVersion;
 import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.AnnotationValue;
 import javax.lang.model.element.Element;
@@ -58,24 +62,30 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 
 /**
- * Utilities for working with language elements.
+ * Utilities for working with java8 and earlier language elements.
  * There is one of these for every ProcessingEnvironment.
+ * 
+ * @see ElementsImpl9
  */
 public class ElementsImpl implements Elements {
 
 	// Used for parsing Javadoc comments: matches initial delimiter, followed by whitespace
 	private static final Pattern INITIAL_DELIMITER = Pattern.compile("^\\s*/\\*+"); //$NON-NLS-1$
 
-	private final BaseProcessingEnvImpl _env;
+	protected final BaseProcessingEnvImpl _env;
 
 	/*
 	 * The processing env creates and caches an ElementsImpl.  Other clients should
 	 * not create their own; they should ask the env for it.
 	 */
-	public ElementsImpl(BaseProcessingEnvImpl env) {
+	protected ElementsImpl(BaseProcessingEnvImpl env) {
 		_env = env;
 	}
 
+	public static ElementsImpl create(BaseProcessingEnvImpl env) {
+		return (SourceVersion.latest().compareTo(SourceVersion.RELEASE_8) <= 0)? new ElementsImpl(env): new ElementsImpl9(env);
+	}
+
 	/**
 	 * Return all the annotation mirrors on this element, including inherited annotations.
 	 * Annotations are inherited only if the annotation type is meta-annotated with @Inherited,
@@ -86,9 +96,9 @@ public class ElementsImpl implements Elements {
 	public List<? extends AnnotationMirror> getAllAnnotationMirrors(Element e) {
 		// if e is a class, walk up its superclass hierarchy looking for @Inherited annotations not already in the list
 		if (e.getKind() == ElementKind.CLASS && e instanceof TypeElementImpl) {
-			List<AnnotationBinding> annotations = new ArrayList<AnnotationBinding>();
+			List<AnnotationBinding> annotations = new ArrayList<>();
 			// A class can only have one annotation of a particular annotation type.
-			Set<ReferenceBinding> annotationTypes = new HashSet<ReferenceBinding>();
+			Set<ReferenceBinding> annotationTypes = new HashSet<>();
 			ReferenceBinding binding = (ReferenceBinding)((TypeElementImpl)e)._binding;
 			boolean checkIfInherited = false;
 			while (null != binding) {
@@ -108,7 +118,7 @@ public class ElementsImpl implements Elements {
 				binding = binding.superclass();
 				checkIfInherited = true;
 			}
-			List<AnnotationMirror> list = new ArrayList<AnnotationMirror>(annotations.size());
+			List<AnnotationMirror> list = new ArrayList<>(annotations.size());
 			for (AnnotationBinding annotation : annotations) {
 				list.add(_env.getFactory().newAnnotationMirror(annotation));
 			}
@@ -144,16 +154,16 @@ public class ElementsImpl implements Elements {
 		}
 		ReferenceBinding binding = (ReferenceBinding)((TypeElementImpl)type)._binding;
 		// Map of element simple name to binding
-		Map<String, ReferenceBinding> types = new HashMap<String, ReferenceBinding>();
+		Map<String, ReferenceBinding> types = new HashMap<>();
 		// Javac implementation does not take field name collisions into account
-		List<FieldBinding> fields = new ArrayList<FieldBinding>();
+		List<FieldBinding> fields = new ArrayList<>();
 		// For methods, need to compare parameters, not just names
-		Map<String, Set<MethodBinding>> methods = new HashMap<String, Set<MethodBinding>>();
-		Set<ReferenceBinding> superinterfaces = new LinkedHashSet<ReferenceBinding>();
+		Map<String, Set<MethodBinding>> methods = new HashMap<>();
+		Set<ReferenceBinding> superinterfaces = new LinkedHashSet<>();
 		boolean ignoreVisibility = true;
 		while (null != binding) {
 			addMembers(binding, ignoreVisibility, types, fields, methods);
-			Set<ReferenceBinding> newfound = new LinkedHashSet<ReferenceBinding>();
+			Set<ReferenceBinding> newfound = new LinkedHashSet<>();
 			collectSuperInterfaces(binding, superinterfaces, newfound);
 			for (ReferenceBinding superinterface : newfound) {
 				addMembers(superinterface, false, types, fields, methods);
@@ -162,7 +172,7 @@ public class ElementsImpl implements Elements {
 			binding = binding.superclass();
 			ignoreVisibility = false;
 		}
-		List<Element> allMembers = new ArrayList<Element>();
+		List<Element> allMembers = new ArrayList<>();
 		for (ReferenceBinding nestedType : types.values()) {
 			allMembers.add(_env.getFactory().newElement(nestedType));
 		}
@@ -229,7 +239,7 @@ public class ElementsImpl implements Elements {
 				if (null == sameNamedMethods) {
 					// New method name.  Create a set for it and add it to the list.
 					// We don't expect many methods with same name, so only 4 slots:
-					sameNamedMethods = new HashSet<MethodBinding>(4);
+					sameNamedMethods = new HashSet<>(4);
 					methods.put(methodName, sameNamedMethods);
 					sameNamedMethods.add(method);
 				}
@@ -553,16 +563,16 @@ public class ElementsImpl implements Elements {
 
 	@Override
 	public PackageElement getPackageElement(CharSequence name) {
-		LookupEnvironment le = _env.getLookupEnvironment();
+		LookupEnvironment le = _env.getLookupEnvironment(); // FIXME(SHMOD): does this lookup need to be module-aware?
 		if (name.length() == 0) {
-			return new PackageElementImpl(_env, le.defaultPackage);
+			return (PackageElement) _env.getFactory().newElement(le.defaultPackage);
 		}
 		char[] packageName = name.toString().toCharArray();
 		PackageBinding packageBinding = le.createPackage(CharOperation.splitOn('.', packageName));
 		if (packageBinding == null) {
 			return null;
 		}
-		return new PackageElementImpl(_env, packageBinding);
+		return (PackageElement) _env.getFactory().newElement(packageBinding);
 	}
 
 	@Override
@@ -702,6 +712,7 @@ public class ElementsImpl implements Elements {
 		}
 	}
 
+	@Override
 	public boolean isFunctionalInterface(TypeElement type) {
 		if (type != null && type.getKind() == ElementKind.INTERFACE) {
 			ReferenceBinding binding = (ReferenceBinding)((TypeElementImpl) type)._binding;
