@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2007, 2014 BEA Systems, Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2007, 2017 BEA Systems, Inc. and others
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *    wharley@bea.com - initial API and implementation
  *    IBM Corporation - fix for 342598
@@ -45,9 +48,11 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ElementValuePair;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ExtraCompilerModifiers;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ModuleBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.SplitPackageBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
@@ -89,7 +94,7 @@ public class Factory {
 		if (null == annotations || 0 == annotations.length) {
 			return Collections.emptyList();
 		}
-		List<AnnotationMirror> list = new ArrayList<AnnotationMirror>(annotations.length);
+		List<AnnotationMirror> list = new ArrayList<>(annotations.length);
 		for (AnnotationBinding annotation : annotations) {
 			if (annotation == null) continue;
 			list.add(newAnnotationMirror(annotation));
@@ -117,7 +122,7 @@ public class Factory {
 		String annoTypeName = annotationClass.getName();
 		if(annoTypeName == null ) return null;
 
-		List<A> list = new ArrayList<A>(annoInstances.length);
+		List<A> list = new ArrayList<>(annoInstances.length);
 		for(AnnotationBinding annoInstance : annoInstances) {
 			if (annoInstance == null)
 				continue;
@@ -319,6 +324,11 @@ public class Factory {
 					ClassFileConstants.AccStrictfp
 				});
 				break;
+			case MODULE :
+				decodeModifiers(result, modifiers, new int[] {
+						ClassFileConstants.ACC_OPEN,
+						ClassFileConstants.ACC_TRANSITIVE
+				});
 			default:
 				break;
 		}
@@ -348,7 +358,7 @@ public class Factory {
 				return new ErrorTypeElement(this._env, referenceBinding);
 			}
 			if (CharOperation.equals(referenceBinding.sourceName, TypeConstants.PACKAGE_INFO_NAME)) {
-				return new PackageElementImpl(_env, referenceBinding.fPackage);
+				return newPackageElement(referenceBinding.fPackage);
 			}
 			return new TypeElementImpl(_env, referenceBinding, kindHint);
 		case Binding.METHOD:
@@ -357,10 +367,12 @@ public class Factory {
 		case Binding.PARAMETERIZED_TYPE:
 			return new TypeElementImpl(_env, ((ParameterizedTypeBinding)binding).genericType(), kindHint);
 		case Binding.PACKAGE:
-			return new PackageElementImpl(_env, (PackageBinding)binding);
+			return newPackageElement((PackageBinding)binding);
 		case Binding.TYPE_PARAMETER:
 			return new TypeParameterElementImpl(_env, (TypeVariableBinding)binding);
 			// TODO: fill in the rest of these
+		case Binding.MODULE:
+			return new ModuleElementImpl(_env, (ModuleBinding) binding);
 		case Binding.IMPORT:
 		case Binding.ARRAY_TYPE:
 		case Binding.BASE_TYPE:
@@ -380,6 +392,12 @@ public class Factory {
 	 */
 	public PackageElement newPackageElement(PackageBinding binding)
 	{
+		if (binding instanceof SplitPackageBinding && binding.enclosingModule != null) {
+			binding = ((SplitPackageBinding) binding).getIncarnation(binding.enclosingModule);
+		}
+		if (binding == null) {
+			return null;
+		}
 		return new PackageElementImpl(_env, binding);
 	}
 	
@@ -396,6 +414,8 @@ public class Factory {
 			return NoTypeImpl.NO_TYPE_VOID;
 		case PACKAGE:
 			return NoTypeImpl.NO_TYPE_PACKAGE;
+		case MODULE:
+			return new NoTypeImpl(kind);
 		default:
 			throw new IllegalArgumentException();
 		}
@@ -487,6 +507,8 @@ public class Factory {
 
 		case Binding.TYPE_PARAMETER:
 			return new TypeVariableImpl(_env, (TypeVariableBinding) binding);
+		case Binding.MODULE:
+			return getNoType(TypeKind.MODULE);
 		}
 		return null;
 	}
@@ -569,17 +591,17 @@ public class Factory {
 				else 
 					return avoidReflectException ? Boolean.FALSE : value;
 			case 'c':
-				return new Character((char)b); // narrowing.
+				return Character.valueOf((char) b); // narrowing.
 			case 'd':
-				return new Double(b); // widening.
+				return Double.valueOf(b); // widening.
 			case 'f':
-				return new Float(b); // widening.
+				return  Float.valueOf(b); // widening.
 			case 'i':
-				return new Integer(b); // widening.
+				return Integer.valueOf(b); // widening.
 			case 'l':
-				return new Long(b); // widening.
+				return Long.valueOf(b); // widening.
 			case 's':
-				return new Short(b); // widening.
+				return Short.valueOf(b); // widening.
 			default:  				
 				throw new IllegalStateException("unknown type " + expectedTypeChar); //$NON-NLS-1$
 			}
@@ -593,19 +615,19 @@ public class Factory {
 			{
 			case 'b':
 				if(nameLen == 4) // byte
-					return new Byte((byte)s); // narrowing.
+					return Byte.valueOf((byte) s); // narrowing.
 				else
 					return avoidReflectException ? Boolean.FALSE : value; // completely wrong.
 			case 'c':
-				return new Character((char)s); // narrowing.
+				return Character.valueOf((char) s); // narrowing.
 			case 'd':
-				return new Double(s); // widening.
+				return Double.valueOf(s); // widening.
 			case 'f':
-				return new Float(s); // widening.
+				return Float.valueOf(s); // widening.
 			case 'i':
-				return new Integer(s); // widening.
+				return Integer.valueOf(s); // widening.
 			case 'l':
-				return new Long(s); // widening.
+				return Long.valueOf(s); // widening.
 			case 's':
 				return value; // exact match
 			default:  				
@@ -621,21 +643,21 @@ public class Factory {
 			{
 			case 'b':
 				if(nameLen == 4) // byte
-					return new Byte((byte)c); // narrowing.
+					return Byte.valueOf((byte) c); // narrowing.
 				else
 					return avoidReflectException ? Boolean.FALSE : value; // completely wrong.
 			case 'c':
 				return value; // exact match
 			case 'd':
-				return new Double(c); // widening.
+				return Double.valueOf(c); // widening.
 			case 'f':
-				return new Float(c); // widening.
+				return Float.valueOf(c); // widening.
 			case 'i':
-				return new Integer(c); // widening.
+				return Integer.valueOf(c); // widening.
 			case 'l':
-				return new Long(c); // widening.
+				return Long.valueOf(c); // widening.
 			case 's':
-				return new Short((short)c); // narrowing.
+				return Short.valueOf((short) c); // narrowing.
 			default:  				
 				throw new IllegalStateException("unknown type " + expectedTypeChar); //$NON-NLS-1$
 			}
@@ -650,21 +672,21 @@ public class Factory {
 			{    
 			case 'b':
 				if(nameLen == 4) // byte
-					return new Byte((byte)i); // narrowing.
+					return Byte.valueOf((byte) i); // narrowing.
 				else
 					return avoidReflectException ? Boolean.FALSE : value; // completely wrong.
 			case 'c':
-				return new Character((char)i); // narrowing
+				return Character.valueOf((char) i); // narrowing
 			case 'd':
-				return new Double(i); // widening.
+				return Double.valueOf(i); // widening.
 			case 'f':
-				return new Float(i); // widening.
+				return Float.valueOf(i); // widening.
 			case 'i':
 				return value; // exact match
 			case 'l':
-				return new Long(i); // widening.
+				return Long.valueOf(i); // widening.
 			case 's':
-				return new Short((short)i); // narrowing.
+				return Short.valueOf((short) i); // narrowing.
 			default:  				
 				throw new IllegalStateException("unknown type " + expectedTypeChar); //$NON-NLS-1$
 			}
@@ -682,9 +704,9 @@ public class Factory {
 				// completely wrong.
 				return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
 			case 'd':
-				return new Double(l); // widening.
+				return Double.valueOf(l); // widening.
 			case 'f':
-				return new Float(l); // widening.			
+				return Float.valueOf(l); // widening.			
 			case 'l': 
 				return value; // exact match.
 		
@@ -707,7 +729,7 @@ public class Factory {
 				// completely wrong.
 				return avoidReflectException ? getMatchingDummyValue(expectedType) : value;
 			case 'd':
-				return new Double(f); // widening.
+				return Double.valueOf(f); // widening.
 			case 'f':
 				return value; // exact match.
 			default:  				
@@ -769,7 +791,8 @@ public class Factory {
 	}
 
 	/* Wrap repeating annotations into their container, return an array of bindings.
-	   Incoming array is not modified.
+	   Incoming array is not modified. The resulting array may be null but will not contain null
+	   entries.
 	*/
 	public static AnnotationBinding [] getPackedAnnotationBindings(AnnotationBinding [] annotations) {
 		
@@ -804,7 +827,7 @@ public class Factory {
 						System.arraycopy(repackagedBindings, 0, repackagedBindings = new AnnotationBinding[length], 0, length);
 					repackagedBindings[j] = null; // so it is not double packed.
 					if (containees == null) {
-						containees = new ArrayList<AnnotationBinding>();
+						containees = new ArrayList<>();
 						containees.add(annotation);
 					}
 					containees.add(otherAnnotation);
@@ -815,14 +838,17 @@ public class Factory {
 				repackagedBindings[i] = new AnnotationBinding(containerType, elementValuePairs);
 			}
 		}
-		if (repackagedBindings == annotations)
-			return annotations;
-		
+
 		int finalTally = 0;
 		for (int i = 0; i < length; i++) {
 			if (repackagedBindings[i] != null)
 				finalTally++;
 		}
+
+		if (repackagedBindings == annotations && finalTally == length) {
+			return annotations;
+		}
+
 		annotations = new AnnotationBinding [finalTally];
 		for (int i = 0, j = 0; i < length; i++) {
 			if (repackagedBindings[i] != null)
@@ -839,7 +865,7 @@ public class Factory {
 		if (length == 0)
 			return annotations;
 		
-		List<AnnotationBinding> unpackedAnnotations = new ArrayList<AnnotationBinding>();
+		List<AnnotationBinding> unpackedAnnotations = new ArrayList<>();
 		for (int i = 0; i < length; i++) {
 			AnnotationBinding annotation = annotations[i];
 			if (annotation == null) continue;
@@ -873,6 +899,6 @@ public class Factory {
 				}
 			}
 		}
-		return (AnnotationBinding[]) unpackedAnnotations.toArray(new AnnotationBinding [unpackedAnnotations.size()]);
+		return unpackedAnnotations.toArray(new AnnotationBinding [unpackedAnnotations.size()]);
 	}	
 }
