@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2011 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2005, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -41,7 +44,6 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
-import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
@@ -55,7 +57,7 @@ public class TypeElementImpl extends ElementImpl implements TypeElement {
 	 *
 	 */
 	private static final class SourceLocationComparator implements Comparator<Element> {
-		private final IdentityHashMap<ElementImpl, Integer> sourceStartCache = new IdentityHashMap<ElementImpl, Integer>();
+		private final IdentityHashMap<ElementImpl, Integer> sourceStartCache = new IdentityHashMap<>();
 
 		@Override
 		public int compare(Element o1, Element o2) {
@@ -146,7 +148,7 @@ public class TypeElementImpl extends ElementImpl implements TypeElement {
 	@Override
 	public List<? extends Element> getEnclosedElements() {
 		ReferenceBinding binding = (ReferenceBinding)_binding;
-		List<Element> enclosed = new ArrayList<Element>(binding.fieldCount() + binding.methods().length + binding.memberTypes().length);
+		List<Element> enclosed = new ArrayList<>(binding.fieldCount() + binding.methods().length + binding.memberTypes().length);
 		for (MethodBinding method : binding.methods()) {
 			ExecutableElement executable = new ExecutableElementImpl(_env, method);
 			enclosed.add(executable);
@@ -195,7 +197,7 @@ public class TypeElementImpl extends ElementImpl implements TypeElement {
 		if (null == binding.superInterfaces() || binding.superInterfaces().length == 0) {
 			return Collections.emptyList();
 		}
-		List<TypeMirror> interfaces = new ArrayList<TypeMirror>(binding.superInterfaces().length);
+		List<TypeMirror> interfaces = new ArrayList<>(binding.superInterfaces().length);
 		for (ReferenceBinding interfaceBinding : binding.superInterfaces()) {
 			TypeMirror interfaceType = _env.getFactory().newTypeMirror(interfaceBinding);
 			if (interfaceType.getKind() == TypeKind.ERROR) {
@@ -263,7 +265,7 @@ public class TypeElementImpl extends ElementImpl implements TypeElement {
 	PackageElement getPackage()
 	{
 		ReferenceBinding binding = (ReferenceBinding)_binding;
-		return _env.getFactory().newPackageElement((PackageBinding)binding.fPackage);
+		return _env.getFactory().newPackageElement(binding.fPackage);
 	}
 
 	@Override
@@ -309,7 +311,7 @@ public class TypeElementImpl extends ElementImpl implements TypeElement {
 		if (variables.length == 0) {
 			return Collections.emptyList();
 		}
-		List<TypeParameterElement> params = new ArrayList<TypeParameterElement>(variables.length); 
+		List<TypeParameterElement> params = new ArrayList<>(variables.length); 
 		for (TypeVariableBinding variable : variables) {
 			params.add(_env.getFactory().newTypeParameterElement(variable, this));
 		}
