@@ -1,13 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2007 - 2012 BEA Systems, Inc. and others
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2007 - 2017 BEA Systems, Inc. and others
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Walter Harley - initial API and implementation
- *    IBM Corporation - fix for 342598
+ *    IBM Corporation - fix for 342598, 382590
+ *    Jean-Marie Henaff <jmhenaff@google.com> (Google) - Bug 481555
  *******************************************************************************/
 
 package org.aspectj.org.eclipse.jdt.internal.compiler.apt.model;
@@ -64,72 +68,176 @@ public class TypesImpl implements Types {
     @Override
     public Element asElement(TypeMirror t) {
         switch(t.getKind()) {
-            case DECLARED :
-            case TYPEVAR :
-                return _env.getFactory().newElement(((TypeMirrorImpl)t).binding());
-            default:
-            	break;
+        case DECLARED :
+        case TYPEVAR :
+            return _env.getFactory().newElement(((TypeMirrorImpl)t).binding());
+        default:
+            break;
         }
         return null;
     }
 
     @Override
-    public TypeMirror asMemberOf(DeclaredType containing, Element element) {
-//        throw new UnsupportedOperationException("NYI: TypesImpl.asMemberOf(" + containing + ", " + element + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
-    	ElementImpl elementImpl = (ElementImpl) element;
-    	DeclaredTypeImpl declaredTypeImpl = (DeclaredTypeImpl) containing;
-    	ReferenceBinding referenceBinding = (ReferenceBinding) declaredTypeImpl._binding;
-    	switch(element.getKind()) {
-    		case CONSTRUCTOR :
-    		case METHOD :
-    			MethodBinding methodBinding = (MethodBinding) elementImpl._binding;
-    			while (referenceBinding != null) {
-    			for (MethodBinding method : referenceBinding.methods()) {
-    				if (CharOperation.equals(method.selector, methodBinding.selector)
-    						&& method.areParameterErasuresEqual(methodBinding)) {
-    					return this._env.getFactory().newTypeMirror(method);
-    				}
-    			}
-                    referenceBinding = referenceBinding.superclass();
-                }
-    			break;
-    		case FIELD :
-    		case ENUM_CONSTANT:
-    			FieldBinding fieldBinding = (FieldBinding) elementImpl._binding;
-                while (referenceBinding != null) {
-    			for (FieldBinding field : referenceBinding.fields()) {
-    				if (CharOperation.equals(field.name, fieldBinding.name)) {
-    					return this._env.getFactory().newTypeMirror(field);
-    				}
-    			}
-                    referenceBinding = referenceBinding.superclass();
-                }
-    			break;
-    		case ENUM :
-    		case ANNOTATION_TYPE :
-    		case INTERFACE :
-    		case CLASS :
-    			ReferenceBinding elementBinding = (ReferenceBinding) elementImpl._binding;
-                while (referenceBinding != null) {
-                    // If referenceBinding is a ParameterizedTypeBinding, this will return only ParameterizedTypeBindings
-                    // for member types, even if the member happens to be a static nested class. That's probably a bug;
-                    // static nested classes are not parameterized by their outer class.
-                    for (ReferenceBinding memberReferenceBinding : referenceBinding.memberTypes()) {
-                        if (CharOperation.equals(elementBinding.compoundName, memberReferenceBinding.compoundName)) {
-                            return this._env.getFactory().newTypeMirror(memberReferenceBinding);
-    			}
-    				}
-                    referenceBinding = referenceBinding.superclass();
-    			}
-    			break;
-    		default:
-                throw new IllegalArgumentException("element " + element + //$NON-NLS-1$
-                        " has unrecognized element kind " + element.getKind()); //$NON-NLS-1$
-    	}
-            throw new IllegalArgumentException("element " + element + //$NON-NLS-1$
-                    " is not a member of the containing type " + containing +  //$NON-NLS-1$
-                    " nor any of its superclasses"); //$NON-NLS-1$
-    }
+	public TypeMirror asMemberOf(DeclaredType containing, Element element) {
+		// throw new UnsupportedOperationException("NYI: TypesImpl.asMemberOf("
+		// + containing + ", " + element + ")"); //$NON-NLS-1$ //$NON-NLS-2$
+		// //$NON-NLS-3$
+		ElementImpl elementImpl = (ElementImpl) element;
+		DeclaredTypeImpl declaredTypeImpl = (DeclaredTypeImpl) containing;
+		ReferenceBinding referenceBinding = (ReferenceBinding) declaredTypeImpl._binding;
+		TypeMirror typeMirror;
+
+		switch (element.getKind()) {
+		case CONSTRUCTOR:
+		case METHOD:
+			typeMirror = findMemberInHierarchy(referenceBinding, elementImpl._binding, new MemberInTypeFinder() {
+				@Override
+				public TypeMirror find(ReferenceBinding typeBinding, Binding memberBinding) {
+					MethodBinding methodBinding = ((MethodBinding) memberBinding);
+					for (MethodBinding method : typeBinding.methods()) {
+						if (CharOperation.equals(method.selector, methodBinding.selector)
+								&& (method.original() == methodBinding
+										|| method.areParameterErasuresEqual(methodBinding))) {
+							return TypesImpl.this._env.getFactory().newTypeMirror(method);
+						}
+					}
+					return null;
+				}
+			});
+
+			if (typeMirror != null) {
+				return typeMirror;
+			}
+			break;
+		case TYPE_PARAMETER:
+			typeMirror = findMemberInHierarchy(referenceBinding, elementImpl._binding, new MemberInTypeFinder() {
+				@Override
+				public TypeMirror find(ReferenceBinding typeBinding, Binding memberBinding) {
+					if (typeBinding instanceof ParameterizedTypeBinding) {
+						TypeVariableBinding variableBinding = ((TypeVariableBinding) memberBinding);
+						ReferenceBinding binding = ((ParameterizedTypeBinding) typeBinding).genericType();
+						if (variableBinding.declaringElement == binding) { // check in advance avoid looking into type parameters unnecessarily.
+							TypeVariableBinding[] typeVariables = binding.typeVariables();
+							TypeBinding[] typeArguments = ((ParameterizedTypeBinding) typeBinding).typeArguments();
+							if (typeVariables.length == typeArguments.length) {
+								for(int i = 0; i < typeVariables.length; i++) {
+									if (typeVariables[i] == memberBinding) {
+										return TypesImpl.this._env.getFactory().newTypeMirror(typeArguments[i]); 
+									}
+								}
+							}
+						}
+					}
+					return null;
+				}
+			});
+
+			if (typeMirror != null) {
+				return typeMirror;
+			}
+			break;
+		case FIELD:
+		case ENUM_CONSTANT:
+			typeMirror = findMemberInHierarchy(referenceBinding, elementImpl._binding, new MemberInTypeFinder() {
+				@Override
+				public TypeMirror find(ReferenceBinding typeBinding, Binding memberBinding) {
+					FieldBinding fieldBinding = (FieldBinding) memberBinding;
+					for (FieldBinding field : typeBinding.fields()) {
+						if (CharOperation.equals(field.name, fieldBinding.name)) {
+							return TypesImpl.this._env.getFactory().newTypeMirror(field);
+						}
+					}
+					return null;
+				}
+			});
+
+			if (typeMirror != null) {
+				return typeMirror;
+			}
+			break;
+		case ENUM:
+		case ANNOTATION_TYPE:
+		case INTERFACE:
+		case CLASS:
+
+			typeMirror = findMemberInHierarchy(referenceBinding, elementImpl._binding, new MemberInTypeFinder() {
+				@Override
+				public TypeMirror find(ReferenceBinding typeBinding, Binding memberBinding) {
+					ReferenceBinding elementBinding = (ReferenceBinding) memberBinding;
+					// If referenceBinding is a ParameterizedTypeBinding, this
+					// will return only ParameterizedTypeBindings
+					// for member types, even if the member happens to be a
+					// static nested class. That's probably a bug;
+					// static nested classes are not parameterized by their
+					// outer class.
+					for (ReferenceBinding memberReferenceBinding : typeBinding.memberTypes()) {
+						if (CharOperation.equals(elementBinding.compoundName, memberReferenceBinding.compoundName)) {
+							return TypesImpl.this._env.getFactory().newTypeMirror(memberReferenceBinding);
+						}
+					}
+					return null;
+				}
+			});
+
+			if (typeMirror != null) {
+				return typeMirror;
+			}
+			break;
+		default:
+			throw new IllegalArgumentException("element " + element + //$NON-NLS-1$
+					" has unrecognized element kind " + element.getKind()); //$NON-NLS-1$
+		}
+		throw new IllegalArgumentException("element " + element + //$NON-NLS-1$
+				" is not a member of the containing type " + containing + //$NON-NLS-1$
+				" nor any of its superclasses"); //$NON-NLS-1$
+	}
+
+	private static interface MemberInTypeFinder {
+		TypeMirror find(ReferenceBinding typeBinding, Binding memberBinding);
+	}
+
+	private TypeMirror findMemberInHierarchy(ReferenceBinding typeBinding, Binding memberBinding,
+			MemberInTypeFinder finder) {
+		TypeMirror result = null;
+
+		if (typeBinding == null) {
+			return null;
+		}
+
+		result = finder.find(typeBinding, memberBinding);
+		if (result != null) {
+			return result;
+		}
+
+		result = findMemberInHierarchy(typeBinding.superclass(), memberBinding, finder);
+		if (result != null) {
+			return result;
+		}
+
+		for (ReferenceBinding superInterface : typeBinding.superInterfaces()) {
+			result = findMemberInHierarchy(superInterface, memberBinding, finder);
+			if (result != null) {
+				return result;
+			}
+		}
+
+		return null;
+	}
+	private void validateRealType(TypeMirror t) {
+		switch (t.getKind()) {
+			case EXECUTABLE:
+			case PACKAGE:
+			case MODULE:
+				throw new IllegalArgumentException(
+						"Executable, package and module are illegal argument for Types.contains(..)"); //$NON-NLS-1$
+			default:
+				break;
+		}
+	}
+	private void validateRealTypes(TypeMirror t1, TypeMirror t2) {
+		validateRealType(t1);
+		validateRealType(t2);
+	}
 
     @Override
     public TypeElement boxedClass(PrimitiveType p) {
@@ -141,69 +249,63 @@ public class TypesImpl implements Types {
 
     @Override
     public TypeMirror capture(TypeMirror t) {
-        throw new UnsupportedOperationException("NYI: TypesImpl.capture(...)"); //$NON-NLS-1$
+    	validateRealType(t);
+    	TypeMirrorImpl typeMirrorImpl = (TypeMirrorImpl) t;
+    	if (typeMirrorImpl._binding instanceof ParameterizedTypeBinding) {
+    		throw new UnsupportedOperationException("NYI: TypesImpl.capture(...)"); //$NON-NLS-1$
+    	}
+        return t;
     }
 
     @Override
     public boolean contains(TypeMirror t1, TypeMirror t2) {
-    	switch(t1.getKind()) {
-    		case EXECUTABLE :
-    		case PACKAGE :
-    			throw new IllegalArgumentException("Executable and package are illegal argument for Types.contains(..)"); //$NON-NLS-1$
-    		default:
-    			break;
-    	}
-    	switch(t2.getKind()) {
-    		case EXECUTABLE :
-    		case PACKAGE :
-    			throw new IllegalArgumentException("Executable and package are illegal argument for Types.contains(..)"); //$NON-NLS-1$
-    		default:
-    			break;
-    	}
+    	validateRealTypes(t1, t2);
         throw new UnsupportedOperationException("NYI: TypesImpl.contains(" + t1 + ", " + t2 + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
     }
 
     @Override
     public List<? extends TypeMirror> directSupertypes(TypeMirror t) {
-        switch(t.getKind()) {
-            case PACKAGE :
-            case EXECUTABLE :
-                throw new IllegalArgumentException("Invalid type mirror for directSupertypes"); //$NON-NLS-1$
-            default:
-                break;
-        }
+    	validateRealType(t);
         TypeMirrorImpl typeMirrorImpl = (TypeMirrorImpl) t;
         Binding binding = typeMirrorImpl._binding;
         if (binding instanceof ReferenceBinding) {
-        	ReferenceBinding referenceBinding = (ReferenceBinding) binding;
-        	ArrayList<TypeMirror> list = new ArrayList<TypeMirror>();
-        	ReferenceBinding superclass = referenceBinding.superclass();
-			if (superclass != null) {
-        		list.add(this._env.getFactory().newTypeMirror(superclass));
-        	}
-			for (ReferenceBinding interfaceBinding : referenceBinding.superInterfaces()) {
-        		list.add(this._env.getFactory().newTypeMirror(interfaceBinding));
-			}
-			return Collections.unmodifiableList(list);
+            ReferenceBinding referenceBinding = (ReferenceBinding) binding;
+            ArrayList<TypeMirror> list = new ArrayList<>();
+            ReferenceBinding superclass = referenceBinding.superclass();
+            if (superclass != null) {
+                list.add(this._env.getFactory().newTypeMirror(superclass));
+            }
+            for (ReferenceBinding interfaceBinding : referenceBinding.superInterfaces()) {
+                list.add(this._env.getFactory().newTypeMirror(interfaceBinding));
+            }
+            return Collections.unmodifiableList(list);
         }
         return Collections.emptyList();
     }
 
     @Override
     public TypeMirror erasure(TypeMirror t) {
-    	TypeMirrorImpl typeMirrorImpl = (TypeMirrorImpl) t;
-    	Binding binding = typeMirrorImpl._binding;
-    	if (binding instanceof ReferenceBinding) {
-    		return _env.getFactory().newTypeMirror(((ReferenceBinding) binding).erasure());
-    	}
-    	if (binding instanceof ArrayBinding) {
-    		TypeBinding typeBinding = (TypeBinding) binding;
-    		return _env.getFactory().newTypeMirror(
-    				this._env.getLookupEnvironment().createArrayType(
-    						typeBinding.leafComponentType().erasure(),
-    						typeBinding.dimensions()));
-    	}
-    	return t;
+    	validateRealType(t);
+        TypeMirrorImpl typeMirrorImpl = (TypeMirrorImpl) t;
+        Binding binding = typeMirrorImpl._binding;
+        if (binding instanceof ReferenceBinding) {
+        	TypeBinding type = ((ReferenceBinding) binding).erasure();
+        	if (type.isGenericType()) {
+        		type = _env.getLookupEnvironment().convertToRawType(type, false);
+        	}
+            return _env.getFactory().newTypeMirror(type);
+        }
+        if (binding instanceof ArrayBinding) {
+            TypeBinding typeBinding = (TypeBinding) binding;
+            TypeBinding leafType = typeBinding.leafComponentType().erasure();
+            if (leafType.isGenericType()) {
+            	leafType = _env.getLookupEnvironment().convertToRawType(leafType, false);
+            }
+            return _env.getFactory().newTypeMirror(
+                    this._env.getLookupEnvironment().createArrayType(leafType,
+                            typeBinding.dimensions()));
+        }
+        return t;
     }
 
     @Override
@@ -211,15 +313,15 @@ public class TypesImpl implements Types {
         TypeMirrorImpl typeMirrorImpl = (TypeMirrorImpl) componentType;
         TypeBinding typeBinding = (TypeBinding) typeMirrorImpl._binding;
         return (ArrayType) _env.getFactory().newTypeMirror(
-        		this._env.getLookupEnvironment().createArrayType(
-        				typeBinding.leafComponentType(),
-        				typeBinding.dimensions() + 1));
+                this._env.getLookupEnvironment().createArrayType(
+                        typeBinding.leafComponentType(),
+                        typeBinding.dimensions() + 1));
     }
 
     /*
      * (non-Javadoc)
      * Create a type instance by parameterizing a type element. If the element is a member type,
-     * its container must not be generic (if it were, you would need to use the form of
+     * its container won't be parameterized (if it needs to be, you would need to use the form of
      * getDeclaredType that takes a container TypeMirror). If typeArgs is empty, and typeElem
      * is not generic, then you should use TypeElem.asType().  If typeArgs is empty and typeElem
      * is generic, this method will create the raw type.
@@ -249,8 +351,14 @@ public class TypesImpl implements Types {
             }
             typeArguments[i] = (TypeBinding) binding;
         }
+
+        ReferenceBinding enclosing = elementBinding.enclosingType();
+        if (enclosing != null) {
+            enclosing = this._env.getLookupEnvironment().createRawType(enclosing, null);
+        }
+
         return (DeclaredType) _env.getFactory().newTypeMirror(
-                this._env.getLookupEnvironment().createParameterizedType(elementBinding, typeArguments, null));
+                this._env.getLookupEnvironment().createParameterizedType(elementBinding, typeArguments, enclosing));
     }
 
     /* (non-Javadoc)
@@ -322,12 +430,12 @@ public class TypesImpl implements Types {
             TypeMirrorImpl extendsBoundMirrorType = (TypeMirrorImpl) extendsBound;
             TypeBinding typeBinding = (TypeBinding) extendsBoundMirrorType._binding;
             return (WildcardType) _env.getFactory().newTypeMirror(
-            		this._env.getLookupEnvironment().createWildcard(
-	                    null,
-	                    0,
-	                    typeBinding,
-	                    null,
-	                    Wildcard.EXTENDS));
+                    this._env.getLookupEnvironment().createWildcard(
+                            null,
+                            0,
+                            typeBinding,
+                            null,
+                            Wildcard.EXTENDS));
         }
         if (superBound != null) {
             TypeMirrorImpl superBoundMirrorType = (TypeMirrorImpl) superBound;
@@ -352,8 +460,9 @@ public class TypesImpl implements Types {
      */
     @Override
     public boolean isAssignable(TypeMirror t1, TypeMirror t2) {
+    	validateRealTypes(t1, t2);
         if (!(t1 instanceof TypeMirrorImpl) || !(t2 instanceof TypeMirrorImpl)) {
-            return false;
+        	return false;
         }
         Binding b1 = ((TypeMirrorImpl)t1).binding();
         Binding b2 = ((TypeMirrorImpl)t2).binding();
@@ -371,10 +480,18 @@ public class TypesImpl implements Types {
 
     @Override
     public boolean isSameType(TypeMirror t1, TypeMirror t2) {
-    	if (t1.getKind() == TypeKind.WILDCARD || t2.getKind() == TypeKind.WILDCARD) {
+        if (t1 instanceof NoTypeImpl) {
+            if (t2 instanceof NoTypeImpl) {
+                return ((NoTypeImpl) t1).getKind() == ((NoTypeImpl) t2).getKind();
+            }
+            return false;
+        } else if (t2 instanceof NoTypeImpl) {
+            return false;
+        }
+        if (t1.getKind() == TypeKind.WILDCARD || t2.getKind() == TypeKind.WILDCARD) {
             // Wildcard types are never equal, according to the spec of this method
-    		return false;
-    	}
+            return false;
+        }
         if (t1 == t2) {
             return true;
         }
@@ -399,28 +516,29 @@ public class TypesImpl implements Types {
 
     @Override
     public boolean isSubsignature(ExecutableType m1, ExecutableType m2) {
-		MethodBinding methodBinding1 = (MethodBinding) ((ExecutableTypeImpl) m1)._binding;
-		MethodBinding methodBinding2 = (MethodBinding) ((ExecutableTypeImpl) m2)._binding;
-		if (!CharOperation.equals(methodBinding1.selector, methodBinding2.selector))
-			return false;
-		return methodBinding1.areParameterErasuresEqual(methodBinding2) && methodBinding1.areTypeVariableErasuresEqual(methodBinding2);
-	}
+        MethodBinding methodBinding1 = (MethodBinding) ((ExecutableTypeImpl) m1)._binding;
+        MethodBinding methodBinding2 = (MethodBinding) ((ExecutableTypeImpl) m2)._binding;
+        if (!CharOperation.equals(methodBinding1.selector, methodBinding2.selector))
+            return false;
+        return methodBinding1.areParameterErasuresEqual(methodBinding2) && methodBinding1.areTypeVariableErasuresEqual(methodBinding2);
+    }
 
     /* (non-Javadoc)
      * @return true if t1 is a subtype of t2, or if t1 == t2.
      */
     @Override
     public boolean isSubtype(TypeMirror t1, TypeMirror t2) {
-    	if (t1 instanceof NoTypeImpl) {
-    		if (t2 instanceof NoTypeImpl) {
-    			return ((NoTypeImpl) t1).getKind() == ((NoTypeImpl) t2).getKind();
-    		}
-    		return false;
-    	} else if (t2 instanceof NoTypeImpl) {
-    		return false;
-    	}
-        if (!(t1 instanceof TypeMirrorImpl) || !(t2 instanceof TypeMirrorImpl)) {
+    	validateRealTypes(t1, t2);
+        if (t1 instanceof NoTypeImpl) {
+            if (t2 instanceof NoTypeImpl) {
+                return ((NoTypeImpl) t1).getKind() == ((NoTypeImpl) t2).getKind();
+            }
             return false;
+        } else if (t2 instanceof NoTypeImpl) {
+            return false;
+        }
+        if (!(t1 instanceof TypeMirrorImpl) || !(t2 instanceof TypeMirrorImpl)) {
+        	throw new IllegalArgumentException();
         }
         if (t1 == t2) {
             return true;
@@ -432,7 +550,7 @@ public class TypesImpl implements Types {
         }
         if (!(b1 instanceof TypeBinding) || !(b2 instanceof TypeBinding)) {
             // package, method, import, etc.
-            return false;
+        	 throw new IllegalArgumentException();
         }
         if (b1.kind() == Binding.BASE_TYPE || b2.kind() == Binding.BASE_TYPE) {
             if (b1.kind() != b2.kind()) {
