@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -59,6 +62,7 @@ public BinaryExpression(BinaryExpression expression) {
 	this.sourceStart = expression.sourceStart;
 	this.sourceEnd = expression.sourceEnd;
 }
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	// keep implementation in sync with CombinedBinaryExpression#analyseCode
 	try {
@@ -68,13 +72,13 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 								this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits())
 							.unconditionalInits();
 		} else {
-			this.left.checkNPE(currentScope, flowContext, flowInfo);
 			flowInfo = this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+			this.left.checkNPE(currentScope, flowContext, flowInfo);
 			if (((this.bits & OperatorMASK) >> OperatorSHIFT) != AND) {
 				flowContext.expireNullCheckedFieldInfo();
 			}
-			this.right.checkNPE(currentScope, flowContext, flowInfo);
 			flowInfo = this.right.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits();
+			this.right.checkNPE(currentScope, flowContext, flowInfo);
 			if (((this.bits & OperatorMASK) >> OperatorSHIFT) != AND) {
 				flowContext.expireNullCheckedFieldInfo();
 			}
@@ -113,6 +117,7 @@ public void computeConstant(BlockScope scope, int leftId, int rightId) {
 	}
 }
 
+@Override
 public Constant optimizedBooleanConstant() {
 	return this.optimizedBooleanConstant == null ? this.constant : this.optimizedBooleanConstant;
 }
@@ -123,6 +128,7 @@ public Constant optimizedBooleanConstant() {
 // given the current focus of CombinedBinaryExpression on strings concatenation,
 // we do not provide a general, non-recursive implementation of generateCode,
 // but rely upon generateOptimizedStringConcatenationCreation instead
+@Override
 public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
 	int pc = codeStream.position;
 	if (this.constant != Constant.NotAConstant) {
@@ -582,6 +588,7 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
  * Boolean operator code generation
  *	Optimized operations are: <, <=, >, >=, &, |, ^
  */
+@Override
 public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
 	if ((this.constant != Constant.NotAConstant) && (this.constant.typeID() == TypeIds.T_boolean)) {
 		super.generateOptimizedBoolean(
@@ -1547,6 +1554,7 @@ public void generateOptimizedLogicalXor(BlockScope currentScope, CodeStream code
 	codeStream.recordPositionsFrom(codeStream.position, this.sourceEnd);
 }
 
+@Override
 public void generateOptimizedStringConcatenation(BlockScope blockScope, CodeStream codeStream, int typeID) {
 	// keep implementation in sync with CombinedBinaryExpression
 	// #generateOptimizedStringConcatenation
@@ -1579,6 +1587,7 @@ public void generateOptimizedStringConcatenation(BlockScope blockScope, CodeStre
 	}
 }
 
+@Override
 public void generateOptimizedStringConcatenationCreation(BlockScope blockScope, CodeStream codeStream, int typeID) {
 	// keep implementation in sync with CombinedBinaryExpression
 	// #generateOptimizedStringConcatenationCreation
@@ -1613,6 +1622,7 @@ public void generateOptimizedStringConcatenationCreation(BlockScope blockScope,
 	}
 }
 
+@Override
 public boolean isCompactableOperation() {
 	return true;
 }
@@ -1784,6 +1794,7 @@ public void optimizedBooleanConstant(int leftId, int operator, int rightId) {
 	}
 }
 
+@Override
 public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output) {
 	// keep implementation in sync with
 	// CombinedBinaryExpression#printExpressionNoParenthesis
@@ -1791,6 +1802,7 @@ public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output
 	return this.right.printExpression(0, output);
 }
 
+@Override
 public TypeBinding resolveType(BlockScope scope) {
 	// keep implementation in sync with CombinedBinaryExpression#resolveType
 	// and nonRecursiveResolveTypeUpwards
@@ -1901,6 +1913,7 @@ public TypeBinding resolveType(BlockScope scope) {
 	return this.resolvedType;
 }
 
+@Override
 public void traverse(ASTVisitor visitor, BlockScope scope) {
 	if (visitor.visit(this, scope)) {
 		this.left.traverse(visitor, scope);
