@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -37,6 +40,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.flow.FlowContext;
 import org.aspectj.org.eclipse.jdt.internal.compiler.flow.FlowInfo;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.Constant;
+import org.aspectj.org.eclipse.jdt.internal.compiler.impl.IrritantSet;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BlockScope;
@@ -68,6 +72,7 @@ public CastExpression(Expression expression, TypeReference type) {
 	type.bits |= ASTNode.IgnoreRawTypeCheck; // no need to worry about raw type usage
 }
 
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	FlowInfo result = this.expression
 		.analyseCode(currentScope, flowContext, flowInfo)
@@ -266,27 +271,46 @@ public static void checkNeedForArgumentCasts(BlockScope scope, int operator, int
 	}
 }
 
-public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
+@Override
+public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
+	if((this.resolvedType.tagBits & TagBits.AnnotationNonNull) != 0) {
+		return true;
+	}
 	checkNPEbyUnboxing(scope, flowContext, flowInfo);
-	return this.expression.checkNPE(scope, flowContext, flowInfo);
+	return this.expression.checkNPE(scope, flowContext, flowInfo, ttlForFieldCheck);
 }
 
 private static void checkAlternateBinding(BlockScope scope, Expression receiver, TypeBinding receiverType, MethodBinding binding, Expression[] arguments, TypeBinding[] originalArgumentTypes, TypeBinding[] alternateArgumentTypes, final InvocationSite invocationSite) {
 		InvocationSite fakeInvocationSite = new InvocationSite(){
+			@Override
 			public TypeBinding[] genericTypeArguments() { return null; }
+			@Override
 			public boolean isSuperAccess(){ return invocationSite.isSuperAccess(); }
+			@Override
 			public boolean isTypeAccess() { return invocationSite.isTypeAccess(); }
+			@Override
 			public void setActualReceiverType(ReferenceBinding actualReceiverType) { /* ignore */}
+			@Override
 			public void setDepth(int depth) { /* ignore */}
+			@Override
 			public void setFieldIndex(int depth){ /* ignore */}
+			@Override
 			public int sourceStart() { return 0; }
+			@Override
 			public int sourceEnd() { return 0; }
+			@Override
 			public TypeBinding invocationTargetType() { return invocationSite.invocationTargetType(); }
+			@Override
 			public boolean receiverIsImplicitThis() { return invocationSite.receiverIsImplicitThis();}
+			@Override
 			public InferenceContext18 freshInferenceContext(Scope someScope) { return invocationSite.freshInferenceContext(someScope); }
+			@Override
 			public ExpressionContext getExpressionContext() { return invocationSite.getExpressionContext(); }
+			@Override
 			public boolean isQualifiedSuper() { return invocationSite.isQualifiedSuper(); }
+			@Override
 			public boolean checkingPotentialCompatibility() { return false; }
+			@Override
 			public void acceptPotentiallyCompatibleMethods(MethodBinding[] methods) {/* ignore */}
 		};
 		MethodBinding bindingIfNoCast;
@@ -319,12 +343,32 @@ private static void checkAlternateBinding(BlockScope scope, Expression receiver,
 			for (int i = 0; i < argumentLength; i++) {
 				if (TypeBinding.notEquals(originalArgumentTypes[i], alternateArgumentTypes[i])
                        /*&& !originalArgumentTypes[i].needsUncheckedConversion(alternateArgumentTypes[i])*/) {
-					scope.problemReporter().unnecessaryCast((CastExpression)arguments[i]);
+					if (!preventsUnlikelyTypeWarning(originalArgumentTypes[i], alternateArgumentTypes[i], receiverType, binding, scope))
+						scope.problemReporter().unnecessaryCast((CastExpression)arguments[i]);
 				}
 			}
 		}
 }
 
+private static boolean preventsUnlikelyTypeWarning(TypeBinding castedType, TypeBinding uncastedType, TypeBinding receiverType, MethodBinding binding, BlockScope scope) {
+	if (!scope.compilerOptions().isAnyEnabled(IrritantSet.UNLIKELY_ARGUMENT_TYPE))
+		return false;
+	if (binding.isStatic() || binding.parameters.length != 1)
+		return false;
+	// would using the uncastedType be considered as dangerous?
+	UnlikelyArgumentCheck argumentChecks = UnlikelyArgumentCheck.determineCheckForNonStaticSingleArgumentMethod(
+			uncastedType, scope, binding.selector, receiverType, binding.parameters);
+	if (argumentChecks != null && argumentChecks.isDangerous(scope)) {
+		// does the cast help?
+		argumentChecks = UnlikelyArgumentCheck.determineCheckForNonStaticSingleArgumentMethod(
+				castedType, scope, binding.selector, receiverType, binding.parameters);
+		if (argumentChecks == null || !argumentChecks.isDangerous(scope))
+			return true;
+	}
+	return false;
+}
+
+@Override
 public boolean checkUnsafeCast(Scope scope, TypeBinding castType, TypeBinding expressionType, TypeBinding match, boolean isNarrowing) {
 	if (TypeBinding.equalsEquals(match, castType)) {
 		if (!isNarrowing && TypeBinding.equalsEquals(match, this.resolvedType.leafComponentType()) // do not tag as unnecessary when recursing through upper bounds
@@ -439,6 +483,7 @@ public boolean checkUnsafeCast(Scope scope, TypeBinding castType, TypeBinding ex
  * @param codeStream org.aspectj.org.eclipse.jdt.internal.compiler.codegen.CodeStream
  * @param valueRequired boolean
  */
+@Override
 public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
 	int pc = codeStream.position;
 	boolean annotatedCast = (this.type.bits & ASTNode.HasTypeAnnotations) != 0;
@@ -447,7 +492,7 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 		if (valueRequired || needRuntimeCheckcast || annotatedCast) { // Added for: 1F1W9IG: IVJCOM:WINNT - Compiler omits casting check
 			codeStream.generateConstant(this.constant, this.implicitConversion);
 			if (needRuntimeCheckcast || annotatedCast) {
-				codeStream.checkcast(this.type, this.resolvedType);
+				codeStream.checkcast(this.type, this.resolvedType, pc);
 			}
 			if (!valueRequired) {
 				// the resolveType cannot be double or long
@@ -459,7 +504,7 @@ public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean
 	}
 	this.expression.generateCode(currentScope, codeStream, annotatedCast || valueRequired || needRuntimeCheckcast);
 	if (annotatedCast || (needRuntimeCheckcast && TypeBinding.notEquals(this.expression.postConversionType(currentScope), this.resolvedType.erasure()))) { // no need to issue a checkcast if already done as genericCast
-		codeStream.checkcast(this.type, this.resolvedType);
+		codeStream.checkcast(this.type, this.resolvedType, pc);
 	}
 	if (valueRequired) {
 		codeStream.generateImplicitConversion(this.implicitConversion);
@@ -488,10 +533,12 @@ public Expression innermostCastedExpression(){
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#localVariableBinding()
  */
+@Override
 public LocalVariableBinding localVariableBinding() {
 	return this.expression.localVariableBinding();
 }
 
+@Override
 public int nullStatus(FlowInfo flowInfo, FlowContext flowContext) {
 	if ((this.implicitConversion & TypeIds.BOXING) != 0)
 		return FlowInfo.NON_NULL;
@@ -501,6 +548,7 @@ public int nullStatus(FlowInfo flowInfo, FlowContext flowContext) {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#optimizedBooleanConstant()
  */
+@Override
 public Constant optimizedBooleanConstant() {
 	switch(this.resolvedType.id) {
 		case T_boolean :
@@ -510,6 +558,7 @@ public Constant optimizedBooleanConstant() {
 	return Constant.NotAConstant;
 }
 
+@Override
 public StringBuffer printExpression(int indent, StringBuffer output) {
 	int parenthesesCount = (this.bits & ASTNode.ParenthesizedMASK) >> ASTNode.ParenthesizedSHIFT;
 	String suffix = ""; //$NON-NLS-1$
@@ -522,6 +571,7 @@ public StringBuffer printExpression(int indent, StringBuffer output) {
 	return this.expression.printExpression(0, output).append(suffix);
 }
 
+@Override
 public TypeBinding resolveType(BlockScope scope) {
 	// compute a new constant if the cast is effective
 
@@ -598,6 +648,7 @@ public TypeBinding resolveType(BlockScope scope) {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#setExpectedType(org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding)
  */
+@Override
 public void setExpectedType(TypeBinding expectedType) {
 	this.expectedType = expectedType;
 }
@@ -627,6 +678,7 @@ private boolean isIndirectlyUsed() {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#tagAsNeedCheckCast()
  */
+@Override
 public void tagAsNeedCheckCast() {
 	this.bits |= ASTNode.GenerateCheckcast;
 }
@@ -634,10 +686,12 @@ public void tagAsNeedCheckCast() {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#tagAsUnnecessaryCast(Scope, TypeBinding)
  */
+@Override
 public void tagAsUnnecessaryCast(Scope scope, TypeBinding castType) {
 	this.bits |= ASTNode.UnnecessaryCast;
 }
 
+@Override
 public void traverse(ASTVisitor visitor, BlockScope blockScope) {
 	if (visitor.visit(this, blockScope)) {
 		this.type.traverse(visitor, blockScope);
