@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -68,6 +71,7 @@ public class ConditionalExpression extends OperatorExpression implements IPolyEx
 		this.sourceEnd = valueIfFalse.sourceEnd;
 	}
 
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 			FlowInfo flowInfo) {
 		int initialComplaintLevel = (flowInfo.reachMode() & FlowInfo.UNREACHABLE) != 0 ? Statement.COMPLAINED_FAKE_REACHABLE : Statement.NOT_COMPLAINED;
@@ -187,7 +191,8 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return mergedInfo;
 	}
 
-	public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
+	@Override
+	public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
 		if ((this.nullStatus & FlowInfo.NULL) != 0)
 			scope.problemReporter().expressionNullReference(this);
 		else if ((this.nullStatus & FlowInfo.POTENTIALLY_NULL) != 0)
@@ -237,6 +242,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 	 * @param codeStream org.aspectj.org.eclipse.jdt.internal.compiler.codegen.CodeStream
 	 * @param valueRequired boolean
 	*/
+	@Override
 	public void generateCode(
 		BlockScope currentScope,
 		CodeStream codeStream,
@@ -327,6 +333,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 	/**
 	 * Optimized boolean code generation for the conditional operator ?:
 	*/
+	@Override
 	public void generateOptimizedBoolean(
 		BlockScope currentScope,
 		CodeStream codeStream,
@@ -337,7 +344,8 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		int pc = codeStream.position;
 
 		if ((this.constant != Constant.NotAConstant) && (this.constant.typeID() == T_boolean) // constant
-			|| ((this.valueIfTrue.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) != T_boolean) { // non boolean values
+			|| ((this.valueIfTrue.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) != T_boolean
+			|| ((this.valueIfFalse.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) != T_boolean) { // non boolean values
 			super.generateOptimizedBoolean(currentScope, codeStream, trueLabel, falseLabel, valueRequired);
 			return;
 		}
@@ -418,17 +426,20 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		codeStream.recordPositionsFrom(pc, this.sourceEnd);
 	}
 
+	@Override
 	public int nullStatus(FlowInfo flowInfo, FlowContext flowContext) {
 		if ((this.implicitConversion & TypeIds.BOXING) != 0)
 			return FlowInfo.NON_NULL;
 		return this.nullStatus;
 	}
 
+	@Override
 	public Constant optimizedBooleanConstant() {
 
 		return this.optimizedBooleanConstant == null ? this.constant : this.optimizedBooleanConstant;
 	}
 
+	@Override
 	public StringBuffer printExpressionNoParenthesis(int indent, StringBuffer output) {
 
 		this.condition.printExpression(indent, output).append(" ? "); //$NON-NLS-1$
@@ -436,6 +447,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return this.valueIfFalse.printExpression(0, output);
 	}
 
+	@Override
 	public TypeBinding resolveType(BlockScope scope) {
 		// JLS3 15.25
 		LookupEnvironment env = scope.environment();
@@ -478,7 +490,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 				return this.resolvedType = null;
 		}
 		if (isPolyExpression()) {
-			if (this.expectedType == null) {
+			if (this.expectedType == null || !this.expectedType.isProperType(true)) {
 				return new PolyTypeBinding(this);
 			}
 			return this.resolvedType = computeConversions(scope, this.expectedType) ? this.expectedType : null;
@@ -706,14 +718,17 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return ok;
 	}
 
+	@Override
 	public void setExpectedType(TypeBinding expectedType) {
 		this.expectedType = expectedType;
 	}
 	
+	@Override
 	public void setExpressionContext(ExpressionContext context) {
 		this.expressionContext = context;
 	}
 
+	@Override
 	public ExpressionContext getExpressionContext() {
 		return this.expressionContext;
 	}
@@ -732,6 +747,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return allPolys;
 	}
 
+	@Override
 	public boolean isPertinentToApplicability(TypeBinding targetType, MethodBinding method) {
 		return this.valueIfTrue.isPertinentToApplicability(targetType, method) 
 				&& this.valueIfFalse.isPertinentToApplicability(targetType, method);
@@ -748,6 +764,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return this.valueIfTrue.isFunctionalType() || this.valueIfFalse.isFunctionalType(); // Even if only one arm is functional type, this will require a functional interface target
 	}
 	
+	@Override
 	public boolean isPolyExpression() throws UnsupportedOperationException {
 		
 		if (!this.use18specifics)
@@ -762,7 +779,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		if (this.originalValueIfTrueType == null || this.originalValueIfFalseType == null) // resolution error.
 			return false;
 		
-		if (this.originalValueIfTrueType.kind() == Binding.POLY_TYPE || this.originalValueIfFalseType.kind() == Binding.POLY_TYPE)
+		if (this.valueIfTrue.isPolyExpression() || this.valueIfFalse.isPolyExpression())
 			return true;
 		
 		// "... unless both operands produce primitives (or boxed primitives)":
@@ -775,6 +792,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 		return this.isPolyExpression = true;
 	}
 	
+	@Override
 	public boolean isCompatibleWith(TypeBinding left, Scope scope) {
 		return isPolyExpression() ? this.valueIfTrue.isCompatibleWith(left, scope) && this.valueIfFalse.isCompatibleWith(left, scope) :
 			super.isCompatibleWith(left, scope);
@@ -790,6 +808,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 			super.isBoxingCompatibleWith(targetType, scope);
 	}	
 	
+	@Override
 	public boolean sIsMoreSpecific(TypeBinding s, TypeBinding t, Scope scope) {
 		if (super.sIsMoreSpecific(s, t, scope))
 			return true;
@@ -798,6 +817,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext,
 				false;
 	}
 
+	@Override
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		if (visitor.visit(this, scope)) {
 			this.condition.traverse(visitor, scope);
