@@ -1,9 +1,13 @@
+// AspectJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -145,10 +149,7 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 		}
 
 		// nullity and mark as assigned
-		if (classScope.environment().usesNullTypeAnnotations())
-			analyseArguments18(flowInfo, this.arguments, this.binding);
-		else
-			analyseArguments(flowInfo, this.arguments, this.binding);
+		analyseArguments(classScope.environment(), flowInfo, this.arguments, this.binding);
 
 		// propagate to constructor call
 		if (this.constructorCall != null) {
@@ -207,7 +208,7 @@ public void analyseCode(ClassScope classScope, InitializationFlowContext initial
 								((this.bits & ASTNode.IsDefaultConstructor) != 0)
 									? (ASTNode) this.scope.referenceType().declarationOf(field.original())
 									: this);
-					} else if (field.isNonNull()) {
+					} else if (field.isNonNull() || field.type.isFreeTypeVariable()) {
 						FieldDeclaration fieldDecl = this.scope.referenceType().declarationOf(field.original());
 						if (!isValueProvidedUsingAnnotation(fieldDecl))
 							this.scope.problemReporter().uninitializedNonNullField(
@@ -266,6 +267,7 @@ boolean isValueProvidedUsingAnnotation(FieldDeclaration fieldDecl) {
  * @param classScope org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ClassScope
  * @param classFile org.aspectj.org.eclipse.jdt.internal.compiler.codegen.ClassFile
  */
+@Override
 public void generateCode(ClassScope classScope, ClassFile classFile) {
 	int problemResetPC = 0;
 	if (this.ignoreFurtherInvestigation) {
@@ -417,7 +419,7 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 
 		if (needFieldInitializations && preInitSyntheticFields){
 			generateSyntheticFieldInitializationsIfNecessary(this.scope, codeStream, declaringClass);
-			codeStream.recordPositionsFrom(0, this.bodyStart);
+			codeStream.recordPositionsFrom(0, this.bodyStart > 0 ? this.bodyStart : this.sourceStart);
 		}
 		// generate constructor call
 		if (this.constructorCall != null) {
@@ -453,7 +455,7 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 		}
 		// local variable attributes
 		codeStream.exitUserScope(this.scope);
-		codeStream.recordPositionsFrom(0, this.bodyEnd);
+		codeStream.recordPositionsFrom(0, this.bodyEnd > 0 ? this.bodyEnd : this.sourceStart);
 		try {
 			classFile.completeCodeAttribute(codeAttributeOffset);
 		} catch(NegativeArraySizeException e) {
@@ -469,22 +471,28 @@ private void internalGenerateCode(ClassScope classScope, ClassFile classFile) {
 	classFile.completeMethodInfo(this.binding, methodAttributeOffset, attributeNumber);
 }
 
+@Override
 public void getAllAnnotationContexts(int targetType, List allAnnotationContexts) {
-	AnnotationCollector collector = new AnnotationCollector(null, targetType, allAnnotationContexts);
+	TypeReference fakeReturnType = new SingleTypeReference(this.selector, 0);
+	fakeReturnType.resolvedType = this.binding.declaringClass;
+	AnnotationCollector collector = new AnnotationCollector(fakeReturnType, targetType, allAnnotationContexts);
 	for (int i = 0, max = this.annotations.length; i < max; i++) {
 		Annotation annotation = this.annotations[i];
 		annotation.traverse(collector, (BlockScope) null);
 	}
 }
 
+@Override
 public boolean isConstructor() {
 	return true;
 }
 
+@Override
 public boolean isDefaultConstructor() {
 	return (this.bits & ASTNode.IsDefaultConstructor) != 0;
 }
 
+@Override
 public boolean isInitializationMethod() {
 	return true;
 }
@@ -523,6 +531,7 @@ public boolean isRecursive(ArrayList visited) {
 	return targetConstructor.isRecursive(visited);
 }
 
+@Override
 public void parseStatements(Parser parser, CompilationUnitDeclaration unit) {
 	//fill up the constructor body with its statements
 	if (((this.bits & ASTNode.IsDefaultConstructor) != 0) && this.constructorCall == null){
@@ -535,6 +544,7 @@ public void parseStatements(Parser parser, CompilationUnitDeclaration unit) {
 
 }
 
+@Override
 public StringBuffer printBody(int indent, StringBuffer output) {
 	output.append(" {"); //$NON-NLS-1$
 	if (this.constructorCall != null) {
@@ -552,6 +562,7 @@ public StringBuffer printBody(int indent, StringBuffer output) {
 	return output;
 }
 
+@Override
 public void resolveJavadoc() {
 	if (this.binding == null || this.javadoc != null) {
 		super.resolveJavadoc();
@@ -577,6 +588,7 @@ public void resolveJavadoc() {
  * Type checking for constructor, just another method, except for special check
  * for recursive constructor invocations.
  */
+@Override
 public void resolveStatements() {
 	SourceTypeBinding sourceType = this.scope.enclosingSourceType();
 	if (!CharOperation.equals(sourceType.sourceName, this.selector)){
@@ -605,6 +617,7 @@ public void resolveStatements() {
 	super.resolveStatements();
 }
 
+@Override
 public void traverse(ASTVisitor visitor, ClassScope classScope) {
 	if (visitor.visit(this, classScope)) {
 		if (this.javadoc != null) {
@@ -641,6 +654,7 @@ public void traverse(ASTVisitor visitor, ClassScope classScope) {
 	}
 	visitor.endVisit(this, classScope);
 }
+@Override
 public TypeParameter[] typeParameters() {
     return this.typeParameters;
 }
