@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2015 GK Software AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2011, 2016 GK Software AG and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Stephan Herrmann - initial API and implementation
@@ -133,9 +136,11 @@ public class FakedTrackingVariable extends LocalDeclaration {
 			flowInfo.markNullStatus(this.binding, nullStatus); // mark that this flow has seen the resource
 	}
 	
+	@Override
 	public void generateCode(BlockScope currentScope, CodeStream codeStream)
 	{ /* NOP - this variable is completely dummy, ie. for analysis only. */ }
 
+	@Override
 	public void resolve (BlockScope scope) {
 		// only need the binding, which is used as reference in FlowInfo methods.
 		this.binding = new LocalVariableBinding(
@@ -900,7 +905,7 @@ public class FakedTrackingVariable extends LocalDeclaration {
 		}
 		if (this.recordedLocations == null)
 			this.recordedLocations = new HashMap();
-		this.recordedLocations.put(location, new Integer(nullStatus));
+		this.recordedLocations.put(location, Integer.valueOf(nullStatus));
 	}
 
 	public boolean reportRecordedErrors(Scope scope, int mergedStatus, boolean atDeadEnd) {
