@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2011 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -20,6 +23,7 @@ public LongLiteralMinValue(char[] token, char[] reducedForm, int start, int end)
 	super(token, reducedForm, start, end);
 	this.constant = LongConstant.fromValue(Long.MIN_VALUE);
 }
+@Override
 public void computeConstant() {
 	/*precomputed at creation time*/}
 }
