@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2017 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -154,7 +157,7 @@ public class QualifiedTypeReference extends TypeReference {
 				}
 				if (currentType.isGenericType()) {
 					qualifiedType = scope.environment().createRawType(currentType, qualifiedType);
-				} else if (currentType.isStatic()) {
+				} else if (!currentType.hasEnclosingInstanceContext()) {
 					qualifiedType = currentType; // parameterization of enclosing is irrelevant in this case
 				} else {
 					boolean rawQualified = qualifiedType.isRawType();
