@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -66,6 +69,7 @@ public ReturnStatement(Expression expression, int sourceStart, int sourceEnd, bo
 	this.implicitReturn = implicitReturn;
 }
 
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {	// here requires to generate a sequence of finally blocks invocations depending corresponding
 	// to each of the traversed try statements, so that execution will terminate properly.
 
@@ -171,6 +175,7 @@ public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, Fl
 	currentScope.checkUnclosedCloseables(flowInfo, flowContext, this, currentScope);
 	// inside conditional structure respect that a finally-block may conditionally be entered directly from here
 	flowContext.recordAbruptExit();
+	flowContext.expireNullCheckedFieldInfo();
 	return FlowInfo.DEAD_END;
 }
 @Override
@@ -208,6 +213,7 @@ void checkAgainstNullAnnotation(BlockScope scope, FlowContext flowContext, FlowI
  * @param currentScope org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BlockScope
  * @param codeStream org.aspectj.org.eclipse.jdt.internal.compiler.codegen.CodeStream
  */
+@Override
 public void generateCode(BlockScope currentScope, CodeStream codeStream) {
 	if ((this.bits & ASTNode.IsReachable) == 0) {
 		return;
@@ -287,6 +293,7 @@ public void prepareSaveValueLocation(TryStatement targetTryStatement){
 	this.saveValueVariable = targetTryStatement.secretReturnValue;
 }
 
+@Override
 public StringBuffer printStatement(int tab, StringBuffer output){
 	printIndent(tab, output).append("return "); //$NON-NLS-1$
 	if (this.expression != null )
@@ -294,6 +301,7 @@ public StringBuffer printStatement(int tab, StringBuffer output){
 	return output.append(';');
 }
 
+@Override
 public void resolve(BlockScope scope) {
 	MethodScope methodScope = scope.methodScope();
 	MethodBinding methodBinding;
@@ -382,6 +390,7 @@ public void resolve(BlockScope scope) {
 	}
 }
 
+@Override
 public void traverse(ASTVisitor visitor, BlockScope scope) {
 	if (visitor.visit(this, scope)) {
 		if (this.expression != null)
