@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2012 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -30,6 +33,7 @@ public class SingleTypeReference extends TypeReference {
 
 	}
 
+	@Override
 	public TypeReference augmentTypeWithAdditionalDimensions(int additionalDimensions, Annotation[][] additionalAnnotations, boolean isVarargs) {
 		int totalDimensions = this.dimensions() + additionalDimensions;
 		Annotation [][] allAnnotations = getMergedAnnotationsOnDimensions(additionalDimensions, additionalAnnotations);
@@ -41,9 +45,11 @@ public class SingleTypeReference extends TypeReference {
 		return arrayTypeReference;
 	}
 
+	@Override
 	public char[] getLastToken() {
 		return this.token;
 	}
+	@Override
 	protected TypeBinding getTypeBinding(Scope scope) {
 		if (this.resolvedType != null)
 			return this.resolvedType;
@@ -69,6 +75,7 @@ public class SingleTypeReference extends TypeReference {
 		return this.resolvedType;
 	}
 
+	@Override
 	public char [][] getTypeName() {
 		return new char[][] { this.token };
 	}
@@ -87,6 +94,7 @@ public class SingleTypeReference extends TypeReference {
 			   this.token == VOID;	    
 	}
 	
+	@Override
 	public StringBuffer printExpression(int indent, StringBuffer output){
 		if (this.annotations != null && this.annotations[0] != null) {
 			printAnnotations(this.annotations[0], output);
@@ -124,6 +132,7 @@ public class SingleTypeReference extends TypeReference {
 		return this.resolvedType = memberType;
 	}
 
+	@Override
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		if (visitor.visit(this, scope)) {
 			if (this.annotations != null) {
@@ -135,6 +144,7 @@ public class SingleTypeReference extends TypeReference {
 		visitor.endVisit(this, scope);
 	}
 
+	@Override
 	public void traverse(ASTVisitor visitor, ClassScope scope) {
 		if (visitor.visit(this, scope)) {
 			if (this.annotations != null) {
