@@ -1,10 +1,14 @@
+// AspectJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann - Contributions for
@@ -38,6 +42,7 @@ public class TypeDeclaration extends Statement implements ProblemSeverities, Ref
 
 	public int modifiers = ClassFileConstants.AccDefault;
 	public int modifiersSourceStart;
+	public int functionalExpressionsCount = 0;
 	public Annotation[] annotations;
 	public Annotation[] originalAnnotations; // AspectJ Extension
 	public char[] name;
@@ -76,6 +81,7 @@ public TypeDeclaration(CompilationResult compilationResult){
 /*
  *	We cause the compilation task to abort to a given extent.
  */
+@Override
 public void abort(int abortLevel, CategorizedProblem problem) {
 	switch (abortLevel) {
 		case AbortCompilation :
@@ -197,6 +203,7 @@ public MethodDeclaration addMissingAbstractMethodFor(MethodBinding methodBinding
  *	Flow analysis for a local innertype
  *
  */
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	if (this.ignoreFurtherInvestigation)
 		return flowInfo;
@@ -306,6 +313,7 @@ public boolean checkConstructors(Parser parser) {
 	return hasConstructor;
 }
 
+@Override
 public CompilationResult compilationResult() {
 	return this.compilationResult;
 }
@@ -511,6 +519,7 @@ public TypeDeclaration declarationOfType(char[][] typeName) {
 	return null;
 }
 
+@Override
 public CompilationUnitDeclaration getCompilationUnitDeclaration() {
 	if (this.scope != null) {
 		return this.scope.compilationUnitScope().referenceContext;
@@ -620,6 +629,7 @@ public void generateCode(ClassFile enclosingClassFile) {
 /**
  * Bytecode generation for a local inner type (API as a normal statement code gen)
  */
+@Override
 public void generateCode(BlockScope blockScope, CodeStream codeStream) {
 	if ((this.bits & ASTNode.IsReachable) == 0) {
 		return;
@@ -665,6 +675,7 @@ public void generateCode(CompilationUnitScope unitScope) {
 	generateCode((ClassFile) null);
 }
 
+@Override
 public boolean hasErrors() {
 	return this.ignoreFurtherInvestigation;
 }
@@ -745,6 +756,22 @@ private void internalAnalyseCode(FlowContext flowContext, FlowInfo flowInfo) {
 			}
 		}
 	}
+	if (this.scope.compilerOptions().complianceLevel >= ClassFileConstants.JDK9) {
+		// synthesize <clinit> if one is not present. Required to initialize
+		// synthetic final fields as modifying final fields outside of a <clinit>
+		// is disallowed in Java 9
+		if (this.methods == null || !this.methods[0].isClinit()) {
+			Clinit clinit = new Clinit(this.compilationResult);
+			clinit.declarationSourceStart = clinit.sourceStart = this.sourceStart;
+			clinit.declarationSourceEnd = clinit.sourceEnd = this.sourceEnd;
+			clinit.bodyEnd = this.sourceEnd;
+			int length = this.methods == null ? 0 : this.methods.length;
+			AbstractMethodDeclaration[] methodDeclarations = new AbstractMethodDeclaration[length + 1];
+			methodDeclarations[0] = clinit;
+			if (this.methods != null)
+				System.arraycopy(this.methods, 0, methodDeclarations, 1, length);
+		}
+	}
 	if (this.methods != null) {
 		UnconditionalFlowInfo outerInfo = flowInfo.unconditionalFieldLessCopy();
 		FlowInfo constructorInfo = nonStaticFieldInfo.unconditionalInits().discardNonFieldInitializations().addInitializationsFrom(outerInfo);
@@ -920,6 +947,7 @@ public void parseMethods(Parser parser, CompilationUnitDeclaration unit) {
 	}
 }
 
+@Override
 public StringBuffer print(int indent, StringBuffer output) {
 	if (this.javadoc != null) {
 		this.javadoc.print(indent, output);
@@ -1014,11 +1042,18 @@ public StringBuffer printHeader(int indent, StringBuffer output) {
 	return output;
 }
 
+@Override
 public StringBuffer printStatement(int tab, StringBuffer output) {
 	return print(tab, output);
 }
 
-
+/*
+ * Keep track of number of lambda/method reference expressions in this type declaration.
+ * Return the 0 based "ordinal" in the TypeDeclaration.
+ */
+public int record(FunctionalExpression expression) {
+	return this.functionalExpressionsCount++;
+}
 
 public void resolve() {
 	SourceTypeBinding sourceType = this.binding;
@@ -1027,6 +1062,13 @@ public void resolve() {
 		return;
 	}
 	try {
+		if (CharOperation.equals(this.name, TypeConstants.VAR)) {
+			if (this.scope.compilerOptions().sourceLevel < ClassFileConstants.JDK10) {
+				this.scope.problemReporter().varIsReservedTypeNameInFuture(this);
+			} else {
+				this.scope.problemReporter().varIsReservedTypeName(this);
+			}
+		}
 		// resolve annotations and check @Deprecated annotation
 		long annotationTagBits = sourceType.getAnnotationTagBits();
 		if ((annotationTagBits & TagBits.AnnotationDeprecated) == 0
@@ -1236,6 +1278,17 @@ public void resolve() {
 				reporter.javadocMissing(this.sourceStart, this.sourceEnd, severity, javadocModifiers);
 			}
 		}
+		updateNestInfo();
+		FieldDeclaration[] fieldsDecls = this.fields;
+		if (fieldsDecls != null) {
+			for (FieldDeclaration fieldDeclaration : fieldsDecls)
+				fieldDeclaration.resolveJavadoc(this.initializerScope);
+		}
+		AbstractMethodDeclaration[] methodDecls = this.methods;
+		if (methodDecls != null) {
+			for (AbstractMethodDeclaration methodDeclaration : methodDecls)
+				methodDeclaration.resolveJavadoc();
+		}
 	} catch (AbortType e) {
 		this.ignoreFurtherInvestigation = true;
 		return;
@@ -1245,6 +1298,7 @@ public void resolve() {
 /**
  * Resolve a local type declaration
  */
+@Override
 public void resolve(BlockScope blockScope) {
 
 	// need to build its scope first and proceed with binding's creation
@@ -1331,10 +1385,12 @@ public void resolve(CompilationUnitScope upperScope) {
 	updateMaxFieldCount();
 }
 
+@Override
 public void tagAsHavingErrors() {
 	this.ignoreFurtherInvestigation = true;
 }
 
+@Override
 public void tagAsHavingIgnoredMandatoryErrors(int problemId) {
 	// Nothing to do for this context;
 }
@@ -1398,6 +1454,7 @@ public void traverse(ASTVisitor visitor, CompilationUnitScope unitScope) {
 /**
  *	Iteration for a local inner type
  */
+@Override
 public void traverse(ASTVisitor visitor, BlockScope blockScope) {
 	try {
 		if (visitor.visit(this, blockScope)) {
@@ -1527,6 +1584,20 @@ void updateMaxFieldCount() {
 	}
 }
 
+private SourceTypeBinding findNestHost() {
+	ClassScope classScope = this.scope.enclosingTopMostClassScope();
+	return classScope != null ? classScope.referenceContext.binding : null;
+}
+
+void updateNestInfo() {
+	if (this.binding == null)
+		return;
+	SourceTypeBinding nestHost = findNestHost();
+	if (nestHost != null && !this.binding.equals(nestHost)) {// member
+		this.binding.setNestHost(nestHost);
+		nestHost.addNestMember(this.binding);
+	}
+}
 public boolean isPackageInfo() {
 	return CharOperation.equals(this.name,  TypeConstants.PACKAGE_INFO_NAME);
 }
