@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -28,14 +32,20 @@
  *                          Bug 383624 - [1.8][compiler] Revive code generation support for type annotations (from Olivier's work)
  *                          Bug 409236 - [1.8][compiler] Type annotations on intersection cast types dropped by code generator
  *                          Bug 415399 - [1.8][compiler] Type annotations on constructor results dropped by the code generator
+ *      Jesper S Møller <jesper@selskabet.org> -  Contributions for
+ *                          bug 527554 - [18.3] Compiler support for JEP 286 Local-Variable Type
+ *                          bug 529556 - [18.3] Add content assist support for 'var' as a type
+ *                          
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 
 import java.util.ArrayList;
 import java.util.List;
 
+import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching.CheckMode;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.AnnotationContext;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.AnnotationTargetTypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.flow.FlowContext;
@@ -53,10 +63,11 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ProblemReasons;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Scope;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Substitution;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeIds;
-import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.ProblemSeverities;
 @SuppressWarnings({"rawtypes", "unchecked"})
 public abstract class TypeReference extends Expression {
@@ -219,19 +230,24 @@ static class AnnotationCollector extends ASTVisitor {
 		}
 		return true;
 	}
+	@Override
 	public boolean visit(MarkerAnnotation annotation, BlockScope scope) {
 		return internalVisit(annotation);
 	}
+	@Override
 	public boolean visit(NormalAnnotation annotation, BlockScope scope) {
 		return internalVisit(annotation);
 	}
+	@Override
 	public boolean visit(SingleMemberAnnotation annotation, BlockScope scope) {
 		return internalVisit(annotation);
 	}
+	@Override
 	public boolean visit(Wildcard wildcard, BlockScope scope) {
 		this.currentWildcard = wildcard;
 		return true;
 	}
+	@Override
 	public boolean visit(Argument argument, BlockScope scope) {
 		if ((argument.bits & ASTNode.IsUnionType) == 0) {
 			return true;
@@ -245,6 +261,7 @@ static class AnnotationCollector extends ASTVisitor {
 		}
 		return false;
 	}
+	@Override
 	public boolean visit(Argument argument, ClassScope scope) {
 		if ((argument.bits & ASTNode.IsUnionType) == 0) {
 			return true;
@@ -258,6 +275,7 @@ static class AnnotationCollector extends ASTVisitor {
 		}
 		return false;
 	}
+	@Override
 	public boolean visit(LocalDeclaration localDeclaration, BlockScope scope) {
 		for (int i = 0, max = this.localVariable.initializationCount; i < max; i++) {
 			int startPC = this.localVariable.initializationPCs[i << 1];
@@ -268,6 +286,7 @@ static class AnnotationCollector extends ASTVisitor {
 		}
 		return false;
 	}
+	@Override
 	public void endVisit(Wildcard wildcard, BlockScope scope) {
 		this.currentWildcard = null;
 	}
@@ -332,6 +351,7 @@ public Annotation[][] annotations = null;
 public void aboutToResolve(Scope scope) {
 	// default implementation: do nothing
 }
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	return flowInfo;
 }
@@ -504,7 +524,11 @@ protected TypeBinding internalResolveType(Scope scope, int location) {
 	if (type == null) {
 		return null; // detected cycle while resolving hierarchy
 	} else if ((hasError = !type.isValidBinding()) == true) {
-		reportInvalidType(scope);
+		if (this.isTypeNameVar(scope)) {
+			reportVarIsNotAllowedHere(scope);
+		} else {
+			reportInvalidType(scope);
+		}
 		switch (type.problemId()) {
 			case ProblemReasons.NotFound :
 			case ProblemReasons.NotVisible :
@@ -540,6 +564,7 @@ protected TypeBinding internalResolveType(Scope scope, int location) {
 		return this.resolvedType; // pick up value that may have been changed in resolveAnnotations(..)
 	}
 }
+@Override
 public boolean isTypeReference() {
 	return true;
 }
@@ -567,6 +592,10 @@ protected void reportInvalidType(Scope scope) {
 	scope.problemReporter().invalidType(this, this.resolvedType);
 }
 
+protected void reportVarIsNotAllowedHere(Scope scope) {
+	scope.problemReporter().varIsNotAllowedHere(this);
+}
+
 public TypeBinding resolveSuperType(ClassScope scope) {
 	// assumes the implementation of resolveType(ClassScope) will call back to detect cycles
 	TypeBinding superType = resolveType(scope);
@@ -582,6 +611,7 @@ public TypeBinding resolveSuperType(ClassScope scope) {
 	return superType;
 }
 
+@Override
 public final TypeBinding resolveType(BlockScope blockScope) {
 	return resolveType(blockScope, true /* checkbounds if any */);
 }
@@ -594,6 +624,7 @@ public TypeBinding resolveType(BlockScope scope, boolean checkBounds, int locati
 	return internalResolveType(scope, location);
 }
 
+@Override
 public TypeBinding resolveType(ClassScope scope) {
 	return resolveType(scope, 0);
 }
@@ -614,8 +645,8 @@ public TypeBinding resolveTypeArgument(ClassScope classScope, ReferenceBinding g
 	boolean pauseHierarchyCheck = false;
 	try {
 		if (ref.isHierarchyBeingConnected()) {
+			pauseHierarchyCheck = (ref.tagBits & TagBits.PauseHierarchyCheck) == 0;
 			ref.tagBits |= TagBits.PauseHierarchyCheck;
-			pauseHierarchyCheck = true;
 		}
 	    return resolveType(classScope, Binding.DefaultLocationTypeArgument);
 	} finally {
@@ -625,8 +656,10 @@ public TypeBinding resolveTypeArgument(ClassScope classScope, ReferenceBinding g
 	}
 }
 
+@Override
 public abstract void traverse(ASTVisitor visitor, BlockScope scope);
 
+@Override
 public abstract void traverse(ASTVisitor visitor, ClassScope scope);
 
 protected void resolveAnnotations(Scope scope, int location) {
@@ -663,7 +696,7 @@ protected void resolveAnnotations(Scope scope, int location) {
 			&& !this.resolvedType.isTypeVariable()
 			&& !this.resolvedType.isWildcard()
 			&& location != 0
-			&& scope.hasDefaultNullnessFor(location)) 
+			&& scope.hasDefaultNullnessFor(location, this.sourceStart)) 
 	{
 		if (location == Binding.DefaultLocationTypeBound && this.resolvedType.id == TypeIds.T_JavaLangObject) {
 			scope.problemReporter().implicitObjectBoundNoNullDefault(this);
@@ -677,42 +710,39 @@ protected void resolveAnnotations(Scope scope, int location) {
 public int getAnnotatableLevels() {
 	return 1;
 }
-/** Check all typeArguments against null constraints on their corresponding type variables. */
-protected void checkNullConstraints(Scope scope, TypeReference[] typeArguments) {
-	if (scope.environment().usesNullTypeAnnotations()
-			&& typeArguments != null)
-	{
-		TypeVariableBinding[] typeVariables = this.resolvedType.original().typeVariables();
+/** Check all typeArguments for illegal null annotations on base types. */
+protected void checkIllegalNullAnnotations(Scope scope, TypeReference[] typeArguments) {
+	if (scope.environment().usesNullTypeAnnotations() && typeArguments != null) {
 		for (int i = 0; i < typeArguments.length; i++) {
 			TypeReference arg = typeArguments[i];
 			if (arg.resolvedType != null)
-				arg.checkNullConstraints(scope, typeVariables, i);
+				arg.checkIllegalNullAnnotation(scope);
 		}
 	}
 }
 /** Check whether this type reference conforms to the null constraints defined for the corresponding type variable. */
-protected void checkNullConstraints(Scope scope, TypeBinding[] variables, int rank) {
+protected void checkNullConstraints(Scope scope, Substitution substitution, TypeBinding[] variables, int rank) {
 	if (variables != null && variables.length > rank) {
 		TypeBinding variable = variables[rank];
 		if (variable.hasNullTypeAnnotations()) {
-			if (NullAnnotationMatching.analyse(variable, this.resolvedType, null, -1, CheckMode.BOUND_CHECK).isAnyMismatch())
+			if (NullAnnotationMatching.analyse(variable, this.resolvedType, null, substitution, -1, null, CheckMode.BOUND_CHECK).isAnyMismatch())
 				scope.problemReporter().nullityMismatchTypeArgument(variable, this.resolvedType, this);
     	}
 	}
-	if (this.resolvedType.leafComponentType().isBaseType() && hasNullTypeAnnotation(AnnotationPosition.LEAF_TYPE)) {
-		scope.problemReporter().illegalAnnotationForBaseType(this, this.annotations[0], this.resolvedType.tagBits & TagBits.AnnotationNullMASK);
-	}
+	checkIllegalNullAnnotation(scope);
+}
+protected void checkIllegalNullAnnotation(Scope scope) {
+	if (this.resolvedType.leafComponentType().isBaseType() && hasNullTypeAnnotation(AnnotationPosition.LEAF_TYPE))
+		scope.problemReporter().illegalAnnotationForBaseType(this, this.annotations[0], this.resolvedType.tagBits & TagBits.AnnotationNullMASK);	
 }
 /** Retrieve the null annotation that has been translated to the given nullTagBits. */
 public Annotation findAnnotation(long nullTagBits) {
 	if (this.annotations != null) {
 		Annotation[] innerAnnotations = this.annotations[this.annotations.length-1];
 		if (innerAnnotations != null) {
-			int annId = nullTagBits == TagBits.AnnotationNonNull ? TypeIds.T_ConfiguredAnnotationNonNull : TypeIds.T_ConfiguredAnnotationNullable;
+			int annBit = nullTagBits == TagBits.AnnotationNonNull ? TypeIds.BitNonNullAnnotation : TypeIds.BitNullableAnnotation;
 			for (int i = 0; i < innerAnnotations.length; i++) {
-				if (innerAnnotations[i] != null 
-						&& innerAnnotations[i].resolvedType != null 
-						&& innerAnnotations[i].resolvedType.id == annId)
+				if (innerAnnotations[i] != null && innerAnnotations[i].hasNullBit(annBit))
 					return innerAnnotations[i];
 			}
 		}
@@ -722,13 +752,13 @@ public Annotation findAnnotation(long nullTagBits) {
 public boolean hasNullTypeAnnotation(AnnotationPosition position) {
 	if (this.annotations != null) {
 		if (position == AnnotationPosition.MAIN_TYPE) {
-		Annotation[] innerAnnotations = this.annotations[this.annotations.length-1];
+			Annotation[] innerAnnotations = this.annotations[this.annotations.length-1];
 			return containsNullAnnotation(innerAnnotations);
 		} else {
 			for (Annotation[] someAnnotations: this.annotations) {
 				if (containsNullAnnotation(someAnnotations))
-			return true;
-	}
+					return true;
+			}
 		}
 	}
 	return false;
@@ -736,10 +766,7 @@ public boolean hasNullTypeAnnotation(AnnotationPosition position) {
 public static boolean containsNullAnnotation(Annotation[] annotations) {
 	if (annotations != null) {
 		for (int i = 0; i < annotations.length; i++) {
-			if (annotations[i] != null 
-					&& annotations[i].resolvedType != null 
-					&& (annotations[i].resolvedType.id == TypeIds.T_ConfiguredAnnotationNonNull
-						|| annotations[i].resolvedType.id == TypeIds.T_ConfiguredAnnotationNullable))
+			if (annotations[i] != null && (annotations[i].hasNullBit(TypeIds.BitNonNullAnnotation|TypeIds.BitNullableAnnotation)))
 				return true;
 		}
 	}
@@ -752,4 +779,17 @@ public TypeReference[] getTypeReferences() {
 public boolean isBaseTypeReference() {
 	return false;
 }
+/**
+ * Checks to see if the declaration uses 'var' as type name 
+ * @param scope Relevant scope, for error reporting
+ * @return true, if source level is Java 10 or above and the type name is just 'var', false otherwise 
+ */
+public boolean isTypeNameVar(Scope scope) {
+	CompilerOptions compilerOptions = scope != null ? scope.compilerOptions() : null;
+	if (compilerOptions != null && compilerOptions.sourceLevel < ClassFileConstants.JDK10) {
+		return false;
+	}
+	char[][] typeName = this.getTypeName();
+	return typeName.length == 1 && CharOperation.equals(typeName[0], TypeConstants.VAR);
+}
 }
