@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -42,6 +45,7 @@ public class WhileStatement extends Statement {
 		this.sourceEnd = e;
 	}
 
+	@Override
 	public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 
 		this.breakLabel = new BranchLabel();
@@ -103,6 +107,7 @@ public class WhileStatement extends Statement {
 					this.continueLabel,
 					currentScope,
 					true);
+			loopingContext.copyNullCheckedFieldsFrom(condLoopContext);
 			if (isConditionFalse) {
 				actionInfo = FlowInfo.DEAD_END;
 			} else {
@@ -129,6 +134,11 @@ public class WhileStatement extends Statement {
 				if ((combinedTagBits & FlowInfo.UNREACHABLE_OR_DEAD) != 0)
 					this.continueLabel = null;
 				exitBranch.addInitializationsFrom(condInfo.initsWhenFalse());
+				actionInfo = actionInfo.mergedWith(loopingContext.initsOnContinue.unconditionalInits());
+				condLoopContext.complainOnDeferredNullChecks(currentScope,
+						actionInfo, false);
+				loopingContext.complainOnDeferredNullChecks(currentScope,
+						actionInfo, false);
 			} else {
 				condLoopContext.complainOnDeferredFinalChecks(currentScope,
 						condInfo);
@@ -174,6 +184,7 @@ public class WhileStatement extends Statement {
 	 * @param currentScope org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BlockScope
 	 * @param codeStream org.aspectj.org.eclipse.jdt.internal.compiler.codegen.CodeStream
 	 */
+	@Override
 	public void generateCode(BlockScope currentScope, CodeStream codeStream) {
 
 		if ((this.bits & IsReachable) == 0) {
@@ -257,6 +268,7 @@ public class WhileStatement extends Statement {
 		codeStream.recordPositionsFrom(pc, this.sourceStart);
 	}
 
+	@Override
 	public void resolve(BlockScope scope) {
 
 		TypeBinding type = this.condition.resolveTypeExpecting(scope, TypeBinding.BOOLEAN);
@@ -265,6 +277,7 @@ public class WhileStatement extends Statement {
 			this.action.resolve(scope);
 	}
 
+	@Override
 	public StringBuffer printStatement(int tab, StringBuffer output) {
 
 		printIndent(tab, output).append("while ("); //$NON-NLS-1$
@@ -276,6 +289,7 @@ public class WhileStatement extends Statement {
 		return output;
 	}
 
+	@Override
 	public void traverse(
 		ASTVisitor visitor,
 		BlockScope blockScope) {
