@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,27 +16,26 @@
  *								Bug 440477 - [null] Infrastructure for feeding external annotations into compilation
  *								Bug 440687 - [compiler][batch][null] improve command line option for external annotations
  *     Andy Clement (GoPivotal, Inc) aclement@gopivotal.com - Contributions for
- *         bug 407191 - [1.8] Binary access support for type annotations
+ *         						bug 407191 - [1.8] Binary access support for type annotations
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.classfmt;
 
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.Arrays;
-import java.util.zip.ZipEntry;
-import java.util.zip.ZipFile;
+import java.util.function.Predicate;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.AnnotationTargetTypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.AttributeNamesConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.Constant;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BinaryTypeBinding.ExternalAnnotationStatus;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
-import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeIds;
+import org.aspectj.org.eclipse.jdt.internal.compiler.util.JRTUtil;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
 
 public class ClassFileReader extends ClassFileStruct implements IBinaryType {
@@ -45,6 +48,8 @@ public class ClassFileReader extends ClassFileStruct implements IBinaryType {
 	private AnnotationInfo[] annotations;
 	private TypeAnnotationInfo[] typeAnnotations;
 	private FieldInfo[] fields;
+	private ModuleInfo moduleDeclaration;
+	public char[] moduleName;
 	private int fieldsCount;
 
 	// initialized in case the .class file is a nested type
@@ -65,7 +70,9 @@ public class ClassFileReader extends ClassFileStruct implements IBinaryType {
 	private char[][][] missingTypeNames;
 	private int enclosingNameAndTypeIndex;
 	private char[] enclosingMethod;
-	private ExternalAnnotationProvider annotationProvider;
+	private char[] nestHost;
+	private int nestMembersCount;
+	private char[][] nestMembers;
 
 private static String printTypeModifiers(int modifiers) {
 	java.io.ByteArrayOutputStream out = new java.io.ByteArrayOutputStream();
@@ -114,6 +121,23 @@ public static ClassFileReader read(
 		return read(zip, filename, false);
 }
 
+public static ClassFileReader readFromJrt(
+		File jrt,
+		IModule module,
+		String filename)
+
+		throws ClassFormatException, java.io.IOException {
+		return JRTUtil.getClassfile(jrt, filename, module);
+	}
+public static ClassFileReader readFromModule(
+		File jrt,
+		String moduleName,
+		String filename,
+		Predicate<String> moduleNameFilter)
+
+		throws ClassFormatException, java.io.IOException {
+		return JRTUtil.getClassfile(jrt, filename, moduleName, moduleNameFilter);
+}
 public static ClassFileReader read(
 	java.util.zip.ZipFile zip,
 	String filename,
@@ -230,10 +254,22 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 					this.constantPoolOffsets[i] = readOffset;
 					readOffset += ClassFileConstants.ConstantMethodTypeFixedSize;
 					break;
+				case ClassFileConstants.DynamicTag :
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantDynamicFixedSize;
+					break;
 				case ClassFileConstants.InvokeDynamicTag :
 					this.constantPoolOffsets[i] = readOffset;
 					readOffset += ClassFileConstants.ConstantInvokeDynamicFixedSize;
 					break;
+				case ClassFileConstants.ModuleTag:
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantModuleFixedSize;
+					break;
+				case ClassFileConstants.PackageTag:
+					this.constantPoolOffsets[i] = readOffset;
+					readOffset += ClassFileConstants.ConstantPackageFixedSize;
+					break;
 			}
 		}
 		// Read and validate access flags
@@ -242,7 +278,9 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 
 		// Read the classname, use exception handlers to catch bad format
 		this.classNameIndex = u2At(readOffset);
-		this.className = getConstantClassNameAt(this.classNameIndex);
+		if (this.classNameIndex != 0) {
+			this.className = getConstantClassNameAt(this.classNameIndex);
+		}
 		readOffset += 2;
 
 		// Read the superclass name, can be null for java.lang.Object
@@ -271,7 +309,7 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 			FieldInfo field;
 			this.fields = new FieldInfo[this.fieldsCount];
 			for (int i = 0; i < this.fieldsCount; i++) {
-				field = FieldInfo.createField(this.reference, this.constantPoolOffsets, readOffset);
+				field = FieldInfo.createField(this.reference, this.constantPoolOffsets, readOffset, this.version);
 				this.fields[i] = field;
 				readOffset += field.sizeInBytes();
 			}
@@ -284,8 +322,8 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 			boolean isAnnotationType = (this.accessFlags & ClassFileConstants.AccAnnotation) != 0;
 			for (int i = 0; i < this.methodsCount; i++) {
 				this.methods[i] = isAnnotationType
-					? AnnotationMethodInfo.createAnnotationMethod(this.reference, this.constantPoolOffsets, readOffset)
-					: MethodInfo.createMethod(this.reference, this.constantPoolOffsets, readOffset);
+					? AnnotationMethodInfo.createAnnotationMethod(this.reference, this.constantPoolOffsets, readOffset, this.version)
+					: MethodInfo.createMethod(this.reference, this.constantPoolOffsets, readOffset, this.version);
 				readOffset += this.methods[i].sizeInBytes();
 			}
 		}
@@ -390,67 +428,59 @@ public ClassFileReader(byte[] classFileBytes, char[] fileName, boolean fullyInit
 								missingTypeOffset += 2;
 							}
 						}
+					} else if (CharOperation.equals(attributeName, AttributeNamesConstants.ModuleName)) {
+						this.moduleDeclaration = ModuleInfo.createModule(this.reference, this.constantPoolOffsets, readOffset);
+						this.moduleName = this.moduleDeclaration.name();
 					}
+					break;
+				case 'N' :
+					if (CharOperation.equals(attributeName, AttributeNamesConstants.NestHost)) {
+						utf8Offset =
+							this.constantPoolOffsets[u2At(this.constantPoolOffsets[u2At(readOffset + 6)] + 1)];
+ 						this.nestHost = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
+					} else if (CharOperation.equals(attributeName, AttributeNamesConstants.NestMembers)) {
+						int offset = readOffset + 6;
+						this.nestMembersCount = u2At(offset);
+						if (this.nestMembersCount != 0) {
+							offset += 2;
+							this.nestMembers = new char[this.nestMembersCount][];
+							for (int j = 0; j < this.nestMembersCount; j++) {
+								utf8Offset =
+									this.constantPoolOffsets[u2At(this.constantPoolOffsets[u2At(offset)] + 1)];
+		 						this.nestMembers[j] = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
+		 						offset += 2;
+							}
+						}
+					}
+					break;
 			}
 			readOffset += (6 + u4At(readOffset + 2));
 		}
+		if (this.moduleDeclaration != null && this.annotations != null) {
+			this.moduleDeclaration.setAnnotations(this.annotations, this.tagBits, fullyInitialize);
+			this.annotations = null;
+		}
 		if (fullyInitialize) {
 			initialize();
 		}
 	} catch(ClassFormatException e) {
 		throw e;
 	} catch (Exception e) {
-		throw new ClassFormatException(
+		throw new ClassFormatException(e,
+			this.classFileName,
 			ClassFormatException.ErrTruncatedInput,
 			readOffset);
 	}
 }
 
-/** Auxiliary interface for {@link #setExternalAnnotationProvider(String,String,ZipFile,ZipFileProducer)}. */
-public interface ZipFileProducer { ZipFile produce() throws IOException; }
-
-/**
- * Create and remember a provider for external annotations using the given basePath,
- * which is either a directory holding .eea text files, or a zip file of entries of the same format.
- * @param basePath resolved filesystem path of either directory or zip file
- * @param qualifiedBinaryTypeName slash-separated type name
- * @param zipFile an existing zip file for the same basePath, or null. 
- * 		Output: wl be filled with 
- * @param producer an optional helper to produce the zipFile when needed.
- * @return the client provided zip file; 
- * 		or else a fresh new zip file, to let clients cache it, if desired; 
- * 		or null to signal that basePath is not a zip file, but a directory.
- * @throws IOException any unexpected errors during file access. File not found while
- *		accessing an individual file if basePath is a directory <em>is</em> expected,
- *		and simply answered with null. If basePath is neither a directory nor a zip file,
- *		this is unexpected.
- */
-public ZipFile setExternalAnnotationProvider(String basePath, String qualifiedBinaryTypeName, ZipFile zipFile, ZipFileProducer producer) throws IOException {
-	String qualifiedBinaryFileName = qualifiedBinaryTypeName + ExternalAnnotationProvider.ANNOTATION_FILE_SUFFIX;
-	if (zipFile == null) {
-		File annotationBase = new File(basePath);
-		if (annotationBase.isDirectory()) {
-			try {
-				String filePath = annotationBase.getAbsolutePath()+'/'+qualifiedBinaryFileName;
-				this.annotationProvider = new ExternalAnnotationProvider(new FileInputStream(filePath), String.valueOf(getName()));
-			} catch (FileNotFoundException e) {
-				// expected, no need to report an error here
-			}
-			return null; // no zipFile
-		}
-		if (!annotationBase.exists())
-			return null; // no zipFile, treat as not-yet-created directory
-		zipFile = (producer != null ? producer.produce() : new ZipFile(annotationBase));
-	}
-	ZipEntry entry = zipFile.getEntry(qualifiedBinaryFileName);
-	if (entry != null)
-		this.annotationProvider = new ExternalAnnotationProvider(zipFile.getInputStream(entry), String.valueOf(getName()));
-	return zipFile;
-}
-public boolean hasAnnotationProvider() {
-	return this.annotationProvider != null;
+public char[] getNestHost() {
+	return this.nestHost;
 }
 
+@Override
+public ExternalAnnotationStatus getExternalAnnotationStatus() {
+	return ExternalAnnotationStatus.NOT_EEA_CONFIGURED;
+}
 /**
  * Conditionally add external annotations to the mix.
  * If 'member' is given it must be either of IBinaryField or IBinaryMethod, in which case we're seeking annotations for that member.
@@ -458,23 +488,6 @@ public boolean hasAnnotationProvider() {
  */
 @Override
 public ITypeAnnotationWalker enrichWithExternalAnnotationsFor(ITypeAnnotationWalker walker, Object member, LookupEnvironment environment) {
-	if (walker == ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER && this.annotationProvider != null) {
-		if (member == null) {
-			return this.annotationProvider.forTypeHeader(environment);
-		} else if (member instanceof IBinaryField) {
-			IBinaryField field = (IBinaryField) member;
-			char[] fieldSignature = field.getGenericSignature();
-			if (fieldSignature == null)
-				fieldSignature = field.getTypeName();
-			return this.annotationProvider.forField(field.getName(), fieldSignature, environment);
-		} else if (member instanceof IBinaryMethod) {
-			IBinaryMethod method = (IBinaryMethod) member;
-			char[] methodSignature = method.getGenericSignature();
-			if (methodSignature == null)
-				methodSignature = method.getMethodDescriptor();
-			return this.annotationProvider.forMethod(method.isConstructor() ? TypeConstants.INIT : method.getSelector(), methodSignature, environment);
-		}
-	}
 	return walker;
 }
 
@@ -500,11 +513,12 @@ private void decodeAnnotations(int offset, boolean runtimeVisible) {
 			long standardTagBits = newInfo.standardAnnotationTagBits;
 			if (standardTagBits != 0) {
 				this.tagBits |= standardTagBits;
-			} else {
-				if (newInfos == null)
-					newInfos = new AnnotationInfo[numberOfAnnotations - i];
-				newInfos[newInfoCount++] = newInfo;
+				if (this.version < ClassFileConstants.JDK9 || (standardTagBits & TagBits.AnnotationDeprecated) == 0)
+					continue;
 			}
+			if (newInfos == null)
+				newInfos = new AnnotationInfo[numberOfAnnotations - i];
+			newInfos[newInfoCount++] = newInfo;
 		}
 		if (newInfos == null)
 			return; // nothing to record in this.annotations
@@ -550,6 +564,7 @@ private void decodeTypeAnnotations(int offset, boolean runtimeVisible) {
 /**
  * @return the annotations or null if there is none.
  */
+@Override
 public IBinaryAnnotation[] getAnnotations() {
 	return this.annotations;
 }
@@ -557,6 +572,7 @@ public IBinaryAnnotation[] getAnnotations() {
 /**
  * @return the type annotations or null if there is none.
  */
+@Override
 public IBinaryTypeAnnotation[] getTypeAnnotations() {
 	return this.typeAnnotations;
 }
@@ -582,6 +598,7 @@ public int[] getConstantPoolOffsets() {
 	return this.constantPoolOffsets;
 }
 
+@Override
 public char[] getEnclosingMethod() {
 	if (this.enclosingNameAndTypeIndex <= 0) {
 		return null;
@@ -606,6 +623,7 @@ public char[] getEnclosingMethod() {
  * Answer the resolved compoundName of the enclosing type
  * or null if the receiver is a top level type.
  */
+@Override
 public char[] getEnclosingTypeName() {
 	return this.enclosingTypeName;
 }
@@ -614,17 +632,36 @@ public char[] getEnclosingTypeName() {
  * Answer the receiver's this.fields or null if the array is empty.
  * @return org.aspectj.org.eclipse.jdt.internal.compiler.api.IBinaryField[]
  */
+@Override
 public IBinaryField[] getFields() {
 	return this.fields;
 }
+/**
+ * @see IBinaryType#getModule()
+ */
+@Override
+public char[] getModule() {
+	return this.moduleName;
+}
+/**
+ * Returns the module declaration that this class file represents. This will be 
+ * null for non module-info class files.
+ * 
+ * @return the module declaration this represents
+ */
+public IBinaryModule getModuleDeclaration() {
+	return this.moduleDeclaration;
+}
 
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IDependent#getFileName()
  */
+@Override
 public char[] getFileName() {
 	return this.classFileName;
 }
 
+@Override
 public char[] getGenericSignature() {
 	return this.signature;
 }
@@ -656,20 +693,14 @@ public char[] getInnerSourceName() {
 	return null;
 }
 
-/**
- * Answer the resolved names of the receiver's interfaces in the
- * class file format as specified in section 4.2 of the Java 2 VM spec
- * or null if the array is empty.
- *
- * For example, java.lang.String is java/lang/String.
- * @return char[][]
- */
+@Override
 public char[][] getInterfaceNames() {
 	return this.interfaceNames;
 }
 
 // AspectJ start - original method has added boolean parameter, this new one has the original signature and simply
 // passes in false.  This is all needed due to the support for inter type inner types
+@Override
 public IBinaryNestedType[] getMemberTypes() {
 	return getMemberTypes(false);
 }
@@ -741,6 +772,7 @@ public IBinaryNestedType[] getMemberTypes(boolean keepIncorrectlyNamedInners) {
  * Answer the receiver's this.methods or null if the array is empty.
  * @return org.aspectj.org.eclipse.jdt.internal.compiler.api.env.IBinaryMethod[]
  */
+@Override
 public IBinaryMethod[] getMethods() {
 	return this.methods;
 }
@@ -792,6 +824,7 @@ public static void main(String[] args) throws ClassFormatException, IOException
 	System.err.println('}');
 }
 */
+@Override
 public char[][][] getMissingTypeNames() {
 	return this.missingTypeNames;
 }
@@ -802,6 +835,7 @@ public char[][][] getMissingTypeNames() {
  * Set the AccDeprecated and AccSynthetic bits if necessary
  * @return int
  */
+@Override
 public int getModifiers() {
 	int modifiers;
 	if (this.innerInfo != null) {
@@ -814,17 +848,12 @@ public int getModifiers() {
 	return modifiers;
 }
 
-/**
- * Answer the resolved name of the type in the
- * class file format as specified in section 4.2 of the Java 2 VM spec.
- *
- * For example, java.lang.String is java/lang/String.
- * @return char[]
- */
+@Override
 public char[] getName() {
 	return this.className;
 }
 
+@Override
 public char[] getSourceName() {
 	if (this.sourceName != null)
 		return this.sourceName;
@@ -847,18 +876,12 @@ public char[] getSourceName() {
 	return this.sourceName = name;
 }
 
-/**
- * Answer the resolved name of the receiver's superclass in the
- * class file format as specified in section 4.2 of the Java 2 VM spec
- * or null if it does not have one.
- *
- * For example, java.lang.String is java/lang/String.
- * @return char[]
- */
+@Override
 public char[] getSuperclassName() {
 	return this.superclassName;
 }
 
+@Override
 public long getTagBits() {
 	return this.tagBits;
 }
@@ -982,6 +1005,9 @@ public boolean hasStructuralChanges(byte[] newBytes, boolean orderRequired, bool
 		// annotations
 		if (hasStructuralAnnotationChanges(getAnnotations(), newClassFile.getAnnotations()))
 			return true;
+		if (this.version >= ClassFileConstants.JDK1_8
+				&& hasStructuralTypeAnnotationChanges(getTypeAnnotations(), newClassFile.getTypeAnnotations()))
+			return true;
 
 		// generic signature
 		if (!CharOperation.equals(getGenericSignature(), newClassFile.getGenericSignature()))
@@ -1108,43 +1134,48 @@ private boolean hasStructuralAnnotationChanges(IBinaryAnnotation[] currentAnnota
 	if (currentAnnotationsLength != otherAnnotationsLength)
 		return true;
 	for (int i = 0; i < currentAnnotationsLength; i++) {
-		if (!CharOperation.equals(currentAnnotations[i].getTypeName(), otherAnnotations[i].getTypeName()))
-			return true;
-		IBinaryElementValuePair[] currentPairs = currentAnnotations[i].getElementValuePairs();
-		IBinaryElementValuePair[] otherPairs = otherAnnotations[i].getElementValuePairs();
-		int currentPairsLength = currentPairs == null ? 0 : currentPairs.length;
-		int otherPairsLength = otherPairs == null ? 0 : otherPairs.length;
-		if (currentPairsLength != otherPairsLength)
-			return true;
-		for (int j = 0; j < currentPairsLength; j++) {
-			if (!CharOperation.equals(currentPairs[j].getName(), otherPairs[j].getName()))
-				return true;
-			final Object value = currentPairs[j].getValue();
-			final Object value2 = otherPairs[j].getValue();
-			if (value instanceof Object[]) {
-				Object[] currentValues = (Object[]) value;
-				if (value2 instanceof Object[]) {
-					Object[] currentValues2 = (Object[]) value2;
-					final int length = currentValues.length;
-					if (length != currentValues2.length) {
-						return true;
-					}
-					for (int n = 0; n < length; n++) {
-						if (!currentValues[n].equals(currentValues2[n])) {
-							return true;
-						}
-					}
-					return false;
-				}
-				return true;
-			} else if (!value.equals(value2)) {
-				return true;
-			}
-		}
+		Boolean match = matchAnnotations(currentAnnotations[i], otherAnnotations[i]);
+		if (match != null)
+			return match.booleanValue();
 	}
 	return false;
 }
-
+private Boolean matchAnnotations(IBinaryAnnotation currentAnnotation, IBinaryAnnotation otherAnnotation) {
+	if (!CharOperation.equals(currentAnnotation.getTypeName(), otherAnnotation.getTypeName()))
+		return true;
+	IBinaryElementValuePair[] currentPairs = currentAnnotation.getElementValuePairs();
+	IBinaryElementValuePair[] otherPairs = otherAnnotation.getElementValuePairs();
+	int currentPairsLength = currentPairs == null ? 0 : currentPairs.length;
+	int otherPairsLength = otherPairs == null ? 0 : otherPairs.length;
+	if (currentPairsLength != otherPairsLength)
+		return Boolean.TRUE;
+	for (int j = 0; j < currentPairsLength; j++) {
+		if (!CharOperation.equals(currentPairs[j].getName(), otherPairs[j].getName()))
+			return Boolean.TRUE;
+		final Object value = currentPairs[j].getValue();
+		final Object value2 = otherPairs[j].getValue();
+		if (value instanceof Object[]) {
+			Object[] currentValues = (Object[]) value;
+			if (value2 instanceof Object[]) {
+				Object[] currentValues2 = (Object[]) value2;
+				final int length = currentValues.length;
+				if (length != currentValues2.length) {
+					return Boolean.TRUE;
+				}
+				for (int n = 0; n < length; n++) {
+					if (!currentValues[n].equals(currentValues2[n])) {
+						return Boolean.TRUE;
+					}
+				}
+				return Boolean.FALSE;
+			}
+			return Boolean.TRUE;
+		} else if (!value.equals(value2)) {
+			return Boolean.TRUE;
+		}
+	}
+	return null;
+}
 private boolean hasStructuralFieldChanges(FieldInfo currentFieldInfo, FieldInfo otherFieldInfo) {
 	// generic signature
 	if (!CharOperation.equals(currentFieldInfo.getGenericSignature(), otherFieldInfo.getGenericSignature()))
@@ -1155,6 +1186,9 @@ private boolean hasStructuralFieldChanges(FieldInfo currentFieldInfo, FieldInfo
 		return true;
 	if (hasStructuralAnnotationChanges(currentFieldInfo.getAnnotations(), otherFieldInfo.getAnnotations()))
 		return true;
+	if (this.version >= ClassFileConstants.JDK1_8
+			&& hasStructuralTypeAnnotationChanges(currentFieldInfo.getTypeAnnotations(), otherFieldInfo.getTypeAnnotations()))
+		return true;
 	if (!CharOperation.equals(currentFieldInfo.getName(), otherFieldInfo.getName()))
 		return true;
 	if (!CharOperation.equals(currentFieldInfo.getTypeName(), otherFieldInfo.getTypeName()))
@@ -1208,9 +1242,12 @@ private boolean hasStructuralMethodChanges(MethodInfo currentMethodInfo, MethodI
 	if (currentAnnotatedParamsCount != otherAnnotatedParamsCount)
 		return true;
 	for (int i=0; i<currentAnnotatedParamsCount; i++) {
-		if (hasStructuralAnnotationChanges(currentMethodInfo.getParameterAnnotations(i), otherMethodInfo.getParameterAnnotations(i)))
+		if (hasStructuralAnnotationChanges(currentMethodInfo.getParameterAnnotations(i, this.classFileName), otherMethodInfo.getParameterAnnotations(i, this.classFileName)))
 			return true;
 	}
+	if (this.version >= ClassFileConstants.JDK1_8
+			&& hasStructuralTypeAnnotationChanges(currentMethodInfo.getTypeAnnotations(), otherMethodInfo.getTypeAnnotations()))
+		return true;
 
 	if (!CharOperation.equals(currentMethodInfo.getSelector(), otherMethodInfo.getSelector()))
 		return true;
@@ -1233,6 +1270,45 @@ private boolean hasStructuralMethodChanges(MethodInfo currentMethodInfo, MethodI
 	return false;
 }
 
+private boolean hasStructuralTypeAnnotationChanges(IBinaryTypeAnnotation[] currentTypeAnnotations, IBinaryTypeAnnotation[] otherTypeAnnotations) {
+	if (otherTypeAnnotations != null) {
+		// copy so we can delete matched annotations:
+		int len = otherTypeAnnotations.length;
+		System.arraycopy(otherTypeAnnotations, 0, otherTypeAnnotations = new IBinaryTypeAnnotation[len], 0, len);
+	}
+	if (currentTypeAnnotations != null) {
+		loopCurrent:
+		for (IBinaryTypeAnnotation currentAnnotation : currentTypeAnnotations) {
+			if (!affectsSignature(currentAnnotation)) continue;
+			if (otherTypeAnnotations == null)
+				return true;
+			for (int i = 0; i < otherTypeAnnotations.length; i++) {
+				IBinaryTypeAnnotation otherAnnotation = otherTypeAnnotations[i];
+				if (otherAnnotation != null && matchAnnotations(currentAnnotation.getAnnotation(), otherAnnotation.getAnnotation()) == Boolean.TRUE) {
+					otherTypeAnnotations[i] = null; // matched
+					continue loopCurrent;
+				}
+			}
+			return true; // not matched
+		}
+	}
+	if (otherTypeAnnotations != null) {
+		for (IBinaryTypeAnnotation otherAnnotation : otherTypeAnnotations) {
+			if (affectsSignature(otherAnnotation))
+				return true;
+		}
+	}
+	return false;
+}
+
+private boolean affectsSignature(IBinaryTypeAnnotation typeAnnotation) {
+	if (typeAnnotation == null) return false;
+	int targetType = typeAnnotation.getTargetType();
+	if (targetType >= AnnotationTargetTypeConstants.LOCAL_VARIABLE && targetType <= AnnotationTargetTypeConstants.METHOD_REFERENCE_TYPE_ARGUMENT)
+		return false; // affects detail within a block
+	return true;
+}
+
 /**
  * This method is used to fully initialize the contents of the receiver. All methodinfos, fields infos
  * will be therefore fully initialized and we can get rid of the bytes.
@@ -1262,32 +1338,19 @@ private void initialize() throws ClassFormatException {
 		throw exception;
 	}
 }
-
-/**
- * Answer true if the receiver is an anonymous type, false otherwise
- *
- * @return <CODE>boolean</CODE>
- */
+@Override
 public boolean isAnonymous() {
 	if (this.innerInfo == null) return false;
 	char[] innerSourceName = this.innerInfo.getSourceName();
 	return (innerSourceName == null || innerSourceName.length == 0);
 }
 
-/**
- * Answer whether the receiver contains the resolved binary form
- * or the unresolved source form of the type.
- * @return boolean
- */
+@Override
 public boolean isBinaryType() {
 	return true;
 }
 
-/**
- * Answer true if the receiver is a local type, false otherwise
- *
- * @return <CODE>boolean</CODE>
- */
+@Override
 public boolean isLocal() {
 	if (this.innerInfo == null) return false;
 	if (this.innerInfo.getEnclosingTypeName() != null) return false;
@@ -1295,11 +1358,7 @@ public boolean isLocal() {
 	return (innerSourceName != null && innerSourceName.length > 0);
 }
 
-/**
- * Answer true if the receiver is a member type, false otherwise
- *
- * @return <CODE>boolean</CODE>
- */
+@Override
 public boolean isMember() {
 	if (this.innerInfo == null) return false;
 	if (this.innerInfo.getEnclosingTypeName() == null) return false;
@@ -1321,16 +1380,20 @@ public boolean isNestedType() {
  *
  * @return char[]
  */
+@Override
 public char[] sourceFileName() {
 	return this.sourceFileName;
 }
 
+@Override
 public String toString() {
 	java.io.ByteArrayOutputStream out = new java.io.ByteArrayOutputStream();
 	java.io.PrintWriter print = new java.io.PrintWriter(out);
 	print.println(getClass().getName() + "{"); //$NON-NLS-1$
 	print.println(" this.className: " + new String(getName())); //$NON-NLS-1$
 	print.println(" this.superclassName: " + (getSuperclassName() == null ? "null" : new String(getSuperclassName()))); //$NON-NLS-2$ //$NON-NLS-1$
+	if (this.moduleName != null)
+		print.println(" this.moduleName: " + (new String(this.moduleName))); //$NON-NLS-1$
 	print.println(" access_flags: " + printTypeModifiers(accessFlags()) + "(" + accessFlags() + ")"); //$NON-NLS-1$ //$NON-NLS-3$ //$NON-NLS-2$
 	print.flush();
 	return out.toString();
