@@ -1,12 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2015 GK Software AG.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2017 GK Software AG.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Stephan Herrmann - initial API and implementation
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.classfmt;
 
@@ -28,7 +33,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
 
 public class ExternalAnnotationProvider {
 
-	public static final String ANNOTION_FILE_EXTENSION= "eea"; //$NON-NLS-1$
+	public static final String ANNOTATION_FILE_EXTENSION= "eea"; //$NON-NLS-1$
 	public static final String CLASS_PREFIX = "class "; //$NON-NLS-1$
 	public static final String SUPER_PREFIX = "super "; //$NON-NLS-1$
 
@@ -44,7 +49,7 @@ public class ExternalAnnotationProvider {
 	 */
 	public static final char NO_ANNOTATION = '@';
 
-	static final String ANNOTATION_FILE_SUFFIX = ".eea"; //$NON-NLS-1$
+	public static final String ANNOTATION_FILE_SUFFIX = ".eea"; //$NON-NLS-1$
 
 	private static final String TYPE_PARAMETER_PREFIX = " <"; //$NON-NLS-1$
 
@@ -67,8 +72,7 @@ public class ExternalAnnotationProvider {
 	}
 
 	private void initialize(InputStream input) throws IOException {
-		LineNumberReader reader = new LineNumberReader(new InputStreamReader(input));
-		try {
+		try (LineNumberReader reader = new LineNumberReader(new InputStreamReader(input))) {
 			assertClassHeader(reader.readLine(), this.typeName);
 
 			String line;
@@ -117,26 +121,24 @@ public class ExternalAnnotationProvider {
 				}
 				if (rawSig == null || annotSig == null) {
 					if (errLine == -1) errLine = reader.getLineNumber();
-					throw new IOException("Illegal format for annotation file at line "+errLine); //$NON-NLS-1$
+					throw new IOException("Illegal format in annotation file for "+this.typeName+" at line "+errLine); //$NON-NLS-1$ //$NON-NLS-2$
 				}
 				// discard optional meta data (separated by whitespace):
 				annotSig = trimTail(annotSig);
 				if (isSuper) {
 					if (this.supertypeAnnotationSources == null)
-						this.supertypeAnnotationSources = new HashMap<String, String>();
+						this.supertypeAnnotationSources = new HashMap<>();
 					this.supertypeAnnotationSources.put('L'+selector+rawSig+';', annotSig);
 				} else if (rawSig.contains("(")) { //$NON-NLS-1$
 					if (this.methodAnnotationSources == null)
-						this.methodAnnotationSources = new HashMap<String, String>();
+						this.methodAnnotationSources = new HashMap<>();
 					this.methodAnnotationSources.put(selector+rawSig, annotSig);
 				} else {
 					if (this.fieldAnnotationSources == null)
-						this.fieldAnnotationSources = new HashMap<String, String>();
+						this.fieldAnnotationSources = new HashMap<>();
 					this.fieldAnnotationSources.put(selector+':'+rawSig, annotSig);
 				}
 			} while (((line = pendingLine) != null) || (line = reader.readLine()) != null);
-		} finally {
-			reader.close();
 		}
 	}
 
@@ -144,10 +146,10 @@ public class ExternalAnnotationProvider {
 	 * Assert that the given line is a class header for 'typeName' (slash-separated qualified name).
 	 */
 	public static void assertClassHeader(String line, String typeName) throws IOException {
-		if (line.startsWith(CLASS_PREFIX)) {
+		if (line != null && line.startsWith(CLASS_PREFIX)) {
 			line = line.substring(CLASS_PREFIX.length());
 		} else {
-			throw new IOException("missing class header in annotation file"); //$NON-NLS-1$
+			throw new IOException("missing class header in annotation file for "+typeName); //$NON-NLS-1$
 		}
 		if (!trimTail(line).equals(typeName)) {
 			throw new IOException("mismatching class name in annotation file, expected "+typeName+", but header said "+line); //$NON-NLS-1$ //$NON-NLS-2$
@@ -199,6 +201,7 @@ public class ExternalAnnotationProvider {
 		return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
 	}
 
+	@Override
 	public String toString() {
 		StringBuilder sb = new StringBuilder();
 		sb.append("External Annotations for ").append(this.typeName).append('\n'); //$NON-NLS-1$
@@ -214,6 +217,10 @@ public class ExternalAnnotationProvider {
 		public IBinaryElementValuePair[] getElementValuePairs() {
 			return ElementValuePairInfo.NoMembers;
 		}
+		@Override
+		public boolean isExternalAnnotation() {
+			return true;
+		}
 		protected char[] getBinaryTypeName(char[][] name) {
 			return CharOperation.concat('L', CharOperation.concatWith(name, '/'), ';');
 		}
@@ -246,6 +253,7 @@ public class ExternalAnnotationProvider {
 		public DispatchingAnnotationWalker(LookupEnvironment environment) {
 			this.environment = environment;
 		}
+		@Override
 		public ITypeAnnotationWalker toTypeParameter(boolean isClassTypeParameter, int rank) {
 			String source = ExternalAnnotationProvider.this.typeParametersAnnotationSource;
 			if (source != null) {
@@ -255,11 +263,13 @@ public class ExternalAnnotationProvider {
 			}
 			return this;
 		}
+		@Override
 		public ITypeAnnotationWalker toTypeParameterBounds(boolean isClassTypeParameter, int parameterRank) {
 			if (this.typeParametersWalker != null)
 				return this.typeParametersWalker.toTypeParameterBounds(isClassTypeParameter, parameterRank);
 			return this;
 		}
+		@Override
 		public ITypeAnnotationWalker toSupertype(short index, char[] superTypeSignature) {
 			Map<String, String> sources = ExternalAnnotationProvider.this.supertypeAnnotationSources;
 			if (sources != null) {
@@ -270,17 +280,28 @@ public class ExternalAnnotationProvider {
 			return this;
 		}
 		// the rest is borrowed from EMPTY_ANNOTATION_WALKER:
+		@Override
 		public ITypeAnnotationWalker toField() { return this; }
+		@Override
 		public ITypeAnnotationWalker toThrows(int rank) { return this; }
+		@Override
 		public ITypeAnnotationWalker toTypeArgument(int rank) { return this; }
+		@Override
 		public ITypeAnnotationWalker toMethodParameter(short index) { return this; }
+		@Override
 		public ITypeAnnotationWalker toTypeBound(short boundIndex) { return this; }
+		@Override
 		public ITypeAnnotationWalker toMethodReturn() { return this; }
+		@Override
 		public ITypeAnnotationWalker toReceiver() { return this; }
+		@Override
 		public ITypeAnnotationWalker toWildcardBound() { return this; }
+		@Override
 		public ITypeAnnotationWalker toNextArrayDimension() { return this; }
+		@Override
 		public ITypeAnnotationWalker toNextNestedType() { return this; }
-		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId) { return NO_ANNOTATIONS; }
+		@Override
+		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId, boolean mayApplyArrayContentsDefaultNullness) { return NO_ANNOTATIONS; }
 	}
 
 	abstract class BasicAnnotationWalker implements ITypeAnnotationWalker {
@@ -303,6 +324,7 @@ public class ExternalAnnotationProvider {
 			if (this.wrapper == null)
 				this.wrapper = new SignatureWrapper(this.source);
 			this.wrapper.start = start;
+			this.wrapper.bracket = -1;
 			return this.wrapper;
 		}
 
@@ -341,16 +363,17 @@ public class ExternalAnnotationProvider {
 			int next = this.prevTypeArgStart;
 			switch (this.source[next]) {
 				case '*': 
+					next = skipNullAnnotation(next+1);
 					break;
 				case '-': 
 				case '+':
-					next++;
+					next = skipNullAnnotation(next+1);
 					//$FALL-THROUGH$
 				default:
 					next = wrapperWithStart(next).computeEnd();
+					next++;
 			}
-			next++;
-		    this.prevTypeArgStart = next;
+			this.prevTypeArgStart = next;
 		    return new MethodAnnotationWalker(this.source, next,	this.environment);
 		}
 
@@ -359,7 +382,8 @@ public class ExternalAnnotationProvider {
 			switch (this.source[this.pos]) {
 				case '-': 
 				case '+':
-					return new MethodAnnotationWalker(this.source, this.pos+1, this.environment);
+					int newPos = skipNullAnnotation(this.pos+1);
+					return new MethodAnnotationWalker(this.source, newPos, this.environment);
 				default: // includes unbounded '*'
 					return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
 			}			
@@ -368,10 +392,7 @@ public class ExternalAnnotationProvider {
 		@Override
 		public ITypeAnnotationWalker toNextArrayDimension() {
 			if (this.source[this.pos] == '[') {
-				int newPos = this.pos+1;
-				switch (this.source[newPos]) {
-					case NULLABLE: case NONNULL: newPos++; break;
-				}
+				int newPos = skipNullAnnotation(this.pos+1);
 				return new MethodAnnotationWalker(this.source, newPos, this.environment);
 			}
 			return ITypeAnnotationWalker.EMPTY_ANNOTATION_WALKER;
@@ -383,7 +404,7 @@ public class ExternalAnnotationProvider {
 		}
 
 		@Override
-		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId) {
+		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId, boolean mayApplyArrayContentsDefaultNullness) {
 			if (this.pos != -1 && this.pos < this.source.length-2) {
 				switch (this.source[this.pos]) {
 					case 'T':
@@ -400,7 +421,18 @@ public class ExternalAnnotationProvider {
 						}
 				}				
 			}
-			return null;
+			return NO_ANNOTATIONS;
+		}
+		int skipNullAnnotation(int cur) {
+			if (cur >= this.source.length)
+				return cur;
+			switch (this.source[cur]) {
+				case NONNULL:
+				case NULLABLE:
+					return cur+1;
+				default:
+					return cur; 
+			}
 		}
 	}
 
@@ -439,6 +471,25 @@ public class ExternalAnnotationProvider {
 								if ((depth == 0) && (i +1 < length) && (this.source[i+1] != Util.C_COLON))
 									pendingVariable = true;
 								break;
+							case Util.C_COLON :
+								if (depth == 0)
+									pendingVariable = true; // end of variable name
+								// skip optional bound ReferenceTypeSignature
+								i++; // peek next
+								while (i < length && this.source[i] == Util.C_ARRAY)
+									i++;
+								if (i < length && this.source[i] == Util.C_RESOLVED) {
+									int currentdepth = depth;
+									while (i < length && (currentdepth != depth || this.source[i] != Util.C_NAME_END)) {
+										if(this.source[i] == Util.C_GENERIC_START)
+											currentdepth++;
+										if(this.source[i] == Util.C_GENERIC_END)
+											currentdepth--;
+										i++;
+									}
+								}
+								i--; // unget
+								break;
 							default:
 								if (pendingVariable) {
 									pendingVariable = false;
@@ -506,7 +557,7 @@ public class ExternalAnnotationProvider {
 		}
 
 		@Override
-		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId) {
+		public IBinaryAnnotation[] getAnnotationsAtCursor(int currentTypeId, boolean mayApplyArrayContentsDefaultNullness) {
 			if (this.pos != -1 && this.pos < this.source.length-1) {
 				switch (this.source[this.pos]) {
 					case NULLABLE:
@@ -515,7 +566,7 @@ public class ExternalAnnotationProvider {
 						return new IBinaryAnnotation[]{ ExternalAnnotationProvider.this.NONNULL_ANNOTATION };
 				}				
 			}
-			return super.getAnnotationsAtCursor(currentTypeId);
+			return super.getAnnotationsAtCursor(currentTypeId, mayApplyArrayContentsDefaultNullness);
 		}
 	}
 
@@ -564,11 +615,10 @@ public class ExternalAnnotationProvider {
 		int typeEnd(int start) {
 			while (this.source[start] == '[') {
 				start++;
-				char an = this.source[start];
-				if (an == NULLABLE || an == NONNULL)
-					start++;
+				start = skipNullAnnotation(start);
 			}
-			int end = wrapperWithStart(start).computeEnd();
+			SignatureWrapper wrapper1 = wrapperWithStart(start);
+			int end = wrapper1.skipAngleContents(wrapper1.computeEnd());
 			return end;
 		}
 		
