@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -20,7 +23,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.ConstantPool;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryAnnotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryMethod;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryTypeAnnotation;
-import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
 
 @SuppressWarnings("rawtypes")
 public class MethodInfo extends ClassFileStruct implements IBinaryMethod, Comparable {
@@ -30,15 +33,16 @@ public class MethodInfo extends ClassFileStruct implements IBinaryMethod, Compar
 	protected int accessFlags;
 	protected int attributeBytes;
 	protected char[] descriptor;
-	protected char[][] exceptionNames;
+	protected volatile char[][] exceptionNames;
 	protected char[] name;
 	protected char[] signature;
 	protected int signatureUtf8Offset;
 	protected long tagBits;
-	protected char[][] argumentNames;
+	protected volatile char[][] argumentNames;
+	protected long version;
 
-public static MethodInfo createMethod(byte classFileBytes[], int offsets[], int offset) {
-	MethodInfo methodInfo = new MethodInfo(classFileBytes, offsets, offset);
+public static MethodInfo createMethod(byte classFileBytes[], int offsets[], int offset, long version) {
+	MethodInfo methodInfo = new MethodInfo(classFileBytes, offsets, offset, version);
 	int attributesCount = methodInfo.u2At(6);
 	int readOffset = 8;
 	AnnotationInfo[] annotations = null;
@@ -147,22 +151,25 @@ static AnnotationInfo[] decodeMethodAnnotations(int offset, boolean runtimeVisib
 	if (numberOfAnnotations > 0) {
 		AnnotationInfo[] annos = decodeAnnotations(offset + 8, runtimeVisible, numberOfAnnotations, methodInfo);
 		if (runtimeVisible){
-			int numStandardAnnotations = 0;
+			int numRetainedAnnotations = 0;
 			for( int i=0; i<numberOfAnnotations; i++ ){
 				long standardAnnoTagBits = annos[i].standardAnnotationTagBits;
 				methodInfo.tagBits |= standardAnnoTagBits;
 				if(standardAnnoTagBits != 0){
-					annos[i] = null;
-					numStandardAnnotations ++;
+					if (methodInfo.version < ClassFileConstants.JDK9 || (standardAnnoTagBits & TagBits.AnnotationDeprecated) == 0) { // must retain enhanced deprecation
+						annos[i] = null;
+						continue;
+					}
 				}
+				numRetainedAnnotations++;
 			}
 
-			if( numStandardAnnotations != 0 ){
-				if( numStandardAnnotations == numberOfAnnotations )
+			if(numRetainedAnnotations != numberOfAnnotations){
+				if(numRetainedAnnotations == 0)
 					return null;
 
 				// need to resize
-				AnnotationInfo[] temp = new AnnotationInfo[numberOfAnnotations - numStandardAnnotations ];
+				AnnotationInfo[] temp = new AnnotationInfo[numRetainedAnnotations];
 				int tmpIndex = 0;
 				for (int i = 0; i < numberOfAnnotations; i++)
 					if (annos[i] != null)
@@ -214,18 +221,22 @@ static AnnotationInfo[][] decodeParamAnnotations(int offset, boolean runtimeVisi
  * @param classFileBytes byte[]
  * @param offsets int[]
  * @param offset int
+ * @param version class file version 
  */
-protected MethodInfo (byte classFileBytes[], int offsets[], int offset) {
+protected MethodInfo (byte classFileBytes[], int offsets[], int offset, long version) {
 	super(classFileBytes, offsets, offset);
 	this.accessFlags = -1;
 	this.signatureUtf8Offset = -1;
+	this.version = version;
 }
+@Override
 public int compareTo(Object o) {
 	MethodInfo otherMethod = (MethodInfo) o;
 	int result = new String(getSelector()).compareTo(new String(otherMethod.getSelector()));
 	if (result != 0) return result;
 	return new String(getMethodDescriptor()).compareTo(new String(otherMethod.getMethodDescriptor()));
 }
+@Override
 public boolean equals(Object o) {
 	if (!(o instanceof MethodInfo)) {
 		return false;
@@ -234,41 +245,38 @@ public boolean equals(Object o) {
 	return CharOperation.equals(getSelector(), otherMethod.getSelector())
 			&& CharOperation.equals(getMethodDescriptor(), otherMethod.getMethodDescriptor());
 }
+@Override
 public int hashCode() {
 	return CharOperation.hashCode(getSelector()) + CharOperation.hashCode(getMethodDescriptor());
 }
-/**
- * @return the annotations or null if there is none.
- */
+
+@Override
 public IBinaryAnnotation[] getAnnotations() {
 	return null;
 }
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IGenericMethod#getArgumentNames()
  */
+@Override
 public char[][] getArgumentNames() {
 	if (this.argumentNames == null) {
 		readCodeAttribute();
 	}
 	return this.argumentNames;
 }
+@Override
 public Object getDefaultValue() {
 	return null;
 }
-/**
- * Answer the resolved names of the exception types in the
- * class file format as specified in section 4.2 of the Java 2 VM spec
- * or null if the array is empty.
- *
- * For example, java.lang.String is java/lang/String.
- * @return char[][]
- */
+
+@Override
 public char[][] getExceptionTypeNames() {
 	if (this.exceptionNames == null) {
 		readExceptionAttributes();
 	}
 	return this.exceptionNames;
 }
+@Override
 public char[] getGenericSignature() {
 	if (this.signatureUtf8Offset != -1) {
 		if (this.signature == null) {
@@ -279,15 +287,8 @@ public char[] getGenericSignature() {
 	}
 	return null;
 }
-/**
- * Answer the receiver's method descriptor which describes the parameter &
- * return types as specified in section 4.3.3 of the Java 2 VM spec.
- *
- * For example:
- *   - int foo(String) is (Ljava/lang/String;)I
- *   - void foo(Object[]) is (I)[Ljava/lang/Object;
- * @return char[]
- */
+
+@Override
 public char[] getMethodDescriptor() {
 	if (this.descriptor == null) {
 		// read the name
@@ -302,29 +303,28 @@ public char[] getMethodDescriptor() {
  * Set the AccDeprecated and AccSynthetic bits if necessary
  * @return int
  */
+@Override
 public int getModifiers() {
 	if (this.accessFlags == -1) {
 		// compute the accessflag. Don't forget the deprecated attribute
-		this.accessFlags = u2At(0);
 		readModifierRelatedAttributes();
 	}
 	return this.accessFlags;
 }
-public IBinaryAnnotation[] getParameterAnnotations(int index) {
+@Override
+public IBinaryAnnotation[] getParameterAnnotations(int index, char[] classFileName) {
 	return null;
 }
+@Override
 public int getAnnotatedParametersCount() {
 	return 0;
 }
+@Override
 public IBinaryTypeAnnotation[] getTypeAnnotations() {
 	return null;
 }
-/**
- * Answer the name of the method.
- *
- * For a constructor, answer <init> & <clinit> for a clinit method.
- * @return char[]
- */
+
+@Override
 public char[] getSelector() {
 	if (this.name == null) {
 		// read the name
@@ -333,6 +333,7 @@ public char[] getSelector() {
 	}
 	return this.name;
 }
+@Override
 public long getTagBits() {
 	return this.tagBits;
 }
@@ -353,17 +354,17 @@ protected void initialize() {
  * Answer true if the method is a class initializer, false otherwise.
  * @return boolean
  */
+@Override
 public boolean isClinit() {
-	char[] selector = getSelector();
-	return selector[0] == '<' && selector.length == 8; // Can only match <clinit>
+	return org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.JavaBinaryNames.isClinit(getSelector());
 }
 /**
  * Answer true if the method is a constructor, false otherwise.
  * @return boolean
  */
+@Override
 public boolean isConstructor() {
-	char[] selector = getSelector();
-	return selector[0] == '<' && selector.length == 6; // Can only match <init>
+	return org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.JavaBinaryNames.isConstructor(getSelector());
 }
 /**
  * Return true if the field is a synthetic method, false otherwise.
@@ -372,9 +373,10 @@ public boolean isConstructor() {
 public boolean isSynthetic() {
 	return (getModifiers() & ClassFileConstants.AccSynthetic) != 0;
 }
-private void readExceptionAttributes() {
+private synchronized void readExceptionAttributes() {
 	int attributesCount = u2At(6);
 	int readOffset = 8;
+	char[][] names = null;
 	for (int i = 0; i < attributesCount; i++) {
 		int utf8Offset = this.constantPoolOffsets[u2At(readOffset)] - this.structOffset;
 		char[] attributeName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
@@ -384,15 +386,15 @@ private void readExceptionAttributes() {
 			// place the readOffset at the beginning of the exceptions table
 			readOffset += 8;
 			if (entriesNumber == 0) {
-				this.exceptionNames = noException;
+				names = noException;
 			} else {
-				this.exceptionNames = new char[entriesNumber][];
+				names = new char[entriesNumber][];
 				for (int j = 0; j < entriesNumber; j++) {
 					utf8Offset =
 						this.constantPoolOffsets[u2At(
 							this.constantPoolOffsets[u2At(readOffset)] - this.structOffset + 1)]
 							- this.structOffset;
-					this.exceptionNames[j] = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
+					names[j] = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
 					readOffset += 2;
 				}
 			}
@@ -400,11 +402,14 @@ private void readExceptionAttributes() {
 			readOffset += (6 + u4At(readOffset + 2));
 		}
 	}
-	if (this.exceptionNames == null) {
+	if (names == null) {
 		this.exceptionNames = noException;
+	} else {
+		this.exceptionNames = names;
 	}
 }
-private void readModifierRelatedAttributes() {
+private synchronized void readModifierRelatedAttributes() {
+	int flags = u2At(0);
 	int attributesCount = u2At(6);
 	int readOffset = 8;
 	for (int i = 0; i < attributesCount; i++) {
@@ -415,23 +420,24 @@ private void readModifierRelatedAttributes() {
 			switch(attributeName[0]) {
 				case 'D' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.DeprecatedName))
-						this.accessFlags |= ClassFileConstants.AccDeprecated;
+						flags |= ClassFileConstants.AccDeprecated;
 					break;
 				case 'S' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.SyntheticName))
-						this.accessFlags |= ClassFileConstants.AccSynthetic;
+						flags |= ClassFileConstants.AccSynthetic;
 					break;
 				case 'A' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.AnnotationDefaultName))
-						this.accessFlags |= ClassFileConstants.AccAnnotationDefault;
+						flags |= ClassFileConstants.AccAnnotationDefault;
 					break;
 				case 'V' :
 					if (CharOperation.equals(attributeName, AttributeNamesConstants.VarargsName))
-						this.accessFlags |= ClassFileConstants.AccVarargs;
+						flags |= ClassFileConstants.AccVarargs;
 			}
 		}
 		readOffset += (6 + u4At(readOffset + 2));
 	}
+	this.accessFlags = flags;
 }
 /**
  * Answer the size of the receiver in bytes.
@@ -442,6 +448,7 @@ public int sizeInBytes() {
 	return this.attributeBytes;
 }
 
+@Override
 public String toString() {
 	StringBuffer buffer = new StringBuffer();
 	toString(buffer);
@@ -452,26 +459,9 @@ void toString(StringBuffer buffer) {
 	toStringContent(buffer);
 }
 protected void toStringContent(StringBuffer buffer) {
-	int modifiers = getModifiers();
-	char[] desc = getGenericSignature();
-	if (desc == null)
-		desc = getMethodDescriptor();
-	buffer
-	.append('{')
-	.append(
-		((modifiers & ClassFileConstants.AccDeprecated) != 0 ? "deprecated " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0001) == 1 ? "public " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0002) == 0x0002 ? "private " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0004) == 0x0004 ? "protected " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0008) == 0x000008 ? "static " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0010) == 0x0010 ? "final " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0040) == 0x0040 ? "bridge " : Util.EMPTY_STRING) //$NON-NLS-1$
-			+ ((modifiers & 0x0080) == 0x0080 ? "varargs " : Util.EMPTY_STRING)) //$NON-NLS-1$
-	.append(getSelector())
-	.append(desc)
-	.append('}');
+	BinaryTypeFormatter.methodToStringContent(buffer, this);
 }
-private void readCodeAttribute() {
+private synchronized void readCodeAttribute() {
 	int attributesCount = u2At(6);
 	int readOffset = 8;
 	if (attributesCount != 0) {
@@ -518,7 +508,7 @@ private void decodeLocalVariableAttribute(int offset, int codeLength) {
 	final int length = u2At(readOffset);
 	if (length != 0) {
 		readOffset += 2;
-		this.argumentNames = new char[length][];
+		char[][] names = new char[length][];
 		int argumentNamesIndex = 0;
 		for (int i = 0; i < length; i++) {
 			int startPC = u2At(readOffset);
@@ -527,17 +517,18 @@ private void decodeLocalVariableAttribute(int offset, int codeLength) {
 				int utf8Offset = this.constantPoolOffsets[nameIndex] - this.structOffset;
 				char[] localVariableName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
 				if (!CharOperation.equals(localVariableName, ConstantPool.This)) {
-					this.argumentNames[argumentNamesIndex++] = localVariableName;
+					names[argumentNamesIndex++] = localVariableName;
 				}
 			} else {
 				break;
 			}
 			readOffset += 10;
 		}
-		if (argumentNamesIndex != this.argumentNames.length) {
+		if (argumentNamesIndex != names.length) {
 			// resize
-			System.arraycopy(this.argumentNames, 0, (this.argumentNames = new char[argumentNamesIndex][]), 0, argumentNamesIndex);
+			System.arraycopy(names, 0, (names = new char[argumentNamesIndex][]), 0, argumentNamesIndex);
 		}
+		this.argumentNames = names;
 	}
 }
 private void decodeMethodParameters(int offset, MethodInfo methodInfo) {
@@ -545,18 +536,19 @@ private void decodeMethodParameters(int offset, MethodInfo methodInfo) {
 	final int length = u1At(readOffset);
 	if (length != 0) {
 		readOffset += 1;
-		this.argumentNames = new char[length][];
+		char[][] names = new char[length][];
 		for (int i = 0; i < length; i++) {
 			int nameIndex = u2At(readOffset);
 			if (nameIndex != 0) {
 				int utf8Offset = this.constantPoolOffsets[nameIndex] - this.structOffset;
 				char[] parameterName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
-				this.argumentNames[i] = parameterName;
+				names[i] = parameterName;
 			} else {
-				this.argumentNames[i] = CharOperation.concat(ARG, String.valueOf(i).toCharArray());
+				names[i] = CharOperation.concat(ARG, String.valueOf(i).toCharArray());
 			}
 			readOffset += 4;
 		}
+		this.argumentNames = names;
 	}
 }
 }
