@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -26,6 +29,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TagBits;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeIds;
+import org.aspectj.org.eclipse.jdt.internal.compiler.util.HashtableOfInteger;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
 /**
@@ -43,6 +47,7 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 	public static final int NAMEANDTYPE_INITIAL_SIZE = 272;
 	public static final int CONSTANTPOOL_INITIAL_SIZE = 2000;
 	public static final int CONSTANTPOOL_GROW_SIZE = 6000;
+	public static final int DYNAMIC_INITIAL_SIZE = 10;
 	protected DoubleCache doubleCache;
 	protected FloatCache floatCache;
 	protected IntegerCache intCache;
@@ -51,7 +56,10 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 	protected CharArrayCache stringCache;
 	protected HashtableOfObject methodsAndFieldsCache;
 	protected CharArrayCache classCache;
+	protected CharArrayCache moduleCache;
+	protected CharArrayCache packageCache;
 	protected HashtableOfObject nameAndTypeCacheForFieldsAndMethods;
+	protected HashtableOfInteger dynamicCache;
 	public byte[] poolContent;
 	public int currentIndex = 1;
 	public int currentOffset;
@@ -260,7 +268,7 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 	public static final char[] JAVA_LANG_ANNOTATION_INHERITED = "Ljava/lang/annotation/Inherited;".toCharArray(); //$NON-NLS-1$
 	// java 7  java.lang.SafeVarargs
 	public static final char[] JAVA_LANG_SAFEVARARGS = "Ljava/lang/SafeVarargs;".toCharArray(); //$NON-NLS-1$
-	// java 7 java.lang.invoke.MethodHandle.invokeExact(..)/invokeGeneric(..)
+	// java 7 java.lang.invoke.MethodHandle.invokeExact(..)/invoke(..)
 	public static final char[] JAVA_LANG_INVOKE_METHODHANDLE_POLYMORPHICSIGNATURE = "Ljava/lang/invoke/MethodHandle$PolymorphicSignature;".toCharArray(); //$NON-NLS-1$
 	// Java 8 lambda support
 	public static final char[] METAFACTORY = "metafactory".toCharArray(); //$NON-NLS-1$
@@ -306,7 +314,10 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 		this.stringCache = new CharArrayCache(STRING_INITIAL_SIZE);
 		this.methodsAndFieldsCache = new HashtableOfObject(METHODS_AND_FIELDS_INITIAL_SIZE);
 		this.classCache = new CharArrayCache(CLASS_INITIAL_SIZE);
+		this.moduleCache = new CharArrayCache(5);
+		this.packageCache = new CharArrayCache(5);
 		this.nameAndTypeCacheForFieldsAndMethods = new HashtableOfObject(NAMEANDTYPE_INITIAL_SIZE);
+		this.dynamicCache = new HashtableOfInteger(DYNAMIC_INITIAL_SIZE);
 		this.offsets = new int[5];
 		initialize(classFile);
 	}
@@ -671,6 +682,64 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 		}
 		return index;
 	}
+	public int literalIndexForModule(final char[] moduleName) {
+		int index;
+		if ((index = this.moduleCache.putIfAbsent(moduleName, this.currentIndex)) < 0) {
+			// The entry doesn't exit yet
+			this.currentIndex++;
+			if ((index = -index) > 0xFFFF){
+				this.classFile.referenceBinding.scope.problemReporter().noMoreAvailableSpaceInConstantPool(this.classFile.referenceBinding.scope.referenceType());
+			}
+			// Write the tag first
+			int length = this.offsets.length;
+			if (length <= index) {
+				// resize
+				System.arraycopy(this.offsets, 0, (this.offsets = new int[index * 2]), 0, length);
+			}
+			this.offsets[index] = this.currentOffset;
+			writeU1(ModuleTag);
+			// Then the string index
+			int stringIndexOffset = this.currentOffset;
+			if (this.currentOffset + 2 >= this.poolContent.length) {
+				resizePoolContents(2);
+			}
+			this.currentOffset+=2;
+
+			final int stringIndex = literalIndex(moduleName);
+			this.poolContent[stringIndexOffset++] = (byte) (stringIndex >> 8);
+			this.poolContent[stringIndexOffset] = (byte) stringIndex;
+		}
+		return index;
+	}
+	public int literalIndexForPackage(final char[] packageName) {
+		int index;
+		if ((index = this.packageCache.putIfAbsent(packageName, this.currentIndex)) < 0) {
+			// The entry doesn't exit yet
+			this.currentIndex++;
+			if ((index = -index) > 0xFFFF){
+				this.classFile.referenceBinding.scope.problemReporter().noMoreAvailableSpaceInConstantPool(this.classFile.referenceBinding.scope.referenceType());
+			}
+			// Write the tag first
+			int length = this.offsets.length;
+			if (length <= index) {
+				// resize
+				System.arraycopy(this.offsets, 0, (this.offsets = new int[index * 2]), 0, length);
+			}
+			this.offsets[index] = this.currentOffset;
+			writeU1(PackageTag);
+			// Then the string index
+			int stringIndexOffset = this.currentOffset;
+			if (this.currentOffset + 2 >= this.poolContent.length) {
+				resizePoolContents(2);
+			}
+			this.currentOffset+=2;
+
+			final int stringIndex = literalIndex(packageName);
+			this.poolContent[stringIndexOffset++] = (byte) (stringIndex >> 8);
+			this.poolContent[stringIndexOffset] = (byte) stringIndex;
+		}
+		return index;
+	}
 	public int literalIndexForType(final char[] constantPoolName) {
 		int index;
 		if ((index = this.classCache.putIfAbsent(constantPoolName, this.currentIndex)) < 0) {
@@ -826,20 +895,43 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 
 		return index;
 	}
-	public int literalIndexForInvokeDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
-		int nameAndTypeIndex = literalIndexForNameAndType(selector, descriptor);
-		int index = this.currentIndex++;
-		int length = this.offsets.length;
-		if (length <= index) {
-			// resize
-			System.arraycopy(this.offsets, 0, (this.offsets = new int[index * 2]), 0, length);
+	private int literalIndexForInvokeAndConstantDynamic(int bootStrapIndex, char[] selector, char[] descriptor, int tag) {
+		int index;
+		if ((index = putInDynamicCacheIfAbsent(bootStrapIndex, selector, descriptor, this.currentIndex)) < 0) {
+			this.currentIndex++;
+			if ((index = -index) > 0xFFFF){
+				this.classFile.referenceBinding.scope.problemReporter().noMoreAvailableSpaceInConstantPool(this.classFile.referenceBinding.scope.referenceType());
+			}
+			int length = this.offsets.length;
+			if (length <= index) {
+				// resize
+				System.arraycopy(this.offsets, 0, (this.offsets = new int[index * 2]), 0, length);
+			}
+			this.offsets[index] = this.currentOffset;
+
+			writeU1(tag);
+			int classIndexOffset = this.currentOffset;
+			if (this.currentOffset + 4 >= this.poolContent.length) {
+				resizePoolContents(4);
+			}
+			this.currentOffset+=4;
+
+			int nameAndTypeIndex = literalIndexForNameAndType(selector, descriptor);
+
+			this.poolContent[classIndexOffset++] = (byte) (bootStrapIndex >> 8);
+			this.poolContent[classIndexOffset++] = (byte) bootStrapIndex;
+			this.poolContent[classIndexOffset++] = (byte) (nameAndTypeIndex >> 8);
+			this.poolContent[classIndexOffset] = (byte) nameAndTypeIndex;
 		}
-		this.offsets[index] = this.currentOffset;
-		writeU1(InvokeDynamicTag);
-		writeU2(bootStrapIndex);
-		writeU2(nameAndTypeIndex);
 		return index;
 	}
+	// CONSTANT_Dynamic_info JVMS 4.4.10 /jep 309
+	public int literalIndexForDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
+		return literalIndexForInvokeAndConstantDynamic(bootStrapIndex, selector, descriptor, DynamicTag);
+	}
+	public int literalIndexForInvokeDynamic(int bootStrapIndex, char[] selector, char[] descriptor) {
+		return literalIndexForInvokeAndConstantDynamic(bootStrapIndex, selector, descriptor, InvokeDynamicTag);
+	}
 	public int literalIndexForField(char[] declaringClass, char[] name, char[] signature) {
 		int index;
 		if ((index = putInCacheIfAbsent(declaringClass, name, signature, this.currentIndex)) < 0) {
@@ -1010,6 +1102,47 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 		}
 		return index;
 	}
+	
+	/**
+	 * @param bootstrapIndex the given bootstrap index
+	 * @param selector the given method selector
+	 * @param descriptor the given signature
+	 * @param value the new index
+	 * @return the given index
+	 */	private int putInDynamicCacheIfAbsent(int bootstrapIndex, final char[] selector, final char[] descriptor, final int value) {
+		int index;
+		HashtableOfObject key1Value = (HashtableOfObject) this.dynamicCache.get(bootstrapIndex);
+		if (key1Value == null) {
+			key1Value = new HashtableOfObject();
+			this.dynamicCache.put(bootstrapIndex, key1Value);
+			CachedIndexEntry cachedIndexEntry = new CachedIndexEntry(descriptor, value);
+			index = -value;
+			key1Value.put(selector, cachedIndexEntry);
+		} else {
+			Object key2Value = key1Value.get(selector);
+			if (key2Value == null) {
+				CachedIndexEntry cachedIndexEntry = new CachedIndexEntry(descriptor, value);
+				index = -value;
+				key1Value.put(selector, cachedIndexEntry);
+			} else if (key2Value instanceof CachedIndexEntry) {
+				// adding a second entry
+				CachedIndexEntry entry = (CachedIndexEntry) key2Value;
+				if (CharOperation.equals(descriptor, entry.signature)) {
+					index = entry.index;
+				} else {
+					CharArrayCache charArrayCache = new CharArrayCache();
+					charArrayCache.putIfAbsent(entry.signature, entry.index);
+					index = charArrayCache.putIfAbsent(descriptor, value);
+					key1Value.put(selector, charArrayCache);
+				}
+			} else {
+				CharArrayCache charArrayCache = (CharArrayCache) key2Value;
+				index = charArrayCache.putIfAbsent(descriptor, value);
+			}
+		}
+		return index;
+	}
+
 	/**
 	 * @param key1 the given declaring class name
 	 * @param key2 the given field name or method selector
@@ -1112,15 +1245,11 @@ public class ConstantPool implements ClassFileConstants, TypeIds {
 		this.stringCache.clear();
 		this.methodsAndFieldsCache.clear();
 		this.classCache.clear();
+		this.packageCache.clear();
+		this.moduleCache.clear();
 		this.nameAndTypeCacheForFieldsAndMethods.clear();
+		this.dynamicCache.clear();
 		this.currentIndex = 1;
 		this.currentOffset = 0;
 	}
-	public void resetForAttributeName(char[] attributeName, int constantPoolIndex, int constantPoolOffset) {
-		this.currentIndex = constantPoolIndex;
-		this.currentOffset = constantPoolOffset;
-		if (this.UTF8Cache.get(attributeName) >= constantPoolIndex) {
-			this.UTF8Cache.remove(attributeName);
-		}
-	}
 }
