@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -62,7 +66,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 public class FlowContext implements TypeConstants {
 
 	// preempt marks looping contexts
-	public final static FlowContext NotContinuableContext = new FlowContext(null, null);
+	public final static FlowContext NotContinuableContext = new FlowContext(null, null, true);
 	public ASTNode associatedNode;
 	public FlowContext parent;
 	public FlowInfo initsOnFinally;
@@ -83,10 +87,10 @@ public class FlowContext implements TypeConstants {
 	public TypeBinding[][] providedExpectedTypes = null;
 
 	// record field references known to be non-null
-	//   this array will never shrink, only grow. reset happens by nulling the first cell
-	//   adding elements after reset ensures that the valid part of the array is always null-terminated
+	//   this array will never shrink, only grow. reset happens by nulling expired entries
+	//   this array grows in lock step with timesToLiveForNullCheckInfo, which controls expiration
 	private Reference[] nullCheckedFieldReferences = null;
-	private int timeToLiveForNullCheckInfo = -1;
+	private int[] timesToLiveForNullCheckInfo = null;
 
 	public static final int DEFER_NULL_DIAGNOSTIC = 0x1;
 	public static final int PREEMPT_NULL_DIAGNOSTIC = 0x2;
@@ -122,7 +126,7 @@ public static final int IN_INSTANCEOF = 0x0400;
 // check happened in an instanceof expression
 public static final int CONTEXT_MASK = ~CHECK_MASK & ~HIDE_NULL_COMPARISON_WARNING_MASK;
 
-public FlowContext(FlowContext parent, ASTNode associatedNode) {
+public FlowContext(FlowContext parent, ASTNode associatedNode, boolean inheritNullFieldChecks) {
 	this.parent = parent;
 	this.associatedNode = associatedNode;
 	if (parent != null) {
@@ -131,7 +135,16 @@ public FlowContext(FlowContext parent, ASTNode associatedNode) {
 		}
 		this.initsOnFinally = parent.initsOnFinally;
 		this.conditionalLevel = parent.conditionalLevel;
-		this.nullCheckedFieldReferences = parent.nullCheckedFieldReferences; // re-use list if there is one
+		if (inheritNullFieldChecks)
+			copyNullCheckedFieldsFrom(parent); // re-use list if there is one
+	}
+}
+
+public void copyNullCheckedFieldsFrom(FlowContext other) {
+	Reference[] fieldReferences = other.nullCheckedFieldReferences;
+	if (fieldReferences != null && fieldReferences.length > 0 && fieldReferences[0] != null) {
+		this.nullCheckedFieldReferences = other.nullCheckedFieldReferences;
+		this.timesToLiveForNullCheckInfo = other.timesToLiveForNullCheckInfo;
 	}
 }
 
@@ -142,33 +155,35 @@ public FlowContext(FlowContext parent, ASTNode associatedNode) {
  * @param timeToLive control how many expire events are needed to expire this information
  */
 public void recordNullCheckedFieldReference(Reference reference, int timeToLive) {
-	this.timeToLiveForNullCheckInfo = timeToLive;
 	if (this.nullCheckedFieldReferences == null) {
 		// first entry:
-		this.nullCheckedFieldReferences = new Reference[2];
-		this.nullCheckedFieldReferences[0] = reference;
+		this.nullCheckedFieldReferences = new Reference[] { reference, null };
+		this.timesToLiveForNullCheckInfo = new int[] { timeToLive, -1 };
 	} else {
 		int len = this.nullCheckedFieldReferences.length;
 		// insert into first empty slot:
 		for (int i=0; i<len; i++) {
 			if (this.nullCheckedFieldReferences[i] == null) {
 				this.nullCheckedFieldReferences[i] = reference;
-				if (i+1 < len) {
-					this.nullCheckedFieldReferences[i+1] = null; // lazily mark next as empty
-				}
+				this.timesToLiveForNullCheckInfo[i] = timeToLive;
 				return;
 			}
 		}
-		// grow array:
+		// grow arrays:
 		System.arraycopy(this.nullCheckedFieldReferences, 0, this.nullCheckedFieldReferences=new Reference[len+2], 0, len);
+		System.arraycopy(this.timesToLiveForNullCheckInfo, 0, this.timesToLiveForNullCheckInfo=new int[len+2], 0, len);
 		this.nullCheckedFieldReferences[len] = reference;
+		this.timesToLiveForNullCheckInfo[len] = timeToLive;
 	}
 }
 
 /** If a null checked field has been recorded recently, increase its time to live. */
 public void extendTimeToLiveForNullCheckedField(int t) {
-	if (this.timeToLiveForNullCheckInfo > 0)
-		this.timeToLiveForNullCheckInfo += t;
+	if (this.timesToLiveForNullCheckInfo != null) {
+		for (int i = 0; i < this.timesToLiveForNullCheckInfo.length; i++)
+			if (this.timesToLiveForNullCheckInfo[i] > 0)
+				this.timesToLiveForNullCheckInfo[i] += t;
+}
 }
 
 /**
@@ -179,8 +194,9 @@ public void extendTimeToLiveForNullCheckedField(int t) {
  */	 
 public void expireNullCheckedFieldInfo() {
 	if (this.nullCheckedFieldReferences != null) {
-		if (--this.timeToLiveForNullCheckInfo == 0) {
-			this.nullCheckedFieldReferences[0] = null; // lazily wipe
+		for (int i = 0; i < this.nullCheckedFieldReferences.length; i++) {
+			if (--this.timesToLiveForNullCheckInfo[i] == 0)
+				this.nullCheckedFieldReferences[i] = null;
 		}
 	}
 }
@@ -197,7 +213,7 @@ public boolean isNullcheckedFieldAccess(Reference reference) {
 	for (int i=0; i<len; i++) {
 		Reference checked = this.nullCheckedFieldReferences[i];
 		if (checked == null) {
-			return false;
+			continue;
 		}
 		if (checked.isEquivalent(reference)) {
 			return true;
@@ -523,6 +539,9 @@ public FlowInfo getInitsForFinalBlankInitializationCheck(TypeBinding declaringTy
 			inits = initializationContext.initsBeforeContext;
 			current = initializationContext.initializationParent;
 		} else if (current instanceof ExceptionHandlingFlowContext) {
+			if(current instanceof FieldInitsFakingFlowContext) {
+				return FlowInfo.DEAD_END; // isDefinitelyAssigned will return true for all fields
+			}
 			ExceptionHandlingFlowContext exceptionContext = (ExceptionHandlingFlowContext) current;
 			current = exceptionContext.initializationParent == null ? exceptionContext.parent : exceptionContext.initializationParent;
 		} else {
@@ -530,7 +549,7 @@ public FlowInfo getInitsForFinalBlankInitializationCheck(TypeBinding declaringTy
 		}
 	} while (current != null);
 	// not found
-	return null;
+	throw new IllegalStateException(declaringType.debugName());
 }
 
 /*
@@ -956,6 +975,10 @@ public void recordUsingNullReference(Scope scope, LocalVariableBinding local,
 				return;
 			}
 			if (flowInfo.isPotentiallyNull(local)) {
+				if(local.type.isFreeTypeVariable()) {
+					scope.problemReporter().localVariableFreeTypeVariableReference(local, location);
+					return;
+				}
 				scope.problemReporter().localVariablePotentialNullReference(local, location);
 				return;
 			}
@@ -977,6 +1000,7 @@ public SubRoutineStatement subroutine() {
 	return null;
 }
 
+@Override
 public String toString() {
 	StringBuffer buffer = new StringBuffer();
 	FlowContext current = this;
