@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -105,7 +108,7 @@ public class LoopingFlowContext extends SwitchFlowContext {
 		BranchLabel continueLabel,
 		Scope associatedScope,
 		boolean isPreTest) {
-		super(parent, associatedNode, breakLabel, isPreTest);
+		super(parent, associatedNode, breakLabel, isPreTest, false);
 		this.tagBits |= FlowContext.PREEMPT_NULL_DIAGNOSTIC;
 			// children will defer to this, which may defer to its own parent
 		this.continueLabel = continueLabel;
@@ -161,6 +164,9 @@ public void complainOnDeferredFinalChecks(BlockScope scope, FlowInfo flowInfo) {
  * @param callerFlowInfo the flow info against which checks must be performed
  */
 public void complainOnDeferredNullChecks(BlockScope scope, FlowInfo callerFlowInfo) {
+	complainOnDeferredNullChecks(scope, callerFlowInfo, true);
+}
+public void complainOnDeferredNullChecks(BlockScope scope, FlowInfo callerFlowInfo, boolean updateInitsOnBreak) {
 	for (int i = 0 ; i < this.innerFlowContextsCount ; i++) {
 		this.upstreamNullFlowInfo.
 			addPotentialNullInfoFrom(
@@ -433,16 +439,20 @@ public void complainOnDeferredNullChecks(BlockScope scope, FlowInfo callerFlowIn
 		}
 	}
 	// propagate breaks
-	this.initsOnBreak.addPotentialNullInfoFrom(incomingInfo);
-	for (int i = 0; i < this.breakTargetsCount; i++) {
-		this.breakTargetContexts[i].initsOnBreak.addPotentialNullInfoFrom(incomingInfo);
+	if(updateInitsOnBreak) {
+		this.initsOnBreak.addPotentialNullInfoFrom(incomingInfo);
+		for (int i = 0; i < this.breakTargetsCount; i++) {
+			this.breakTargetContexts[i].initsOnBreak.addPotentialNullInfoFrom(incomingInfo);
+		}
 	}
 }
 
+	@Override
 	public BranchLabel continueLabel() {
 		return this.continueLabel;
 	}
 
+	@Override
 	public String individualToString() {
 		StringBuffer buffer = new StringBuffer("Looping flow context"); //$NON-NLS-1$
 		buffer.append("[initsOnBreak - ").append(this.initsOnBreak.toString()).append(']'); //$NON-NLS-1$
@@ -452,6 +462,7 @@ public void complainOnDeferredNullChecks(BlockScope scope, FlowInfo callerFlowIn
 		return buffer.toString();
 	}
 
+	@Override
 	public boolean isContinuable() {
 		return true;
 	}
@@ -460,6 +471,7 @@ public void complainOnDeferredNullChecks(BlockScope scope, FlowInfo callerFlowIn
 		return this.initsOnContinue != FlowInfo.DEAD_END;
 	}
 
+@Override
 public void recordBreakTo(FlowContext targetContext) {
 	if (targetContext instanceof LabelFlowContext) {
 		int current;
@@ -472,6 +484,7 @@ public void recordBreakTo(FlowContext targetContext) {
 	}
 }
 
+@Override
 public void recordContinueFrom(FlowContext innerFlowContext, FlowInfo flowInfo) {
 	if ((flowInfo.tagBits & FlowInfo.UNREACHABLE_OR_DEAD) == 0)	{
 		if ((this.initsOnContinue.tagBits & FlowInfo.UNREACHABLE_OR_DEAD) == 0) {
@@ -513,6 +526,7 @@ public void recordContinueFrom(FlowContext innerFlowContext, FlowInfo flowInfo)
 	}
 }
 
+	@Override
 	protected boolean recordFinalAssignment(
 		VariableBinding binding,
 		Reference finalAssignment) {
@@ -548,6 +562,7 @@ public void recordContinueFrom(FlowContext innerFlowContext, FlowInfo flowInfo)
 		return true;
 	}
 
+@Override
 protected void recordNullReference(LocalVariableBinding local,
 	ASTNode expression, int checkType, FlowInfo nullInfo) {
 	if (this.nullCount == 0) {
@@ -571,6 +586,7 @@ protected void recordNullReference(LocalVariableBinding local,
 	this.nullCheckTypes[this.nullCount] = checkType;
 	this.nullInfos[this.nullCount++] = nullInfo != null ? nullInfo.unconditionalCopy() : null;
 }
+@Override
 public void recordUnboxing(Scope scope, Expression expression, int nullStatus, FlowInfo flowInfo) {
 	if (nullStatus == FlowInfo.NULL)
 		super.recordUnboxing(scope, expression, nullStatus, flowInfo);
@@ -579,6 +595,7 @@ public void recordUnboxing(Scope scope, Expression expression, int nullStatus, F
 }
 
 /** Record the fact that we see an early exit (in 'reference') while 'trackingVar' is in scope and may be unclosed. */
+@Override
 public boolean recordExitAgainstResource(BlockScope scope, FlowInfo flowInfo, FakedTrackingVariable trackingVar, ASTNode reference) {
 	LocalVariableBinding local = trackingVar.binding;
 	if (flowInfo.isDefinitelyNonNull(local)) {
@@ -596,6 +613,7 @@ public boolean recordExitAgainstResource(BlockScope scope, FlowInfo flowInfo, Fa
 	return true; // handled
 }
 
+@Override
 public void recordUsingNullReference(Scope scope, LocalVariableBinding local,
 		ASTNode location, int checkType, FlowInfo flowInfo) {
 	if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) != 0 ||
@@ -724,6 +742,7 @@ public void recordUsingNullReference(Scope scope, LocalVariableBinding local,
 	}
 }
 
+	@Override
 	void removeFinalAssignmentIfAny(Reference reference) {
 		for (int i = 0; i < this.assignCount; i++) {
 			if (this.finalAssignments[i] == reference) {
@@ -761,6 +780,7 @@ public void recordUsingNullReference(Scope scope, LocalVariableBinding local,
 		return this.escapingExceptionCatchSites != null;
 	}
 
+	@Override
 	protected boolean internalRecordNullityMismatch(Expression expression, TypeBinding providedType, FlowInfo flowInfo, int nullStatus, TypeBinding expectedType, int checkType) {
 		recordProvidedExpectedTypes(providedType, expectedType, this.nullCount);
 		recordNullReference(expression.localVariableBinding(), expression, checkType, flowInfo);
