@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -23,6 +26,8 @@
  *								Bug 438458 - [1.8][null] clean up handling of null type annotations wrt type variables
  *								Bug 440759 - [1.8][null] @NonNullByDefault should never affect wildcards and uses of a type variable
  *								Bug 441693 - [1.8][null] Bogus warning for type argument annotated with @NonNull
+ *     Jesper S Møller - Contributions for bug 381345 : [1.8] Take care of the Java 8 major version
+ *								Bug 527554 - [18.3] Compiler support for JEP 286 Local-Variable Type
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
@@ -30,6 +35,7 @@ import java.util.List;
 import java.util.Set;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ClassFile;
 import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.Constant;
@@ -49,6 +55,8 @@ public final class ArrayBinding extends TypeBinding {
 	// possible bits are TagBits.AnnotationNonNull and TagBits.AnnotationNullable
 	// (only ever set when CompilerOptions.isAnnotationBasedNullAnalysisEnabled == true):
 	public long[] nullTagBitsPerDimension;
+	
+	private MethodBinding clone;
 
 public ArrayBinding(TypeBinding type, int dimensions, LookupEnvironment environment) {
 	this.tagBits |= TagBits.IsArrayType;
@@ -67,6 +75,7 @@ public ArrayBinding(TypeBinding type, int dimensions, LookupEnvironment environm
 	}
 }
 
+@Override
 public TypeBinding closestMatch() {
 	if (isValidBinding()) {
 		return this;
@@ -81,21 +90,15 @@ public TypeBinding closestMatch() {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#collectMissingTypes(java.util.List)
  */
-public List collectMissingTypes(List missingTypes) {
+@Override
+public List<TypeBinding> collectMissingTypes(List<TypeBinding> missingTypes) {
 	if ((this.tagBits & TagBits.HasMissingType) != 0) {
 		missingTypes = this.leafComponentType.collectMissingTypes(missingTypes);
 	}
 	return missingTypes;
 }
 
-/**
- * Collect the substitutes into a map for certain type variables inside the receiver type
- * e.g.   Collection<T>.collectSubstitutes(Collection<List<X>>, Map), will populate Map with: T --> List<X>
- * Constraints:
- *   A << F   corresponds to:   F.collectSubstitutes(..., A, ..., CONSTRAINT_EXTENDS (1))
- *   A = F   corresponds to:      F.collectSubstitutes(..., A, ..., CONSTRAINT_EQUAL (0))
- *   A >> F   corresponds to:   F.collectSubstitutes(..., A, ..., CONSTRAINT_SUPER (2))
-*/
+@Override
 public void collectSubstitutes(Scope scope, TypeBinding actualType, InferenceContext inferenceContext, int constraint) {
 
 	if ((this.tagBits & TagBits.HasTypeVariable) == 0) return;
@@ -123,10 +126,12 @@ public boolean mentionsAny(TypeBinding[] parameters, int idx) {
 	return this.leafComponentType.mentionsAny(parameters, idx);
 }
 
+@Override
 void collectInferenceVariables(Set<InferenceVariable> variables) {
 	this.leafComponentType.collectInferenceVariables(variables);
 }
 
+@Override
 TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding substituteType) {
 	TypeBinding substitutedLeaf = this.leafComponentType.substituteInferenceVariable(var, substituteType);
 	if (TypeBinding.notEquals(substitutedLeaf, this.leafComponentType))
@@ -138,17 +143,14 @@ TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding subst
  * brakets leafUniqueKey
  * p.X[][] --> [[Lp/X;
  */
+@Override
 public char[] computeUniqueKey(boolean isLeaf) {
 	char[] brackets = new char[this.dimensions];
 	for (int i = this.dimensions - 1; i >= 0; i--) brackets[i] = '[';
 	return CharOperation.concat(brackets, this.leafComponentType.computeUniqueKey(isLeaf));
  }
 
-/**
- * Answer the receiver's constant pool name.
- * NOTE: This method should only be used during/after code gen.
- * e.g. '[Ljava/lang/Object;'
- */
+@Override
 public char[] constantPoolName() {
 	if (this.constantPoolName != null)
 		return this.constantPoolName;
@@ -157,6 +159,7 @@ public char[] constantPoolName() {
 	for (int i = this.dimensions - 1; i >= 0; i--) brackets[i] = '[';
 	return this.constantPoolName = CharOperation.concat(brackets, this.leafComponentType.signature());
 }
+@Override
 public String debugName() {
 	if (this.hasTypeAnnotations())
 		return annotatedDebugName();
@@ -166,6 +169,7 @@ public String debugName() {
 	return this.leafComponentType.debugName() + brackets.toString();
 }
 
+@Override
 public String annotatedDebugName() {
 	StringBuffer brackets = new StringBuffer(this.dimensions * 2);
 	brackets.append(this.leafComponentType.annotatedDebugName());
@@ -185,6 +189,7 @@ public String annotatedDebugName() {
 	return brackets.toString();
 }
 
+@Override
 public int dimensions() {
 	return this.dimensions;
 }
@@ -214,16 +219,31 @@ public TypeBinding elementsType() {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#erasure()
  */
+@Override
 public TypeBinding erasure() {
     TypeBinding erasedType = this.leafComponentType.erasure();
     if (TypeBinding.notEquals(this.leafComponentType, erasedType))
         return this.environment.createArrayType(erasedType, this.dimensions);
     return this;
 }
+
+@Override
+public ArrayBinding upwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+	TypeBinding leafType = this.leafComponentType.upwardsProjection(scope, mentionedTypeVariables);
+	return scope.environment().createArrayType(leafType, this.dimensions, this.typeAnnotations);
+}
+
+@Override
+public ArrayBinding downwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+	TypeBinding leafType = this.leafComponentType.downwardsProjection(scope, mentionedTypeVariables);
+	return scope.environment().createArrayType(leafType, this.dimensions, this.typeAnnotations);
+}
+
 public LookupEnvironment environment() {
     return this.environment;
 }
 
+@Override
 public char[] genericTypeSignature() {
 
     if (this.genericTypeSignature == null) {
@@ -234,16 +254,19 @@ public char[] genericTypeSignature() {
     return this.genericTypeSignature;
 }
 
+@Override
 public PackageBinding getPackage() {
 	return this.leafComponentType.getPackage();
 }
 
+@Override
 public int hashCode() {
 	return this.leafComponentType == null ? super.hashCode() : this.leafComponentType.hashCode();
 }
 
 /* Answer true if the receiver type can be assigned to the argument type (right)
 */
+@Override
 public boolean isCompatibleWith(TypeBinding otherType, Scope captureScope) {
 	if (equalsEquals(this, otherType))
 		return true;
@@ -264,6 +287,13 @@ public boolean isCompatibleWith(TypeBinding otherType, Scope captureScope) {
 		case Binding.INTERSECTION_TYPE :
 		    return ((WildcardBinding) otherType).boundCheck(this);
 
+		case Binding.INTERSECTION_TYPE18:
+			for (ReferenceBinding intersecting : ((IntersectionTypeBinding18) otherType).intersectingTypes) {
+				if (!isCompatibleWith(intersecting, captureScope))
+					return false;
+			}
+			return true;
+
 		case Binding.TYPE_PARAMETER :
 			// check compatibility with capture of ? super X
 			if (otherType.isCapture()) {
@@ -289,7 +319,7 @@ public boolean isCompatibleWith(TypeBinding otherType, Scope captureScope) {
 }
 
 @Override
-public boolean isSubtypeOf(TypeBinding otherType) {
+public boolean isSubtypeOf(TypeBinding otherType, boolean simulatingBugJDK8026527) {
 	if (equalsEquals(this, otherType))
 		return true;
 
@@ -299,12 +329,18 @@ public boolean isSubtypeOf(TypeBinding otherType) {
 			if (otherArray.leafComponentType.isBaseType())
 				return false; // relying on the fact that all equal arrays are identical
 			if (this.dimensions == otherArray.dimensions)
-				return this.leafComponentType.isSubtypeOf(otherArray.leafComponentType);
+				return this.leafComponentType.isSubtypeOf(otherArray.leafComponentType, simulatingBugJDK8026527);
 			if (this.dimensions < otherArray.dimensions)
 				return false; // cannot assign 'String[]' into 'Object[][]' but can assign 'byte[][]' into 'Object[]'
 			break;
 		case Binding.BASE_TYPE :
 			return false;
+		case Binding.INTERSECTION_TYPE18:
+			for (ReferenceBinding intersecting : ((IntersectionTypeBinding18) otherType).intersectingTypes) {
+				if (!isSubtypeOf(intersecting, simulatingBugJDK8026527))
+					return false;
+			}
+			return true;
 	}
 	switch (otherType.leafComponentType().id) {
 	    case TypeIds.T_JavaLangObject :
@@ -315,18 +351,22 @@ public boolean isSubtypeOf(TypeBinding otherType) {
 	return false;
 }
 
+@Override
 public boolean isProperType(boolean admitCapture18) {
 	return this.leafComponentType.isProperType(admitCapture18);
 }
 
+@Override
 public int kind() {
 	return ARRAY_TYPE;
 }
 
+@Override
 public TypeBinding leafComponentType(){
 	return this.leafComponentType;
 }
 
+@Override
 public char[] nullAnnotatedReadableName(CompilerOptions options, boolean shortNames) /* java.lang.Object @o.e.j.a.NonNull[] */ {
 	if (this.nullTagBitsPerDimension == null)
 		return shortNames ? shortReadableName() : readableName();
@@ -359,15 +399,12 @@ public char[] nullAnnotatedReadableName(CompilerOptions options, boolean shortNa
 * Answer the problem id associated with the receiver.
 * NoError if the receiver is a valid binding.
 */
+@Override
 public int problemId() {
 	return this.leafComponentType.problemId();
 }
-/**
-* Answer the source name for the type.
-* In the case of member types, as the qualified name from its top level type.
-* For example, for a member type N defined inside M & A: "A.M.N".
-*/
 
+@Override
 public char[] qualifiedSourceName() {
 	char[] brackets = new char[this.dimensions * 2];
 	for (int i = this.dimensions * 2 - 1; i >= 0; i -= 2) {
@@ -376,6 +413,7 @@ public char[] qualifiedSourceName() {
 	}
 	return CharOperation.concat(this.leafComponentType.qualifiedSourceName(), brackets);
 }
+@Override
 public char[] readableName() /* java.lang.Object[] */ {
 	char[] brackets = new char[this.dimensions * 2];
 	for (int i = this.dimensions * 2 - 1; i >= 0; i -= 2) {
@@ -385,6 +423,7 @@ public char[] readableName() /* java.lang.Object[] */ {
 	return CharOperation.concat(this.leafComponentType.readableName(), brackets);
 }
 
+@Override
 public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNullAnnotations) {
 	this.tagBits |= TagBits.HasTypeAnnotations;
 	if (annotations == null || annotations.length == 0)
@@ -400,15 +439,12 @@ public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNull
 		for (int i = 0, length = annotations.length; i < length; i++) {
 			AnnotationBinding annotation = annotations[i];
 			if (annotation != null) {
-				switch (annotation.type.id) {
-					case TypeIds.T_ConfiguredAnnotationNullable :
-						nullTagBits  |= TagBits.AnnotationNullable;
-						this.tagBits |= TagBits.HasNullTypeAnnotation;
-						break;
-					case TypeIds.T_ConfiguredAnnotationNonNull :
-						nullTagBits  |= TagBits.AnnotationNonNull;
-						this.tagBits |= TagBits.HasNullTypeAnnotation;
-						break;
+				if (annotation.type.hasNullBit(TypeIds.BitNullableAnnotation)) {
+					nullTagBits  |= TagBits.AnnotationNullable;
+					this.tagBits |= TagBits.HasNullTypeAnnotation;
+				} else if (annotation.type.hasNullBit(TypeIds.BitNonNullAnnotation)) {
+					nullTagBits  |= TagBits.AnnotationNonNull;
+					this.tagBits |= TagBits.HasNullTypeAnnotation;
 				}
 			} else {
 				// null signals end of annotations for the current dimension in the serialized form.
@@ -422,6 +458,7 @@ public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNull
 		this.tagBits |= this.nullTagBitsPerDimension[0]; // outer-most dimension
 	}
 }
+@Override
 public char[] shortReadableName(){
 	char[] brackets = new char[this.dimensions * 2];
 	for (int i = this.dimensions * 2 - 1; i >= 0; i -= 2) {
@@ -430,6 +467,7 @@ public char[] shortReadableName(){
 	}
 	return CharOperation.concat(this.leafComponentType.shortReadableName(), brackets);
 }
+@Override
 public char[] sourceName() {
 	char[] brackets = new char[this.dimensions * 2];
 	for (int i = this.dimensions * 2 - 1; i >= 0; i -= 2) {
@@ -438,6 +476,7 @@ public char[] sourceName() {
 	}
 	return CharOperation.concat(this.leafComponentType.sourceName(), brackets);
 }
+@Override
 public void swapUnresolved(UnresolvedReferenceBinding unresolvedType, ReferenceBinding resolvedType, LookupEnvironment env) {
 	if (this.leafComponentType == unresolvedType) { //$IDENTITY-COMPARISON$
 		this.leafComponentType = env.convertUnresolvedBinaryToRawType(resolvedType);
@@ -458,9 +497,11 @@ public void swapUnresolved(UnresolvedReferenceBinding unresolvedType, ReferenceB
 		this.tagBits |= this.leafComponentType.tagBits & (TagBits.HasTypeVariable | TagBits.HasDirectWildcard | TagBits.HasMissingType | TagBits.HasCapturedWildcard);
 	}
 }
+@Override
 public String toString() {
 	return this.leafComponentType != null ? debugName() : "NULL TYPE ARRAY"; //$NON-NLS-1$
 }
+@Override
 public TypeBinding unannotated() {
 	return this.hasTypeAnnotations() ? this.environment.getUnannotatedType(this) : this;
 }
@@ -482,4 +523,49 @@ public TypeBinding uncapture(Scope scope) {
 public boolean acceptsNonNullDefault() {
 	return true;
 }
+@Override
+public long updateTagBits() {
+	if (this.leafComponentType != null)
+		this.tagBits |= this.leafComponentType.updateTagBits(); 
+	return super.updateTagBits();
+}
+
+/**
+ * The type of x.clone() is substituted from 'Object' into the type of the receiver array (non-null)
+ */
+public MethodBinding getCloneMethod(final MethodBinding originalMethod) {
+	if (this.clone != null)
+		return this.clone;
+	MethodBinding method = new MethodBinding() {
+		@Override
+		public char[] signature(ClassFile classFile) {
+			return originalMethod.signature(); // for codeGen we need to answer the signature of j.l.Object.clone()
+		}
+	};
+	method.modifiers = originalMethod.modifiers;
+	method.selector = originalMethod.selector;
+	method.declaringClass = originalMethod.declaringClass; // cannot set array binding as declaring class, will be tweaked in CodeStream.getConstantPoolDeclaringClass()
+	method.typeVariables = Binding.NO_TYPE_VARIABLES;
+	method.parameters = originalMethod.parameters;
+	method.thrownExceptions = Binding.NO_EXCEPTIONS;
+	method.tagBits = originalMethod.tagBits;
+	method.returnType = this.environment.globalOptions.sourceLevel >= ClassFileConstants.JDK1_5 ? this : originalMethod.returnType;
+	if (this.environment.globalOptions.isAnnotationBasedNullAnalysisEnabled) {
+		if (this.environment.usesNullTypeAnnotations())
+			method.returnType = this.environment.createAnnotatedType(method.returnType, new AnnotationBinding[] { this.environment.getNonNullAnnotation() });
+		else
+			method.tagBits |= TagBits.AnnotationNonNull;
+	}
+	if ((method.returnType.tagBits & TagBits.HasMissingType) != 0) {
+		method.tagBits |=  TagBits.HasMissingType;
+	}
+	return this.clone = method;
+}
+public static boolean isArrayClone(TypeBinding receiverType, MethodBinding binding) {
+	if (receiverType instanceof ArrayBinding) {
+		MethodBinding clone = ((ArrayBinding) receiverType).clone;
+		return clone != null && binding == clone;
+	}
+	return false;
+}
 }
