@@ -1,12 +1,18 @@
 /*******************************************************************************
- * Copyright (c) 2013, 2015 GK Software AG.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2013, 2017 GK Software AG.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Stephan Herrmann - initial API and implementation
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
+ *     IBM Corporation - Bug fixes
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
@@ -41,21 +47,32 @@ class BoundSet {
 		Set<TypeBound> subBounds;
 		TypeBinding	instantiation;
 		Map<InferenceVariable,TypeBound> inverseBounds; // from right inference variable to bound
-		
+		Set<InferenceVariable> dependencies;
 		public ThreeSets() {
 			// empty, the sets are lazily initialized
 		}
 		/** Add a type bound to the appropriate set. */
 		public boolean addBound(TypeBound bound) {
+			boolean result = addBound1(bound);
+			if(result) {
+				Set<InferenceVariable> set = (this.dependencies == null ? new HashSet<>() : this.dependencies);
+				bound.right.collectInferenceVariables(set);
+				if (this.dependencies == null && set.size() > 0) {
+					this.dependencies = set;
+				}
+			}
+			return result;
+		}
+		private boolean addBound1(TypeBound bound) {
 			switch (bound.relation) {
 				case ReductionResult.SUPERTYPE:
-					if (this.superBounds == null) this.superBounds = new HashSet<TypeBound>();
+					if (this.superBounds == null) this.superBounds = new HashSet<>();
 					return this.superBounds.add(bound);
 				case ReductionResult.SAME:
-					if (this.sameBounds == null) this.sameBounds = new HashSet<TypeBound>();
+					if (this.sameBounds == null) this.sameBounds = new HashSet<>();
 					return this.sameBounds.add(bound);
 				case ReductionResult.SUBTYPE:
-					if (this.subBounds == null) this.subBounds = new HashSet<TypeBound>();
+					if (this.subBounds == null) this.subBounds = new HashSet<>();
 					return this.subBounds.add(bound);
 				default:
 					throw new IllegalArgumentException("Unexpected bound relation in : " + bound); //$NON-NLS-1$
@@ -115,11 +132,7 @@ class BoundSet {
 		}
 		// pre: beta is a prototype
 		public boolean hasDependency(InferenceVariable beta) {
-			if (this.superBounds != null && hasDependency(this.superBounds, beta))
-				return true;
-			if (this.sameBounds != null && hasDependency(this.sameBounds, beta))
-				return true;
-			if (this.subBounds != null && hasDependency(this.subBounds, beta))
+			if(this.dependencies != null && this.dependencies.contains(beta))
 				return true;
 			if (this.inverseBounds != null) {
 				if (this.inverseBounds.containsKey(beta)) {
@@ -129,16 +142,6 @@ class BoundSet {
 			}
 			return false;
 		}
-		// pre: var is a prototype
-		private boolean hasDependency(Set<TypeBound> someBounds, InferenceVariable var) {
-			Iterator<TypeBound> bIt = someBounds.iterator();
-			while (bIt.hasNext()) {
-				TypeBound bound = bIt.next();
-				if (TypeBinding.equalsEquals(bound.right, var) || bound.right.mentionsAny(new TypeBinding[] {var}, -1))
-					return true;
-			}
-			return false;
-		}
 		/** Total number of type bounds in this container. */
 		public int size() {
 			int size = 0;
@@ -171,16 +174,33 @@ class BoundSet {
 		public ThreeSets copy() {
 			ThreeSets copy = new ThreeSets();
 			if (this.superBounds != null)
-				copy.superBounds = new HashSet<TypeBound>(this.superBounds);
+				copy.superBounds = new HashSet<>(this.superBounds);
 			if (this.sameBounds != null)
-				copy.sameBounds = new HashSet<TypeBound>(this.sameBounds);
+				copy.sameBounds = new HashSet<>(this.sameBounds);
 			if (this.subBounds != null)
-				copy.subBounds = new HashSet<TypeBound>(this.subBounds);
+				copy.subBounds = new HashSet<>(this.subBounds);
 			copy.instantiation = this.instantiation;
+			if (this.dependencies != null) {
+				copy.dependencies = new HashSet<>(this.dependencies);
+			}
 			return copy;
 		}
 		public TypeBinding findSingleWrapperType() {
-			TypeBinding wrapperBound = null;
+			if (this.instantiation != null) {
+				if (this.instantiation.isProperType(true)) {
+					switch (this.instantiation.id) {
+						case TypeIds.T_JavaLangByte:
+						case TypeIds.T_JavaLangShort:
+						case TypeIds.T_JavaLangCharacter:
+						case TypeIds.T_JavaLangInteger:
+						case TypeIds.T_JavaLangLong:
+						case TypeIds.T_JavaLangFloat:
+						case TypeIds.T_JavaLangDouble:
+						case TypeIds.T_JavaLangBoolean:
+							return this.instantiation;
+					}
+				}
+			}
 			if (this.subBounds != null) {
 				Iterator<TypeBound> it = this.subBounds.iterator();
 				while(it.hasNext()) {
@@ -195,9 +215,7 @@ class BoundSet {
 							case TypeIds.T_JavaLangFloat:
 							case TypeIds.T_JavaLangDouble:
 							case TypeIds.T_JavaLangBoolean:
-								if (wrapperBound != null)
-									return null;
-								wrapperBound = boundType;
+								return boundType;
 						}
 					}
 				}		
@@ -216,14 +234,12 @@ class BoundSet {
 							case TypeIds.T_JavaLangFloat:
 							case TypeIds.T_JavaLangDouble:
 							case TypeIds.T_JavaLangBoolean:
-								if (wrapperBound != null)
-									return null;
-								wrapperBound = boundType;
+								return boundType;
 						}
 					}
 				}		
 			}
-			return wrapperBound;
+			return null;
 		}
 		/**
 		 * Not per JLS: enhance the given type bounds using the nullHints, if useful.
@@ -294,15 +310,15 @@ class BoundSet {
 		}
 	}
 	// main storage of type bounds:
-	HashMap<InferenceVariable, ThreeSets> boundsPerVariable = new HashMap<InferenceVariable, ThreeSets>();
+	HashMap<InferenceVariable, ThreeSets> boundsPerVariable = new HashMap<>();
 	
 	/**
 	 * 18.1.3 bullet 4: G<α1, ..., αn> = capture(G<A1, ..., An>)
 	 * On both sides we only enter types with nonnull arguments. 
 	 */
-	HashMap<ParameterizedTypeBinding,ParameterizedTypeBinding> captures = new HashMap<ParameterizedTypeBinding, ParameterizedTypeBinding>();
+	HashMap<ParameterizedTypeBinding,ParameterizedTypeBinding> captures = new HashMap<>();
 	/** 18.1.3 bullet 5: throws α */
-	Set<InferenceVariable> inThrows = new HashSet<InferenceVariable>();
+	Set<InferenceVariable> inThrows = new HashSet<>();
 
 	private TypeBound [] incorporatedBounds = new TypeBound[0];
 	private TypeBound [] unincorporatedBounds = new TypeBound [1024];
@@ -410,7 +426,7 @@ class BoundSet {
 				if (three == null)
 					this.boundsPerVariable.put(rightIV, (three = new ThreeSets()));
 				if (three.inverseBounds == null)
-					three.inverseBounds = new HashMap<InferenceVariable,TypeBound>();
+					three.inverseBounds = new HashMap<>();
 				three.inverseBounds.put(rightIV, bound);
 			}
 		}
@@ -577,8 +593,14 @@ class BoundSet {
 							// not per JLS: if the new constraint relates types where at least one has a null annotations,
 							// record all null tagBits as hints for the final inference solution.
 							long nullHints = (newConstraint.left.tagBits | newConstraint.right.tagBits) & TagBits.AnnotationNullMASK;
-							boundI.nullHints |= nullHints;
-							boundJ.nullHints |= nullHints;
+							if (nullHints != 0) {
+								if (TypeBinding.equalsEquals(boundI.left, boundJ.left)
+										|| (boundI.relation == ReductionResult.SAME	&& TypeBinding.equalsEquals(boundI.right, boundJ.left))
+										|| (boundJ.relation == ReductionResult.SAME	&& TypeBinding.equalsEquals(boundI.left, boundJ.right))) {
+									boundI.nullHints |= nullHints;
+									boundJ.nullHints |= nullHints;
+								}
+							}
 						}
 					}
 					ConstraintFormula[] typeArgumentConstraints = deriveTypeArgumentConstraints ? deriveTypeArgumentConstraints(boundI, boundJ) : null;
@@ -634,15 +656,15 @@ class BoundSet {
 							it = three.sameBounds.iterator();
 							while (it.hasNext()) {
 								TypeBound bound = it.next();
-								if (InferenceContext18.SHOULD_WORKAROUND_BUG_JDK_8054721) {
-									if (bound.right instanceof CaptureBinding && bound.right.isProperType(true))
-										continue;
-								}
 								if (!(bound.right instanceof InferenceVariable))
 									return false;
 							}
 						}
-						if (three.subBounds != null && pi.firstBound != null) {
+						if (three.subBounds != null) {
+							TypeBinding bi1 = pi.firstBound;
+							if (bi1 == null) {
+								bi1 = context.object; // implicit bound
+							}
 							// If Bi is Object, α <: R implies ⟨T <: R⟩	(extends wildcard)
 							// α <: R implies ⟨θ Bi <: R⟩				(else) 
 							it = three.subBounds.iterator();
@@ -650,7 +672,6 @@ class BoundSet {
 								TypeBound bound = it.next();
 								if (!(bound.right instanceof InferenceVariable)) {
 									TypeBinding r = bound.right;
-									TypeBinding bi1 = pi.firstBound;
 									ReferenceBinding[] otherBounds = pi.superInterfaces;
 									TypeBinding bi;
 									if (otherBounds == Binding.NO_SUPERINTERFACES) {
@@ -663,13 +684,6 @@ class BoundSet {
 										bi = context.environment.createIntersectionType18(allBounds);
 									}
 									addTypeBoundsFromWildcardBound(context, theta, wildcardBinding.boundKind, t, r, bi);
-									//										if (otherBounds != null) {
-									//											for (int j = 0; j < otherBounds.length; j++) {
-									//												TypeBinding tj = otherBounds[j];
-									//												if (TypeBinding.notEquals(tj, t))
-									//													addTypeBoundsFromWildcardBound(context, wildcardBinding, tj, r, bij);
-									//											}
-									//										}
 								}
 							}
 						}
@@ -747,27 +761,38 @@ class BoundSet {
 		//  α = S and T <: α imply ⟨T <: S⟩
 		InferenceVariable alpha = boundS.left;
 		TypeBinding s = boundS.right;
-		if (TypeBinding.equalsEquals(alpha,boundT.left))
-			return ConstraintTypeFormula.create(s, boundT.right, boundT.relation, boundT.isSoft||boundS.isSoft);
-		if (TypeBinding.equalsEquals(alpha, boundT.right))
-			return ConstraintTypeFormula.create(boundT.right, s, boundT.relation, boundT.isSoft||boundS.isSoft);
+		if (TypeBinding.equalsEquals(alpha, boundT.left)) {
+			TypeBinding t = boundT.right;
+			return ConstraintTypeFormula.create(s, t, boundT.relation, boundT.isSoft||boundS.isSoft);
+		}
+		if (TypeBinding.equalsEquals(alpha, boundT.right)) {
+			TypeBinding t = boundT.left;
+			return ConstraintTypeFormula.create(t, s, boundT.relation, boundT.isSoft||boundS.isSoft);
+		}
 
 		if (boundS.right instanceof InferenceVariable) {
 			// reverse:
 			alpha = (InferenceVariable) boundS.right;
 			s = boundS.left;
-			if (TypeBinding.equalsEquals(alpha, boundT.left))
-				return ConstraintTypeFormula.create(s, boundT.right, boundT.relation, boundT.isSoft||boundS.isSoft);
-			if (TypeBinding.equalsEquals(alpha, boundT.right))
-				return ConstraintTypeFormula.create(boundT.right, s, boundT.relation, boundT.isSoft||boundS.isSoft);			
+			if (TypeBinding.equalsEquals(alpha, boundT.left)) {
+				TypeBinding t = boundT.right;
+				return ConstraintTypeFormula.create(s, t, boundT.relation, boundT.isSoft||boundS.isSoft);
+			}
+			if (TypeBinding.equalsEquals(alpha, boundT.right)) {
+				TypeBinding t = boundT.left;
+				return ConstraintTypeFormula.create(t, s, boundT.relation, boundT.isSoft||boundS.isSoft);
+			}			
 		}
 		
 		//  α = U and S <: T imply ⟨S[α:=U] <: T[α:=U]⟩ 
 		TypeBinding u = boundS.right;
 		if (u.isProperType(true)) {
-			TypeBinding left = (TypeBinding.equalsEquals(alpha, boundT.left)) ? u : boundT.left;
+			boolean substitute = TypeBinding.equalsEquals(alpha, boundT.left);
+			TypeBinding left = substitute ? u : boundT.left;
 			TypeBinding right = boundT.right.substituteInferenceVariable(alpha, u);
-			return ConstraintTypeFormula.create(left, right, boundT.relation, boundT.isSoft||boundS.isSoft);
+			substitute |= TypeBinding.notEquals(right, boundT.right);
+			if (substitute) // avoid redundant constraint
+				return ConstraintTypeFormula.create(left, right, boundT.relation, boundT.isSoft||boundS.isSoft);
 		}
 		return null;
 	}
@@ -822,7 +847,7 @@ class BoundSet {
 		TypeBinding[] tis = t.typeArguments();
 		if (sis == null || tis == null || sis.length != tis.length)
 			return null;
-		List<ConstraintTypeFormula> result = new ArrayList<ConstraintTypeFormula>(); 
+		List<ConstraintTypeFormula> result = new ArrayList<>(); 
 		for (int i = 0; i < sis.length; i++) {
 			TypeBinding si = sis[i];
 			TypeBinding ti = tis[i];
@@ -873,6 +898,8 @@ class BoundSet {
 	public boolean dependsOnResolutionOf(InferenceVariable alpha, InferenceVariable beta) {
 		alpha = alpha.prototype();
 		beta = beta.prototype();
+		if (TypeBinding.equalsEquals(alpha, beta))
+			return true; // An inference variable α depends on the resolution of itself.
 		Iterator<Map.Entry<ParameterizedTypeBinding, ParameterizedTypeBinding>> captureIter = this.captures.entrySet().iterator();
 		boolean betaIsInCaptureLhs = false;
 		while (captureIter.hasNext()) { // TODO: optimization: consider separate index structure (by IV)
@@ -904,6 +931,44 @@ class BoundSet {
 		return false;
 	}
 
+	List<Set<InferenceVariable>> computeConnectedComponents(InferenceVariable[] inferenceVariables) {
+		// create all dependency edges (as bi-directional):
+		Map<InferenceVariable, Set<InferenceVariable>> allEdges = new HashMap<>();
+		for (int i = 0; i < inferenceVariables.length; i++) {
+			InferenceVariable iv1 = inferenceVariables[i];
+			HashSet<InferenceVariable> targetSet = new HashSet<InferenceVariable>();
+			allEdges.put(iv1, targetSet); // eventually ensures: forall iv in inferenceVariables : allEdges.get(iv) != null
+			for (int j = 0; j < i; j++) {
+				InferenceVariable iv2 = inferenceVariables[j];
+				if (dependsOnResolutionOf(iv1, iv2) || dependsOnResolutionOf(iv2, iv1)) {
+					targetSet.add(iv2);
+					allEdges.get(iv2).add(iv1);
+				}
+			}
+		}
+		// collect all connected IVs into one component:
+		Set<InferenceVariable> visited = new HashSet<>();
+		List<Set<InferenceVariable>> allComponents = new ArrayList<>();
+		for (InferenceVariable inferenceVariable : inferenceVariables) {
+			Set<InferenceVariable> component = new HashSet<>();
+			addConnected(component, inferenceVariable, allEdges, visited);
+			if (!component.isEmpty())
+				allComponents.add(component);
+		}
+		return allComponents;
+	}
+
+	private void addConnected(Set<InferenceVariable> component, InferenceVariable seed,
+			Map<InferenceVariable, Set<InferenceVariable>> allEdges, Set<InferenceVariable> visited)
+	{
+		if (visited.add(seed)) {
+			// add all IVs starting from seed and reachable via any in allEdges:
+			component.add(seed);
+			for (InferenceVariable next : allEdges.get(seed))
+				addConnected(component, next, allEdges, visited);
+		}
+	}
+
 	// helper for 18.4
 	public boolean hasCaptureBound(Set<InferenceVariable> variableSet) {
 		Iterator<ParameterizedTypeBinding> captureIter = this.captures.keySet().iterator();
@@ -960,6 +1025,7 @@ class BoundSet {
 	}
 
 	// debugging:
+	@Override
 	public String toString() {
 		StringBuffer buf = new StringBuffer("Type Bounds:\n"); //$NON-NLS-1$
 		TypeBound[] flattened = flatten();
@@ -1010,7 +1076,7 @@ class BoundSet {
 		// ii) B2 contains two bounds of the forms S1 <: α and S2 <: α, where
 		//     S1 and S2 have supertypes (4.10) that are two different parameterizations of the same generic class or interface.
 		if (ts.superBounds != null) {
-			ArrayList<TypeBound> superBounds = new ArrayList<TypeBound>(ts.superBounds);
+			ArrayList<TypeBound> superBounds = new ArrayList<>(ts.superBounds);
 			int len = superBounds.size();
 			for (int i=0; i<len; i++) {
 				TypeBinding s1 = superBounds.get(i).right;
@@ -1103,8 +1169,8 @@ class BoundSet {
 			}
 		}
 		for (InferenceVariable iv : outerVariables) {
-			three = this.boundsPerVariable.get(outerVariables);
-			if (three != null) {
+			three = this.boundsPerVariable.get(iv);
+			if (three != null && three.sameBounds != null) {
 				for (TypeBound bound : three.sameBounds)
 					if (TypeBinding.equalsEquals(bound.right, variable))
 						return iv;
