@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -14,10 +18,15 @@
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Map;
+
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRestriction;
+import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.*;
 
@@ -57,15 +66,22 @@ public class CompilationUnitScope extends Scope {
 		private boolean skipCachingImports;
 	
 	boolean connectingHierarchy;
+	private ArrayList<Invocation> inferredInvocations;
+	/** Cache of interned inference variables. Access only via {@link InferenceVariable#get(TypeBinding, int, InvocationSite, Scope, ReferenceBinding)}. */
+	Map<InferenceVariable.InferenceVarKey, InferenceVariable> uniqueInferenceVariables = new HashMap<>();
 
 public CompilationUnitScope(CompilationUnitDeclaration unit, LookupEnvironment environment) {
-	super(COMPILATION_UNIT_SCOPE, null);
+	this(unit, environment.globalOptions);
 	this.environment = environment;
+}
+
+public CompilationUnitScope(CompilationUnitDeclaration unit, CompilerOptions compilerOptions) {
+	super(COMPILATION_UNIT_SCOPE, null);
 	this.referenceContext = unit;
 	unit.scope = this;
 	this.currentPackageName = unit.currentPackage == null ? CharOperation.NO_CHAR_CHAR : unit.currentPackage.tokens;
 
-	if (compilerOptions().produceReferenceInfo) {
+	if (compilerOptions.produceReferenceInfo) {
 		this.qualifiedReferences = new CompoundNameVector();
 		this.simpleNameReferences = new SimpleNameVector();
 		this.rootReferences = new SimpleNameVector();
@@ -78,6 +94,7 @@ public CompilationUnitScope(CompilationUnitDeclaration unit, LookupEnvironment e
 		this.referencedTypes = null;
 		this.referencedSuperTypes = null;
 	}
+	// client still needs to assign #environment
 }
 public void buildFieldsAndMethods() { // AspectJ Extension - raised to public
 	for (int i = 0, length = this.topLevelTypes.length; i < length; i++)
@@ -88,7 +105,7 @@ void buildTypeBindings(AccessRestriction accessRestriction) {
 	boolean firstIsSynthetic = false;
 	if (this.referenceContext.compilationResult.compilationUnit != null) {
 		char[][] expectedPackageName = this.referenceContext.compilationResult.compilationUnit.getPackageName();
-		if (expectedPackageName != null
+		if (expectedPackageName != null && !this.referenceContext.isModuleInfo()
 				&& !CharOperation.equals(this.currentPackageName, expectedPackageName)) {
 
 			// only report if the unit isn't structurally empty
@@ -103,6 +120,15 @@ void buildTypeBindings(AccessRestriction accessRestriction) {
 	if (this.currentPackageName == CharOperation.NO_CHAR_CHAR) {
 		// environment default package is never null
 		this.fPackage = this.environment.defaultPackage;
+		if (this.referenceContext.isModuleInfo()) { // cannot have a package as per grammar
+			ModuleDeclaration moduleDecl = this.referenceContext.moduleDeclaration;
+			if (moduleDecl != null) {
+				moduleDecl.createScope(this);
+				moduleDecl.checkAndSetModifiers();
+			}
+		} else if (this.environment.module != this.environment.UnNamedModule) {
+			problemReporter().unnamedPackageInNamedModule(this.environment.module);
+		}
 	} else {
 		if ((this.fPackage = this.environment.createPackage(this.currentPackageName)) == null) {
 			if (this.referenceContext.currentPackage != null) {
@@ -133,13 +159,15 @@ void buildTypeBindings(AccessRestriction accessRestriction) {
 	int count = 0;
 	nextType: for (int i = 0; i < typeLength; i++) {
 		TypeDeclaration typeDecl = types[i];
-		if (this.environment.isProcessingAnnotations && this.environment.isMissingType(typeDecl.name))
+		if (this.environment.root.isProcessingAnnotations && this.environment.isMissingType(typeDecl.name))
 			throw new SourceTypeCollisionException(); // resolved a type ref before APT generated the type
 		ReferenceBinding typeBinding = this.fPackage.getType0(typeDecl.name);
+		if (Binding.isValid(typeBinding) && this.fPackage instanceof SplitPackageBinding && !this.environment.module.canAccess(typeBinding.fPackage))
+			typeBinding = null;
 		recordSimpleReference(typeDecl.name); // needed to detect collision cases
-		if (typeBinding != null && typeBinding.isValidBinding() && !(typeBinding instanceof UnresolvedReferenceBinding)) {
+		if (Binding.isValid(typeBinding) && !(typeBinding instanceof UnresolvedReferenceBinding)) {
 			// if its an unresolved binding - its fixed up whenever its needed, see UnresolvedReferenceBinding.resolve()
-			if (this.environment.isProcessingAnnotations)
+			if (this.environment.root.isProcessingAnnotations)
 				throw new SourceTypeCollisionException(); // resolved a type ref before APT generated the type
 			// if a type exists, check that its a valid type
 			// it can be a NotFound problem type if its a secondary type referenced before its primary type found in additional units
@@ -147,11 +175,6 @@ void buildTypeBindings(AccessRestriction accessRestriction) {
 			problemReporter().duplicateTypes(this.referenceContext, typeDecl);
 			continue nextType;
 		}
-		if (this.fPackage != this.environment.defaultPackage && this.fPackage.getPackage(typeDecl.name) != null) {
-			// if a package exists, it must be a valid package - cannot be a NotFound problem package
-			// this is now a warning since a package does not really 'exist' until it contains a type, see JLS v2, 7.4.3
-			problemReporter().typeCollidesWithPackage(this.referenceContext, typeDecl);
-		}
 
 		if ((typeDecl.modifiers & ClassFileConstants.AccPublic) != 0) {
 			char[] mainTypeName;
@@ -175,6 +198,24 @@ void buildTypeBindings(AccessRestriction accessRestriction) {
 		System.arraycopy(this.topLevelTypes, 0, this.topLevelTypes = new SourceTypeBinding[count], 0, count);
 }
 public void checkAndSetImports() { // AspectJ Extension - raised to public
+	// TODO(SHMOD): verify: this block moved here from buildTypeBindings.
+	// package resolving may require all modules to be known
+	TypeDeclaration[] types = this.referenceContext.types;
+	if (types != null) {
+		for (int i = 0; i < types.length; i++) {
+			TypeDeclaration typeDecl = types[i];
+			if (this.fPackage != this.environment.defaultPackage && this.fPackage.getPackage(typeDecl.name, module()) != null) {
+				// if a package exists, it must be a valid package - cannot be a NotFound problem package
+				// this is now a warning since a package does not really 'exist' until it contains a type, see JLS v2, 7.4.3
+				problemReporter().typeCollidesWithPackage(this.referenceContext, typeDecl);
+			}
+		}
+	}
+
+	if (this.referenceContext.moduleDeclaration != null) {
+		this.referenceContext.moduleDeclaration.resolveModuleDirectives(this);
+		this.referenceContext.moduleDeclaration.resolvePackageDirectives(this);
+	}
 	if (this.referenceContext.imports == null) {
 		this.imports = getDefaultImports();
 		return;
@@ -342,7 +383,6 @@ public void connectTypeHierarchy() { // AspectJ Extension - raised to public
 	} finally {
 		this.connectingHierarchy = false;
 	}
-
 }
 void faultInImports() {
 	boolean unresolvedFound = false;
@@ -399,16 +439,19 @@ void faultInImports() {
 			}
 		}
 		if ((importReference.bits & ASTNode.OnDemand) != 0) {
-			if (CharOperation.equals(compoundName, this.currentPackageName)) {
-				problemReporter().unusedImport(importReference); // since skipped, must be reported now
-				continue nextImport;
-			}
-
 			Binding importBinding = findImport(compoundName, compoundName.length);
 			if (!importBinding.isValidBinding()) {
 				problemReporter().importProblem(importReference, importBinding);
 				continue nextImport;
 			}
+			if (importBinding instanceof PackageBinding) {
+				PackageBinding uniquePackage = ((PackageBinding)importBinding).getVisibleFor(module());
+				if (uniquePackage instanceof SplitPackageBinding) {
+					SplitPackageBinding splitPackage = (SplitPackageBinding) uniquePackage;
+					problemReporter().conflictingPackagesFromModules(splitPackage, importReference.sourceStart, importReference.sourceEnd);
+					continue nextImport;
+				}
+			}
 			if (importReference.isStatic() && importBinding instanceof PackageBinding) {
 				problemReporter().cannotImportPackage(importReference);
 				continue nextImport;
@@ -430,6 +473,24 @@ void faultInImports() {
 			if (importBinding instanceof PackageBinding) {
 				problemReporter().cannotImportPackage(importReference);
 				continue nextImport;
+			} else if (this.environment.useModuleSystem && importBinding instanceof ReferenceBinding) {
+				PackageBinding importedPackage = ((ReferenceBinding) importBinding).fPackage;
+				if (importedPackage != null) {
+					if (!importedPackage.isValidBinding()) {
+						problemReporter().importProblem(importReference, importedPackage);
+						continue nextImport;
+			}
+					// re-get to find a possible split package:
+					importedPackage = (PackageBinding) findImport(importedPackage.compoundName, false, true);
+					if (importedPackage != null)
+						importedPackage = importedPackage.getVisibleFor(module());
+					if (importedPackage instanceof SplitPackageBinding) {
+						SplitPackageBinding splitPackage = (SplitPackageBinding) importedPackage;
+						int sourceEnd = (int) importReference.sourcePositions[splitPackage.compoundName.length-1];
+						problemReporter().conflictingPackagesFromModules(splitPackage, importReference.sourceStart, sourceEnd);
+						continue nextImport;
+					}
+				}
 			}
 			// all the code here which checks for valid bindings have been moved to the method 
 			// checkAndRecordImportBinding() since bug 361327
@@ -464,6 +525,11 @@ void faultInImports() {
 }
 public void faultInTypes() {
 	faultInImports();
+	if (this.referenceContext.moduleDeclaration != null) {
+		this.referenceContext.moduleDeclaration.resolveTypeDirectives(this);
+	} else if (this.referenceContext.currentPackage != null) {
+		this.referenceContext.currentPackage.checkPackageConflict(this);
+	}
 
 	for (int i = 0, length = this.topLevelTypes.length; i < length; i++)
 		this.topLevelTypes[i].faultInTypesForFieldsAndMethods();
@@ -478,13 +544,16 @@ public Binding findImport(char[][] compoundName, boolean findStaticImports, bool
 }
 private Binding findImport(char[][] compoundName, int length) {
 	recordQualifiedReference(compoundName);
-
+	ModuleBinding module = module();
 	Binding binding = this.environment.getTopLevelPackage(compoundName[0]);
 	int i = 1;
 	foundNothingOrType: if (binding != null) {
 		PackageBinding packageBinding = (PackageBinding) binding;
 		while (i < length) {
-			binding = packageBinding.getTypeOrPackage(compoundName[i++]);
+			binding = packageBinding.getTypeOrPackage(compoundName[i++], module);
+			if (binding instanceof ReferenceBinding && binding.problemId() == ProblemReasons.NotAccessible) {
+				return this.environment.convertToRawType((TypeBinding) binding, false /*do not force conversion of enclosing types*/);
+			}
 			if (binding == null || !binding.isValidBinding()) {
 				binding = null;
 				break foundNothingOrType;
@@ -494,6 +563,8 @@ private Binding findImport(char[][] compoundName, int length) {
 
 			packageBinding = (PackageBinding) binding;
 		}
+		if (packageBinding.isValidBinding() && !module.canAccess(packageBinding))
+			return new ProblemPackageBinding(compoundName, ProblemReasons.NotAccessible);
 		return packageBinding;
 	}
 
@@ -528,7 +599,7 @@ private Binding findSingleImport(char[][] compoundName, int mask, boolean findSt
 	if (compoundName.length == 1) {
 		// findType records the reference
 		// the name cannot be a package
-		if (compilerOptions().complianceLevel >= ClassFileConstants.JDK1_4)
+		if (compilerOptions().complianceLevel >= ClassFileConstants.JDK1_4 && !this.referenceContext.isModuleInfo())
 			return new ProblemReferenceBinding(compoundName, null, ProblemReasons.NotFound);
 		ReferenceBinding typeBinding = findType(compoundName[0], this.environment.defaultPackage, this.fPackage);
 		if (typeBinding == null)
@@ -546,7 +617,7 @@ private Binding findSingleStaticImport(char[][] compoundName, int mask) {
 
 	char[] name = compoundName[compoundName.length - 1];
 	if (binding instanceof PackageBinding) {
-		Binding temp = ((PackageBinding) binding).getTypeOrPackage(name);
+		Binding temp = ((PackageBinding) binding).getTypeOrPackage(name, module());
 		if (temp != null && temp instanceof ReferenceBinding) // must resolve to a member type or field, not a top level type
 			return new ProblemReferenceBinding(compoundName, (ReferenceBinding) temp, ProblemReasons.InvalidTypeForStaticImport);
 		return binding; // cannot be a package, error is caught in sender
@@ -598,11 +669,11 @@ private MethodBinding findStaticMethod(ReferenceBinding currentType, char[] sele
 }
 ImportBinding[] getDefaultImports() {
 	// initialize the default imports if necessary... share the default java.lang.* import
-	if (this.environment.defaultImports != null) return this.environment.defaultImports;
+	if (this.environment.root.defaultImports != null) return this.environment.root.defaultImports;
 
 	Binding importBinding = this.environment.getTopLevelPackage(TypeConstants.JAVA);
 	if (importBinding != null)
-		importBinding = ((PackageBinding) importBinding).getTypeOrPackage(TypeConstants.JAVA_LANG[1]);
+		importBinding = ((PackageBinding) importBinding).getTypeOrPackage(TypeConstants.JAVA_LANG[1], module());
 
 	if (importBinding == null || !importBinding.isValidBinding()) {
 		// create a proxy for the missing BinaryType
@@ -614,7 +685,7 @@ ImportBinding[] getDefaultImports() {
 		importBinding = missingObject.fPackage;
 	}
 
-	return this.environment.defaultImports = new ImportBinding[] {new ImportBinding(TypeConstants.JAVA_LANG, true, importBinding, null)};
+	return this.environment.root.defaultImports = new ImportBinding[] {new ImportBinding(TypeConstants.JAVA_LANG, true, importBinding, null)};
 }
 // NOT Public API
 public final Binding getImport(char[][] compoundName, boolean onDemand, boolean isStaticImport) {
@@ -633,6 +704,7 @@ public int nextCaptureID() {
 * (unit, type or method) in case the problem handler decides it is necessary
 * to abort.
 */
+@Override
 public ProblemReporter problemReporter() {
 	ProblemReporter problemReporter = this.referenceContext.problemReporter;
 	problemReporter.referenceContext = this.referenceContext;
@@ -815,10 +887,13 @@ public void storeDependencyInfo() {
 		rootRefs[i] = this.rootReferences.elementAt(i);
 	this.referenceContext.compilationResult.rootReferences = rootRefs;
 }
+@Override
 public String toString() {
 	return "--- CompilationUnit Scope : " + new String(this.referenceContext.getFileName()); //$NON-NLS-1$
 }
 private ReferenceBinding typeToRecord(TypeBinding type) {
+	if (type == null)
+		return null;
 	while (type.isArrayType())
 		type = ((ArrayBinding) type).leafComponentType();
 
@@ -827,7 +902,7 @@ private ReferenceBinding typeToRecord(TypeBinding type) {
 		case Binding.TYPE_PARAMETER :
 		case Binding.WILDCARD_TYPE :
 		case Binding.INTERSECTION_TYPE :
-		case Binding.INTERSECTION_CAST_TYPE: // constituents would have been recorded.
+		case Binding.INTERSECTION_TYPE18: // constituents would have been recorded.
 		case Binding.POLY_TYPE: // not a real type, will mutate into one, hopefully soon.
 			return null;
 		case Binding.PARAMETERIZED_TYPE :
@@ -931,7 +1006,6 @@ private int checkAndRecordImportBinding(
 						}
 					} else if (resolved.resolvedImport == referenceBinding) {
 						if (importReference.isStatic() != resolved.isStatic()) {
-							problemReporter().duplicateImport(importReference);
 							recordImportBinding(new ImportBinding(compoundName, false, importBinding, importReference));
 						}
 					}
@@ -989,9 +1063,39 @@ private int checkAndRecordImportBinding(
 	return this.importPtr;
 }
 @Override
-public boolean hasDefaultNullnessFor(int location) {
+public boolean hasDefaultNullnessFor(int location, int sourceStart) {
+	int nonNullByDefaultValue = localNonNullByDefaultValue(sourceStart);
+	if (nonNullByDefaultValue != 0) {
+		return (nonNullByDefaultValue & location) != 0;
+	}
 	if (this.fPackage != null)
-		return (this.fPackage.defaultNullness & location) != 0;
+		return (this.fPackage.getDefaultNullness() & location) != 0;
 	return false;
 }
+
+@Override
+public /* @Nullable */ Binding checkRedundantDefaultNullness(int nullBits, int sourceStart) {
+	Binding target = localCheckRedundantDefaultNullness(nullBits, sourceStart);
+	if (target != null) {
+		return target;
+	}
+	if (this.fPackage != null) {
+		return this.fPackage.findDefaultNullnessTarget(n -> n == nullBits);
+	}
+
+	return null;
+}
+
+public void registerInferredInvocation(Invocation invocation) {
+	if (this.inferredInvocations == null)
+		this.inferredInvocations = new ArrayList<>();
+	this.inferredInvocations.add(invocation);
+}
+public void cleanUpInferenceContexts() {
+	if (this.inferredInvocations == null)
+		return;
+	for (Invocation invocation : this.inferredInvocations)
+		invocation.cleanUpInferenceContexts();
+	this.inferredInvocations = null;
+}
 }
