@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -15,6 +19,8 @@
  *								Bug 435805 - [1.8][compiler][null] Java 8 compiler does not recognize declaration style null annotations
  *								Bug 458396 - NPE in CodeStream.invoke()
  *								Bug 446217 - [null] @NonNullByDefault in package-info.java causes bogus "null type safety" warning
+ *     Till Brychcy - Contribution for
+ *     						    bug 467094 - [1.8][null] TYPE_USE NullAnnotations of array contents are applied to field.
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
@@ -44,7 +50,7 @@ public FieldBinding(FieldBinding initialFieldBinding, ReferenceBinding declaring
 	super(initialFieldBinding.name, initialFieldBinding.type, initialFieldBinding.modifiers, initialFieldBinding.constant());
 	this.declaringClass = declaringClass;
 	this.id = initialFieldBinding.id;
-    if (declaringClass!=null) setAnnotations(initialFieldBinding.getAnnotations());	// New AspectJ Extension - null guard
+    if (declaringClass!=null) setAnnotations(initialFieldBinding.getAnnotations(), false);	// New AspectJ Extension - null guard
 }
 /* API
 * Answer the receiver's binding type from Binding.BindingID.
@@ -169,6 +175,7 @@ public boolean canBeSeenBy(TypeBinding receiverType, InvocationSite invocationSi
  * declaringUniqueKey dot fieldName ) returnTypeUniqueKey
  * p.X { X<T> x} --> Lp/X;.x)p/X<TT;>;
  */
+@Override
 public char[] computeUniqueKey(boolean isLeaf) {
 	// declaring key
 	char[] declaringKey =
@@ -195,6 +202,7 @@ public char[] computeUniqueKey(boolean isLeaf) {
 	System.arraycopy(returnTypeKey, 0, uniqueKey, index, returnTypeLength);
 	return uniqueKey;
 }
+@Override
 public Constant constant() {
 	Constant fieldConstant = this.constant;
 	if (fieldConstant == null) {
@@ -232,6 +240,7 @@ public Constant constant() {
 	return fieldConstant;
 }
 
+@Override
 public Constant constant(Scope scope) {
 	if (this.constant != null)
 		return this.constant;
@@ -245,21 +254,23 @@ public Constant constant(Scope scope) {
 }
 
 public void fillInDefaultNonNullness(FieldDeclaration sourceField, Scope scope) {
+	if (this.type == null || this.type.isBaseType())
+		return;
 	LookupEnvironment environment = scope.environment();
-	if (this.type == null)
-		return;
-	if (environment.usesNullTypeAnnotations() && !this.type.acceptsNonNullDefault())
-		return;
-	if (   !this.type.isBaseType()
-		&& (this.tagBits & TagBits.AnnotationNullMASK) == 0 		// declaration annotation?
-		&& (this.type.tagBits & TagBits.AnnotationNullMASK) == 0)	// type annotation? (java.lang.@Nullable String)
-	{
-		if (environment.usesNullTypeAnnotations())
+	if (environment.usesNullTypeAnnotations()) {
+		if (!this.type.acceptsNonNullDefault())
+			return;
+		if ( (this.type.tagBits & TagBits.AnnotationNullMASK) == 0) {
 			this.type = environment.createAnnotatedType(this.type, new AnnotationBinding[]{environment.getNonNullAnnotation()});
-		else
+		} else if ((this.type.tagBits & TagBits.AnnotationNonNull) != 0) {
+			scope.problemReporter().nullAnnotationIsRedundant(sourceField);
+		}
+	} else {
+		if ( (this.tagBits & TagBits.AnnotationNullMASK) == 0 ) {
 			this.tagBits |= TagBits.AnnotationNonNull;
-	} else if ((this.tagBits & TagBits.AnnotationNonNull) != 0) {
-		scope.problemReporter().nullAnnotationIsRedundant(sourceField);
+		} else if ((this.tagBits & TagBits.AnnotationNonNull) != 0) {
+			scope.problemReporter().nullAnnotationIsRedundant(sourceField);
+		}		
 	}
 }
 
@@ -274,6 +285,7 @@ public final int getAccessFlags() {
 	return this.modifiers & ExtraCompilerModifiers.AccJustFlag;
 }
 
+@Override
 public AnnotationBinding[] getAnnotations() {
 	FieldBinding originalField = original();
 	ReferenceBinding declaringClassBinding = originalField.declaringClass;
@@ -288,6 +300,7 @@ public AnnotationBinding[] getAnnotations() {
  * lazily resolving corresponding annotation nodes, in case of forward references.
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding#getAnnotationTagBits()
  */
+@Override
 public long getAnnotationTagBits() {
 	FieldBinding originalField = original();
 	if ((originalField.tagBits & TagBits.AnnotationResolved) == 0 && originalField.declaringClass instanceof SourceTypeBinding) {
@@ -392,10 +405,12 @@ public final boolean isViewedAsDeprecated() {
 /* Answer true if the receiver is a volatile field
 */
 
+@Override
 public final boolean isVolatile() {
 	return (this.modifiers & ClassFileConstants.AccVolatile) != 0;
 }
 
+@Override
 public final int kind() {
 	return FIELD;
 }
@@ -407,8 +422,9 @@ public final int kind() {
 public FieldBinding original() {
 	return this;
 }
-public void setAnnotations(AnnotationBinding[] annotations) {
-	this.declaringClass.storeAnnotations(this, annotations);
+@Override
+public void setAnnotations(AnnotationBinding[] annotations, boolean forceStore) {
+	this.declaringClass.storeAnnotations(this, annotations, forceStore);
 }
 public FieldDeclaration sourceField() {
 	SourceTypeBinding sourceType;
