@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -15,9 +18,6 @@
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
-import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
-import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Annotation;
-
 public abstract class NestedTypeBinding extends SourceTypeBinding {
 
 	public SourceTypeBinding enclosingType;
@@ -126,20 +126,9 @@ public SyntheticArgumentBinding addSyntheticArgumentAndField(ReferenceBinding ta
 	return synthLocal;
 }
 
-protected void checkRedundantNullnessDefaultRecurse(ASTNode location, Annotation[] annotations, long nullBits, boolean useNullTypeAnnotations) {
-	if (!isPrototype()) throw new IllegalStateException();
-	ReferenceBinding currentType = this.enclosingType;
-	do {
-		if (!((SourceTypeBinding)currentType).checkRedundantNullnessDefaultOne(location, annotations, nullBits, useNullTypeAnnotations)) {
-			return;
-		}
-		currentType = currentType.enclosingType();
-	} while (currentType instanceof SourceTypeBinding);
-	super.checkRedundantNullnessDefaultRecurse(location, annotations, nullBits, useNullTypeAnnotations);
-}
-
 /* Answer the receiver's enclosing type... null if the receiver is a top level type.
 */
+@Override
 public ReferenceBinding enclosingType() {
 	return this.enclosingType;
 }
@@ -147,6 +136,7 @@ public ReferenceBinding enclosingType() {
 /**
  * @return the enclosingInstancesSlotSize
  */
+@Override
 public int getEnclosingInstancesSlotSize() {
 	if (!isPrototype()) throw new IllegalStateException();
 	return this.enclosingInstances == null ? 0 : this.enclosingInstances.length;
@@ -155,6 +145,7 @@ public int getEnclosingInstancesSlotSize() {
 /**
  * @return the outerLocalVariablesSlotSize
  */
+@Override
 public int getOuterLocalVariablesSlotSize() {
 	if (!isPrototype()) throw new IllegalStateException();
 	if (this.outerLocalVariablesSlotSize < 0) {
@@ -235,6 +226,7 @@ public SyntheticArgumentBinding[] syntheticEnclosingInstances() {
 	return this.enclosingInstances;		// is null if no enclosing instances are required
 }
 
+@Override
 public ReferenceBinding[] syntheticEnclosingInstanceTypes() {
 	if (!isPrototype()) throw new IllegalStateException();
 	if (this.enclosingTypes == UNINITIALIZED_REFERENCE_TYPES) {
@@ -251,6 +243,7 @@ public ReferenceBinding[] syntheticEnclosingInstanceTypes() {
 	return this.enclosingTypes;
 }
 
+@Override
 public SyntheticArgumentBinding[] syntheticOuterLocalVariables() {
 	if (!isPrototype()) throw new IllegalStateException();
 	return this.outerLocalVariables;		// is null if no outer locals are required
