@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *      Stephen Herrmann <stephan@cs.tu-berlin.de> -  Contributions for
@@ -30,6 +33,8 @@
  *								Bug 446434 - [1.8][null] Enable interned captures also when analysing null type annotations
  *      Jesper S Moller <jesper@selskabet.org> -  Contributions for
  *								bug 382701 - [1.8][compiler] Implement semantic analysis of Lambda expressions & Reference expression
+ *								bug 527554 - [18.3] Compiler support for JEP 286 Local-Variable Type
+ *
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
@@ -50,7 +55,6 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
  *
  * null is NOT a valid value for a non-public field... it just means the field is not initialized.
  */
-@SuppressWarnings("rawtypes")
 abstract public class TypeBinding extends Binding {
 
 	public int id = TypeIds.NoId;
@@ -61,7 +65,9 @@ abstract public class TypeBinding extends Binding {
 	// jsr 308
 	public static final ReferenceBinding TYPE_USE_BINDING = new ReferenceBinding() { /* used for type annotation resolution. */
 		{ this.id = TypeIds.T_undefined; }
+		@Override
 		public int kind() { return Binding.TYPE_USE; }
+		@Override
 		public boolean hasTypeBit(int bit) { return false; }
 	};
 	
@@ -207,7 +213,7 @@ public TypeBinding closestMatch() {
  * @param missingTypes
  * @return missing types
  */
-public List collectMissingTypes(List missingTypes) {
+public List<TypeBinding> collectMissingTypes(List<TypeBinding> missingTypes) {
 	return missingTypes;
 }
 
@@ -271,6 +277,26 @@ public TypeBinding erasure() {
 	return this;
 }
 
+/**
+ * Perform an upwards type projection as per JLS 4.10.5
+ * @param scope Relevant scope for evaluating type projection
+ * @param mentionedTypeVariables Filter for mentioned type variabled
+ * @returns Upwards type projection of 'this', or null if downwards projection is undefined 
+*/
+public TypeBinding upwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+	return this;
+}
+
+/**
+ * Perform a downwards type projection as per JLS 4.10.5
+ * @param scope Relevant scope for evaluating type projection
+ * @param mentionedTypeVariables Filter for mentioned type variabled
+ * @returns Downwards type projection of 'this', or null if downwards projection is undefined 
+*/
+public TypeBinding downwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+	return this;
+}
+
 /**
  * Find supertype which originates from a given well-known type, or null if not found
  * (using id avoids triggering the load of well-known type: 73740)
@@ -517,6 +543,20 @@ public TypeBinding getErasureCompatibleType(TypeBinding declaringClass) {
 				}
 			}
 			return this; // only occur if passed null declaringClass for arraylength
+		case Binding.INTERSECTION_TYPE18:
+			ReferenceBinding[] intersectingTypes = ((IntersectionTypeBinding18) this).getIntersectingTypes();
+			ReferenceBinding constantPoolType = intersectingTypes[0];
+			if (constantPoolType.id == TypeIds.T_JavaLangObject && intersectingTypes.length > 1)
+				constantPoolType = intersectingTypes[1];
+			if (constantPoolType.erasure().findSuperTypeOriginatingFrom(declaringClass) != null) {
+				return this; // no need for alternate receiver type
+			}
+			for (ReferenceBinding superBinding : intersectingTypes) {
+				if (superBinding.findSuperTypeOriginatingFrom(declaringClass) != null) {
+					return superBinding.getErasureCompatibleType(declaringClass);
+				}
+			}
+			return this; // should only occur if passed null declaringClass for arraylength
 		default :
 			return this;
 	}
@@ -1409,6 +1449,7 @@ public boolean isWildcard() {
 /* API
  * Answer the receiver's binding type from Binding.BindingID.
  */
+@Override
 public int kind() {
 	return Binding.TYPE;
 }
@@ -1526,14 +1567,10 @@ public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNull
 		for (int i = 0, length = annotations.length; i < length; i++) {
 			AnnotationBinding annotation = annotations[i];
 			if (annotation != null) {
-				switch (annotation.type.id) {
-					case TypeIds.T_ConfiguredAnnotationNullable :
-						this.tagBits |= TagBits.AnnotationNullable | TagBits.HasNullTypeAnnotation;
-						break;
-					case TypeIds.T_ConfiguredAnnotationNonNull :
-						this.tagBits |= TagBits.AnnotationNonNull  | TagBits.HasNullTypeAnnotation;
-						break;
-				}
+				if (annotation.type.hasNullBit(TypeIds.BitNullableAnnotation))
+					this.tagBits |= TagBits.AnnotationNullable | TagBits.HasNullTypeAnnotation;
+				else if (annotation.type.hasNullBit(TypeIds.BitNonNullAnnotation))
+					this.tagBits |= TagBits.AnnotationNonNull  | TagBits.HasNullTypeAnnotation;
 			}
 		}
 		// we do accept contradictory tagBits here, to support detecting contradictions caused by type substitution
@@ -1570,11 +1607,12 @@ public TypeVariableBinding[] typeVariables() {
 }
 
 /**
- * Return the single abstract method of a functional interface, or null, if the receiver is not a functional interface as defined in JLS 9.8.
+ * Return the single abstract method of a functional interface, or one of {@code null} or {@link ReferenceBinding#samProblemBinding}, if the receiver is not a functional interface as defined in JLS 9.8.
+ * In particular {@code null} is answered if the receiver is not a reference type, or is a problem type.
  * @param scope scope
  * @param replaceWildcards Should wildcards be replaced following JLS 9.8? Say false for lambdas with explicit argument types which should apply 18.5.3
  *  
- * @return The single abstract method of a functional interface, or null, if the receiver is not a functional interface. 
+ * @return The single abstract method of a functional interface, or one of {@code null} or {@link ReferenceBinding#samProblemBinding}, if the receiver is not a functional interface. 
  */
 public MethodBinding getSingleAbstractMethod(Scope scope, boolean replaceWildcards) {
 	return null;
@@ -1591,6 +1629,10 @@ public static boolean equalsEquals(TypeBinding that, TypeBinding other) {
 		return false;
 	if (that.id != TypeIds.NoId && that.id == other.id)
 		return true;
+	if (that instanceof LocalTypeBinding && other instanceof LocalTypeBinding) {
+		// while a lambda is being resolved, consider a local type as equal to its variant from another lambda copy
+		return ((LocalTypeBinding) that).sourceStart == ((LocalTypeBinding) other).sourceStart;
+	}
 	return false;
 }
 
@@ -1637,7 +1679,7 @@ public boolean sIsMoreSpecific(TypeBinding s, TypeBinding t, Scope scope) {
 	return s.isCompatibleWith(t, scope) && !s.needsUncheckedConversion(t);
 }
 
-public boolean isSubtypeOf(TypeBinding right) {
+public boolean isSubtypeOf(TypeBinding right, boolean simulatingBugJDK8026527) {
 	return isCompatibleWith(right);
 }
 
@@ -1662,7 +1704,7 @@ public SyntheticArgumentBinding[] syntheticOuterLocalVariables() {
 }
 /**
  * Call this before descending into type details to prevent infinite recursion.
- * @return true if a recursion has already been started.
+ * @return true if a recursion was not already started.
  */
 public boolean enterRecursiveFunction() {
 	return true;
@@ -1677,4 +1719,15 @@ public void exitRecursiveFunction() {
 public boolean isFunctionalType() {
 	return false;
 }
+/**
+ * Refresh some tagBits from details into the main type.
+ * Currently handled: TagBits.HasNullTypeAnnotation
+ */
+public long updateTagBits() {
+	return this.tagBits & TagBits.HasNullTypeAnnotation; // subclasses to override
+}
+
+public boolean isFreeTypeVariable() {
+	return false;
+}
 }
