@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2013, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2013, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,12 +14,13 @@
  *								Bug 434602 - Possible error with inferred null annotations leading to contradictory null annotations
  *								Bug 456497 - [1.8][null] during inference nullness from target type is lost against weaker hint from applicability analysis
  *								Bug 456487 - [1.8][null] @Nullable type variant of @NonNull-constrained type parameter causes grief
+ *     Till Brychcy - Contribution for
+ *								Bug 473713 - [1.8][null] Type mismatch: cannot convert from @NonNull A1 to @NonNull A1
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
 import java.util.HashMap;
 
-import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleLookupTable;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
@@ -64,27 +68,69 @@ public class TypeSystem {
 	
 	public final class HashedParameterizedTypes {
 		
-		private final class InternalParameterizedTypeBinding extends ParameterizedTypeBinding {
-						
-			public InternalParameterizedTypeBinding(ReferenceBinding genericType, TypeBinding[] typeArguments, ReferenceBinding enclosingType, LookupEnvironment environment) {
-				super(genericType, typeArguments, enclosingType, environment);
+		private final class PTBKey extends ReferenceBinding { // extends ReferenceBinding so it can be used as wrapper
+			protected ReferenceBinding type; // must ensure the type is resolved
+			public TypeBinding[] arguments;
+			private ReferenceBinding enclosingType;
+			public PTBKey(ReferenceBinding type, TypeBinding[] arguments, ReferenceBinding enclosingType, LookupEnvironment environment) {
+				this.type = type;
+				this.arguments = arguments;
+				this.enclosingType = enclosingType;
+
+				if(environment != null) {
+					// only add as wrapper when used in put()
+					if (type instanceof UnresolvedReferenceBinding)
+						((UnresolvedReferenceBinding) type).addWrapper(this, environment);
+					if (arguments != null) {
+						for (int i = 0, l = arguments.length; i < l; i++) {
+							if (arguments[i] instanceof UnresolvedReferenceBinding)
+								((UnresolvedReferenceBinding) arguments[i]).addWrapper(this, environment);
+							if (arguments[i].hasNullTypeAnnotations())
+								this.tagBits |= TagBits.HasNullTypeAnnotation;
+						}
+					}
+				}
 			}
-			
+			@Override
+			public void swapUnresolved(UnresolvedReferenceBinding unresolvedType, ReferenceBinding resolvedType, LookupEnvironment env) {
+				if (this.type == unresolvedType) { //$IDENTITY-COMPARISON$
+					this.type = resolvedType; // cannot be raw since being parameterized below
+					ReferenceBinding enclosing = resolvedType.enclosingType();
+					if (enclosing != null) {
+						this.enclosingType = (ReferenceBinding) env.convertUnresolvedBinaryToRawType(enclosing); // needed when binding unresolved member type
+					}
+				}
+				if (this.arguments != null) {
+					for (int i = 0, l = this.arguments.length; i < l; i++) {
+						if (this.arguments[i] == unresolvedType) { //$IDENTITY-COMPARISON$
+							this.arguments[i] = env.convertUnresolvedBinaryToRawType(resolvedType);
+						}
+					}
+				}
+			}
+			@Override
 			public boolean equals(Object other) {
-				ParameterizedTypeBinding that = (ParameterizedTypeBinding) other;  // homogeneous container. 
+				PTBKey that = (PTBKey) other;  // homogeneous container. 
 				return this.type == that.type && this.enclosingType == that.enclosingType && Util.effectivelyEqual(this.arguments, that.arguments); //$IDENTITY-COMPARISON$
 			}
-			
+			final int hash(TypeBinding b) {
+				if(b instanceof WildcardBinding || b instanceof TypeVariableBinding) {
+					return System.identityHashCode(b);
+				}
+				return b.hashCode();
+			}
+			@Override
 			public int hashCode() {
-				int hashCode = this.type.hashCode() + 13 * (this.enclosingType != null ? this.enclosingType.hashCode() : 0);
+				final int prime=31;
+				int hashCode = 1 + hash(this.type) + (this.enclosingType != null ? hash(this.enclosingType) : 0);
 				for (int i = 0, length = this.arguments == null ? 0 : this.arguments.length; i < length; i++) {
-					hashCode += (i + 1) * this.arguments[i].id * this.arguments[i].hashCode();
+					hashCode = hashCode * prime + hash(this.arguments[i]);
 				}
 				return hashCode;
 			}
 		}
 		
-		HashMap<ParameterizedTypeBinding, ParameterizedTypeBinding []> hashedParameterizedTypes = new HashMap<ParameterizedTypeBinding, ParameterizedTypeBinding[]>(256);
+		HashMap<PTBKey, ParameterizedTypeBinding []> hashedParameterizedTypes = new HashMap<>(256);
 
 		ParameterizedTypeBinding get(ReferenceBinding genericType, TypeBinding[] typeArguments, ReferenceBinding enclosingType, AnnotationBinding[] annotations) {
 			
@@ -96,7 +142,7 @@ public class TypeSystem {
 			}
 			ReferenceBinding unannotatedEnclosingType = enclosingType == null ? null : (ReferenceBinding) getUnannotatedType(enclosingType);
 			
-			ParameterizedTypeBinding typeParameterization = new InternalParameterizedTypeBinding(unannotatedGenericType, unannotatedTypeArguments, unannotatedEnclosingType, TypeSystem.this.environment);
+			PTBKey key = new PTBKey(unannotatedGenericType, unannotatedTypeArguments, unannotatedEnclosingType, null);
 			ReferenceBinding genericTypeToMatch = unannotatedGenericType, enclosingTypeToMatch = unannotatedEnclosingType;
 			TypeBinding [] typeArgumentsToMatch = unannotatedTypeArguments;
 			if (TypeSystem.this instanceof AnnotatableTypeSystem) {
@@ -104,7 +150,7 @@ public class TypeSystem {
 				enclosingTypeToMatch = enclosingType;
 				typeArgumentsToMatch = typeArguments;
 			}
-			ParameterizedTypeBinding [] parameterizedTypeBindings = this.hashedParameterizedTypes.get(typeParameterization);
+			ParameterizedTypeBinding [] parameterizedTypeBindings = this.hashedParameterizedTypes.get(key);
 			for (int i = 0, length = parameterizedTypeBindings == null ? 0 : parameterizedTypeBindings.length; i < length; i++) {
 				ParameterizedTypeBinding parameterizedType = parameterizedTypeBindings[i];
 				if (parameterizedType.actualType() != genericTypeToMatch) { //$IDENTITY-COMPARISON$
@@ -129,9 +175,9 @@ public class TypeSystem {
 			}
 			ReferenceBinding unannotatedEnclosingType = enclosingType == null ? null : (ReferenceBinding) getUnannotatedType(enclosingType);
 			
-			ParameterizedTypeBinding typeParameterization = new InternalParameterizedTypeBinding(unannotatedGenericType, unannotatedTypeArguments, unannotatedEnclosingType, TypeSystem.this.environment);
+			PTBKey key = new PTBKey(unannotatedGenericType, unannotatedTypeArguments, unannotatedEnclosingType, TypeSystem.this.environment);
 			
-			ParameterizedTypeBinding [] parameterizedTypeBindings = this.hashedParameterizedTypes.get(typeParameterization);
+			ParameterizedTypeBinding [] parameterizedTypeBindings = this.hashedParameterizedTypes.get(key);
 			int slot;
 			if (parameterizedTypeBindings == null) {
 				slot = 0;
@@ -141,7 +187,7 @@ public class TypeSystem {
 				System.arraycopy(parameterizedTypeBindings, 0, parameterizedTypeBindings = new ParameterizedTypeBinding[slot + 1], 0, slot);
 			}
 			parameterizedTypeBindings[slot] = parameterizedType;
-			this.hashedParameterizedTypes.put(typeParameterization, parameterizedTypeBindings);
+			this.hashedParameterizedTypes.put(key, parameterizedTypeBindings);
 		}
 	}	
 	
@@ -162,32 +208,32 @@ public class TypeSystem {
 	// Given a type, answer its unannotated aka naked prototype. This is also a convenient way to "register" a type with TypeSystem and have it id stamped.
 	public final TypeBinding getUnannotatedType(TypeBinding type) {
 		UnresolvedReferenceBinding urb = null;
-		if (type.isUnresolvedType() && CharOperation.indexOf('$', type.sourceName()) > 0) {
+		if (type.isUnresolvedType()) {
 			urb = (UnresolvedReferenceBinding) type;
-			boolean mayTolerateMissingType = this.environment.mayTolerateMissingType;
-			this.environment.mayTolerateMissingType = true;
-			try {
-				type = BinaryTypeBinding.resolveType(type, this.environment, true); // to ensure unique id assignment (when enclosing type is parameterized, inner type is also) 
-			} finally {
-				this.environment.mayTolerateMissingType = mayTolerateMissingType;
+			ReferenceBinding resolvedType = urb.resolvedType;
+			if (resolvedType != null) {
+				type = resolvedType;
 			}
 		}
-		if (type.id == TypeIds.NoId) {
-			if (type.hasTypeAnnotations())
-				throw new IllegalStateException();
-			int typesLength = this.types.length;
-			if (this.typeid == typesLength)
-				System.arraycopy(this.types, 0, this.types = new TypeBinding[typesLength * 2][], 0, typesLength);
-			this.types[type.id = this.typeid++] = new TypeBinding[4];
-			if (urb != null)
+		try {
+			if (type.id == TypeIds.NoId) {
+				if (type.hasTypeAnnotations())
+					throw new IllegalStateException();
+				int typesLength = this.types.length;
+				if (this.typeid == typesLength)
+					System.arraycopy(this.types, 0, this.types = new TypeBinding[typesLength * 2][], 0, typesLength);
+				this.types[type.id = this.typeid++] = new TypeBinding[4];
+			} else {
+				TypeBinding nakedType = this.types[type.id] == null ? null : this.types[type.id][0];
+				if (type.hasTypeAnnotations() && nakedType == null)
+					throw new IllegalStateException();
+				if (nakedType != null)
+					return nakedType;
+				this.types[type.id] = new TypeBinding[4];  // well known type, assigned id elsewhere.
+			}
+		} finally {
+			if (urb != null && urb.id == TypeIds.NoId)
 				urb.id = type.id;
-		} else {
-			TypeBinding nakedType = this.types[type.id] == null ? null : this.types[type.id][0];
-			if (type.hasTypeAnnotations() && nakedType == null)
-				throw new IllegalStateException();
-			if (nakedType != null)
-				return nakedType;
-			this.types[type.id] = new TypeBinding[4];  // well known type, assigned id elsewhere.
 		}
 	
 		return this.types[type.id][0] = type;
@@ -294,6 +340,9 @@ public class TypeSystem {
 	   they may and we need to materialize the unannotated versions and work on them.
 	*/ 
 	public RawTypeBinding getRawType(ReferenceBinding genericType, ReferenceBinding enclosingType) {
+		if (!genericType.hasEnclosingInstanceContext() && enclosingType != null) {
+			enclosingType = (ReferenceBinding) enclosingType.original();
+		}
 		ReferenceBinding unannotatedGenericType = (ReferenceBinding) getUnannotatedType(genericType);
 		ReferenceBinding unannotatedEnclosingType = enclosingType == null ? null : (ReferenceBinding) getUnannotatedType(enclosingType);
 	
@@ -341,7 +390,9 @@ public class TypeSystem {
 		}
 		TypeBinding unannotatedBound = bound == null ? null : getUnannotatedType(bound);
 
-		TypeBinding[] derivedTypes = this.types[unannotatedGenericType.id];  // by construction, cachedInfo != null now.
+		boolean useDerivedTypesOfBound = unannotatedBound instanceof TypeVariableBinding || (unannotatedBound instanceof ParameterizedTypeBinding && !(unannotatedBound instanceof RawTypeBinding));
+		TypeBinding[] derivedTypes = this.types[useDerivedTypesOfBound ? unannotatedBound.id :unannotatedGenericType.id];  // by construction, cachedInfo != null now.
+
 		int i, length = derivedTypes.length;
 		for (i = 0; i < length; i++) {
 			TypeBinding derivedType = derivedTypes[i];
@@ -357,7 +408,7 @@ public class TypeSystem {
 		
 		if (i == length) {
 			System.arraycopy(derivedTypes, 0, derivedTypes = new TypeBinding[length * 2], 0, length);
-			this.types[unannotatedGenericType.id] = derivedTypes;
+			this.types[useDerivedTypesOfBound ? unannotatedBound.id :unannotatedGenericType.id] = derivedTypes;
 		}
 		TypeBinding wildcard = derivedTypes[i] = new WildcardBinding(unannotatedGenericType, rank, unannotatedBound, unannotatedOtherBounds, boundKind, this.environment);
 	
@@ -502,6 +553,9 @@ public class TypeSystem {
 	
 	public void updateCaches(UnresolvedReferenceBinding unresolvedType, ReferenceBinding resolvedType) {
 		final int unresolvedTypeId = unresolvedType.id;
+		if (resolvedType.id != TypeIds.NoId) {
+			unresolvedType.id = resolvedType.id;
+		}
 		if (unresolvedTypeId != TypeIds.NoId) {
 			// https://bugs.eclipse.org/bugs/show_bug.cgi?id=432977
 			TypeBinding[] derivedTypes = this.types[unresolvedTypeId];
@@ -509,7 +563,8 @@ public class TypeSystem {
 				if (derivedTypes[i] == null)
 					break;
 				if (derivedTypes[i] == unresolvedType) { //$IDENTITY-COMPARISON$
-					resolvedType.id = unresolvedTypeId;
+					if(resolvedType.id == TypeIds.NoId)
+						resolvedType.id = unresolvedTypeId;
 					derivedTypes[i] = resolvedType;
 				}
 			}
