@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2016 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann <stephan@cs.tu-berlin.de> - Contributions for
@@ -34,19 +37,24 @@
  *								Bug 456487 - [1.8][null] @Nullable type variant of @NonNull-constrained type parameter causes grief
  *								Bug 462790 - [null] NPE in Expression.computeConversion()
  *								Bug 456532 - [1.8][null] ReferenceBinding.appendNullAnnotation() includes phantom annotations in error messages
+ *     Jesper S Møller <jesper@selskabet.org>  - Contributions for bug 381345 : [1.8] Take care of the Java 8 major version
+ *								Bug 527554 - [18.3] Compiler support for JEP 286 Local-Variable Type
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
 import java.util.Set;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Annotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.NullAnnotationMatching.CheckMode;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.TypeParameter;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.TypeReference;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Wildcard;
 import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants.BoundCheckStatus;
 
 /**
  * Binding for a type parameter, held by source/binary type or method.
@@ -110,105 +118,141 @@ public class TypeVariableBinding extends ReferenceBinding {
 
 	/**
 	 * Returns true if the argument type satisfies all bounds of the type parameter
+	 * @param location if non-null this may be used for reporting errors relating to null type annotations (if enabled)
 	 */
-	public int boundCheck(Substitution substitution, TypeBinding argumentType, Scope scope) {
-		int code = internalBoundCheck(substitution, argumentType, scope);
-		if (code == TypeConstants.MISMATCH) {
+	public TypeConstants.BoundCheckStatus boundCheck(Substitution substitution, TypeBinding argumentType, Scope scope, ASTNode location) {
+		TypeConstants.BoundCheckStatus code = internalBoundCheck(substitution, argumentType, scope, location);
+		if (code == BoundCheckStatus.MISMATCH) {
 			if (argumentType instanceof TypeVariableBinding && scope != null) {
 				TypeBinding bound = ((TypeVariableBinding)argumentType).firstBound;
 				if (bound instanceof ParameterizedTypeBinding) {
-					int code2 = boundCheck(substitution, bound.capture(scope, -1, -1), scope); // no position needed as this capture will never escape this context
-					return Math.min(code, code2);
+					BoundCheckStatus code2 = boundCheck(substitution, bound.capture(scope, -1, -1), scope, location); // no capture position needed as this capture will never escape this context
+					return code.betterOf(code2);
 				}
 			}
 		}
 		return code;
 	}
-	private int internalBoundCheck(Substitution substitution, TypeBinding argumentType, Scope scope) {
+	private TypeConstants.BoundCheckStatus internalBoundCheck(Substitution substitution, TypeBinding argumentType, Scope scope, ASTNode location) {
 		if (argumentType == TypeBinding.NULL || TypeBinding.equalsEquals(argumentType, this)) {
-			return TypeConstants.OK;
+			return BoundCheckStatus.OK;
 		}
 		boolean hasSubstitution = substitution != null;
 		if (!(argumentType instanceof ReferenceBinding || argumentType.isArrayType()))
-			return TypeConstants.MISMATCH;
+			return BoundCheckStatus.MISMATCH;
 		// special case for re-entrant source types (selection, code assist, etc)...
 		// can request additional types during hierarchy walk that are found as source types that also 'need' to connect their hierarchy
 		if (this.superclass == null)
-			return TypeConstants.OK;
+			return BoundCheckStatus.OK;
+
+		BoundCheckStatus nullStatus = BoundCheckStatus.OK;
+		boolean checkNullAnnotations = scope.environment().usesNullTypeAnnotations() && (location == null || (location.bits & ASTNode.InsideJavadoc) == 0);
 
 		if (argumentType.kind() == Binding.WILDCARD_TYPE) {
 			WildcardBinding wildcard = (WildcardBinding) argumentType;
 			switch(wildcard.boundKind) {
 				case Wildcard.EXTENDS :
+					boolean checkedAsOK = false;
 					TypeBinding wildcardBound = wildcard.bound;
 					if (TypeBinding.equalsEquals(wildcardBound, this))
-						return TypeConstants.OK;
+						checkedAsOK = true; // OK per JLS, but may require null checking below
 					boolean isArrayBound = wildcardBound.isArrayType();
 					if (!wildcardBound.isInterface()) {
 						TypeBinding substitutedSuperType = hasSubstitution ? Scope.substitute(substitution, this.superclass) : this.superclass;
-						if (substitutedSuperType.id != TypeIds.T_JavaLangObject) {
-							if (isArrayBound) {
-								if (!wildcardBound.isCompatibleWith(substitutedSuperType, scope))
-									return TypeConstants.MISMATCH;
-							} else {
-								TypeBinding match = wildcardBound.findSuperTypeOriginatingFrom(substitutedSuperType);
-								if (match != null) {
-									if (substitutedSuperType.isProvablyDistinct(match)) {
-										return TypeConstants.MISMATCH;
-									}
+						if (!checkedAsOK) {
+							if (substitutedSuperType.id != TypeIds.T_JavaLangObject) {
+								if (isArrayBound) {
+									if (!wildcardBound.isCompatibleWith(substitutedSuperType, scope))
+										return BoundCheckStatus.MISMATCH;
 								} else {
-									match =  substitutedSuperType.findSuperTypeOriginatingFrom(wildcardBound);
+									TypeBinding match = wildcardBound.findSuperTypeOriginatingFrom(substitutedSuperType);
 									if (match != null) {
-										if (match.isProvablyDistinct(wildcardBound)) {
-											return TypeConstants.MISMATCH;
+										if (substitutedSuperType.isProvablyDistinct(match)) {
+											return BoundCheckStatus.MISMATCH;
 										}
 									} else {
-										if (denotesRelevantSuperClass(wildcardBound) && denotesRelevantSuperClass(substitutedSuperType)) {
-											// non-object real superclass should have produced a valid 'match' above
-											return TypeConstants.MISMATCH;
+										match =  substitutedSuperType.findSuperTypeOriginatingFrom(wildcardBound);
+										if (match != null) {
+											if (match.isProvablyDistinct(wildcardBound)) {
+												return BoundCheckStatus.MISMATCH;
+											}
+										} else {
+											if (denotesRelevantSuperClass(wildcardBound) && denotesRelevantSuperClass(substitutedSuperType)) {
+												// non-object real superclass should have produced a valid 'match' above
+												return BoundCheckStatus.MISMATCH;
+											}
+											// not fully spec-ed in JLS, but based on email communication (2017-09-13):
+											// (a) bound check should apply capture
+											// (b) capture applies glb
+											// (c) and then the glb should be checked for well-formedness (see Scope.isMalformedPair() - this part missing in JLS).
+											// Since we don't do (a), nor (b) for this case, we just directly proceed to (b) here.
+											// For (a) see ParameterizedTypeBinding.boundCheck() - comment added as of this commit
+											// for (b) see CaptureBinding.initializeBounds()  - comment added as of this commit
+											if (Scope.greaterLowerBound(new TypeBinding[] {substitutedSuperType, wildcardBound}, scope, this.environment) == null) {
+												return BoundCheckStatus.MISMATCH;
+											}
 										}
 									}
 								}
 							}
 						}
+						if (checkNullAnnotations && argumentType.hasNullTypeAnnotations()) {
+							nullStatus = nullBoundCheck(scope, argumentType, substitutedSuperType, substitution, location, nullStatus);
+						}
 					}
 					boolean mustImplement = isArrayBound || ((ReferenceBinding)wildcardBound).isFinal();
 					for (int i = 0, length = this.superInterfaces.length; i < length; i++) {
 						TypeBinding substitutedSuperType = hasSubstitution ? Scope.substitute(substitution, this.superInterfaces[i]) : this.superInterfaces[i];
-						if (isArrayBound) {
-							if (!wildcardBound.isCompatibleWith(substitutedSuperType, scope))
-									return TypeConstants.MISMATCH;
-						} else {
-							TypeBinding match = wildcardBound.findSuperTypeOriginatingFrom(substitutedSuperType);
-							if (match != null) {
-								if (substitutedSuperType.isProvablyDistinct(match)) {
-									return TypeConstants.MISMATCH;
+						if (!checkedAsOK) {
+							if (isArrayBound) {
+								if (!wildcardBound.isCompatibleWith(substitutedSuperType, scope))
+										return BoundCheckStatus.MISMATCH;
+							} else {
+								TypeBinding match = wildcardBound.findSuperTypeOriginatingFrom(substitutedSuperType);
+								if (match != null) {
+									if (substitutedSuperType.isProvablyDistinct(match)) {
+										return BoundCheckStatus.MISMATCH;
+									}
+								} else if (mustImplement) {
+										return BoundCheckStatus.MISMATCH; // cannot be extended further to satisfy missing bounds
 								}
-							} else if (mustImplement) {
-									return TypeConstants.MISMATCH; // cannot be extended further to satisfy missing bounds
 							}
 						}
-
+						if (checkNullAnnotations && argumentType.hasNullTypeAnnotations()) {
+							nullStatus = nullBoundCheck(scope, argumentType, substitutedSuperType, substitution, location, nullStatus);
+						}
 					}
+					if (nullStatus != null)
+						return nullStatus;
 					break;
 
 				case Wildcard.SUPER :
 					// if the wildcard is lower-bounded by a type variable that has no relevant upper bound there's nothing to check here (bug 282152):
-					if (wildcard.bound.isTypeVariable() && ((TypeVariableBinding)wildcard.bound).superclass.id == TypeIds.T_JavaLangObject)
-						break;
-					return boundCheck(substitution, wildcard.bound, scope);
-
+					if (wildcard.bound.isTypeVariable() && ((TypeVariableBinding)wildcard.bound).superclass.id == TypeIds.T_JavaLangObject) {
+						return nullBoundCheck(scope, argumentType, null, substitution, location, nullStatus);
+					} else {
+						TypeBinding bound = wildcard.bound;
+						if (checkNullAnnotations && this.environment.containsNullTypeAnnotation(wildcard.typeAnnotations))
+							bound = this.environment.createAnnotatedType(bound.withoutToplevelNullAnnotation(), wildcard.getTypeAnnotations());
+						BoundCheckStatus status = boundCheck(substitution, bound, scope, null); // do not report null-errors against the tweaked bound ...
+						if (status == BoundCheckStatus.NULL_PROBLEM && location != null)
+							scope.problemReporter().nullityMismatchTypeArgument(this, wildcard, location); // ... but against the wildcard
+						return status;
+					}
 				case Wildcard.UNBOUND :
+					if (checkNullAnnotations && argumentType.hasNullTypeAnnotations()) {
+						return nullBoundCheck(scope, argumentType, null, substitution, location, nullStatus);
+					}
 					break;
 			}
-			return TypeConstants.OK;
+			return BoundCheckStatus.OK;
 		}
 		boolean unchecked = false;
 		if (this.superclass.id != TypeIds.T_JavaLangObject) {
 			TypeBinding substitutedSuperType = hasSubstitution ? Scope.substitute(substitution, this.superclass) : this.superclass;
 	    	if (TypeBinding.notEquals(substitutedSuperType, argumentType)) {
 				if (!argumentType.isCompatibleWith(substitutedSuperType, scope)) {
-				    return TypeConstants.MISMATCH;
+				    return BoundCheckStatus.MISMATCH;
 				}
 				TypeBinding match = argumentType.findSuperTypeOriginatingFrom(substitutedSuperType);
 				if (match != null){
@@ -217,12 +261,15 @@ public class TypeVariableBinding extends ReferenceBinding {
 						unchecked = true;
 				}
 	    	}
+			if (checkNullAnnotations) {
+				nullStatus = nullBoundCheck(scope, argumentType, substitutedSuperType, substitution, location, nullStatus);
+			}
 		}
 	    for (int i = 0, length = this.superInterfaces.length; i < length; i++) {
 			TypeBinding substitutedSuperType = hasSubstitution ? Scope.substitute(substitution, this.superInterfaces[i]) : this.superInterfaces[i];
 	    	if (TypeBinding.notEquals(substitutedSuperType, argumentType)) {
 				if (!argumentType.isCompatibleWith(substitutedSuperType, scope)) {
-				    return TypeConstants.MISMATCH;
+				    return BoundCheckStatus.MISMATCH;
 				}
 				TypeBinding match = argumentType.findSuperTypeOriginatingFrom(substitutedSuperType);
 				if (match != null){
@@ -231,15 +278,28 @@ public class TypeVariableBinding extends ReferenceBinding {
 						unchecked = true;
 				}
 	    	}
+			if (checkNullAnnotations) {
+				nullStatus = nullBoundCheck(scope, argumentType, substitutedSuperType, substitution, location, nullStatus);
+			}
 	    }
-	    long nullTagBits = NullAnnotationMatching.validNullTagBits(this.tagBits);
-	    if (nullTagBits != 0) {
-	    	long argBits = NullAnnotationMatching.validNullTagBits(argumentType.tagBits);
-	    	if (argBits != nullTagBits) {
-//	    		System.err.println("TODO(stephan): issue proper error: bound conflict at "+String.valueOf(this.declaringElement.readableName()));
-	    	}
+	    if (checkNullAnnotations && nullStatus != BoundCheckStatus.NULL_PROBLEM) {
+	    	long nullBits = this.tagBits & TagBits.AnnotationNullMASK;
+	    	if (nullBits != 0 && nullBits != (argumentType.tagBits & TagBits.AnnotationNullMASK)) {
+				if (location != null)
+					scope.problemReporter().nullityMismatchTypeArgument(this, argumentType, location);
+				nullStatus = BoundCheckStatus.NULL_PROBLEM;
+			}
 	    }
-	    return unchecked ? TypeConstants.UNCHECKED : TypeConstants.OK;
+	    return unchecked ? BoundCheckStatus.UNCHECKED : nullStatus != null ? nullStatus : BoundCheckStatus.OK;
+	}
+
+	private BoundCheckStatus nullBoundCheck(Scope scope, TypeBinding argumentType, TypeBinding substitutedSuperType, Substitution substitution, ASTNode location, BoundCheckStatus previousStatus) {
+		if (NullAnnotationMatching.analyse(this, argumentType, substitutedSuperType, substitution, -1, null, CheckMode.BOUND_CHECK).isAnyMismatch()) {
+			if (location != null)
+				scope.problemReporter().nullityMismatchTypeArgument(this, argumentType, location);
+			return BoundCheckStatus.NULL_PROBLEM;
+		}
+		return previousStatus;
 	}
 
 	boolean denotesRelevantSuperClass(TypeBinding type) {
@@ -260,6 +320,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#canBeInstantiated()
 	 */
+	@Override
 	public boolean canBeInstantiated() {
 		return false;
 	}
@@ -271,6 +332,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	 *   A = F   corresponds to:      F.collectSubstitutes(..., A, ..., CONSTRAINT_EQUAL (0))
 	 *   A >> F   corresponds to:   F.collectSubstitutes(..., A, ..., CONSTRAINT_SUPER (2))
 	 */
+	@Override
 	public void collectSubstitutes(Scope scope, TypeBinding actualType, InferenceContext inferenceContext, int constraint) {
 
 		//	only infer for type params of the generic method
@@ -311,6 +373,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	 * p.X<T> { ... } --> Lp/X;:TT;
 	 * p.X { <T> void foo() {...} } --> Lp/X;.foo()V:TT;
 	 */
+	@Override
 	public char[] computeUniqueKey(boolean isLeaf) {
 		StringBuffer buffer = new StringBuffer();
 		Binding declaring = this.declaringElement;
@@ -338,6 +401,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		buffer.getChars(0, length, uniqueKey, 0);
 		return uniqueKey;
 	}
+	@Override
 	public char[] constantPoolName() { /* java/lang/Object */
 	    if (this.firstBound != null) {
 			return this.firstBound.constantPoolName();
@@ -345,9 +409,11 @@ public class TypeVariableBinding extends ReferenceBinding {
 	    return this.superclass.constantPoolName(); // java/lang/Object
 	}
 	
+	@Override
 	public TypeBinding clone(TypeBinding enclosingType) {
 		return new TypeVariableBinding(this);
 	}
+	@Override
 	public String annotatedDebugName() {
 		StringBuffer buffer = new StringBuffer(10);
 		buffer.append(super.annotatedDebugName());
@@ -377,11 +443,13 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#debugName()
 	 */
+	@Override
 	public String debugName() {
 		if (this.hasTypeAnnotations())
 			return super.annotatedDebugName();
 	    return new String(this.sourceName);
 	}
+	@Override
 	public TypeBinding erasure() {
 	    if (this.firstBound != null) {
 			return this.firstBound.erasure();
@@ -412,6 +480,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	 * T::Ljava/util/Map;:Ljava/io/Serializable;
 	 * T:LY<TT;>
 	 */
+	@Override
 	public char[] genericTypeSignature() {
 	    if (this.genericTypeSignature != null) return this.genericTypeSignature;
 		return this.genericTypeSignature = CharOperation.concat('T', this.sourceName, ';');
@@ -446,6 +515,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		return true;
 	}
 
+	@Override
 	public boolean hasTypeBit(int bit) {
 		if (this.typeBits == TypeIds.BitUninitialized) {
 			// initialize from bounds
@@ -473,6 +543,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		return false;
 	}
 
+	@Override
 	public boolean isHierarchyConnected() {
 		return (this.modifiers & ExtraCompilerModifiers.AccUnresolved) == 0;
 	}
@@ -503,16 +574,16 @@ public class TypeVariableBinding extends ReferenceBinding {
 	}
 
 	@Override
-	public boolean isSubtypeOf(TypeBinding other) {
+	public boolean isSubtypeOf(TypeBinding other, boolean simulatingBugJDK8026527) {
 		if (isSubTypeOfRTL(other))
 			return true;
-		if (this.firstBound != null && this.firstBound.isSubtypeOf(other))
+		if (this.firstBound != null && this.firstBound.isSubtypeOf(other, simulatingBugJDK8026527))
 			return true;
-		if (this.superclass != null && this.superclass.isSubtypeOf(other))
+		if (this.superclass != null && this.superclass.isSubtypeOf(other, simulatingBugJDK8026527))
 			return true;
 		if (this.superInterfaces != null)
 			for (int i = 0, l = this.superInterfaces.length; i < l; i++)
-		   		if (this.superInterfaces[i].isSubtypeOf(other))
+		   		if (this.superInterfaces[i].isSubtypeOf(other, false))
 					return true;
 		return other.id == TypeIds.T_JavaLangObject;
 	}
@@ -531,7 +602,21 @@ public class TypeVariableBinding extends ReferenceBinding {
 	public void exitRecursiveFunction() {
 		this.inRecursiveFunction = false;
 	}
+
+	// to prevent infinite recursion when inspecting recursive generics:
+	boolean inRecursiveProjectionFunction = false;
 	
+	public boolean enterRecursiveProjectionFunction() {
+		if (this.inRecursiveProjectionFunction)
+			return false;
+		this.inRecursiveProjectionFunction = true;
+		return true;
+	}
+	public void exitRecursiveProjectionFunction() {
+		this.inRecursiveProjectionFunction = false;
+	}
+
+	@Override
 	public boolean isProperType(boolean admitCapture18) {
 		// handle recursive calls:
 		if (this.inRecursiveFunction) // be optimistic, since this node is not an inference variable
@@ -553,6 +638,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		}
 	}
 
+	@Override
 	TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding substituteType) {
 		if (this.inRecursiveFunction) return this;
 		this.inRecursiveFunction = true;
@@ -597,6 +683,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
 	 * Returns true if the type was declared as a type variable
 	 */
+	@Override
 	public boolean isTypeVariable() {
 	    return true;
 	}
@@ -622,10 +709,12 @@ public class TypeVariableBinding extends ReferenceBinding {
 //		return this;
 //	}
 
+	@Override
 	public int kind() {
 		return Binding.TYPE_PARAMETER;
 	}
 	
+	@Override
 	public boolean mentionsAny(TypeBinding[] parameters, int idx) {
 		if (this.inRecursiveFunction)
 			return false; // nothing seen
@@ -646,6 +735,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		}
 	}
 
+	@Override
 	void collectInferenceVariables(Set<InferenceVariable> variables) {
 		if (this.inRecursiveFunction)
 			return; // nothing seen
@@ -679,7 +769,8 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
      * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#readableName()
      */
-    public char[] readableName() {
+    @Override
+	public char[] readableName() {
         return this.sourceName;
     }
 	ReferenceBinding resolve() {
@@ -695,7 +786,9 @@ public class TypeVariableBinding extends ReferenceBinding {
 			long superNullTagBits = resolveType.tagBits & TagBits.AnnotationNullMASK;
 			if (superNullTagBits != 0L) {
 				if (nullTagBits == 0L) {
-					this.tagBits |= (superNullTagBits | TagBits.HasNullTypeAnnotation);
+					if ((superNullTagBits & TagBits.AnnotationNonNull) != 0) {
+						nullTagBits = superNullTagBits;
+					}
 				} else {
 //					System.err.println("TODO(stephan): report proper error: conflict binary TypeVariable vs. first bound");
 				}
@@ -712,7 +805,9 @@ public class TypeVariableBinding extends ReferenceBinding {
 				long superNullTagBits = resolveType.tagBits & TagBits.AnnotationNullMASK;
 				if (superNullTagBits != 0L) {
 					if (nullTagBits == 0L) {
-						this.tagBits |= (superNullTagBits | TagBits.HasNullTypeAnnotation);
+						if ((superNullTagBits & TagBits.AnnotationNonNull) != 0) {
+							nullTagBits = superNullTagBits;
+						}
 					} else {
 //						System.err.println("TODO(stephan): report proper error: conflict binary TypeVariable vs. bound "+i);
 					}
@@ -720,6 +815,9 @@ public class TypeVariableBinding extends ReferenceBinding {
 				interfaces[i] = resolveType;
 			}
 		}
+		if (nullTagBits != 0)
+			this.tagBits |= nullTagBits | TagBits.HasNullTypeAnnotation;
+
 		// refresh the firstBound in case it changed
 		if (this.firstBound != null) {
 			if (TypeBinding.equalsEquals(this.firstBound, oldSuperclass)) {
@@ -732,6 +830,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		return this;
 	}
 	
+	@Override
 	public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNullAnnotations) {
 		if (getClass() == TypeVariableBinding.class) {
 			// TVB only: if the declaration itself carries type annotations,
@@ -746,13 +845,16 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
      * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#shortReadableName()
      */
-    public char[] shortReadableName() {
+    @Override
+	public char[] shortReadableName() {
         return readableName();
     }
+	@Override
 	public ReferenceBinding superclass() {
 		return this.superclass;
 	}
 	
+	@Override
 	public ReferenceBinding[] superInterfaces() {
 		return this.superInterfaces;
 	}
@@ -760,6 +862,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	/**
 	 * @see java.lang.Object#toString()
 	 */
+	@Override
 	public String toString() {
 		if (this.hasTypeAnnotations())
 			return annotatedDebugName();
@@ -815,6 +918,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 	    return readableName;
 	}
 
+	@Override
 	protected void appendNullAnnotation(StringBuffer nameBuffer, CompilerOptions options) {
 		int oldSize = nameBuffer.length();
 		super.appendNullAnnotation(nameBuffer, options);
@@ -836,6 +940,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 		}
 	}
 
+	@Override
 	public TypeBinding unannotated() {
 		return this.hasTypeAnnotations() ? this.environment.getUnannotatedType(this) : this;
 	}
@@ -866,26 +971,31 @@ public class TypeVariableBinding extends ReferenceBinding {
 			long superNullTagBits = NullAnnotationMatching.validNullTagBits(this.firstBound.tagBits);
 			if (superNullTagBits != 0L) {
 				if (nullTagBits == 0L) {
-					nullTagBits |= superNullTagBits;
+					if ((superNullTagBits & TagBits.AnnotationNonNull) != 0) {
+						nullTagBits = superNullTagBits;
+					}
 				} else if (superNullTagBits != nullTagBits) {
-					this.firstBound = nullMismatchOnBound(parameter, this.firstBound, superNullTagBits, nullTagBits, scope);
+					if(parameter != null)
+						this.firstBound = nullMismatchOnBound(parameter, this.firstBound, superNullTagBits, nullTagBits, scope);
 				}
 			}
 		}	
 		ReferenceBinding[] interfaces = this.superInterfaces;
 		int length;
-		if ((length = interfaces.length) != 0) {
+		if (interfaces != null && (length = interfaces.length) != 0) {
 			for (int i = length; --i >= 0;) {
 				ReferenceBinding resolveType = interfaces[i];
 				long superNullTagBits = NullAnnotationMatching.validNullTagBits(resolveType.tagBits);
 				if (superNullTagBits != 0L) {
 					if (nullTagBits == 0L) {
-						nullTagBits |= superNullTagBits;
+						if ((superNullTagBits & TagBits.AnnotationNonNull) != 0) {
+							nullTagBits = superNullTagBits;
+						}
 					} else if (superNullTagBits != nullTagBits) {
-						interfaces[i] = (ReferenceBinding) nullMismatchOnBound(parameter, resolveType, superNullTagBits, nullTagBits, scope);
+						if(parameter != null)
+							interfaces[i] = (ReferenceBinding) nullMismatchOnBound(parameter, resolveType, superNullTagBits, nullTagBits, scope);
 					}
 				}
-				interfaces[i] = resolveType;
 			}
 		}
 		if (nullTagBits != 0)
@@ -927,8 +1037,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 			TypeBinding [] annotatedTypes = getDerivedTypesForDeferredInitialization();
 			for (int i = 0, length = annotatedTypes == null ? 0 : annotatedTypes.length; i < length; i++) {
 				TypeVariableBinding annotatedType = (TypeVariableBinding) annotatedTypes[i];
-				if (annotatedType.firstBound == null)
-					annotatedType.firstBound = firstBound;
+				annotatedType.firstBound = firstBound;
 			}
 		}
 		if (firstBound != null && firstBound.hasNullTypeAnnotations())
@@ -944,8 +1053,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 			TypeBinding [] annotatedTypes = getDerivedTypesForDeferredInitialization();
 			for (int i = 0, length = annotatedTypes == null ? 0 : annotatedTypes.length; i < length; i++) {
 				TypeVariableBinding annotatedType = (TypeVariableBinding) annotatedTypes[i];
-				if (annotatedType.superclass == null)
-					annotatedType.superclass = superclass;
+				annotatedType.superclass = superclass;
 			}
 		}
 		return superclass;
@@ -959,8 +1067,7 @@ public class TypeVariableBinding extends ReferenceBinding {
 			TypeBinding [] annotatedTypes = getDerivedTypesForDeferredInitialization();
 			for (int i = 0, length = annotatedTypes == null ? 0 : annotatedTypes.length; i < length; i++) {
 				TypeVariableBinding annotatedType = (TypeVariableBinding) annotatedTypes[i];
-				if (annotatedType.superInterfaces == null)
-					annotatedType.superInterfaces = superInterfaces;
+				annotatedType.superInterfaces = superInterfaces;
 			}
 		}
 		return superInterfaces;
@@ -1000,7 +1107,43 @@ public class TypeVariableBinding extends ReferenceBinding {
 		return (currentNullBits & ~declarationNullBits) != 0;
 	}
 
+	@Override
 	public boolean acceptsNonNullDefault() {
 		return false;
 	}
+
+	@Override
+	public long updateTagBits() {
+		if (!this.inRecursiveFunction) {
+			this.inRecursiveFunction = true;
+			try {
+				if (this.superclass != null)
+					this.tagBits |= this.superclass.updateTagBits();
+				if (this.superInterfaces != null)
+					for (TypeBinding superIfc : this.superInterfaces)
+						this.tagBits |= superIfc.updateTagBits();
+			} finally {
+				this.inRecursiveFunction = false;
+			}
+		}
+		return super.updateTagBits();
+	}
+
+	@Override
+	public boolean isFreeTypeVariable() {
+		return this.environment.usesNullTypeAnnotations() 
+				&& this.environment.globalOptions.pessimisticNullAnalysisForFreeTypeVariablesEnabled 
+				&& (this.tagBits & TagBits.AnnotationNullMASK) == 0;	
+	}
+
+	@Override
+	public ReferenceBinding upwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+		return this;
+	}
+
+	@Override
+	public ReferenceBinding downwardsProjection(Scope scope, TypeBinding[] mentionedTypeVariables) {
+		return this;
+	}
+
 }
