@@ -1,10 +1,13 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * Copyright (c) 2005, 2017 IBM Corporation and others.
  *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
+ * which accompanies this distribution, and is available at
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
+ * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Stephan Herrmann - Contribution for
@@ -28,6 +31,7 @@ import java.util.List;
 import java.util.Set;
 
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Annotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Wildcard;
 import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
@@ -67,10 +71,12 @@ public class WildcardBinding extends ReferenceBinding {
 		this.typeBits = TypeIds.BitUninitialized;
 	}
 
+	@Override
 	TypeBinding bound() {
 		return this.bound;
 	}
 	
+	@Override
 	int boundKind() {
 		return this.boundKind;
 	}
@@ -89,15 +95,136 @@ public class WildcardBinding extends ReferenceBinding {
 		}
 		return this.environment.createIntersectionType18(allBounds);
 	}
+	
+	
+	@Override
+	public void setTypeAnnotations(AnnotationBinding[] annotations, boolean evalNullAnnotations) {
+		this.tagBits |= TagBits.HasTypeAnnotations;
+		if (annotations != null && annotations.length != 0) {
+			this.typeAnnotations = annotations;
+		}
+		if (evalNullAnnotations) {
+			evaluateNullAnnotations(null, null);
+		}
+	}
 
+	/**
+	 * evaluate null type annotations and check / copy nullTagBits from bound and typevariable.
+	 * may be invoked repeatedly.
+	 * @param scope (may be null, if wildcard is null)
+	 * @param wildcard (may be null. if non-null, errors are reported and type annotations are dropped from this.bound in case of conflicts.)
+	 */
+	public void evaluateNullAnnotations(Scope scope, Wildcard wildcard) {
+		long nullTagBits = determineNullBitsFromDeclaration(scope, wildcard);
+		if (nullTagBits == 0L) {
+			TypeVariableBinding typeVariable2 = typeVariable();
+			if (typeVariable2 != null) {
+				long typeVariableNullTagBits = typeVariable2.tagBits & TagBits.AnnotationNullMASK;
+				if (typeVariableNullTagBits != 0L) {
+					nullTagBits = typeVariableNullTagBits;
+				}
+			}
+		}
+		if (nullTagBits != 0)
+			this.tagBits = (this.tagBits & ~TagBits.AnnotationNullMASK) | nullTagBits | TagBits.HasNullTypeAnnotation;
+	}
+
+	/**
+	 * compute the nullTagBits from type annotations and bound.
+	 * @param scope (may be null, if wildcard is null)
+	 * @param wildcard (may be null. if non-null, errors are reported and type annotations are dropped from this.bound in case of conflicts.)
+	 */
+	public long determineNullBitsFromDeclaration(Scope scope, Wildcard wildcard) {
+		long nullTagBits = 0L;
+		AnnotationBinding [] annotations = this.typeAnnotations;
+		if (annotations != null) {
+			for (int i = 0, length = annotations.length; i < length; i++) {
+				AnnotationBinding annotation = annotations[i];
+				if (annotation != null) {
+					if (annotation.type.hasNullBit(TypeIds.BitNullableAnnotation)) {
+						if ((nullTagBits & TagBits.AnnotationNonNull) == 0) {
+							nullTagBits |= TagBits.AnnotationNullable;
+						} else {
+							if (wildcard != null) {
+								Annotation annotation1 = wildcard.findAnnotation(TagBits.AnnotationNullable);
+								if (annotation1 != null)
+									scope.problemReporter().contradictoryNullAnnotations(annotation1);
+							}
+						}
+					} else if (annotation.type.hasNullBit(TypeIds.BitNonNullAnnotation)) {
+						if ((nullTagBits & TagBits.AnnotationNullable) == 0) {
+							nullTagBits |= TagBits.AnnotationNonNull;
+						} else {
+							if (wildcard != null) {
+								Annotation annotation1 = wildcard.findAnnotation(TagBits.AnnotationNonNull);
+								if (annotation1 != null)
+									scope.problemReporter().contradictoryNullAnnotations(annotation1);
+							}
+						}
+					}
+				}
+			}
+		}
+		if (this.bound != null && this.bound.isValidBinding()) {
+			long boundNullTagBits = this.bound.tagBits & TagBits.AnnotationNullMASK;
+			if (boundNullTagBits != 0L) {
+				if (this.boundKind == Wildcard.SUPER) {
+					if ((boundNullTagBits & TagBits.AnnotationNullable) != 0) {
+						if (nullTagBits == 0L) {
+							nullTagBits = TagBits.AnnotationNullable;
+						} else if (wildcard != null && (nullTagBits & TagBits.AnnotationNonNull) != 0) {
+							Annotation annotation = wildcard.bound.findAnnotation(boundNullTagBits);
+							if (annotation == null) { // false alarm, implicit annotation is no conflict, but should be removed:
+								// may not be reachable, how could we have an implicit @Nullable (not via @NonNullByDefault)?
+								TypeBinding newBound = this.bound.withoutToplevelNullAnnotation();
+								this.bound = newBound;
+								wildcard.bound.resolvedType = newBound;
+							} else {
+								scope.problemReporter().contradictoryNullAnnotationsOnBounds(annotation, nullTagBits);
+							}
+						}
+					}
+				} else {
+					if ((boundNullTagBits & TagBits.AnnotationNonNull) != 0) {
+						if (nullTagBits == 0L) {
+							nullTagBits = TagBits.AnnotationNonNull;
+						} else if (wildcard != null && (nullTagBits & TagBits.AnnotationNullable) != 0) {
+							Annotation annotation = wildcard.bound.findAnnotation(boundNullTagBits);
+							if (annotation == null) { // false alarm, implicit annotation is no conflict, but should be removed:
+								TypeBinding newBound = this.bound.withoutToplevelNullAnnotation();
+								this.bound = newBound;
+								wildcard.bound.resolvedType = newBound;
+							} else {
+								scope.problemReporter().contradictoryNullAnnotationsOnBounds(annotation, nullTagBits);
+							}
+						}
+					}
+					if (nullTagBits == 0L && this.otherBounds != null) {
+						for (int i = 0, length = this.otherBounds.length; i < length; i++) {
+							if ((this.otherBounds[i].tagBits & TagBits.AnnotationNonNull) != 0) { // can this happen?
+								nullTagBits = TagBits.AnnotationNonNull;
+								break;
+							}
+						}
+					}
+				}
+			}
+		}
+		return nullTagBits;
+	}
+
+	
+	@Override
 	public ReferenceBinding actualType() {
 		return this.genericType;
 	}
 	
+	@Override
 	TypeBinding[] additionalBounds() {
 		return this.otherBounds;
 	}
 	
+	@Override
 	public int kind() {
 		return this.otherBounds == null ? Binding.WILDCARD_TYPE : Binding.INTERSECTION_TYPE;
 	}
@@ -124,6 +251,7 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#canBeInstantiated()
 	 */
+	@Override
 	public boolean canBeInstantiated() {
 		// cannot be asked per construction
 		return false;
@@ -132,7 +260,8 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#collectMissingTypes(java.util.List)
 	 */
-	public List collectMissingTypes(List missingTypes) {
+	@Override
+	public List<TypeBinding> collectMissingTypes(List<TypeBinding> missingTypes) {
 		if ((this.tagBits & TagBits.HasMissingType) != 0) {
 			missingTypes = this.bound.collectMissingTypes(missingTypes);
 		}
@@ -147,6 +276,7 @@ public class WildcardBinding extends ReferenceBinding {
 	 *   A = F   corresponds to:      F.collectSubstitutes(..., A, ..., CONSTRAINT_EQUAL (0))
 	 *   A >> F   corresponds to:   F.collectSubstitutes(..., A, ..., CONSTRAINT_SUPER (2))
 	 */
+	@Override
 	public void collectSubstitutes(Scope scope, TypeBinding actualType, InferenceContext inferenceContext, int constraint) {
 
 		if ((this.tagBits & TagBits.HasTypeVariable) == 0) return;
@@ -387,6 +517,7 @@ public class WildcardBinding extends ReferenceBinding {
 	 * genericTypeKey {rank}*|+|- [boundKey]
 	 * p.X<T> { X<?> ... } --> Lp/X<TT;>;{0}*
 	 */
+	@Override
 	public char[] computeUniqueKey(boolean isLeaf) {
 		char[] genericTypeKey = this.genericType.computeUniqueKey(false/*not a leaf*/);
 		char[] wildCardKey;
@@ -411,14 +542,17 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#constantPoolName()
 	 */
+	@Override
 	public char[] constantPoolName() {
 		return erasure().constantPoolName();
 	}
 
+	@Override
 	public TypeBinding clone(TypeBinding immaterial) {
 		return new WildcardBinding(this.genericType, this.rank, this.bound, this.otherBounds, this.boundKind, this.environment);
 	}
 	
+	@Override
 	public String annotatedDebugName() {
 		StringBuffer buffer = new StringBuffer(16);
 		AnnotationBinding [] annotations = getTypeAnnotations();
@@ -444,14 +578,13 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#debugName()
 	 */
+	@Override
 	public String debugName() {
 	    return toString();
 	}
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#erasure()
-     */
-    public TypeBinding erasure() {
+    @Override
+	public TypeBinding erasure() {
     	if (this.otherBounds == null) {
 	    	if (this.boundKind == Wildcard.EXTENDS)
 		        return this.bound.erasure();
@@ -466,10 +599,8 @@ public class WildcardBinding extends ReferenceBinding {
     		: this.bound.erasure();
     }
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#signature()
-     */
-    public char[] genericTypeSignature() {
+    @Override
+	public char[] genericTypeSignature() {
         if (this.genericSignature == null) {
             switch (this.boundKind) {
                 case Wildcard.UNBOUND :
@@ -485,10 +616,12 @@ public class WildcardBinding extends ReferenceBinding {
         return this.genericSignature;
     }
 
+	@Override
 	public int hashCode() {
 		return this.genericType.hashCode();
 	}
 
+	@Override
 	public boolean hasTypeBit(int bit) {
 		if (this.typeBits == TypeIds.BitUninitialized) {
 			// initialize from upper bounds
@@ -525,7 +658,8 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
      * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#isSuperclassOf(org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding)
      */
-    public boolean isSuperclassOf(ReferenceBinding otherType) {
+    @Override
+	public boolean isSuperclassOf(ReferenceBinding otherType) {
         if (this.boundKind == Wildcard.SUPER) {
             if (this.bound instanceof ReferenceBinding) {
                 return ((ReferenceBinding) this.bound).isSuperclassOf(otherType);
@@ -536,10 +670,8 @@ public class WildcardBinding extends ReferenceBinding {
         return false;
     }
 
-    /**
-     * Returns true if the current type denotes an intersection type: Number & Comparable<?>
-     */
-    public boolean isIntersectionType() {
+    @Override
+	public boolean isIntersectionType() {
     	return this.otherBounds != null;
     }
 
@@ -560,6 +692,7 @@ public class WildcardBinding extends ReferenceBinding {
     	return null;
     }
 
+	@Override
 	public boolean isHierarchyConnected() {
 		return this.superclass != null && this.superInterfaces != null;
 	}
@@ -579,6 +712,7 @@ public class WildcardBinding extends ReferenceBinding {
 		this.inRecursiveFunction = false;
 	}
 
+	@Override
 	public boolean isProperType(boolean admitCapture18) {
 		if (this.inRecursiveFunction)
 			return true;
@@ -598,6 +732,7 @@ public class WildcardBinding extends ReferenceBinding {
 		}
 	}
 
+	@Override
 	TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding substituteType) {
 		boolean haveSubstitution = false;
 		TypeBinding currentBound = this.bound;
@@ -629,28 +764,23 @@ public class WildcardBinding extends ReferenceBinding {
 		return this;
 	}
 
-	/**
-	 * Returns true if the type is a wildcard
-	 */
+	@Override
 	public boolean isUnboundWildcard() {
 	    return this.boundKind == Wildcard.UNBOUND;
 	}
 
-    /**
-	 * Returns true if the type is a wildcard
-	 */
+	@Override
 	public boolean isWildcard() {
 	    return true;
 	}
 
+	@Override
 	int rank() {
 		return this.rank;
 	}
 	
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding#readableName()
-     */
-    public char[] readableName() {
+    @Override
+	public char[] readableName() {
         switch (this.boundKind) {
             case Wildcard.UNBOUND :
                 return TypeConstants.WILDCARD_NAME;
@@ -671,7 +801,8 @@ public class WildcardBinding extends ReferenceBinding {
         }
     }
 
-    public char[] nullAnnotatedReadableName(CompilerOptions options, boolean shortNames) {
+    @Override
+	public char[] nullAnnotatedReadableName(CompilerOptions options, boolean shortNames) {
     	StringBuffer buffer = new StringBuffer(10);
     	appendNullAnnotation(buffer, options);
         switch (this.boundKind) {
@@ -704,6 +835,7 @@ public class WildcardBinding extends ReferenceBinding {
 
 		this.tagBits &= ~TagBits.HasUnresolvedTypeVariables;
 		BinaryTypeBinding.resolveType(this.genericType, this.environment, false /* no raw conversion */);
+
 		switch(this.boundKind) {
 			case Wildcard.EXTENDS :
 				TypeBinding resolveType = BinaryTypeBinding.resolveType(this.bound, this.environment, true /* raw conversion */);
@@ -722,13 +854,14 @@ public class WildcardBinding extends ReferenceBinding {
 				break;
 			case Wildcard.UNBOUND :
 		}
+		if (this.environment.usesNullTypeAnnotations()) {
+			evaluateNullAnnotations(null, null);
+		}
 		return this;
 	}
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding#shortReadableName()
-     */
-    public char[] shortReadableName() {
+    @Override
+	public char[] shortReadableName() {
         switch (this.boundKind) {
             case Wildcard.UNBOUND :
                 return TypeConstants.WILDCARD_NAME;
@@ -752,7 +885,8 @@ public class WildcardBinding extends ReferenceBinding {
     /**
      * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding#signature()
      */
-    public char[] signature() {
+    @Override
+	public char[] signature() {
      	// should not be called directly on a wildcard; signature should only be asked on
     	// original methods or type erasures (which cannot denote wildcards at first level)
 		if (this.signature == null) {
@@ -766,10 +900,8 @@ public class WildcardBinding extends ReferenceBinding {
 		return this.signature;
     }
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#sourceName()
-     */
-    public char[] sourceName() {
+    @Override
+	public char[] sourceName() {
         switch (this.boundKind) {
             case Wildcard.UNBOUND :
                 return TypeConstants.WILDCARD_NAME;
@@ -780,10 +912,8 @@ public class WildcardBinding extends ReferenceBinding {
         }
     }
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding#superclass()
-     */
-    public ReferenceBinding superclass() {
+    @Override
+	public ReferenceBinding superclass() {
 		if (this.superclass == null) {
 			TypeBinding superType = null;
 			if (this.boundKind == Wildcard.EXTENDS && !this.bound.isInterface()) {
@@ -794,16 +924,14 @@ public class WildcardBinding extends ReferenceBinding {
 			}
 			this.superclass = superType instanceof ReferenceBinding && !superType.isInterface()
 				? (ReferenceBinding) superType
-				: this.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null);
+				: this.environment.getResolvedJavaBaseType(TypeConstants.JAVA_LANG_OBJECT, null);
 		}
 
 		return this.superclass;
     }
 
-    /* (non-Javadoc)
-     * @see org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding#superInterfaces()
-     */
-    public ReferenceBinding[] superInterfaces() {
+    @Override
+	public ReferenceBinding[] superInterfaces() {
         if (this.superInterfaces == null) {
         	if (typeVariable() != null) {
         		this.superInterfaces = this.typeVariable.superInterfaces();
@@ -831,6 +959,7 @@ public class WildcardBinding extends ReferenceBinding {
         return this.superInterfaces;
     }
 
+	@Override
 	public void swapUnresolved(UnresolvedReferenceBinding unresolvedType, ReferenceBinding resolvedType, LookupEnvironment env) {
 		boolean affected = false;
 		if (this.genericType == unresolvedType) { //$IDENTITY-COMPARISON$
@@ -856,6 +985,7 @@ public class WildcardBinding extends ReferenceBinding {
 	/**
 	 * @see java.lang.Object#toString()
 	 */
+	@Override
 	public String toString() {
 		if (this.hasTypeAnnotations())
 			return annotatedDebugName();
@@ -886,6 +1016,7 @@ public class WildcardBinding extends ReferenceBinding {
 		return this.typeVariable;
 	}
 
+	@Override
 	public TypeBinding unannotated() {
 		return this.hasTypeAnnotations() ? this.environment.getUnannotatedType(this) : this;
 	}
@@ -938,7 +1069,26 @@ public class WildcardBinding extends ReferenceBinding {
 		return false;
 	}
 
+	@Override
 	public boolean acceptsNonNullDefault() {
 		return false;
 	}
+
+	@Override
+	public long updateTagBits() {
+		if (!this.inRecursiveFunction) {
+			this.inRecursiveFunction = true;
+			try {
+				if (this.bound != null)
+					this.tagBits |= this.bound.updateTagBits();
+				if (this.otherBounds != null) {
+					for (int i = 0, length = this.otherBounds.length; i < length; i++)
+						this.tagBits |= this.otherBounds[i].updateTagBits();
+				}
+			} finally {
+				this.inRecursiveFunction = false;
+			}
+		}
+		return super.updateTagBits();
+	}
 }
