@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -82,6 +85,7 @@ public RecoveredType(TypeDeclaration typeDeclaration, RecoveredElement parent, i
 
 	this.preserveContent = parser().methodRecoveryActivated || parser().statementRecoveryActivated;
 }
+@Override
 public RecoveredElement add(AbstractMethodDeclaration methodDeclaration, int bracketBalanceValue) {
 
 	/* do not consider a method starting passed the type end (if set)
@@ -135,6 +139,7 @@ public RecoveredElement add(AbstractMethodDeclaration methodDeclaration, int bra
 	if (methodDeclaration.declarationSourceEnd == 0) return element;
 	return this;
 }
+@Override
 public RecoveredElement add(Block nestedBlockDeclaration,int bracketBalanceValue) {
 	this.pendingTypeParameters = null;
 	resetPendingModifiers();
@@ -145,6 +150,7 @@ public RecoveredElement add(Block nestedBlockDeclaration,int bracketBalanceValue
 	}
 	return this.add(new Initializer(nestedBlockDeclaration, mods), bracketBalanceValue);
 }
+@Override
 public RecoveredElement add(FieldDeclaration fieldDeclaration, int bracketBalanceValue) {
 	this.pendingTypeParameters = null;
 
@@ -203,6 +209,7 @@ public RecoveredElement add(FieldDeclaration fieldDeclaration, int bracketBalanc
 	if (fieldDeclaration.declarationSourceEnd == 0) return element;
 	return this;
 }
+@Override
 public RecoveredElement add(TypeDeclaration memberTypeDeclaration, int bracketBalanceValue) {
 	this.pendingTypeParameters = null;
 
@@ -273,6 +280,7 @@ public void add(TypeParameter[] parameters, int startPos) {
 	this.pendingTypeParameters = parameters;
 	this.pendingTypeParametersStart = startPos;
 }
+@Override
 public RecoveredElement addAnnotationName(int identifierPtr, int identifierLengthPtr, int annotationStart, int bracketBalanceValue) {
 	if (this.pendingAnnotations == null) {
 		this.pendingAnnotations = new RecoveredAnnotation[5];
@@ -294,6 +302,7 @@ public RecoveredElement addAnnotationName(int identifierPtr, int identifierLengt
 
 	return element;
 }
+@Override
 public void addModifier(int flag, int modifiersSourceStart) {
 	this.pendingModifiers |= flag;
 
@@ -350,6 +359,7 @@ public boolean bodyStartsAtHeaderEnd(){
 /*
  * Answer the enclosing type node, or null if none
  */
+@Override
 public RecoveredType enclosingType(){
 	RecoveredElement current = this.parent;
 	while (current != null){
@@ -386,15 +396,44 @@ public int lastMemberEnd() {
 
 	return lastMemberEnd;
 }
+@Override
+public int getLastStart() {
+	int lastMemberStart = this.typeDeclaration.bodyStart;
+
+	if (this.fieldCount > 0) {
+		FieldDeclaration lastField = this.fields[this.fieldCount - 1].fieldDeclaration;
+		if (lastMemberStart < lastField.declarationSourceStart && lastField.declarationSourceStart != 0) {
+			lastMemberStart = lastField.declarationSourceStart;
+		}
+	}
+
+	if (this.methodCount > 0) {
+		AbstractMethodDeclaration lastMethod = this.methods[this.methodCount - 1].methodDeclaration;
+		if (lastMemberStart < lastMethod.declarationSourceStart && lastMethod.declarationSourceStart != 0) {
+			lastMemberStart = lastMethod.declarationSourceStart;
+		}
+	}
+
+	if (this.memberTypeCount > 0) {
+		TypeDeclaration lastType = this.memberTypes[this.memberTypeCount - 1].typeDeclaration;
+		if (lastMemberStart < lastType.declarationSourceStart && lastType.declarationSourceStart != 0) {
+			lastMemberStart = lastType.declarationSourceStart;
+		}
+	}
+
+	return lastMemberStart;
+}
 public char[] name(){
 	return this.typeDeclaration.name;
 }
 /*
  * Answer the associated parsed structure
  */
+@Override
 public ASTNode parseTree(){
 	return this.typeDeclaration;
 }
+@Override
 public void resetPendingModifiers() {
 	this.pendingAnnotations = null;
 	this.pendingAnnotationCount = 0;
@@ -404,9 +443,11 @@ public void resetPendingModifiers() {
 /*
  * Answer the very source end of the corresponding parse node
  */
+@Override
 public int sourceEnd(){
 	return this.typeDeclaration.declarationSourceEnd;
 }
+@Override
 public String toString(int tab) {
 	StringBuffer result = new StringBuffer(tabString(tab));
 	result.append("Recovered type:\n"); //$NON-NLS-1$
@@ -444,10 +485,12 @@ public String toString(int tab) {
 /*
  * Update the bodyStart of the corresponding parse node
  */
+@Override
 public void updateBodyStart(int bodyStart){
 	this.foundOpeningBrace = true;
 	this.typeDeclaration.bodyStart = bodyStart;
 }
+@Override
 public Statement updatedStatement(int depth, Set knownTypes){
 
 	// ignore closed anonymous type
@@ -467,7 +510,7 @@ public Statement updatedStatement(int depth, Set knownTypes){
 	}
 	return updatedType;
 }
-public TypeDeclaration updatedTypeDeclaration(int depth, Set knownTypes){
+public TypeDeclaration updatedTypeDeclaration(int depth, Set<TypeDeclaration> knownTypes){
 	if (depth >= MAX_TYPE_DEPTH) return null;
 
 	if(knownTypes.contains(this.typeDeclaration)) return null;
@@ -541,8 +584,13 @@ public TypeDeclaration updatedTypeDeclaration(int depth, Set knownTypes){
 		// may need to update the declarationSourceEnd of the last field
 		if (this.fields[this.fieldCount - 1].fieldDeclaration.declarationSourceEnd == 0){
 			int temp = bodyEnd();
-			this.fields[this.fieldCount - 1].fieldDeclaration.declarationSourceEnd = temp;
-			this.fields[this.fieldCount - 1].fieldDeclaration.declarationEnd = temp;
+			FieldDeclaration fieldDeclaration = this.fields[this.fieldCount - 1].fieldDeclaration;
+			if (temp == 0 && fieldDeclaration.sourceEnd > 0) {
+				temp = fieldDeclaration.sourceEnd;
+				if (lastEnd > temp) lastEnd = temp;
+			}
+			fieldDeclaration.declarationSourceEnd = temp;
+			fieldDeclaration.declarationEnd = temp;
 		}
 		for (int i = 0; i < this.fieldCount; i++){
 			fieldDeclarations[existingCount + i] = this.fields[i].updatedFieldDeclaration(depth, knownTypes);
@@ -664,6 +712,7 @@ public TypeDeclaration updatedTypeDeclaration(int depth, Set knownTypes){
  * Update the corresponding parse node from parser state which
  * is about to disappear because of restarting recovery
  */
+@Override
 public void updateFromParserState(){
 
 	// anymous type and enum constant doesn't need to be updated
@@ -721,6 +770,7 @@ public void updateFromParserState(){
  * A closing brace got consumed, might have closed the current element,
  * in which case both the currentElement is exited
  */
+@Override
 public RecoveredElement updateOnClosingBrace(int braceStart, int braceEnd){
 	if ((--this.bracketBalance <= 0) && (this.parent != null)){
 		this.updateSourceEndIfNecessary(braceStart, braceEnd);
@@ -733,6 +783,7 @@ public RecoveredElement updateOnClosingBrace(int braceStart, int braceEnd){
  * An opening brace got consumed, might be the expected opening one of the current element,
  * in which case the bodyStart is updated.
  */
+@Override
 public RecoveredElement updateOnOpeningBrace(int braceStart, int braceEnd){
 	/* in case the opening brace is not close enough to the signature, ignore it */
 	if (this.bracketBalance == 0){
@@ -772,12 +823,14 @@ public RecoveredElement updateOnOpeningBrace(int braceStart, int braceEnd){
 	}
 	return super.updateOnOpeningBrace(braceStart, braceEnd);
 }
+@Override
 public void updateParseTree(){
 	updatedTypeDeclaration(0, new HashSet());
 }
 /*
  * Update the declarationSourceEnd of the corresponding parse node
  */
+@Override
 public void updateSourceEndIfNecessary(int start, int end){
 	if (this.typeDeclaration.declarationSourceEnd == 0){
 		this.bodyEnd = 0;
