@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2006, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -147,6 +150,7 @@ public class RecoveryScanner extends Scanner {
 		this.data.removedTokenUsed[this.data.removedTokensPtr] = false;
 	}
 
+	@Override
 	protected int getNextToken0() throws InvalidInputException {
 		if(this.pendingTokensPtr > -1) {
 			int pendingToken = this.pendingTokens[this.pendingTokensPtr--];
@@ -221,16 +225,19 @@ public class RecoveryScanner extends Scanner {
 		return currentToken;
 	}
 
+	@Override
 	public char[] getCurrentIdentifierSource() {
 		if(this.fakeTokenSource != null) return this.fakeTokenSource;
 		return super.getCurrentIdentifierSource();
 	}
 
+	@Override
 	public char[] getCurrentTokenSourceString() {
 		if(this.fakeTokenSource != null) return this.fakeTokenSource;
 		return super.getCurrentTokenSourceString();
 	}
 
+	@Override
 	public char[] getCurrentTokenSource() {
 		if(this.fakeTokenSource != null) return this.fakeTokenSource;
 		return super.getCurrentTokenSource();
