@@ -1,9 +1,13 @@
+// ASPECTJ
 /*******************************************************************************
  * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -2545,6 +2549,7 @@ public class DiagnoseParser implements ParserBasicInformation, TerminalTokens, C
     	}
 	}
 
+	@Override
 	public String toString() {
 		StringBuffer res = new StringBuffer();
 
@@ -2553,6 +2558,7 @@ public class DiagnoseParser implements ParserBasicInformation, TerminalTokens, C
 		return res.toString();
 	}
 
+	@Override
 	public boolean atConflictScenario(int token) {
 		/* There is too much voodoo that goes on here in DiagnoseParser (multiple machines, lexer stream reset etc.)
 		   So we take a simple minded view that we will always ask for disambiguation, except there is one scenario 
@@ -2562,4 +2568,10 @@ public class DiagnoseParser implements ParserBasicInformation, TerminalTokens, C
 		*/ 
 		return (token == TokenNameLPAREN || token == TokenNameAT || (token == TokenNameLESS && !this.lexStream.awaitingColonColon()));
 	}
+
+	@Override
+	public boolean isParsingModuleDeclaration() {
+		//
+		return this.parser.isParsingModuleDeclaration();
+	}
 }
