@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -146,6 +149,15 @@ private IMemberValuePair[] getTargetElementTypes(long tagBits) {
 	if ((tagBits & TagBits.AnnotationForPackage) != 0) {
 		values.add(elementType + new String(TypeConstants.UPPER_PACKAGE));
 	}
+	if ((tagBits & TagBits.AnnotationForTypeUse) != 0) {
+		values.add(elementType + new String(TypeConstants.TYPE_USE_TARGET));
+	}
+	if ((tagBits & TagBits.AnnotationForTypeParameter) != 0) {
+		values.add(elementType + new String(TypeConstants.TYPE_PARAMETER_TARGET));
+	}
+	if ((tagBits & TagBits.AnnotationForModule) != 0) {
+		values.add(elementType + new String(TypeConstants.UPPER_MODULE));
+	}
 	final Object value;
 	if (values.size() == 0) {
 		if ((tagBits & TagBits.AnnotationTarget) != 0)
@@ -159,12 +171,15 @@ private IMemberValuePair[] getTargetElementTypes(long tagBits) {
 	}
 	return new IMemberValuePair[] {
 		new IMemberValuePair() {
+			@Override
 			public int getValueKind() {
 				return IMemberValuePair.K_QUALIFIED_NAME;
 			}
+			@Override
 			public Object getValue() {
 				return value;
 			}
+			@Override
 			public String getMemberName() {
 				return new String(TypeConstants.VALUE);
 			}
@@ -188,12 +203,15 @@ private IMemberValuePair[] getRetentionPolicy(long tagBits) {
 	return 
 		new IMemberValuePair[] {
 			new IMemberValuePair() {
+				@Override
 				public int getValueKind() {
 					return IMemberValuePair.K_QUALIFIED_NAME;
 				}
+				@Override
 				public Object getValue() {
 					return value;
 				}
+				@Override
 				public String getMemberName() {
 					return new String(TypeConstants.VALUE);
 				}
@@ -343,7 +361,7 @@ private void generateMethodInfos(IType type, IBinaryType typeInfo, HashMap newEl
 			// ignore
 		}
 		for (int j = startIndex; j < max; j++) {
-			IBinaryAnnotation[] parameterAnnotations = methodInfo.getParameterAnnotations(j - startIndex);
+			IBinaryAnnotation[] parameterAnnotations = methodInfo.getParameterAnnotations(j - startIndex, typeInfo.getFileName());
 			if (parameterAnnotations != null) {
 				LocalVariable localVariable = new LocalVariable(
 						method,
@@ -398,13 +416,6 @@ private void generateTypeParameterInfos(BinaryMember parent, char[] signature, H
 		newElements.put(typeParameter, info);
 	}
 }
-/**
- * Returns true iff the <code>readBinaryChildren</code> has already
- * been called.
- */
-boolean hasReadBinaryChildren() {
-	return this.binaryChildren != null;
-}
 /**
  * Creates the handles for <code>BinaryMember</code>s defined in this
  * <code>ClassFile</code> and adds them to the
