@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -87,7 +90,7 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 		initializeASTParser();
 	}
 	private void initializeASTParser() {
-		this.parser = ASTParser.newParser(AST.JLS8);
+		this.parser = ASTParser.newParser(AST.JLS11);
 	}
 	/**
 	 * Returns the children of <code>source</code> which are affected by this operation.
@@ -187,9 +190,11 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 	/**
 	 * @see MultiOperation
 	 */
+	@Override
 	protected String getMainTaskName() {
 		return Messages.operation_copyResourceProgress;
 	}
+	@Override
 	protected ISchedulingRule getSchedulingRule() {
 		if (this.elementsToProcess == null)
 			return null;
@@ -404,6 +409,7 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 	 * <code>processPackageFragmentResource</code>, depending on the type of
 	 * <code>element</code>.
 	 */
+	@Override
 	protected void processElement(IJavaElement element) throws JavaModelException {
 		IJavaElement dest = getDestinationParent(element);
 		switch (element.getElementType()) {
@@ -423,6 +429,7 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 	 * Overridden to allow special processing of <code>JavaElementDelta</code>s
 	 * and <code>fResultElements</code>.
 	 */
+	@Override
 	protected void processElements() throws JavaModelException {
 		this.createdElements = new ArrayList(this.elementsToProcess.length);
 		try {
@@ -745,6 +752,7 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 	 *		does not match the number of elements that were supplied.
 	 * </ul>
 	 */
+	@Override
 	protected IJavaModelStatus verify() {
 		IJavaModelStatus status = super.verify();
 		if (!status.isOK()) {
@@ -759,6 +767,7 @@ public class CopyResourceElementsOperation extends MultiOperation implements Suf
 	/**
 	 * @see MultiOperation
 	 */
+	@Override
 	protected void verify(IJavaElement element) throws JavaModelException {
 		if (element == null || !element.exists())
 			error(IJavaModelStatusConstants.ELEMENT_DOES_NOT_EXIST, element);
