@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -19,18 +22,22 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.util.*;
+import java.util.Map.Entry;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
 import org.aspectj.org.eclipse.jdt.core.*;
+import org.aspectj.org.eclipse.jdt.internal.core.DeltaProcessor.RootInfo;
 import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager.PerProjectInfo;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.indexer.Indexer;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.indexer.IndexerEvent;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.JavaIndex;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
 
 /**
  * Keep the global states used during Java element delta processing.
  */
-@SuppressWarnings({ "rawtypes", "unchecked" })
-public class DeltaProcessingState implements IResourceChangeListener {
+public class DeltaProcessingState implements IResourceChangeListener, Indexer.Listener {
 
 	/*
 	 * Collection of listeners for Java element deltas
@@ -49,8 +56,8 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	/*
 	 * The delta processor for the current thread.
 	 */
-	private ThreadLocal deltaProcessors = new ThreadLocal();
-	
+	private ThreadLocal<DeltaProcessor> deltaProcessors = new ThreadLocal<>();
+
 	public void doNotUse() {
 		// reset the delta processor of the current thread to avoid to keep it in memory
 		// https://bugs.eclipse.org/bugs/show_bug.cgi?id=269476
@@ -58,63 +65,63 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	}
 
 	/* A table from IPath (from a classpath entry) to DeltaProcessor.RootInfo */
-	public HashMap roots = new HashMap();
+	public Map<IPath, RootInfo> roots = new LinkedHashMap<>();
 
 	/* A table from IPath (from a classpath entry) to ArrayList of DeltaProcessor.RootInfo
 	 * Used when an IPath corresponds to more than one root */
-	public HashMap otherRoots = new HashMap();
+	public Map<IPath, List<RootInfo>> otherRoots = new HashMap<>();
 
 	/* A table from IPath (from a classpath entry) to DeltaProcessor.RootInfo
 	 * from the last time the delta processor was invoked. */
-	public HashMap oldRoots = new HashMap();
+	public Map<IPath, RootInfo> oldRoots = new LinkedHashMap<>();
 
 	/* A table from IPath (from a classpath entry) to ArrayList of DeltaProcessor.RootInfo
 	 * from the last time the delta processor was invoked.
 	 * Used when an IPath corresponds to more than one root */
-	public HashMap oldOtherRoots = new HashMap();
+	public Map<IPath, List<RootInfo>> oldOtherRoots = new HashMap<>();
 
 	/* A table from IPath (a source attachment path from a classpath entry) to IPath (a root path) */
-	public HashMap sourceAttachments = new HashMap();
+	public Map<IPath, IPath> sourceAttachments = new HashMap<>();
 
 	/* A table from IJavaProject to IJavaProject[] (the list of direct dependent of the key) */
-	public HashMap projectDependencies = new HashMap();
+	public Map<IJavaProject, IJavaProject[]> projectDependencies = new HashMap<>();
 
 	/* Whether the roots tables should be recomputed */
 	public boolean rootsAreStale = true;
 
 	/* Threads that are currently running initializeRoots() */
-	private Set initializingThreads = Collections.synchronizedSet(new HashSet());
+	private Set<Thread> initializingThreads = Collections.synchronizedSet(new HashSet<>());
 
 	/* A table from file system absoulte path (String) to timestamp (Long) */
-	public Hashtable externalTimeStamps;
+	public Hashtable<IPath, Long> externalTimeStamps;
 
 	/*
 	 * Map from IProject to ClasspathChange
 	 * Note these changes need to be kept on the delta processing state to ensure we don't loose them
 	 * (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=271102 Java model corrupt after switching target platform)
 	 */
-	private HashMap classpathChanges = new HashMap();
+	private Map<IProject, ClasspathChange> classpathChanges = new LinkedHashMap<>();
 
 	/* A table from JavaProject to ClasspathValidation */
-	private HashMap classpathValidations = new HashMap();
+	private Map<JavaProject, ClasspathValidation> classpathValidations = new LinkedHashMap<>();
 
 	/* A table from JavaProject to ProjectReferenceChange */
-	private HashMap projectReferenceChanges = new HashMap();
+	private Set<IJavaProject> projectReferenceChanges = new LinkedHashSet<>();
 
 	/* A table from JavaProject to ExternalFolderChange */
-	private HashMap externalFolderChanges = new HashMap();
+	private Map<JavaProject, ExternalFolderChange> externalFolderChanges = new LinkedHashMap<>();
 
 	/**
 	 * Workaround for bug 15168 circular errors not reported
 	 * This is a cache of the projects before any project addition/deletion has started.
 	 */
-	private HashSet javaProjectNamesCache;
+	private Set<String> javaProjectNamesCache;
 
 	/*
 	 * A list of IJavaElement used as a scope for external archives refresh during POST_CHANGE.
 	 * This is null if no refresh is needed.
 	 */
-	private HashSet externalElementsToRefresh;
+	private Set<IJavaElement> externalElementsToRefresh;
 
 	/*
 	 * Need to clone defensively the listener information, in case some listener is reacting to some notification iteration by adding/changing/removing
@@ -148,7 +155,7 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	 */
 	public synchronized void addForRefresh(IJavaElement externalElement) {
 		if (this.externalElementsToRefresh == null) {
-			this.externalElementsToRefresh = new HashSet();
+			this.externalElementsToRefresh = new LinkedHashSet<>();
 		}
 		this.externalElementsToRefresh.add(externalElement);
 	}
@@ -172,7 +179,7 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	}
 
 	public DeltaProcessor getDeltaProcessor() {
-		DeltaProcessor deltaProcessor = (DeltaProcessor)this.deltaProcessors.get();
+		DeltaProcessor deltaProcessor = this.deltaProcessors.get();
 		if (deltaProcessor != null) return deltaProcessor;
 		deltaProcessor = new DeltaProcessor(this, JavaModelManager.getJavaModelManager());
 		this.deltaProcessors.set(deltaProcessor);
@@ -181,7 +188,7 @@ public class DeltaProcessingState implements IResourceChangeListener {
 
 	public ClasspathChange addClasspathChange(IProject project, IClasspathEntry[] oldRawClasspath, IPath oldOutputLocation, IClasspathEntry[] oldResolvedClasspath) {
 		synchronized (this.classpathChanges) {
-			ClasspathChange change = (ClasspathChange) this.classpathChanges.get(project);
+			ClasspathChange change = this.classpathChanges.get(project);
 			if (change == null) {
 				change = new ClasspathChange((JavaProject) JavaModelManager.getJavaModelManager().getJavaModel().getJavaProject(project), oldRawClasspath, oldOutputLocation, oldResolvedClasspath);
 				this.classpathChanges.put(project, change);
@@ -196,23 +203,23 @@ public class DeltaProcessingState implements IResourceChangeListener {
 			return change;
 		}
 	}
-	
+
 	public ClasspathChange getClasspathChange(IProject project) {
 		synchronized (this.classpathChanges) {
-			return (ClasspathChange) this.classpathChanges.get(project);
+			return this.classpathChanges.get(project);
 		}
 	}
-	
-	public HashMap removeAllClasspathChanges() {
+
+	public Map<IProject, ClasspathChange> removeAllClasspathChanges() {
 		synchronized (this.classpathChanges) {
-			HashMap result = this.classpathChanges;
-			this.classpathChanges = new HashMap(result.size());
+			Map<IProject, ClasspathChange> result = this.classpathChanges;
+			this.classpathChanges = new LinkedHashMap<>(result.size());
 			return result;
 		}
 	}
 
 	public synchronized ClasspathValidation addClasspathValidation(JavaProject project) {
-		ClasspathValidation validation = (ClasspathValidation) this.classpathValidations.get(project);
+		ClasspathValidation validation = this.classpathValidations.get(project);
 		if (validation == null) {
 			validation = new ClasspathValidation(project);
 			this.classpathValidations.put(project, validation);
@@ -221,25 +228,21 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	}
 
 	public synchronized void addExternalFolderChange(JavaProject project, IClasspathEntry[] oldResolvedClasspath) {
-		ExternalFolderChange change = (ExternalFolderChange) this.externalFolderChanges.get(project);
+		ExternalFolderChange change = this.externalFolderChanges.get(project);
 		if (change == null) {
 			change = new ExternalFolderChange(project, oldResolvedClasspath);
 			this.externalFolderChanges.put(project, change);
-	    }
+		}
 	}
 
-	public synchronized void addProjectReferenceChange(JavaProject project, IClasspathEntry[] oldResolvedClasspath) {
-		ProjectReferenceChange change = (ProjectReferenceChange) this.projectReferenceChanges.get(project);
-		if (change == null) {
-			change = new ProjectReferenceChange(project, oldResolvedClasspath);
-			this.projectReferenceChanges.put(project, change);
-	    }
+	public synchronized void addProjectReferenceChange(IJavaProject project) {
+		this.projectReferenceChanges.add(project);
 	}
 
 	public void initializeRoots(boolean initAfterLoad) {
 
 		// recompute root infos only if necessary
-		HashMap[] rootInfos = null;
+		RootInfos rootInfos = null;
 		if (this.rootsAreStale) {
 			Thread currentThread = Thread.currentThread();
 			boolean addedCurrentThread = false;
@@ -265,31 +268,28 @@ public class DeltaProcessingState implements IResourceChangeListener {
 			this.oldRoots = this.roots;
 			this.oldOtherRoots = this.otherRoots;
 			if (this.rootsAreStale && rootInfos != null) { // double check again
-				this.roots = rootInfos[0];
-				this.otherRoots = rootInfos[1];
-				this.sourceAttachments = rootInfos[2];
-				this.projectDependencies = rootInfos[3];
+				this.roots = rootInfos.roots;
+				this.otherRoots = rootInfos.otherRoots;
+				this.sourceAttachments = rootInfos.sourceAttachments;
+				this.projectDependencies = rootInfos.projectDependencies;
 				this.rootsAreStale = false;
 			}
 		}
 	}
 
 	synchronized void initializeRootsWithPreviousSession() {
-		HashMap[] rootInfos = getRootInfos(true/*use previous session values*/);
+		RootInfos rootInfos = getRootInfos(true/*use previous session values*/);
 		if (rootInfos != null) {
-			this.roots = rootInfos[0];
-			this.otherRoots = rootInfos[1];
-			this.sourceAttachments = rootInfos[2];
-			this.projectDependencies = rootInfos[3];
+			this.roots = rootInfos.roots;
+			this.otherRoots = rootInfos.otherRoots;
+			this.sourceAttachments = rootInfos.sourceAttachments;
+			this.projectDependencies = rootInfos.projectDependencies;
 			this.rootsAreStale = false;
 		}
 	}
 
-	private HashMap[] getRootInfos(boolean usePreviousSession) {
-		HashMap newRoots = new HashMap();
-		HashMap newOtherRoots = new HashMap();
-		HashMap newSourceAttachments = new HashMap();
-		HashMap newProjectDependencies = new HashMap();
+	private RootInfos getRootInfos(boolean usePreviousSession) {
+		RootInfos ri = new RootInfos();
 
 		IJavaModel model = JavaModelManager.getJavaModelManager().getJavaModel();
 		IJavaProject[] projects;
@@ -318,7 +318,7 @@ public class DeltaProcessingState implements IResourceChangeListener {
 				IClasspathEntry entry = classpath[j];
 				if (entry.getEntryKind() == IClasspathEntry.CPE_PROJECT) {
 					IJavaProject key = model.getJavaProject(entry.getPath().segment(0)); // TODO (jerome) reuse handle
-					IJavaProject[] dependents = (IJavaProject[]) newProjectDependencies.get(key);
+					IJavaProject[] dependents = ri.projectDependencies.get(key);
 					if (dependents == null) {
 						dependents = new IJavaProject[] {project};
 					} else {
@@ -326,19 +326,19 @@ public class DeltaProcessingState implements IResourceChangeListener {
 						System.arraycopy(dependents, 0, dependents = new IJavaProject[dependentsLength+1], 0, dependentsLength);
 						dependents[dependentsLength] = project;
 					}
-					newProjectDependencies.put(key, dependents);
+					ri.projectDependencies.put(key, dependents);
 					continue;
 				}
 
 				// root path
 				IPath path = entry.getPath();
-				if (newRoots.get(path) == null) {
-					newRoots.put(path, new DeltaProcessor.RootInfo(project, path, ((ClasspathEntry)entry).fullInclusionPatternChars(), ((ClasspathEntry)entry).fullExclusionPatternChars(), entry.getEntryKind()));
+				if (ri.roots.get(path) == null) {
+					ri.roots.put(path, new DeltaProcessor.RootInfo(project, path, ((ClasspathEntry)entry).fullInclusionPatternChars(), ((ClasspathEntry)entry).fullExclusionPatternChars(), entry.getEntryKind()));
 				} else {
-					ArrayList rootList = (ArrayList)newOtherRoots.get(path);
+					List<RootInfo> rootList = ri.otherRoots.get(path);
 					if (rootList == null) {
-						rootList = new ArrayList();
-						newOtherRoots.put(path, rootList);
+						rootList = new ArrayList<>();
+						ri.otherRoots.put(path, rootList);
 					}
 					rootList.add(new DeltaProcessor.RootInfo(project, path, ((ClasspathEntry)entry).fullInclusionPatternChars(), ((ClasspathEntry)entry).fullExclusionPatternChars(), entry.getEntryKind()));
 				}
@@ -359,11 +359,11 @@ public class DeltaProcessingState implements IResourceChangeListener {
 					sourceAttachmentPath = entry.getSourceAttachmentPath();
 				}
 				if (sourceAttachmentPath != null) {
-					newSourceAttachments.put(sourceAttachmentPath, path);
+					ri.sourceAttachments.put(sourceAttachmentPath, path);
 				}
 			}
 		}
-		return new HashMap[] {newRoots, newOtherRoots, newSourceAttachments, newProjectDependencies};
+		return ri;
 	}
 
 	public synchronized ClasspathValidation[] removeClasspathValidations() {
@@ -384,17 +384,14 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	    return updates;
 	}
 
-	public synchronized ProjectReferenceChange[] removeProjectReferenceChanges() {
-	    int length = this.projectReferenceChanges.size();
-	    if (length == 0) return null;
-	    ProjectReferenceChange[]  updates = new ProjectReferenceChange[length];
-	    this.projectReferenceChanges.values().toArray(updates);
-	    this.projectReferenceChanges.clear();
-	    return updates;
+	public synchronized Set<IJavaProject> removeProjectReferenceChanges() {
+		Set<IJavaProject> result = this.projectReferenceChanges;
+		this.projectReferenceChanges = new HashSet<>();
+		return result;
 	}
 
-	public synchronized HashSet removeExternalElementsToRefresh() {
-		HashSet result = this.externalElementsToRefresh;
+	public synchronized Set<IJavaElement> removeExternalElementsToRefresh() {
+		Set<IJavaElement> result = this.externalElementsToRefresh;
 		this.externalElementsToRefresh = null;
 		return result;
 	}
@@ -459,15 +456,18 @@ public class DeltaProcessingState implements IResourceChangeListener {
 		}
 	}
 
+	@Override
 	public void resourceChanged(final IResourceChangeEvent event) {
 		for (int i = 0; i < this.preResourceChangeListenerCount; i++) {
 			// wrap callbacks with Safe runnable for subsequent listeners to be called when some are causing grief
 			final IResourceChangeListener listener = this.preResourceChangeListeners[i];
 			if ((this.preResourceChangeEventMasks[i] & event.getType()) != 0)
 				SafeRunner.run(new ISafeRunnable() {
+					@Override
 					public void handleException(Throwable exception) {
 						Util.log(exception, "Exception occurred in listener of pre Java resource change notification"); //$NON-NLS-1$
 					}
+					@Override
 					public void run() throws Exception {
 						listener.resourceChanged(event);
 					}
@@ -488,9 +488,9 @@ public class DeltaProcessingState implements IResourceChangeListener {
 
 	}
 
-	public Hashtable getExternalLibTimeStamps() {
+	public Hashtable<IPath, Long> getExternalLibTimeStamps() {
 		if (this.externalTimeStamps == null) {
-			Hashtable timeStamps = new Hashtable();
+			Hashtable<IPath, Long> timeStamps = new Hashtable<>();
 			File timestampsFile = getTimeStampsFile();
 			DataInputStream in = null;
 			try {
@@ -499,7 +499,7 @@ public class DeltaProcessingState implements IResourceChangeListener {
 				while (size-- > 0) {
 					String key = in.readUTF();
 					long timestamp = in.readLong();
-					timeStamps.put(Path.fromPortableString(key), new Long(timestamp));
+					timeStamps.put(Path.fromPortableString(key), Long.valueOf(timestamp));
 				}
 			} catch (IOException e) {
 				if (timestampsFile.exists())
@@ -529,15 +529,15 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	 * Returns the list of java projects before resource delta processing
 	 * has started.
 	 */
-	public synchronized HashSet getOldJavaProjecNames() {
+	public synchronized Set<String> getOldJavaProjecNames() {
 		if (this.javaProjectNamesCache == null) {
-			HashSet result = new HashSet();
 			IJavaProject[] projects;
 			try {
 				projects = JavaModelManager.getJavaModelManager().getJavaModel().getJavaProjects();
 			} catch (JavaModelException e) {
 				return this.javaProjectNamesCache;
 			}
+			HashSet<String> result = new LinkedHashSet<>();
 			for (int i = 0, length = projects.length; i < length; i++) {
 				IJavaProject project = projects[i];
 				result.add(project.getElementName());
@@ -557,31 +557,31 @@ public class DeltaProcessingState implements IResourceChangeListener {
 
 	public void saveExternalLibTimeStamps() throws CoreException {
 		if (this.externalTimeStamps == null) return;
-		
+
 		// cleanup to avoid any leak ( https://bugs.eclipse.org/bugs/show_bug.cgi?id=244849 )
-		HashSet toRemove = new HashSet();
+		HashSet<IPath> toRemove = new HashSet<>();
 		if (this.roots != null) {
-			Enumeration keys = this.externalTimeStamps.keys();
+			Enumeration<IPath> keys = this.externalTimeStamps.keys();
 			while (keys.hasMoreElements()) {
-				Object key = keys.nextElement();
+				IPath key = keys.nextElement();
 				if (this.roots.get(key) == null) {
 					toRemove.add(key);
 				}
 			}
 		}
-		
+
 		File timestamps = getTimeStampsFile();
 		DataOutputStream out = null;
 		try {
 			out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(timestamps)));
 			out.writeInt(this.externalTimeStamps.size() - toRemove.size());
-			Iterator entries = this.externalTimeStamps.entrySet().iterator();
+			Iterator<Entry<IPath, Long>> entries = this.externalTimeStamps.entrySet().iterator();
 			while (entries.hasNext()) {
-				Map.Entry entry = (Map.Entry) entries.next();
-				IPath key = (IPath) entry.getKey();
+				Entry<IPath, Long> entry = entries.next();
+				IPath key = entry.getKey();
 				if (!toRemove.contains(key)) {
 					out.writeUTF(key.toPortableString());
-					Long timestamp = (Long) entry.getValue();
+					Long timestamp = entry.getValue();
 					out.writeLong(timestamp.longValue());
 				}
 			}
@@ -603,8 +603,8 @@ public class DeltaProcessingState implements IResourceChangeListener {
 	 * Update the roots that are affected by the addition or the removal of the given container resource.
 	 */
 	public synchronized void updateRoots(IPath containerPath, IResourceDelta containerDelta, DeltaProcessor deltaProcessor) {
-		Map updatedRoots;
-		Map otherUpdatedRoots;
+		Map<IPath, RootInfo> updatedRoots;
+		Map<IPath, List<RootInfo>> otherUpdatedRoots;
 		if (containerDelta.getKind() == IResourceDelta.REMOVED) {
 			updatedRoots = this.oldRoots;
 			otherUpdatedRoots = this.oldOtherRoots;
@@ -614,25 +614,25 @@ public class DeltaProcessingState implements IResourceChangeListener {
 		}
 		int containerSegmentCount = containerPath.segmentCount();
 		boolean containerIsProject = containerSegmentCount == 1;
-		Iterator iterator = updatedRoots.entrySet().iterator();
+		Iterator<Entry<IPath, RootInfo>> iterator = updatedRoots.entrySet().iterator();
 		while (iterator.hasNext()) {
-			Map.Entry entry = (Map.Entry) iterator.next();
-			IPath path = (IPath) entry.getKey();
+			Entry<IPath, RootInfo> entry = iterator.next();
+			IPath path = entry.getKey();
 			if (containerPath.isPrefixOf(path) && !containerPath.equals(path)) {
 				IResourceDelta rootDelta = containerDelta.findMember(path.removeFirstSegments(containerSegmentCount));
 				if (rootDelta == null) continue;
-				DeltaProcessor.RootInfo rootInfo = (DeltaProcessor.RootInfo) entry.getValue();
+				DeltaProcessor.RootInfo rootInfo = entry.getValue();
 
 				if (!containerIsProject
 						|| !rootInfo.project.getPath().isPrefixOf(path)) { // only consider folder roots that are not included in the container
 					deltaProcessor.updateCurrentDeltaAndIndex(rootDelta, IJavaElement.PACKAGE_FRAGMENT_ROOT, rootInfo);
 				}
 
-				ArrayList rootList = (ArrayList)otherUpdatedRoots.get(path);
+				List<RootInfo> rootList = otherUpdatedRoots.get(path);
 				if (rootList != null) {
-					Iterator otherProjects = rootList.iterator();
+					Iterator<RootInfo> otherProjects = rootList.iterator();
 					while (otherProjects.hasNext()) {
-						rootInfo = (DeltaProcessor.RootInfo)otherProjects.next();
+						rootInfo = otherProjects.next();
 						if (!containerIsProject
 								|| !rootInfo.project.getPath().isPrefixOf(path)) { // only consider folder roots that are not included in the container
 							deltaProcessor.updateCurrentDeltaAndIndex(rootDelta, IJavaElement.PACKAGE_FRAGMENT_ROOT, rootInfo);
@@ -643,4 +643,28 @@ public class DeltaProcessingState implements IResourceChangeListener {
 		}
 	}
 
+	@Override
+	public void consume(IndexerEvent event) {
+		if (JavaIndex.isEnabled()) {
+			DeltaProcessor processor = getDeltaProcessor();
+			JavaElementDelta delta = (JavaElementDelta) event.getDelta();
+			delta.ignoreFromTests = true;
+			processor.notifyAndFire(delta);
+			this.deltaProcessors.set(null);
+		}
+	}
+
+	private static final class RootInfos {
+		final Map<IPath, RootInfo> roots;
+		final Map<IPath, List<RootInfo>> otherRoots;
+		final Map<IPath, IPath> sourceAttachments;
+		final Map<IJavaProject, IJavaProject[]> projectDependencies;
+
+		public RootInfos() {
+			this.roots = new LinkedHashMap<>();
+			this.otherRoots = new HashMap<>();
+			this.sourceAttachments = new HashMap<>();
+			this.projectDependencies = new HashMap<>();
+		}
+	}
 }
