@@ -1,17 +1,26 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.core;
 
+import java.io.IOException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.jar.Manifest;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
@@ -20,9 +29,21 @@ import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IStatus;
-import org.aspectj.org.eclipse.jdt.core.*;
+import org.aspectj.org.eclipse.jdt.core.IClasspathEntry;
+import org.aspectj.org.eclipse.jdt.core.IJavaElement;
+import org.aspectj.org.eclipse.jdt.core.IModuleDescription;
+import org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot;
+import org.aspectj.org.eclipse.jdt.core.JavaCore;
+import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.IReader;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.JavaIndex;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdResourceFile;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdType;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdZipEntry;
 import org.aspectj.org.eclipse.jdt.internal.core.util.HashtableOfArrayToObject;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
 
@@ -39,7 +60,7 @@ import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
 @SuppressWarnings({"rawtypes", "unchecked"})
 public class JarPackageFragmentRoot extends PackageFragmentRoot {
 
-	private final static ArrayList EMPTY_LIST = new ArrayList();
+	protected final static ArrayList EMPTY_LIST = new ArrayList();
 
 	/**
 	 * The path to the jar file
@@ -48,6 +69,10 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	 */
 	protected final IPath jarPath;
 
+	boolean knownToBeModuleLess;
+
+	private boolean multiVersion;
+
 	/**
 	 * Constructs a package fragment root which is the root of the Java package directory hierarchy
 	 * based on a JAR file that is not contained in a <code>IJavaProject</code> and
@@ -71,24 +96,92 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	 * These are all of the directory zip entries, and any directories implied
 	 * by the path of class files contained in the jar of this package fragment root.
 	 */
+	@Override
 	protected boolean computeChildren(OpenableElementInfo info, IResource underlyingResource) throws JavaModelException {
-		HashtableOfArrayToObject rawPackageInfo = new HashtableOfArrayToObject();
-		IJavaElement[] children;
-		ZipFile jar = null;
+		final HashtableOfArrayToObject rawPackageInfo = new HashtableOfArrayToObject();
+		final Map<String, String> overridden = new HashMap<>();
+		IJavaElement[] children = NO_ELEMENTS;
 		try {
-			Object file = JavaModel.getTarget(getPath(), true);
-			long level = Util.getJdkLevel(file);
-			String compliance = CompilerOptions.versionFromJdkLevel(level);
-			jar = getJar();
-
 			// always create the default package
 			rawPackageInfo.put(CharOperation.NO_STRINGS, new ArrayList[] { EMPTY_LIST, EMPTY_LIST });
 
-			for (Enumeration e= jar.entries(); e.hasMoreElements();) {
-				ZipEntry member= (ZipEntry) e.nextElement();
-				initRawPackageInfo(rawPackageInfo, member.getName(), member.isDirectory(), compliance);
+			boolean usedIndex = false;
+			if (JavaIndex.isEnabled()) {
+				JavaIndex index = JavaIndex.getIndex();
+				try (IReader reader = index.getNd().acquireReadLock()) {
+					IPath resourcePath = JavaIndex.getLocationForElement(this); 
+					if (!resourcePath.isEmpty()) {
+						NdResourceFile resourceFile = index.getResourceFile(resourcePath.toString().toCharArray());
+						if (index.isUpToDate(resourceFile)) {
+							usedIndex = true;
+							long level = resourceFile.getJdkLevel();
+							String compliance = CompilerOptions.versionFromJdkLevel(level);
+							// Locate all the non-classfile entries
+							for (NdZipEntry next : resourceFile.getZipEntries()) {
+								String filename = next.getFileName().getString();
+								initRawPackageInfo(rawPackageInfo, filename, filename.endsWith("/"), compliance); //$NON-NLS-1$
+							}
+	
+							// Locate all the classfile entries
+							for (NdType type : resourceFile.getTypes()) {	
+								String path = new String(type.getTypeId().getBinaryName()) + ".class"; //$NON-NLS-1$
+								initRawPackageInfo(rawPackageInfo, path, false, compliance);
+							}
+						}
+					}
+				}
 			}
 
+			// If we weren't able to compute the set of children from the index (either the index was disabled or didn't
+			// contain an up-to-date entry for this .jar) then fetch it directly from the .jar
+			if (!usedIndex) {
+				Object file = JavaModel.getTarget(getPath(), true);
+				long classLevel = Util.getJdkLevel(file);
+				String projectCompliance = this.getJavaProject().getOption(JavaCore.COMPILER_COMPLIANCE, true);
+				long projectLevel = CompilerOptions.versionToJdkLevel(projectCompliance);
+				ZipFile jar = null;
+				try {
+					jar = getJar();
+					String version = "META-INF/versions/";  //$NON-NLS-1$
+					List<String> versions = new ArrayList<>();
+					if (projectLevel >= ClassFileConstants.JDK9 && jar.getEntry(version) != null) {
+						int earliestJavaVersion = ClassFileConstants.MAJOR_VERSION_9;
+						long latestJDK = CompilerOptions.releaseToJDKLevel(projectCompliance);
+						int latestJavaVer = (int) (latestJDK >> 16);
+
+						for(int i = latestJavaVer; i >= earliestJavaVersion; i--) {
+							String s = "" + + (i - 44); //$NON-NLS-1$
+							String versionPath = version + s;
+							if (jar.getEntry(versionPath) != null) {
+								versions.add(s);
+							}
+						}
+					}
+					
+					String[] supportedVersions = versions.toArray(new String[versions.size()]);
+					if (supportedVersions.length > 0) {
+						this.multiVersion = true;
+					}
+					int length = version.length();
+					for (Enumeration<? extends ZipEntry> e= jar.entries(); e.hasMoreElements();) {
+						ZipEntry member= e.nextElement();
+						String name = member.getName();
+						if (this.multiVersion && name.length() > (length + 2) && name.startsWith(version)) {
+							int end = name.indexOf('/', length);
+							if (end >= name.length()) continue;
+							String versionPath = name.substring(0, end);
+							String ver = name.substring(length, end);
+							if(versions.contains(ver) && org.aspectj.org.eclipse.jdt.internal.compiler.util.Util.isClassFileName(name)) {
+								name = name.substring(end + 1);
+								overridden.put(name, versionPath);
+							}
+						}
+						initRawPackageInfo(rawPackageInfo, name, member.isDirectory(), CompilerOptions.versionFromJdkLevel(classLevel));
+					}
+				}  finally {
+					JavaModelManager.getJavaModelManager().closeZipFile(jar);
+				}
+			}
 			// loop through all of referenced packages, creating package fragments if necessary
 			// and cache the entry names in the rawPackageInfo table
 			children = new IJavaElement[rawPackageInfo.size()];
@@ -108,23 +201,36 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 			} else {
 				throw new JavaModelException(e);
 			}
-		} finally {
-			JavaModelManager.getJavaModelManager().closeZipFile(jar);
 		}
-
 		info.setChildren(children);
 		((JarPackageFragmentRootInfo) info).rawPackageInfo = rawPackageInfo;
+		((JarPackageFragmentRootInfo) info).overriddenClasses = overridden;
 		return true;
 	}
+	protected IJavaElement[] createChildren(final HashtableOfArrayToObject rawPackageInfo) {
+		IJavaElement[] children;
+		// loop through all of referenced packages, creating package fragments if necessary
+		// and cache the entry names in the rawPackageInfo table
+		children = new IJavaElement[rawPackageInfo.size()];
+		int index = 0;
+		for (int i = 0, length = rawPackageInfo.keyTable.length; i < length; i++) {
+			String[] pkgName = (String[]) rawPackageInfo.keyTable[i];
+			if (pkgName == null) continue;
+			children[index++] = getPackageFragment(pkgName);
+		}
+		return children;
+	}
 	/**
 	 * Returns a new element info for this element.
 	 */
+	@Override
 	protected Object createElementInfo() {
 		return new JarPackageFragmentRootInfo();
 	}
 	/**
 	 * A Jar is always K_BINARY.
 	 */
+	@Override
 	protected int determineKind(IResource underlyingResource) {
 		return IPackageFragmentRoot.K_BINARY;
 	}
@@ -135,6 +241,7 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	 *
 	 * @see Object#equals
 	 */
+	@Override
 	public boolean equals(Object o) {
 		if (this == o)
 			return true;
@@ -144,6 +251,7 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 		}
 		return false;
 	}
+	@Override
 	public String getElementName() {
 		return this.jarPath.lastSegment();
 	}
@@ -158,15 +266,18 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	/**
 	 * @see IPackageFragmentRoot
 	 */
+	@Override
 	public int getKind() {
 		return IPackageFragmentRoot.K_BINARY;
 	}
+	@Override
 	int internalKind() throws JavaModelException {
 		return IPackageFragmentRoot.K_BINARY;
 	}
 	/**
 	 * Returns an array of non-java resources contained in the receiver.
 	 */
+	@Override
 	public Object[] getNonJavaResources() throws JavaModelException {
 		// We want to show non java resources of the default package at the root (see PR #1G58NB8)
 		Object[] defaultPkgResources =  ((JarPackageFragment) getPackageFragment(CharOperation.NO_STRINGS)).storedNonJavaResources();
@@ -180,9 +291,40 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 		}
 		return nonJavaResources;
 	}
+	@Override
 	public PackageFragment getPackageFragment(String[] pkgName) {
 		return new JarPackageFragment(this, pkgName);
 	}
+	@Override
+	public PackageFragment getPackageFragment(String[] pkgName, String mod) {
+		return new JarPackageFragment(this, pkgName); // Overridden in JImageModuleFragmentBridge
+	}
+
+	@Override
+	public String getClassFilePath(String classname) {
+		if (this.multiVersion) {
+			JarPackageFragmentRootInfo elementInfo;
+			try {
+				elementInfo = (JarPackageFragmentRootInfo) getElementInfo();
+				String versionPath = elementInfo.overriddenClasses.get(classname);
+				return versionPath == null ? classname : versionPath + '/' + classname;
+			} catch (JavaModelException e) {
+				// move on
+			}
+		}
+		return classname;
+	}
+	@Override
+	public IModuleDescription getModuleDescription() {
+		if (this.knownToBeModuleLess)
+			return null;
+		IModuleDescription module = super.getModuleDescription();
+		if (module == null)
+			this.knownToBeModuleLess = true;
+		return module;
+	}
+
+	@Override
 	public IPath internalPath() {
 		if (isExternal()) {
 			return this.jarPath;
@@ -190,6 +332,7 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 			return super.internalPath();
 		}
 	}
+	@Override
 	public IResource resource(PackageFragmentRoot root) {
 		if (this.resource == null) {
 			// external jar
@@ -202,6 +345,7 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	/**
 	 * @see IJavaElement
 	 */
+	@Override
 	public IResource getUnderlyingResource() throws JavaModelException {
 		if (isExternal()) {
 			if (!exists()) throw newNotPresentException();
@@ -210,11 +354,21 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 			return super.getUnderlyingResource();
 		}
 	}
+	@Override
 	public int hashCode() {
 		return this.jarPath.hashCode();
 	}
-	private void initRawPackageInfo(HashtableOfArrayToObject rawPackageInfo, String entryName, boolean isDirectory, String compliance) {
-		int lastSeparator = isDirectory ? entryName.length()-1 : entryName.lastIndexOf('/');
+	protected void initRawPackageInfo(HashtableOfArrayToObject rawPackageInfo, String entryName, boolean isDirectory, String compliance) {
+		int lastSeparator;
+		if (isDirectory) {
+			if (entryName.charAt(entryName.length() - 1) == '/') {
+				lastSeparator = entryName.length() - 1;
+			} else {
+				lastSeparator = entryName.length();
+			}
+		} else {
+			lastSeparator = entryName.lastIndexOf('/');
+		}
 		String[] pkgName = Util.splitOn('/', entryName, 0, lastSeparator);
 		String[] existing = null;
 		int length = pkgName.length;
@@ -259,25 +413,28 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 	/**
 	 * @see IPackageFragmentRoot
 	 */
+	@Override
 	public boolean isArchive() {
 		return true;
 	}
 	/**
 	 * @see IPackageFragmentRoot
 	 */
+	@Override
 	public boolean isExternal() {
 		return resource() == null;
 	}
 	/**
 	 * Jars and jar entries are all read only
 	 */
+	@Override
 	public boolean isReadOnly() {
 		return true;
 	}
-
 	/**
 	 * Returns whether the corresponding resource or associated file exists
 	 */
+	@Override
 	protected boolean resourceExists(IResource underlyingResource) {
 		if (underlyingResource == null) {
 			return
@@ -289,6 +446,7 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 		}
 	}
 
+	@Override
 	protected void toStringAncestors(StringBuffer buffer) {
 		if (isExternal())
 			// don't show project as it is irrelevant for external jar files.
@@ -307,4 +465,28 @@ public class JarPackageFragmentRoot extends PackageFragmentRoot {
 		return null;
 	}
 
+	@Override
+	public Manifest getManifest() {
+		ZipFile jar = null;
+		try {
+			jar = getJar();
+			ZipEntry mfEntry = jar.getEntry(TypeConstants.META_INF_MANIFEST_MF);
+			if (mfEntry != null)
+				return new Manifest(jar.getInputStream(mfEntry));
+		} catch (CoreException | IOException e) {
+			// must do without manifest
+		} finally {
+			JavaModelManager.getJavaModelManager().closeZipFile(jar);
+		}
+		return null;
+	}
+
+//	@Override
+//	public boolean isModule() {
+//	 	try {
+//	 		return ((PackageFragmentRootInfo) getElementInfo()).isModule(resource(), this);
+//	 	} catch (JavaModelException e) {
+//	 		return false;
+//	 	}
+//	}
 }
