@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2011 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -171,18 +174,21 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 	/**
 	 * @see IStatus
 	 */
+	@Override
 	public IStatus[] getChildren() {
 		return this.children;
 	}
 	/**
 	 * @see IJavaModelStatus
 	 */
+	@Override
 	public IJavaElement[] getElements() {
 		return this.elements;
 	}
 	/**
 	 * Returns the message that is relevant to the code of this status.
 	 */
+	@Override
 	public String getMessage() {
 		Throwable exception = getException();
 		if (exception == null) {
@@ -394,6 +400,22 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 				case DEPRECATED_VARIABLE :
 					javaProject = (IJavaProject)this.elements[0];
 					return Messages.bind(Messages.classpath_deprecated_variable, new String[] {this.path.segment(0).toString(), javaProject.getElementName(), this.string});
+				case TEST_SOURCE_REQUIRES_SEPARATE_OUTPUT_LOCATION:
+					javaProject = (IJavaProject)this.elements[0];
+					projectName = javaProject.getElementName();
+					newPath = this.path;
+					if (this.path.segment(0).toString().equals(projectName)) {
+						newPath = this.path.removeFirstSegments(1);
+					}
+					return Messages.bind(Messages.classpath_testSourceRequiresSeparateOutputFolder, new String[] {newPath.makeRelative().toString(), projectName});
+				case TEST_OUTPUT_FOLDER_MUST_BE_SEPARATE_FROM_MAIN_OUTPUT_FOLDERS:
+					javaProject = (IJavaProject)this.elements[0];
+					projectName = javaProject.getElementName();
+					newPath = this.path;
+					if (this.path.segment(0).toString().equals(projectName)) {
+						newPath = this.path.removeFirstSegments(1);
+					}
+					return Messages.bind(Messages.classpath_testOutputFolderMustBeSeparateFromMainOutputFolders, new String[] {newPath.makeRelative().toString(), projectName});
 			}
 			if (this.string != null) {
 				return this.string;
@@ -412,12 +434,14 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 	/**
 	 * @see IJavaModelStatus#getPath()
 	 */
+	@Override
 	public IPath getPath() {
 		return this.path;
 	}
 	/**
 	 * @see IStatus#getSeverity()
 	 */
+	@Override
 	public int getSeverity() {
 		if (this.children == NO_CHILDREN) return super.getSeverity();
 		int severity = -1;
@@ -433,12 +457,14 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 	 * @see IJavaModelStatus#getString()
 	 * @deprecated
 	 */
+	@Override
 	public String getString() {
 		return this.string;
 	}
 	/**
 	 * @see IJavaModelStatus#isDoesNotExist()
 	 */
+	@Override
 	public boolean isDoesNotExist() {
 		int code = getCode();
 		return code == ELEMENT_DOES_NOT_EXIST || code == ELEMENT_NOT_ON_CLASSPATH;
@@ -446,18 +472,21 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 	/**
 	 * @see IStatus#isMultiStatus()
 	 */
+	@Override
 	public boolean isMultiStatus() {
 		return this.children != NO_CHILDREN;
 	}
 	/**
 	 * @see IStatus#isOK()
 	 */
+	@Override
 	public boolean isOK() {
 		return getCode() == OK;
 	}
 	/**
 	 * @see IStatus#matches(int)
 	 */
+	@Override
 	public boolean matches(int mask) {
 		if (! isMultiStatus()) {
 			return matches(this, mask);
@@ -493,6 +522,7 @@ public class JavaModelStatus extends Status implements IJavaModelStatus, IJavaMo
 	 * Returns a printable representation of this exception for debugging
 	 * purposes.
 	 */
+	@Override
 	public String toString() {
 		if (this == VERIFIED_OK){
 			return "JavaModelStatus[OK]"; //$NON-NLS-1$
