@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2014, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -137,10 +140,12 @@ public class LambdaExpression extends SourceType {
 		return elementInfo;
 	}
 	
+	@Override
 	protected void closing(Object info) throws JavaModelException {
 		// nothing to do, not backed by model ATM.
 	}
 	
+	@Override
 	public boolean equals(Object o) {
 		if (this == o)
 			return true;
@@ -159,21 +164,22 @@ public class LambdaExpression extends SourceType {
 		return false;
 	}
 	
+	@Override
 	public int hashCode() {
 		return Util.combineHashCodes(super.hashCode(), this.sourceStart);
 	}
 	
+	@Override
 	public Object getElementInfo(IProgressMonitor monitor) throws JavaModelException {
 		return this.elementInfo;
 	}
 
+	@Override
 	protected char getHandleMementoDelimiter() {
 		return JavaElement.JEM_LAMBDA_EXPRESSION;
 	}
 	
-	/*
-	 * @see JavaElement#getHandleMemento(StringBuffer)
-	 */
+	@Override
 	protected void getHandleMemento(StringBuffer buff) {
 		getHandleMemento(buff, true, true);
 		// lambda method and lambda expression cannot share the same memento - add a trailing discriminator.
@@ -196,6 +202,7 @@ public class LambdaExpression extends SourceType {
 			this.lambdaMethod.getHandleMemento(buff, false);
 	}
 	
+	@Override
 	public IJavaElement getHandleFromMemento(String token, MementoTokenizer memento, WorkingCopyOwner workingCopyOwner) {
 
 		if (token.charAt(0) != JEM_LAMBDA_METHOD)
@@ -239,14 +246,17 @@ public class LambdaExpression extends SourceType {
 		}
 	}
 
+	@Override
 	public IJavaElement[] getChildren() throws JavaModelException {
 		return new IJavaElement[] { this.lambdaMethod };
 	}
 
+	@Override
 	public boolean isLocal() {
 		return true;
 	}
 	
+	@Override
 	public JavaElement resolved(Binding binding) {
 		ResolvedLambdaExpression resolvedHandle = new ResolvedLambdaExpression(this.parent, this, new String(binding.computeUniqueKey()));
 		return resolvedHandle;
@@ -266,6 +276,7 @@ public class LambdaExpression extends SourceType {
 		return false;
 	}
 
+	@Override
 	public void toStringName(StringBuffer buffer) {
 		super.toStringName(buffer);
 		buffer.append("<lambda #"); //$NON-NLS-1$
@@ -290,6 +301,7 @@ public class LambdaExpression extends SourceType {
 		return this;
 	}
 
+	@Override
 	public String[] getSuperInterfaceTypeSignatures() throws JavaModelException {
 		return new String[] { this.interphase };
 	}
