@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,11 +16,14 @@ package org.aspectj.org.eclipse.jdt.internal.core;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Map;
+import java.util.jar.Manifest;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
 import org.aspectj.org.eclipse.jdt.core.*;
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.env.AutomaticModuleNaming;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.core.util.MementoTokenizer;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Messages;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
@@ -55,6 +61,7 @@ protected PackageFragmentRoot(IResource resource, JavaProject project) {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public void attachSource(IPath sourcePath, IPath rootPath, IProgressMonitor monitor) throws JavaModelException {
 	try {
 		verifyAttachSource(sourcePath);
@@ -150,6 +157,7 @@ public void attachSource(IPath sourcePath, IPath rootPath, IProgressMonitor moni
 /**
  * @see Openable
  */
+@Override
 protected boolean buildStructure(OpenableElementInfo info, IProgressMonitor pm, Map newElements, IResource underlyingResource) throws JavaModelException {
 	((PackageFragmentRootInfo) info).setRootKind(determineKind(underlyingResource));
 	return computeChildren(info, underlyingResource);
@@ -166,9 +174,8 @@ SourceMapper createSourceMapper(IPath sourcePath, IPath rootPath) throws JavaMod
 
 	return mapper;
 }
-/*
- * @see org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot#delete
- */
+
+@Override
 public void delete(
 	int updateResourceFlags,
 	int updateModelFlags,
@@ -196,9 +203,19 @@ protected boolean computeChildren(OpenableElementInfo info, IResource underlying
 			char[][] inclusionPatterns = fullInclusionPatternChars();
 			char[][] exclusionPatterns = fullExclusionPatternChars();
 			computeFolderChildren(rootFolder, !Util.isExcluded(rootFolder, inclusionPatterns, exclusionPatterns), CharOperation.NO_STRINGS, vChildren, inclusionPatterns, exclusionPatterns);
-			IJavaElement[] children = new IJavaElement[vChildren.size()];
-			vChildren.toArray(children);
-			info.setChildren(children);
+//			char[] suffix = getKind() == K_SOURCE ? SuffixConstants.SUFFIX_java : SuffixConstants.SUFFIX_class;
+//			char[] moduleInfoName = CharOperation.concat(TypeConstants.MODULE_INFO_NAME, suffix);
+//			IResource module = rootFolder.findMember(String.valueOf(moduleInfoName), true);
+//			if (module != null && module.exists()) {
+//				vChildren.add(new ClassFile(getPackageFragment(CharOperation.NO_STRINGS), String.valueOf(TypeConstants.MODULE_INFO_NAME)));
+//			}
+			if (!vChildren.isEmpty()) {
+				IJavaElement[] children = new IJavaElement[vChildren.size()];
+				vChildren.toArray(children);
+				info.setChildren(children);
+			} else {
+				info.setChildren(JavaElement.NO_ELEMENTS);
+			}
 		}
 	} catch (JavaModelException e) {
 		//problem resolving children; structure remains unknown
@@ -209,7 +226,7 @@ protected boolean computeChildren(OpenableElementInfo info, IResource underlying
 }
 
 /**
- * Starting at this folder, create package fragments and add the fragments that are not exclused
+ * Starting at this folder, create package fragments and add the fragments that are not excluded
  * to the collection of children.
  *
  * @exception JavaModelException  The resource associated with this package fragment does not exist
@@ -271,9 +288,7 @@ protected void computeFolderChildren(IContainer folder, boolean isIncluded, Stri
 	}
 }
 
-/*
- * @see org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot#copy
- */
+@Override
 public void copy(
 	IPath destination,
 	int updateResourceFlags,
@@ -290,6 +305,7 @@ public void copy(
 /**
  * Returns a new element info for this element.
  */
+@Override
 protected Object createElementInfo() {
 	return new PackageFragmentRootInfo();
 }
@@ -297,6 +313,7 @@ protected Object createElementInfo() {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public IPackageFragment createPackageFragment(String pkgName, boolean force, IProgressMonitor monitor) throws JavaModelException {
 	CreatePackageFragmentOperation op = new CreatePackageFragmentOperation(this, pkgName, force);
 	op.runOperation(monitor);
@@ -324,6 +341,7 @@ protected int determineKind(IResource underlyingResource) throws JavaModelExcept
  * same parent, same resources, and occurrence count.
  *
  */
+@Override
 public boolean equals(Object o) {
 	if (this == o)
 		return true;
@@ -411,6 +429,7 @@ public char[][] fullInclusionPatternChars() {
 		return null;
 	}
 }
+@Override
 public String getElementName() {
 	IResource res = resource();
 	if (res instanceof IFolder)
@@ -420,18 +439,21 @@ public String getElementName() {
 /**
  * @see IJavaElement
  */
+@Override
 public int getElementType() {
 	return PACKAGE_FRAGMENT_ROOT;
 }
 /**
  * @see JavaElement#getHandleMemento()
  */
+@Override
 protected char getHandleMementoDelimiter() {
 	return JavaElement.JEM_PACKAGEFRAGMENTROOT;
 }
 /*
  * @see JavaElement
  */
+@Override
 public IJavaElement getHandleFromMemento(String token, MementoTokenizer memento, WorkingCopyOwner owner) {
 	switch (token.charAt(0)) {
 		case JEM_PACKAGEFRAGMENT:
@@ -439,7 +461,7 @@ public IJavaElement getHandleFromMemento(String token, MementoTokenizer memento,
 			if (memento.hasMoreTokens()) {
 				token = memento.nextToken();
 				char firstChar = token.charAt(0);
-				if (firstChar == JEM_CLASSFILE || firstChar == JEM_COMPILATIONUNIT || firstChar == JEM_COUNT) {
+				if (firstChar == JEM_CLASSFILE || firstChar == JEM_MODULAR_CLASSFILE || firstChar == JEM_COMPILATIONUNIT || firstChar == JEM_COUNT) {
 					pkgName = CharOperation.NO_STRINGS;
 				} else {
 					pkgName = Util.splitOn('.', token, 0, token.length());
@@ -461,6 +483,7 @@ public IJavaElement getHandleFromMemento(String token, MementoTokenizer memento,
 /**
  * @see JavaElement#getHandleMemento(StringBuffer)
  */
+@Override
 protected void getHandleMemento(StringBuffer buff) {
 	IPath path;
 	IResource underlyingResource = getResource();
@@ -478,10 +501,15 @@ protected void getHandleMemento(StringBuffer buff) {
 	((JavaElement)getParent()).getHandleMemento(buff);
 	buff.append(getHandleMementoDelimiter());
 	escapeMementoName(buff, path.toString());
+	if (org.aspectj.org.eclipse.jdt.internal.compiler.util.Util.isJrt(path.toOSString())) {
+		buff.append(JavaElement.JEM_MODULE);
+		escapeMementoName(buff, getElementName());
+	}
 }
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public int getKind() throws JavaModelException {
 	return ((PackageFragmentRootInfo)getElementInfo()).getRootKind();
 }
@@ -502,6 +530,7 @@ int internalKind() throws JavaModelException {
 /**
  * Returns an array of non-java resources contained in the receiver.
  */
+@Override
 public Object[] getNonJavaResources() throws JavaModelException {
 	return ((PackageFragmentRootInfo) getElementInfo()).getNonJavaResources(getJavaProject(), resource(), this);
 }
@@ -509,6 +538,7 @@ public Object[] getNonJavaResources() throws JavaModelException {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public IPackageFragment getPackageFragment(String packageName) {
 	// tolerate package names with spaces (e.g. 'x . y') (http://bugs.eclipse.org/bugs/show_bug.cgi?id=21957)
 	String[] pkgName = Util.getTrimmedSimpleNames(packageName);
@@ -517,6 +547,9 @@ public IPackageFragment getPackageFragment(String packageName) {
 public PackageFragment getPackageFragment(String[] pkgName) {
 	return new PackageFragment(this, pkgName);
 }
+public PackageFragment getPackageFragment(String[] pkgName, String mod) {
+	return new PackageFragment(this, pkgName); // Overridden in JImageModuleFragmentBridge
+}
 /**
  * Returns the package name for the given folder
  * (which is a decendent of this root).
@@ -539,6 +572,7 @@ protected String getPackageName(IFolder folder) {
 /**
  * @see IJavaElement
  */
+@Override
 public IPath getPath() {
 	return internalPath();
 }
@@ -549,6 +583,7 @@ public IPath internalPath() {
 /*
  * @see IPackageFragmentRoot
  */
+@Override
 public IClasspathEntry getRawClasspathEntry() throws JavaModelException {
 
 	IClasspathEntry rawEntry = null;
@@ -566,6 +601,7 @@ public IClasspathEntry getRawClasspathEntry() throws JavaModelException {
 /*
  * @see IPackageFragmentRoot
  */
+@Override
 public IClasspathEntry getResolvedClasspathEntry() throws JavaModelException {
 	IClasspathEntry resolvedEntry = null;
 	JavaProject project = (JavaProject)getJavaProject();
@@ -581,6 +617,7 @@ public IClasspathEntry getResolvedClasspathEntry() throws JavaModelException {
 }
 
 
+@Override
 public IResource resource() {
 	if (this.resource != null) // perf improvement to avoid message send in resource()
 		return this.resource;
@@ -589,6 +626,7 @@ public IResource resource() {
 /*
  * @see IJavaElement
  */
+@Override
 public IResource resource(PackageFragmentRoot root) {
 	return this.resource;
 }
@@ -596,6 +634,7 @@ public IResource resource(PackageFragmentRoot root) {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public IPath getSourceAttachmentPath() throws JavaModelException {
 	if (getKind() != K_BINARY) return null;
 
@@ -641,6 +680,7 @@ public void setSourceMapper(SourceMapper mapper) throws JavaModelException {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public IPath getSourceAttachmentRootPath() throws JavaModelException {
 	if (getKind() != K_BINARY) return null;
 
@@ -674,6 +714,7 @@ public IPath getSourceAttachmentRootPath() throws JavaModelException {
 /**
  * @see JavaElement
  */
+@Override
 public SourceMapper getSourceMapper() {
 	SourceMapper mapper;
 	try {
@@ -699,6 +740,7 @@ public SourceMapper getSourceMapper() {
 /**
  * @see IJavaElement
  */
+@Override
 public IResource getUnderlyingResource() throws JavaModelException {
 	if (!exists()) throw newNotPresentException();
 	return resource();
@@ -707,11 +749,13 @@ public IResource getUnderlyingResource() throws JavaModelException {
 /**
  * @see IParent
  */
+@Override
 public boolean hasChildren() throws JavaModelException {
 	// a package fragment root always has the default package as a child
 	return true;
 }
 
+@Override
 public int hashCode() {
 	return resource().hashCode();
 }
@@ -727,6 +771,7 @@ public boolean ignoreOptionalProblems() {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public boolean isArchive() {
 	return false;
 }
@@ -734,6 +779,7 @@ public boolean isArchive() {
 /**
  * @see IPackageFragmentRoot
  */
+@Override
 public boolean isExternal() {
 	return false;
 }
@@ -757,9 +803,8 @@ protected IStatus validateOnClasspath() {
 	}
 	return new JavaModelStatus(IJavaModelStatusConstants.ELEMENT_NOT_ON_CLASSPATH, this);
 }
-/*
- * @see org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot#move
- */
+
+@Override
 public void move(
 	IPath destination,
 	int updateResourceFlags,
@@ -776,6 +821,7 @@ public void move(
 /**
  * @private Debugging purposes
  */
+@Override
 protected void toStringInfo(int tab, StringBuffer buffer, Object info, boolean showResolvedInfo) {
 	buffer.append(tabString(tab));
 	IPath path = getPath();
@@ -795,6 +841,7 @@ protected void toStringInfo(int tab, StringBuffer buffer, Object info, boolean s
 	}
 }
 
+@Override
 protected IStatus validateExistence(IResource underlyingResource) {
 	// check whether this pkg fragment root can be opened
 	IStatus status = validateOnClasspath();
@@ -823,5 +870,93 @@ protected void verifyAttachSource(IPath sourcePath) throws JavaModelException {
 		throw new JavaModelException(new JavaModelStatus(IJavaModelStatusConstants.RELATIVE_PATH, sourcePath));
 	}
 }
+/**
+ * Returns the relative path within an archive for the given class file name. In certain
+ * kind of archives, such as a JMOD file, class files are stored in a nested folder, as opposed
+ * to directly under the root. It is the responsibility of such package fragment roots to
+ * provide the custom behavior.
+ *
+ * @param classname
+ * @return the relative path for the class file within the archive
+ */
+public String getClassFilePath(String classname) {
+	return classname;
+}
+@Override
+public IModuleDescription getModuleDescription() {
+	try {
+		IJavaElement[] pkgs = getChildren();
+		for (int j = 0, length = pkgs.length; j < length; j++) {
+			// only look in the default package
+			if (pkgs[j].getElementName().length() == 0) {
+				OpenableElementInfo info = null;
+				if (getKind() == IPackageFragmentRoot.K_SOURCE) {
+					ICompilationUnit unit = ((PackageFragment) pkgs[j])
+							.getCompilationUnit(TypeConstants.MODULE_INFO_FILE_NAME_STRING);
+					if (unit instanceof CompilationUnit && unit.exists()) {
+						info = (CompilationUnitElementInfo) ((CompilationUnit) unit)
+								.getElementInfo();
+						if (info != null)
+							return info.getModule();
+					}
+				} else {
+					IModularClassFile classFile = ((IPackageFragment)pkgs[j]).getModularClassFile();
+					if (classFile.exists()) {
+						return classFile.getModule();
+					}
+				}
+				break;
+			}
+		}
+	} catch (JavaModelException e) {
+		Util.log(e);
+	}
+	return null;
+}
 
+public IModuleDescription getAutomaticModuleDescription() throws JavaModelException {
+	return getAutomaticModuleDescription(getResolvedClasspathEntry());
+}
+
+IModuleDescription getAutomaticModuleDescription(IClasspathEntry classpathEntry) {
+	String elementName = getElementName();
+	Manifest manifest = null;
+	switch (classpathEntry.getEntryKind()) {
+		case IClasspathEntry.CPE_SOURCE:
+			manifest = ((JavaProject) getJavaProject()).getManifest();
+			elementName = getJavaProject().getElementName();
+			break;
+		case IClasspathEntry.CPE_LIBRARY:
+			manifest = getManifest();
+			break;
+		case IClasspathEntry.CPE_PROJECT:
+			JavaProject javaProject = (JavaProject) getJavaModel().getJavaProject(classpathEntry.getPath().lastSegment());
+			manifest = javaProject.getManifest();
+			elementName = javaProject.getElementName();
+			break;
+	}
+	boolean nameFromManifest = true;
+	char[] moduleName = AutomaticModuleNaming.determineAutomaticModuleNameFromManifest(manifest);
+	if (moduleName == null) {
+		nameFromManifest = false;
+		moduleName = AutomaticModuleNaming.determineAutomaticModuleNameFromFileName(elementName, true, isArchive());
+	}
+	return new AbstractModule.AutoModule(this, String.valueOf(moduleName), nameFromManifest);
+}
+
+/** @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IModulePathEntry#hasCompilationUnit(String, String) */
+public boolean hasCompilationUnit(String qualifiedPackageName, String moduleName) {
+	IPackageFragment fragment = getPackageFragment(qualifiedPackageName.replace('/', '.'));
+	try {
+		if (fragment.exists())
+			return fragment.containsJavaResources();
+	} catch (JavaModelException e) {
+		// silent
+	}
+	return false;
+}
+/** Convenience lookup, though currently only JarPackageFragmentRoot is searched for a manifest. */
+public Manifest getManifest() {
+	return null;
+}
 }
