@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -13,6 +16,8 @@
 package org.aspectj.org.eclipse.jdt.internal.core;
 
 import java.io.IOException;
+import java.nio.file.FileVisitResult;
+import java.nio.file.attribute.BasicFileAttributes;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -35,12 +40,13 @@ import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 import org.aspectj.org.eclipse.jdt.core.Flags;
-import org.aspectj.org.eclipse.jdt.core.IClassFile;
 import org.aspectj.org.eclipse.jdt.core.IField;
 import org.aspectj.org.eclipse.jdt.core.IJavaElement;
 import org.aspectj.org.eclipse.jdt.core.IJavaProject;
 import org.aspectj.org.eclipse.jdt.core.IMember;
 import org.aspectj.org.eclipse.jdt.core.IMethod;
+import org.aspectj.org.eclipse.jdt.core.IModuleDescription;
+import org.aspectj.org.eclipse.jdt.core.IOrdinaryClassFile;
 import org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot;
 import org.aspectj.org.eclipse.jdt.core.ISourceRange;
 import org.aspectj.org.eclipse.jdt.core.IType;
@@ -59,17 +65,21 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ImportReference;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
+import org.aspectj.org.eclipse.jdt.internal.compiler.util.JRTUtil;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SuffixConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
 import org.aspectj.org.eclipse.jdt.internal.core.util.ReferenceInfoAdapter;
 
 /**
- * A SourceMapper maps source code in a ZIP file to binary types in
- * a JAR. The SourceMapper uses the fuzzy parser to identify source
- * fragments in a .java file, and attempts to match the source code
- * with children in a binary type. A SourceMapper is associated
- * with a JarPackageFragment by an AttachSourceOperation.
+ * A SourceMapper maps source code in a ZIP file to binary types or
+ * binary modules in a JAR. The SourceMapper uses the fuzzy parser 
+ * to identify source fragments in a .java file, and attempts to match
+ * the source code with children in a binary type.
+ * Since a module has no children in the Java Model no such matching
+ * happens in that case.
+ * A SourceMapper is associated with a JarPackageFragment by an AttachSourceOperation.
  *
  * @see org.aspectj.org.eclipse.jdt.internal.core.JarPackageFragment
  */
@@ -103,6 +113,7 @@ public class SourceMapper
 			this.name = name;
 		}
 
+		@Override
 		public int hashCode() {
 			final int prime = 31;
 			int result = 1;
@@ -111,6 +122,7 @@ public class SourceMapper
 			return result;
 		}
 
+		@Override
 		public boolean equals(Object obj) {
 			if (this == obj)
 				return true;
@@ -131,6 +143,7 @@ public class SourceMapper
 				return false;
 			return true;
 		}
+		@Override
 		public String toString() {
 			StringBuffer buffer = new StringBuffer();
 			buffer.append('(').append(this.parent).append('.').append(this.name).append(')');
@@ -147,9 +160,9 @@ public class SourceMapper
 	protected ArrayList rootPaths;
 
 	/**
-	 * The binary type source is being mapped for
+	 * The binary type or module source is being mapped for
 	 */
-	protected BinaryType binaryType;
+	protected NamedMember binaryTypeOrModule;
 
 	/**
 	 * The location of the zip file containing source.
@@ -228,7 +241,7 @@ public class SourceMapper
 	protected IJavaElement searchedElement;
 
 	/**
-	 * imports references
+	 * imports references (keyed by binaryTypeOrModule)
 	 */
 	private HashMap importsTable;
 	private HashMap importsCounterTable;
@@ -242,6 +255,13 @@ public class SourceMapper
 	int[] typeModifiers;
 	int typeDepth;
 
+	/**
+	 * Module information
+	 */
+	SourceRange moduleNameRange;
+	int moduleDeclarationStart;
+	int moduleModifiers;
+	
 	/**
 	 *  Anonymous counter in case we want to map the source of an anonymous class.
 	 */
@@ -281,6 +301,7 @@ public class SourceMapper
 			// use no encoding
 		}
 		if (rootPath != null) {
+			this.rootPath = rootPath;
 			this.rootPaths = new ArrayList();
 			this.rootPaths.add(rootPath);
 		}
@@ -295,6 +316,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void acceptImport(
 			int declarationStart,
 			int declarationEnd,
@@ -303,13 +325,13 @@ public class SourceMapper
 			char[][] tokens,
 			boolean onDemand,
 			int modifiers) {
-		char[][] imports = (char[][]) this.importsTable.get(this.binaryType);
+		char[][] imports = (char[][]) this.importsTable.get(this.binaryTypeOrModule);
 		int importsCounter;
 		if (imports == null) {
 			imports = new char[5][];
 			importsCounter = 0;
 		} else {
-			importsCounter = ((Integer) this.importsCounterTable.get(this.binaryType)).intValue();
+			importsCounter = ((Integer) this.importsCounterTable.get(this.binaryTypeOrModule)).intValue();
 		}
 		if (imports.length == importsCounter) {
 			System.arraycopy(
@@ -327,13 +349,14 @@ public class SourceMapper
 			name[nameLength + 1] = '*';
 		}
 		imports[importsCounter++] = name;
-		this.importsTable.put(this.binaryType, imports);
-		this.importsCounterTable.put(this.binaryType, new Integer(importsCounter));
+		this.importsTable.put(this.binaryTypeOrModule, imports);
+		this.importsCounterTable.put(this.binaryTypeOrModule, Integer.valueOf(importsCounter));
 	}
 
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void acceptLineSeparatorPositions(int[] positions) {
 		//do nothing
 	}
@@ -341,6 +364,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void acceptPackage(ImportReference importReference) {
 		//do nothing
 	}
@@ -348,6 +372,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void acceptProblem(CategorizedProblem problem) {
 		//do nothing
 	}
@@ -453,11 +478,67 @@ public class SourceMapper
 		return -1;
 	}
 
-	private synchronized void computeAllRootPaths(IType type) {
+	class JrtPackageNamesAdderVisitor implements JRTUtil.JrtFileVisitor<java.nio.file.Path> {
+
+		public final HashSet firstLevelPackageNames;
+		final IPackageFragmentRoot root;
+		public String sourceLevel = null;
+		public String complianceLevel = null;
+		public boolean containsADefaultPackage;
+		public boolean containsJavaSource;
+
+		JrtPackageNamesAdderVisitor(HashSet firstLevelPackageNames, String sourceLevel, String complianceLevel,
+				boolean containsADefaultPackage, boolean containsJavaSource, IPackageFragmentRoot root) {
+			this.firstLevelPackageNames = firstLevelPackageNames;
+			this.root = root;
+			this.sourceLevel = sourceLevel;
+			this.complianceLevel = complianceLevel;
+			this.containsADefaultPackage = containsADefaultPackage;
+			this.containsJavaSource = containsJavaSource;
+		}
+		
+		@Override
+		public FileVisitResult visitPackage(java.nio.file.Path dir, java.nio.file.Path mod, BasicFileAttributes attrs) throws IOException {
+			return FileVisitResult.CONTINUE;
+		}
+		
+		@Override
+		public FileVisitResult visitFile(java.nio.file.Path file, java.nio.file.Path mod, BasicFileAttributes attrs) throws IOException {
+			String entryName = file.toString();
+			if (Util.isClassFileName(entryName)) {
+				int index = entryName.indexOf('/');
+				if (index != -1) {
+					String firstLevelPackageName = entryName.substring(0, index);
+					if (!this.firstLevelPackageNames.contains(firstLevelPackageName)) {
+						if (this.sourceLevel == null) {
+							IJavaProject project = this.root.getJavaProject();
+							this.sourceLevel = project.getOption(JavaCore.COMPILER_SOURCE, true);
+							this.complianceLevel = project.getOption(JavaCore.COMPILER_COMPLIANCE, true);
+						}
+						IStatus status = JavaConventions.validatePackageName(firstLevelPackageName, this.sourceLevel, this.complianceLevel);
+						if (status.isOK() || status.getSeverity() == IStatus.WARNING) {
+							this.firstLevelPackageNames.add(firstLevelPackageName);
+						}
+					}
+				} else {
+					this.containsADefaultPackage = true;
+				}
+			} else if (!this.containsJavaSource && org.aspectj.org.eclipse.jdt.internal.core.util.Util.isJavaLikeFileName(entryName)) {
+				this.containsJavaSource = true;
+			}
+			return FileVisitResult.CONTINUE;
+		}
+
+		@Override
+		public FileVisitResult visitModule(java.nio.file.Path mod) throws IOException {
+			return FileVisitResult.CONTINUE;
+		}
+	}
+	private synchronized void computeAllRootPaths(IJavaElement typeOrModule) {
 		if (this.areRootPathsComputed) {
 			return;
 		}
-		IPackageFragmentRoot root = (IPackageFragmentRoot) type.getPackageFragment().getParent();
+		IPackageFragmentRoot root = (IPackageFragmentRoot) typeOrModule.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
 		IPath pkgFragmentRootPath = root.getPath();
 		final HashSet tempRoots = new HashSet();
 		long time = 0;
@@ -471,7 +552,22 @@ public class SourceMapper
 
 		String sourceLevel = null;
 		String complianceLevel = null;
-		if (root.isArchive()) {
+		if (Util.isJrt(pkgFragmentRootPath.toOSString())) {
+			try {
+				JrtPackageNamesAdderVisitor jrtPackageNamesAdderVisitor = new JrtPackageNamesAdderVisitor(firstLevelPackageNames, 
+						sourceLevel, complianceLevel, containsADefaultPackage, containsJavaSource, root);
+				org.aspectj.org.eclipse.jdt.internal.compiler.util.JRTUtil.walkModuleImage(root.getPath().toFile(), jrtPackageNamesAdderVisitor, JRTUtil.NOTIFY_FILES);
+				sourceLevel = jrtPackageNamesAdderVisitor.sourceLevel;
+				complianceLevel = jrtPackageNamesAdderVisitor.complianceLevel;
+				containsADefaultPackage = jrtPackageNamesAdderVisitor.containsADefaultPackage;
+				containsJavaSource = jrtPackageNamesAdderVisitor.containsJavaSource;
+			} catch (IOException e) {
+				// We are not reading any specific file, so, move on for now
+				if (VERBOSE) {
+					e.printStackTrace();
+				}
+			}
+		} else if (root.isArchive()) {
 			JavaModelManager manager = JavaModelManager.getJavaModelManager();
 			ZipFile zip = null;
 			try {
@@ -593,6 +689,7 @@ public class SourceMapper
 			ArrayList sortedRoots = new ArrayList(tempRoots);
 			if (size > 1) {
 				Collections.sort(sortedRoots, new Comparator() {
+					@Override
 					public int compare(Object o1, Object o2) {
 						IPath path1 = (IPath) o1;
 						IPath path2 = (IPath) o2;
@@ -659,6 +756,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterType(TypeInfo typeInfo) {
 
 		this.typeDepth++;
@@ -721,7 +819,7 @@ public class SourceMapper
 		if (typeInfo.name.length == 0) {
 			this.anonymousCounter++;
 			if (this.anonymousCounter == this.anonymousClassName) {
-				this.types[this.typeDepth] = getType(this.binaryType.getElementName());
+				this.types[this.typeDepth] = getType(this.binaryTypeOrModule.getElementName());
 			} else {
 				this.types[this.typeDepth] = getType(new String(typeInfo.name));
 			}
@@ -756,10 +854,37 @@ public class SourceMapper
 		// categories
 		addCategories(currentType, typeInfo.categories);
 	}
+	
+	@Override
+	public void enterModule(ModuleInfo moduleInfo) {
+		this.moduleNameRange =
+			new SourceRange(moduleInfo.nameSourceStart, moduleInfo.nameSourceEnd - moduleInfo.nameSourceStart + 1);
+		this.moduleDeclarationStart = moduleInfo.declarationStart;
+	
+	
+		// module type modifiers
+		this.moduleModifiers = moduleInfo.modifiers;
+
+		if (this.binaryTypeOrModule instanceof IModuleDescription) {
+			// categories
+			addCategories(this.binaryTypeOrModule, moduleInfo.categories);
+		}
+	}
+
+	@Override
+	public void exitModule(int declarationEnd) {
+		setSourceRange(
+			this.binaryTypeOrModule,
+			new SourceRange(
+				this.moduleDeclarationStart,
+				declarationEnd - this.moduleDeclarationStart + 1),
+			this.moduleNameRange);
+	}
 
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterCompilationUnit() {
 		// do nothing
 	}
@@ -767,6 +892,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterConstructor(MethodInfo methodInfo) {
 		enterAbstractMethod(methodInfo);
 	}
@@ -774,6 +900,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterField(FieldInfo fieldInfo) {
 		if (this.typeDepth >= 0) {
 			this.memberDeclarationStart[this.typeDepth] = fieldInfo.declarationStart;
@@ -792,6 +919,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterInitializer(
 		int declarationSourceStart,
 		int modifiers) {
@@ -801,6 +929,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void enterMethod(MethodInfo methodInfo) {
 		enterAbstractMethod(methodInfo);
 	}
@@ -817,7 +946,7 @@ public class SourceMapper
 				IType declaringType = currentType.getDeclaringType();
 				String declaringTypeName = declaringType.getElementName();
 				if (declaringTypeName.length() == 0) {
-					IClassFile classFile = declaringType.getClassFile();
+					IOrdinaryClassFile classFile = declaringType.getClassFile();
 					int length = parameterTypes != null ? parameterTypes.length : 0;
 					char[][] newParameterTypes = new char[length+1][];
 					declaringTypeName = classFile.getElementName();
@@ -893,6 +1022,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitType(int declarationEnd) {
 		if (this.typeDepth >= 0) {
 			IType currentType = this.types[this.typeDepth];
@@ -909,6 +1039,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitCompilationUnit(int declarationEnd) {
 		//do nothing
 	}
@@ -916,6 +1047,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitConstructor(int declarationEnd) {
 		exitAbstractMethod(declarationEnd);
 	}
@@ -923,6 +1055,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitField(int initializationStart, int declarationEnd, int declarationSourceEnd) {
 		if (this.typeDepth >= 0) {
 			IType currentType = this.types[this.typeDepth];
@@ -938,6 +1071,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitInitializer(int declarationEnd) {
 		// implements abstract method
 	}
@@ -945,6 +1079,7 @@ public class SourceMapper
 	/**
 	 * @see ISourceElementRequestor
 	 */
+	@Override
 	public void exitMethod(int declarationEnd, Expression defaultValue) {
 		exitAbstractMethod(declarationEnd);
 	}
@@ -992,12 +1127,28 @@ public class SourceMapper
 	 * folder) used to create the given type (e.g. "A.java" for x/y/A$Inner.class)
 	 */
 	public char[] findSource(IType type, String simpleSourceFileName) {
+		PackageFragment pkgFrag = (PackageFragment) type.getPackageFragment();
+		String name = org.aspectj.org.eclipse.jdt.internal.core.util.Util.concatWith(pkgFrag.names, simpleSourceFileName, '/');
+		return internalFindSource((NamedMember) type, name);
+	}
+
+	/**
+	 * Locates and returns source code for the given (binary) module, in this
+	 * SourceMapper's ZIP file, or returns <code>null</code> if source
+	 * code cannot be found.
+	 */
+	public char[] findSource(IModuleDescription module) {
+		if (!module.isBinary()) {
+			return null;
+		}
+		return internalFindSource((NamedMember) module, TypeConstants.MODULE_INFO_FILE_NAME_STRING);
+	}
+
+	private char[] internalFindSource(NamedMember typeOrModule, String name) {
 		long time = 0;
 		if (VERBOSE) {
 			time = System.currentTimeMillis();
 		}
-		PackageFragment pkgFrag = (PackageFragment) type.getPackageFragment();
-		String name = org.aspectj.org.eclipse.jdt.internal.core.util.Util.concatWith(pkgFrag.names, simpleSourceFileName, '/');
 
 		char[] source = null;
 
@@ -1007,19 +1158,25 @@ public class SourceMapper
 
 			if (this.rootPath != null) {
 				source = getSourceForRootPath(this.rootPath, name);
+				if (source == null) {
+					source = getSourceForRootPath("", name); //$NON-NLS-1$
+				}
 			}
-	
-			if (source == null) {
-				computeAllRootPaths(type);
-				if (this.rootPaths != null) {
-					loop: for (Iterator iterator = this.rootPaths.iterator(); iterator.hasNext(); ) {
-						String currentRootPath = (String) iterator.next();
-						if (!currentRootPath.equals(this.rootPath)) {
-							source = getSourceForRootPath(currentRootPath, name);
-							if (source != null) {
-								// remember right root path
-								this.rootPath = currentRootPath;
-								break loop;
+
+			if (source == null) { // proceed with automatic root path detection ...
+				// ... but not for multi-module roots
+				if (!(typeOrModule.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT) instanceof JrtPackageFragmentRoot)) {
+					computeAllRootPaths(typeOrModule);
+					if (this.rootPaths != null) {
+						loop: for (Iterator iterator = this.rootPaths.iterator(); iterator.hasNext(); ) {
+							String currentRootPath = (String) iterator.next();
+							if (!currentRootPath.equals(this.rootPath)) {
+								source = getSourceForRootPath(currentRootPath, name);
+								if (source != null) {
+									// remember right root path
+									this.rootPath = currentRootPath;
+									break loop;
+								}
 							}
 						}
 					}
@@ -1029,7 +1186,7 @@ public class SourceMapper
 			javaModelManager.flushZipFiles(this); // clean up cached zip files.
 		}
 		if (VERBOSE) {
-			System.out.println("spent " + (System.currentTimeMillis() - time) + "ms for " + type.getElementName()); //$NON-NLS-1$ //$NON-NLS-2$
+			System.out.println("spent " + (System.currentTimeMillis() - time) + "ms for " + typeOrModule.getElementName()); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 		return source;
 	}
@@ -1233,8 +1390,11 @@ public class SourceMapper
 	 * as well.
 	 */
 	protected IType getType(String typeName) {
+		if (!(this.binaryTypeOrModule instanceof IType))
+			return null;
+		IType type = (IType) this.binaryTypeOrModule;
 		if (typeName.length() == 0) {
-			IJavaElement classFile = this.binaryType.getParent();
+			IJavaElement classFile = type.getParent();
 			String classFileName = classFile.getElementName();
 			StringBuffer newClassFileName = new StringBuffer();
 			int lastDollar = classFileName.lastIndexOf('$');
@@ -1243,10 +1403,10 @@ public class SourceMapper
 			newClassFileName.append(Integer.toString(this.anonymousCounter));
 			PackageFragment pkg = (PackageFragment) classFile.getParent();
 			return new BinaryType(new ClassFile(pkg, newClassFileName.toString()), typeName);
-		} else if (this.binaryType.getElementName().equals(typeName))
-			return this.binaryType;
+		} else if (type.getElementName().equals(typeName))
+			return type;
 		else
-			return this.binaryType.getType(typeName);
+			return ((this.typeDepth <= 1) ? type : this.types[this.typeDepth - 1]).getType(typeName);
 	}
 
 	/**
@@ -1364,10 +1524,10 @@ public class SourceMapper
 	}
 
 	/**
-	 * Maps the given source code to the given binary type and its children.
+	 * Maps the given source code to the given binary type or module and its children.
 	 */
-	public void mapSource(IType type, char[] contents, IBinaryType info) {
-		this.mapSource(type, contents, info, null);
+	public void mapSource(NamedMember typeOrModule, char[] contents, IBinaryType info) {
+		this.mapSource(typeOrModule, contents, info, null);
 	}
 
 	/**
@@ -1376,18 +1536,18 @@ public class SourceMapper
 	 * given java element without storing it.
 	 */
 	public synchronized ISourceRange mapSource(
-		IType type,
+		NamedMember typeOrModule,
 		char[] contents,
-		IBinaryType info,
+		IBinaryType info, // null for modules
 		IJavaElement elementToFind) {
 
-		this.binaryType = (BinaryType) type;
+		this.binaryTypeOrModule = typeOrModule;
 
 		// check whether it is already mapped
-		if (this.sourceRanges.get(type) != null) return (elementToFind != null) ? getNameRange(elementToFind) : null;
+		if (this.sourceRanges.get(typeOrModule) != null) return (elementToFind != null) ? getNameRange(elementToFind) : null;
 
-		this.importsTable.remove(this.binaryType);
-		this.importsCounterTable.remove(this.binaryType);
+		this.importsTable.remove(this.binaryTypeOrModule);
+		this.importsCounterTable.remove(this.binaryTypeOrModule);
 		this.searchedElement = elementToFind;
 		this.types = new IType[1];
 		this.typeDeclarationStarts = new int[1];
@@ -1408,32 +1568,40 @@ public class SourceMapper
 		try {
 			IProblemFactory factory = new DefaultProblemFactory();
 			SourceElementParser parser = null;
+			boolean doFullParse = false;
 			this.anonymousClassName = 0;
-			if (info == null) {
-				try {
-					info = (IBinaryType) this.binaryType.getElementInfo();
-				} catch(JavaModelException e) {
-					return null;
+			String sourceFileName;
+			if (this.binaryTypeOrModule instanceof BinaryType) {
+				if (info == null) {
+					try {
+						info = (IBinaryType) this.binaryTypeOrModule.getElementInfo();
+					} catch(JavaModelException e) {
+						return null;
+					}
 				}
-			}
-			boolean isAnonymousClass = info.isAnonymous();
-			char[] fullName = info.getName();
-			if (isAnonymousClass) {
-				String eltName = this.binaryType.getParent().getElementName();
-				eltName = eltName.substring(eltName.lastIndexOf('$') + 1, eltName.length());
-				try {
-					this.anonymousClassName = Integer.parseInt(eltName);
-				} catch(NumberFormatException e) {
-					// ignore
+				sourceFileName = ((BinaryType) this.binaryTypeOrModule).sourceFileName(info);
+				boolean isAnonymousClass = info.isAnonymous();
+				
+				char[] fullName = info.getName();
+				if (isAnonymousClass) {
+					String eltName = this.binaryTypeOrModule.getParent().getElementName();
+					eltName = eltName.substring(eltName.lastIndexOf('$') + 1, eltName.length());
+					try {
+						this.anonymousClassName = Integer.parseInt(eltName);
+					} catch(NumberFormatException e) {
+						// ignore
+					}
 				}
+				doFullParse = hasToRetrieveSourceRangesForLocalClass(fullName);
+			} else {
+				sourceFileName = TypeConstants.MODULE_INFO_CLASS_NAME_STRING; 
 			}
-			boolean doFullParse = hasToRetrieveSourceRangesForLocalClass(fullName);
 			parser = new SourceElementParser(this, factory, new CompilerOptions(this.options), doFullParse, true/*optimize string literals*/);
 			parser.javadocParser.checkDocComment = false; // disable javadoc parsing
-			IJavaElement javaElement = this.binaryType.getCompilationUnit();
-			if (javaElement == null) javaElement = this.binaryType.getParent();
+			IJavaElement javaElement = this.binaryTypeOrModule.getCompilationUnit();
+			if (javaElement == null) javaElement = this.binaryTypeOrModule.getParent();
 			parser.parseCompilationUnit(
-				new BasicCompilationUnit(contents, null, this.binaryType.sourceFileName(info), javaElement),
+				new BasicCompilationUnit(contents, null, sourceFileName, javaElement),
 				doFullParse,
 				null/*no progress*/);
 			if (elementToFind != null) {
@@ -1446,7 +1614,7 @@ public class SourceMapper
 			if (elementToFind != null) {
 				this.sourceRanges = oldSourceRanges;
 			}
-			this.binaryType = null;
+			this.binaryTypeOrModule = null;
 			this.searchedElement = null;
 			this.types = null;
 			this.typeDeclarationStarts = null;
@@ -1497,10 +1665,10 @@ public class SourceMapper
 	/**
 	 * Return a char[][] array containing the imports of the attached source for the binary type
 	 */
-	public char[][] getImports(BinaryType type) {
-		char[][] imports = (char[][]) this.importsTable.get(type);
+	public char[][] getImports(Member typeOrModule) {
+		char[][] imports = (char[][]) this.importsTable.get(typeOrModule);
 		if (imports != null) {
-			int importsCounter = ((Integer) this.importsCounterTable.get(type)).intValue();
+			int importsCounter = ((Integer) this.importsCounterTable.get(typeOrModule)).intValue();
 			if (imports.length != importsCounter) {
 				System.arraycopy(
 					imports,
@@ -1509,7 +1677,7 @@ public class SourceMapper
 					0,
 					importsCounter);
 			}
-			this.importsTable.put(type, imports);
+			this.importsTable.put(typeOrModule, imports);
 		}
 		return imports;
 	}
