@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -26,6 +29,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryTypeAnnotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.ITypeAnnotationWalker;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BinaryTypeBinding.ExternalAnnotationStatus;
 import org.aspectj.org.eclipse.jdt.internal.core.search.indexing.IIndexConstants;
 
 public class HierarchyBinaryType implements IBinaryType {
@@ -52,18 +56,38 @@ public HierarchyBinaryType(int modifiers, char[] qualification, char[] sourceNam
 	this.typeParameterSignatures = typeParameterSignatures;
 	CharOperation.replace(this.name, '.', '/');
 }
+
+public HierarchyBinaryType(int modifiers, char[] binaryName, char[] sourceName, char[] enclosingTypeBinaryName, char[][] typeParameterSignatures) {
+	this.modifiers = modifiers;
+	this.sourceName = sourceName;
+	this.name = binaryName;
+	this.enclosingTypeName = enclosingTypeBinaryName;
+	this.typeParameterSignatures = typeParameterSignatures;
+
+	if (typeParameterSignatures != null) {
+		for (char[] next : typeParameterSignatures) {
+			if (next == null) {
+				throw new IllegalArgumentException("Parameter's type signature must not be null"); //$NON-NLS-1$
+			}
+		}
+	}
+}
+
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType
  */
+@Override
 public IBinaryAnnotation[] getAnnotations() {
 	return null;
 }
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType
  */
+@Override
 public IBinaryTypeAnnotation[] getTypeAnnotations() {
 	return null;
 }
+@Override
 public char[] getEnclosingMethod() {
 	return null;
 }
@@ -74,21 +98,25 @@ public char[] getEnclosingMethod() {
  *
  * For example, java.lang.String is java/lang/String.
  */
+@Override
 public char[] getEnclosingTypeName() {
 	return this.enclosingTypeName;
 }
 /**
  * Answer the receiver's fields or null if the array is empty.
  */
+@Override
 public IBinaryField[] getFields() {
 	return null;
 }
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IDependent#getFileName()
  */
+@Override
 public char[] getFileName() {
 	return null;
 }
+@Override
 public char[] getGenericSignature() {
 	if (this.typeParameterSignatures != null && this.genericSignature == null) {
 		StringBuffer buffer = new StringBuffer();
@@ -116,6 +144,7 @@ public char[] getGenericSignature() {
  *
  * For example, java.lang.String is java/lang/String.
  */
+@Override
 public char[][] getInterfaceNames() {
 	return this.superInterfaces;
 }
@@ -125,12 +154,14 @@ public char[][] getInterfaceNames() {
  * This nested type info is extracted from the inner class attributes.
  * Ask the name environment to find a member type using its compound name.
  */
+@Override
 public IBinaryNestedType[] getMemberTypes() {
 	return null;
 }
 /**
  * Answer the receiver's methods or null if the array is empty.
  */
+@Override
 public IBinaryMethod[] getMethods() {
 	return null;
 }
@@ -138,6 +169,7 @@ public IBinaryMethod[] getMethods() {
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType#getMissingTypeNames()
  */
+@Override
 public char[][][] getMissingTypeNames() {
 	return null;
 }
@@ -146,6 +178,7 @@ public char[][][] getMissingTypeNames() {
  * Answer an int whose bits are set according the access constants
  * defined by the VM spec.
  */
+@Override
 public int getModifiers() {
 	return this.modifiers;
 }
@@ -156,10 +189,12 @@ public int getModifiers() {
  *
  * For example, java.lang.String is java/lang/String.
  */
+@Override
 public char[] getName() {
 	return this.name;
 }
 
+@Override
 public char[] getSourceName() {
 	return this.sourceName;
 }
@@ -170,14 +205,17 @@ public char[] getSourceName() {
  *
  * For example, java.lang.String is java/lang/String.
  */
+@Override
 public char[] getSuperclassName() {
 	return this.superclass;
 }
 
 // TODO (jerome) please verify that we don't need the tagbits for the receiver
+@Override
 public long getTagBits() {
 	return 0;
 }
+@Override
 public boolean isAnonymous() {
 	return false; // index did not record this information (since unused for hierarchies)
 }
@@ -185,17 +223,21 @@ public boolean isAnonymous() {
  * Answer whether the receiver contains the resolved binary form
  * or the unresolved source form of the type.
  */
+@Override
 public boolean isBinaryType() {
 	return true;
 }
 
+@Override
 public boolean isLocal() {
 	return false;  // index did not record this information (since unused for hierarchies)
 }
+@Override
 public boolean isMember() {
 	return false;  // index did not record this information (since unused for hierarchies)
 }
 
+
 public void recordSuperType(char[] superTypeName, char[] superQualification, char superClassOrInterface){
 
 	// index encoding of p.A$B was B/p.A$, rebuild the proper name
@@ -214,26 +256,36 @@ public void recordSuperType(char[] superTypeName, char[] superQualification, cha
 		if (TypeDeclaration.kind(this.modifiers) == TypeDeclaration.INTERFACE_DECL) return;
 		char[] encodedName = CharOperation.concat(superQualification, superTypeName, '/');
 		CharOperation.replace(encodedName, '.', '/');
-		this.superclass = encodedName;
+		recordSuperclass(encodedName);
 	} else {
 		char[] encodedName = CharOperation.concat(superQualification, superTypeName, '/');
 		CharOperation.replace(encodedName, '.', '/');
-		if (this.superInterfaces == NoInterface){
-			this.superInterfaces = new char[][] { encodedName };
-		} else {
-			int length = this.superInterfaces.length;
-			System.arraycopy(this.superInterfaces, 0, this.superInterfaces = new char[length+1][], 0, length);
-			this.superInterfaces[length] = encodedName;
-		}
+		recordInterface(encodedName);
+	}
+}
+
+public void recordSuperclass(char[] binaryName) {
+	this.superclass = binaryName;
+}
+
+public void recordInterface(char[] binaryName) {
+	if (this.superInterfaces == NoInterface){
+		this.superInterfaces = new char[][] { binaryName };
+	} else {
+		int length = this.superInterfaces.length;
+		System.arraycopy(this.superInterfaces, 0, this.superInterfaces = new char[length+1][], 0, length);
+		this.superInterfaces[length] = binaryName;
 	}
 }
 
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType
  */
+@Override
 public char[] sourceFileName() {
 	return null;
 }
+@Override
 public String toString() {
 	StringBuffer buffer = new StringBuffer();
 	if (this.modifiers == ClassFileConstants.AccPublic) {
@@ -269,7 +321,17 @@ public String toString() {
 	}
 	return buffer.toString();
 }
+@Override
 public ITypeAnnotationWalker enrichWithExternalAnnotationsFor(ITypeAnnotationWalker walker, Object member, LookupEnvironment environment) {
 	return walker;
 }
+@Override
+public char[] getModule() {
+	// TODO Java 9 Auto-generated method stub
+	return null;
+}
+@Override
+public ExternalAnnotationStatus getExternalAnnotationStatus() {
+	return ExternalAnnotationStatus.NOT_EEA_CONFIGURED;
+}
 }
