@@ -1,26 +1,46 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.core.hierarchy;
 
-import java.util.*;
+import java.util.ArrayDeque;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Comparator;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.Map;
+import java.util.Set;
 
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IWorkspaceRoot;
+import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.NullProgressMonitor;
-import org.eclipse.core.runtime.SubProgressMonitor;
-import org.aspectj.org.eclipse.jdt.core.*;
+import org.eclipse.core.runtime.SubMonitor;
+import org.aspectj.org.eclipse.jdt.core.IJavaElement;
+import org.aspectj.org.eclipse.jdt.core.IJavaProject;
+import org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot;
+import org.aspectj.org.eclipse.jdt.core.IType;
+import org.aspectj.org.eclipse.jdt.core.JavaCore;
+import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
-import org.aspectj.org.eclipse.jdt.core.search.*;
+import org.aspectj.org.eclipse.jdt.core.search.IJavaSearchConstants;
+import org.aspectj.org.eclipse.jdt.core.search.IJavaSearchScope;
+import org.aspectj.org.eclipse.jdt.core.search.SearchParticipant;
+import org.aspectj.org.eclipse.jdt.core.search.SearchPattern;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
@@ -28,10 +48,27 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.problem.DefaultProblemFacto
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.HashtableOfObjectToInt;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SuffixConstants;
-import org.aspectj.org.eclipse.jdt.internal.core.*;
+import org.aspectj.org.eclipse.jdt.internal.core.ClassFile;
+import org.aspectj.org.eclipse.jdt.internal.core.IPathRequestor;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaProject;
+import org.aspectj.org.eclipse.jdt.internal.core.Member;
+import org.aspectj.org.eclipse.jdt.internal.core.Openable;
+import org.aspectj.org.eclipse.jdt.internal.core.PackageFragment;
+import org.aspectj.org.eclipse.jdt.internal.core.SearchableEnvironment;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.IReader;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.Nd;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.indexer.Indexer;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.JavaIndex;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.JavaNames;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdType;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdTypeId;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdTypeInterface;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.NdTypeSignature;
 import org.aspectj.org.eclipse.jdt.internal.core.search.IndexQueryRequestor;
 import org.aspectj.org.eclipse.jdt.internal.core.search.JavaSearchParticipant;
 import org.aspectj.org.eclipse.jdt.internal.core.search.SubTypeSearchJob;
+import org.aspectj.org.eclipse.jdt.internal.core.search.UnindexedSearchScope;
 import org.aspectj.org.eclipse.jdt.internal.core.search.indexing.IIndexConstants;
 import org.aspectj.org.eclipse.jdt.internal.core.search.indexing.IndexManager;
 import org.aspectj.org.eclipse.jdt.internal.core.search.matching.MatchLocator;
@@ -85,6 +122,7 @@ public class IndexBasedHierarchyBuilder extends HierarchyBuilder implements Suff
 			}
 			return name;
 		}
+		@Override
 		public String toString(){
 			StringBuffer buffer = new StringBuffer("Queue:\n"); //$NON-NLS-1$
 			for (int i = this.start; i <= this.end; i++){
@@ -99,6 +137,7 @@ public IndexBasedHierarchyBuilder(TypeHierarchy hierarchy, IJavaSearchScope scop
 	this.binariesFromIndexMatches = new HashMap(10);
 	this.scope = scope;
 }
+@Override
 public void build(boolean computeSubtypes) {
 	JavaModelManager manager = JavaModelManager.getJavaModelManager();
 	try {
@@ -110,10 +149,7 @@ public void build(boolean computeSubtypes) {
 			IType focusType = getType();
 			boolean focusIsObject = focusType.getElementName().equals(new String(IIndexConstants.OBJECT));
 			int amountOfWorkForSubtypes = focusIsObject ? 5 : 80; // percentage of work needed to get possible subtypes
-			IProgressMonitor possibleSubtypesMonitor =
-				this.hierarchy.progressMonitor == null ?
-					null :
-					new SubProgressMonitor(this.hierarchy.progressMonitor, amountOfWorkForSubtypes);
+			SubMonitor possibleSubtypesMonitor = this.hierarchy.progressMonitor.split(amountOfWorkForSubtypes);
 			HashSet localTypes = new HashSet(10); // contains the paths that have potential subtypes that are local/anonymous types
 			String[] allPossibleSubtypes;
 			if (((Member)focusType).getOuterMostLocalContext() == null) {
@@ -124,10 +160,7 @@ public void build(boolean computeSubtypes) {
 				allPossibleSubtypes = CharOperation.NO_STRINGS;
 			}
 			if (allPossibleSubtypes != null) {
-				IProgressMonitor buildMonitor =
-					this.hierarchy.progressMonitor == null ?
-						null :
-						new SubProgressMonitor(this.hierarchy.progressMonitor, 100 - amountOfWorkForSubtypes);
+				SubMonitor buildMonitor = this.hierarchy.progressMonitor.split(100 - amountOfWorkForSubtypes);
 				this.hierarchy.initialize(allPossibleSubtypes.length);
 				buildFromPotentialSubtypes(allPossibleSubtypes, localTypes, buildMonitor);
 			}
@@ -140,6 +173,7 @@ public void build(boolean computeSubtypes) {
 	}
 }
 private void buildForProject(JavaProject project, ArrayList potentialSubtypes, org.aspectj.org.eclipse.jdt.core.ICompilationUnit[] workingCopies, HashSet localTypes, IProgressMonitor monitor) throws JavaModelException {
+	SubMonitor subMonitor = SubMonitor.convert(monitor, 10);
 	// resolve
 	int openablesLength = potentialSubtypes.size();
 	if (openablesLength > 0) {
@@ -162,7 +196,9 @@ private void buildForProject(JavaProject project, ArrayList potentialSubtypes, o
 			}
 			indexes.put(openables[i], index);
 		}
+		subMonitor.split(1);
 		Arrays.sort(openables, new Comparator() {
+			@Override
 			public int compare(Object a, Object b) {
 				int aIndex = indexes.get(a);
 				int bIndex = indexes.get(b);
@@ -204,7 +240,8 @@ private void buildForProject(JavaProject project, ArrayList potentialSubtypes, o
 				// top level or member type
 				if (!inProjectOfFocusType) {
 					char[] typeQualifiedName = focusType.getTypeQualifiedName('.').toCharArray();
-					String[] packageName = ((PackageFragment) focusType.getPackageFragment()).names;
+					PackageFragment fragment = (PackageFragment) focusType.getPackageFragment();
+					String[] packageName = fragment.names;
 					if (searchableEnvironment.findType(typeQualifiedName, Util.toCharArrays(packageName)) == null) {
 						// focus type is not visible in this project: no need to go further
 						return;
@@ -220,17 +257,18 @@ private void buildForProject(JavaProject project, ArrayList potentialSubtypes, o
 				}
 				localTypes = new HashSet();
 				localTypes.add(openable.getPath().toString());
-				this.hierarchyResolver.resolve(new Openable[] {openable}, localTypes, monitor);
+				this.hierarchyResolver.resolve(new Openable[] {openable}, localTypes, subMonitor.split(9));
 				return;
 			}
 		}
-		this.hierarchyResolver.resolve(openables, localTypes, monitor);
+		this.hierarchyResolver.resolve(openables, localTypes, subMonitor.split(9));
 	}
 }
 /**
  * Configure this type hierarchy based on the given potential subtypes.
  */
 private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet localTypes, IProgressMonitor monitor) {
+	SubMonitor subMonitor = SubMonitor.convert(monitor, 100);
 	IType focusType = getType();
 
 	// substitute compilation units with working copies
@@ -268,6 +306,7 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 		length++;
 	}
 
+	subMonitor.split(5);
 	/*
 	 * Sort in alphabetical order so that potential subtypes are grouped per project
 	 */
@@ -275,11 +314,13 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 
 	ArrayList potentialSubtypes = new ArrayList();
 	try {
+		SubMonitor loopMonitor = subMonitor.split(95);
 		// create element infos for subtypes
 		HandleFactory factory = new HandleFactory();
 		IJavaProject currentProject = null;
-		if (monitor != null) monitor.beginTask("", length*2 /* 1 for build binding, 1 for connect hierarchy*/); //$NON-NLS-1$
 		for (int i = 0; i < length; i++) {
+			loopMonitor.setWorkRemaining(length - i + 1);
+			IJavaProject nextProject = null;
 			try {
 				String resourcePath = allPotentialSubTypes[i];
 
@@ -303,18 +344,23 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 					currentProject = project;
 					potentialSubtypes = new ArrayList(5);
 				} else if (!currentProject.equals(project)) {
+					nextProject = project;
 					// build current project
-					buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, monitor);
-					currentProject = project;
+					buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, loopMonitor.split(1));
 					potentialSubtypes = new ArrayList(5);
 				}
 
 				potentialSubtypes.add(handle);
 			} catch (JavaModelException e) {
 				continue;
+			} finally {
+				if (nextProject != null) {
+					currentProject = nextProject;
+				}
 			}
 		}
 
+		loopMonitor.setWorkRemaining(2);
 		// build last project
 		try {
 			if (currentProject == null) {
@@ -326,11 +372,13 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 					potentialSubtypes.add(focusType.getCompilationUnit());
 				}
 			}
-			buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, monitor);
+			buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, loopMonitor.split(1));
 		} catch (JavaModelException e) {
 			// ignore
 		}
 
+		loopMonitor.setWorkRemaining(1);
+
 		// Compute hierarchy of focus type if not already done (case of a type with potential subtypes that are not real subtypes)
 		if (!this.hierarchy.contains(focusType)) {
 			try {
@@ -341,7 +389,7 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 				} else {
 					potentialSubtypes.add(focusType.getCompilationUnit());
 				}
-				buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, monitor);
+				buildForProject((JavaProject)currentProject, potentialSubtypes, workingCopies, localTypes, loopMonitor.split(1));
 			} catch (JavaModelException e) {
 				// ignore
 			}
@@ -352,14 +400,16 @@ private void buildFromPotentialSubtypes(String[] allPotentialSubTypes, HashSet l
 			this.hierarchy.addRootClass(focusType);
 		}
 	} finally {
-		if (monitor != null) monitor.done();
+		SubMonitor.done(monitor);
 	}
 }
+@Override
 protected ICompilationUnit createCompilationUnitFromPath(Openable handle, IFile file) {
 	ICompilationUnit unit = super.createCompilationUnitFromPath(handle, file);
 	this.cuToHandle.put(unit, handle);
 	return unit;
 }
+@Override
 protected IBinaryType createInfoFromClassFile(Openable classFile, IResource file) {
 	String documentPath = classFile.getPath().toString();
 	IBinaryType binaryType = (IBinaryType)this.binariesFromIndexMatches.get(documentPath);
@@ -370,6 +420,7 @@ protected IBinaryType createInfoFromClassFile(Openable classFile, IResource file
 		return super.createInfoFromClassFile(classFile, file);
 	}
 }
+@Override
 protected IBinaryType createInfoFromClassFileInJar(Openable classFile) {
 	String filePath = (((ClassFile)classFile).getType().getFullyQualifiedName('$')).replace('.', '/') + SuffixConstants.SUFFIX_STRING_class;
 	IPackageFragmentRoot root = classFile.getPackageFragmentRoot();
@@ -390,9 +441,9 @@ protected IBinaryType createInfoFromClassFileInJar(Openable classFile) {
  * Returns null if they could not be determine.
  */
 private String[] determinePossibleSubTypes(final HashSet localTypes, IProgressMonitor monitor) {
-
 	class PathCollector implements IPathRequestor {
 		HashSet paths = new HashSet(10);
+		@Override
 		public void acceptPath(String path, boolean containsLocalTypes) {
 			this.paths.add(path);
 			if (containsLocalTypes) {
@@ -402,18 +453,13 @@ private String[] determinePossibleSubTypes(final HashSet localTypes, IProgressMo
 	}
 	PathCollector collector = new PathCollector();
 
-	try {
-		if (monitor != null) monitor.beginTask("", MAXTICKS); //$NON-NLS-1$
-		searchAllPossibleSubTypes(
-			getType(),
-			this.scope,
-			this.binariesFromIndexMatches,
-			collector,
-			IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH,
-			monitor);
-	} finally {
-		if (monitor != null) monitor.done();
-	}
+	searchAllPossibleSubTypes(
+		getType(),
+		this.scope,
+		this.binariesFromIndexMatches,
+		collector,
+		IJavaSearchConstants.WAIT_UNTIL_READY_TO_SEARCH,
+		monitor);
 
 	HashSet paths = collector.paths;
 	int length = paths.size();
@@ -438,9 +484,104 @@ private String[] determinePossibleSubTypes(final HashSet localTypes, IProgressMo
  * @param binariesFromIndexMatches
  * @param pathRequestor
  * @param waitingPolicy
- * @param progressMonitor
+ * @param monitor
  */
 public static void searchAllPossibleSubTypes(
+	IType type,
+	IJavaSearchScope scope,
+	final Map binariesFromIndexMatches,
+	final IPathRequestor pathRequestor,
+	int waitingPolicy,	// WaitUntilReadyToSearch | ForceImmediateSearch | CancelIfNotReadyToSearch
+	final IProgressMonitor monitor) {
+
+	if (JavaIndex.isEnabled()) {
+		SubMonitor subMonitor = SubMonitor.convert(monitor, 2);
+		newSearchAllPossibleSubTypes(type, scope, binariesFromIndexMatches, pathRequestor, waitingPolicy,
+				subMonitor.split(1));
+		legacySearchAllPossibleSubTypes(type, UnindexedSearchScope.filterEntriesCoveredByTheNewIndex(scope),
+				binariesFromIndexMatches, pathRequestor, waitingPolicy, subMonitor.split(1));
+	} else {
+		legacySearchAllPossibleSubTypes(type, scope, binariesFromIndexMatches, pathRequestor, waitingPolicy,
+				monitor);
+	}
+}
+
+private static void newSearchAllPossibleSubTypes(IType type, IJavaSearchScope scope2, Map binariesFromIndexMatches2,
+		IPathRequestor pathRequestor, int waitingPolicy, IProgressMonitor progressMonitor) {
+	SubMonitor subMonitor = SubMonitor.convert(progressMonitor, 2);
+	JavaIndex index = JavaIndex.getIndex();
+
+	Indexer.getInstance().waitForIndex(waitingPolicy, subMonitor.split(1));
+
+	Nd nd = index.getNd();
+	char[] fieldDefinition = JavaNames.fullyQualifiedNameToFieldDescriptor(type.getFullyQualifiedName().toCharArray());
+
+	IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
+
+	try (IReader reader = nd.acquireReadLock()) {
+		NdTypeId foundType = index.findType(fieldDefinition);
+
+		if (foundType == null) {
+			return;
+		}
+
+		ArrayDeque<NdType> typesToVisit = new ArrayDeque<>();
+		Set<NdType> discoveredTypes = new HashSet<>();
+		typesToVisit.addAll(foundType.getTypes());
+		discoveredTypes.addAll(typesToVisit);
+
+		while (!typesToVisit.isEmpty()) {
+			NdType nextType = typesToVisit.removeFirst();
+			NdTypeId typeId = nextType.getTypeId();
+
+			String typePath = new String(JavaNames.getIndexPathFor(nextType, root));
+			if (!scope2.encloses(typePath)) {
+				continue;
+			}
+
+			subMonitor.setWorkRemaining(Math.max(typesToVisit.size(), 3000)).split(1);
+
+			boolean isLocalClass = nextType.isLocal() || nextType.isAnonymous();
+			pathRequestor.acceptPath(typePath, isLocalClass);
+
+			HierarchyBinaryType binaryType = (HierarchyBinaryType)binariesFromIndexMatches2.get(typePath);
+			if (binaryType == null) {
+				binaryType = createBinaryTypeFrom(nextType);
+				binariesFromIndexMatches2.put(typePath, binaryType);
+			}
+
+			for (NdType subType : typeId.getSubTypes()) {
+				if (discoveredTypes.add(subType)) {
+					typesToVisit.add(subType);
+				}
+			}
+		}
+	}
+}
+
+private static HierarchyBinaryType createBinaryTypeFrom(NdType type) {
+	char[] enclosingTypeName = null;
+	NdTypeSignature enclosingType = type.getDeclaringType();
+	if (enclosingType != null) {
+		enclosingTypeName = enclosingType.getRawType().getBinaryName();
+	}
+	char[][] typeParameters = type.getTypeParameterSignatures();
+	NdTypeId typeId = type.getTypeId();
+	HierarchyBinaryType result = new HierarchyBinaryType(type.getModifiers(), typeId.getBinaryName(),
+		type.getSourceName(), enclosingTypeName, typeParameters.length == 0 ? null : typeParameters);
+
+	NdTypeSignature superClass = type.getSuperclass();
+	if (superClass != null) {
+		result.recordSuperclass(superClass.getRawType().getBinaryName());
+	}
+
+	for (NdTypeInterface interf : type.getInterfaces()) {
+		result.recordInterface(interf.getInterface().getRawType().getBinaryName());
+	}
+	return result;
+}
+
+private static void legacySearchAllPossibleSubTypes(
 	IType type,
 	IJavaSearchScope scope,
 	final Map binariesFromIndexMatches,
@@ -448,6 +589,8 @@ public static void searchAllPossibleSubTypes(
 	int waitingPolicy,	// WaitUntilReadyToSearch | ForceImmediateSearch | CancelIfNotReadyToSearch
 	final IProgressMonitor progressMonitor) {
 
+	SubMonitor subMonitor = SubMonitor.convert(progressMonitor, 100);
+
 	/* embed constructs inside arrays so as to pass them to (inner) collector */
 	final Queue queue = new Queue();
 	final HashtableOfObject foundSuperNames = new HashtableOfObject(5);
@@ -456,6 +599,7 @@ public static void searchAllPossibleSubTypes(
 
 	/* use a special collector to collect paths and queue new subtype names */
 	IndexQueryRequestor searchRequestor = new IndexQueryRequestor() {
+		@Override
 		public boolean acceptIndexMatch(String documentPath, SearchPattern indexRecord, SearchParticipant participant, AccessRuleSet access) {
 			SuperTypeReferencePattern record = (SuperTypeReferencePattern)indexRecord;
 			boolean isLocalOrAnonymous = record.enclosingTypeName == IIndexConstants.ONE_ZERO;
@@ -508,11 +652,10 @@ public static void searchAllPossibleSubTypes(
 		scope,
 		searchRequestor);
 
-	int ticks = 0;
 	queue.add(type.getElementName().toCharArray());
 	try {
 		while (queue.start <= queue.end) {
-			if (progressMonitor != null && progressMonitor.isCanceled()) return;
+			subMonitor.setWorkRemaining(Math.max(queue.end - queue.start + 1, 100));
 
 			// all subclasses of OBJECT are actually all types
 			char[] currentTypeName = queue.retrieve();
@@ -521,22 +664,7 @@ public static void searchAllPossibleSubTypes(
 
 			// search all index references to a given supertype
 			pattern.superSimpleName = currentTypeName;
-			indexManager.performConcurrentJob(job, waitingPolicy, progressMonitor == null ? null : new NullProgressMonitor() {
-				// don't report progress since this is too costly for deep hierarchies (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=34078 )
-				// just handle isCanceled() (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=179511 )
-				public void setCanceled(boolean value) {
-					progressMonitor.setCanceled(value);
-				}
-				public boolean isCanceled() {
-					return progressMonitor.isCanceled();
-				}
-				// and handle subTask(...) (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=34078 )
-				public void subTask(String name) {
-					progressMonitor.subTask(name);
-				}
-			});
-			if (progressMonitor != null && ++ticks <= MAXTICKS)
-				progressMonitor.worked(1);
+			indexManager.performConcurrentJob(job, waitingPolicy, subMonitor.split(1));
 
 			// in case, we search all subtypes, no need to search further
 			if (currentTypeName == null) break;
