@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2005, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -97,6 +100,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 			return CharOperation.hashCode(computeUniqueKey());
 		}
 
+		@Override
 		public boolean equals(Object obj) {
 			if (!(obj instanceof SyntheticLocalVariableBinding))
 				return false;
@@ -108,6 +112,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 	Binding compilerBinding;
 
 	char[][] compoundName;
+	char[] moduleName;
 	int dimension;
 	LookupEnvironment environment;
 	ReferenceBinding genericType;
@@ -155,6 +160,11 @@ public class BindingKeyResolver extends BindingKeyParser {
 		return this.compoundName;
 	}
 
+	public char[] moduleName() {
+		return this.moduleName;
+	}
+
+	@Override
 	public void consumeAnnotation() {
 		int size = this.types.size();
 		if (size == 0) return;
@@ -178,10 +188,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeArrayDimension(char[] brakets) {
 		this.dimension = brakets.length;
 	}
 
+	@Override
 	public void consumeBaseType(char[] baseTypeSig) {
 		this.compoundName = new char[][] {getKey().toCharArray()};
 		TypeBinding baseTypeBinding = getBaseTypeBinding(baseTypeSig);
@@ -190,10 +202,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeCapture18ID(int id, int position) {
 		consumeAnyCapture(id, position);
 	}
 
+	@Override
 	public void consumeCapture(final int position) {
 		consumeAnyCapture(-1, position);
 	}
@@ -246,41 +260,49 @@ public class BindingKeyResolver extends BindingKeyParser {
 				}
 				return false;
 			}
+			@Override
 			public boolean visit(SingleNameReference singleNameReference, BlockScope blockScope) {
 				if (checkType(singleNameReference.resolvedType))
 					return false;
 				return super.visit(singleNameReference, blockScope);
 			}
+			@Override
 			public boolean visit(QualifiedNameReference qualifiedNameReference, BlockScope blockScope) {
 				if (checkType(qualifiedNameReference.resolvedType))
 					return false;
 				return super.visit(qualifiedNameReference, blockScope);
 			}
+			@Override
 			public boolean visit(MessageSend messageSend, BlockScope blockScope) {
 				if (checkType(messageSend.resolvedType))
 					return false;
 				return super.visit(messageSend, blockScope);
 			}
+			@Override
 			public boolean visit(FieldReference fieldReference, BlockScope blockScope) {
 				if (checkType(fieldReference.resolvedType))
 					return false;
 				return super.visit(fieldReference, blockScope);
 			}
+			@Override
 			public boolean visit(ConditionalExpression conditionalExpression, BlockScope blockScope) {
 				if (checkType(conditionalExpression.resolvedType))
 					return false;
 				return super.visit(conditionalExpression, blockScope);
 			}
+			@Override
 			public boolean visit(CastExpression castExpression, BlockScope blockScope) {
 				if (checkType(castExpression.resolvedType))
 					return false;
 				return super.visit(castExpression, blockScope);
 			}
+			@Override
 			public boolean visit(Assignment assignment, BlockScope blockScope) {
 				if (checkType(assignment.resolvedType))
 					return false;
 				return super.visit(assignment, blockScope);
 			}
+			@Override
 			public boolean visit(ArrayReference arrayReference, BlockScope blockScope) {
 				if (checkType(arrayReference.resolvedType))
 					return false;
@@ -292,10 +314,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 		this.typeBinding = captureFinder.capture;
 	}
 
+	@Override
 	public void consumeException() {
 		this.types = new ArrayList();
 	}
 
+	@Override
 	public void consumeField(char[] fieldName) {
 		if (this.typeBinding == null)
 			return;
@@ -310,6 +334,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeParameterizedGenericMethod() {
 		if (this.methodBinding == null)
 			return;
@@ -326,7 +351,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 		this.compilerBinding = this.methodBinding;
 	}
 
+	@Override
 	public void consumeLocalType(char[] uniqueKey) {
+		if (this.parsedUnit == null) {
+			this.typeBinding = null;
+			return;
+		}
  		LocalTypeBinding[] localTypeBindings  = this.parsedUnit.localTypes;
  		for (int i = 0; i < this.parsedUnit.localTypeCount; i++)
  			if (CharOperation.equals(uniqueKey, localTypeBindings[i].computeUniqueKey(false/*not a leaf*/))) {
@@ -335,6 +365,7 @@ public class BindingKeyResolver extends BindingKeyParser {
  			}
 	}
 
+	@Override
 	public void consumeLocalVar(char[] varName, int occurrenceCount, int argumentPosition) {
 		if (this.scope == null) {
 			if (this.methodBinding == null)
@@ -376,6 +407,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeMethod(char[] selector, char[] signature) {
 		if (this.typeBinding == null)
 			return;
@@ -399,7 +431,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 					for (int j = 0; j < length; j++) {
 						parameterTypes[j] = getType(typeParameters[j]);
 					}
-					PolymorphicMethodBinding polymorphicMethod = this.environment.createPolymorphicMethod(method, parameterTypes);
+					PolymorphicMethodBinding polymorphicMethod = this.environment.createPolymorphicMethod(method, parameterTypes, this.scope);
 					this.methodBinding = polymorphicMethod;
 					this.methodBinding = this.environment.updatePolymorphicMethodReturnType(
 							polymorphicMethod,
@@ -460,15 +492,18 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 		return binding;
 	}
+	@Override
 	public void consumeMemberType(char[] simpleTypeName) {
 		this.typeBinding = getTypeBinding(simpleTypeName);
 	}
 
+	@Override
 	public void consumePackage(char[] pkgName) {
 		this.compoundName = CharOperation.splitOn('/', pkgName);
-		this.compilerBinding = new PackageBinding(this.compoundName, null, this.environment);
+		this.compilerBinding = new PackageBinding(this.compoundName, null, this.environment, this.environment.module); //TODO(SHMOD) enclosingModule
 	}
 
+	@Override
 	public void consumeParameterizedType(char[] simpleTypeName, boolean isRaw) {
 		if (this.typeBinding == null)
 			return;
@@ -501,10 +536,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 	}
 
 
+	@Override
 	public void consumeParser(BindingKeyParser parser) {
 		this.types.add(parser);
 	}
 
+	@Override
 	public void consumeScope(int scopeNumber) {
 		if (this.scope == null) {
 			if (this.methodBinding == null)
@@ -516,18 +553,22 @@ public class BindingKeyResolver extends BindingKeyParser {
 		this.scope = (BlockScope) this.scope.subscopes[scopeNumber];
 	}
 
+	@Override
 	public void consumeRawType() {
 		if (this.typeBinding == null) return;
 		this.typeBinding = this.environment.convertToRawType(this.typeBinding, false /*do not force conversion of enclosing types*/);
 	}
+	@Override
 	public void consumeSecondaryType(char[] simpleTypeName) {
 		this.secondarySimpleName = simpleTypeName;
 	}
 
+	@Override
 	public void consumeFullyQualifiedName(char[] fullyQualifiedName) {
 		this.compoundName = CharOperation.splitOn('/', fullyQualifiedName);
 	}
 
+	@Override
 	public void consumeTopLevelType() {
 		char[] fileName;
 		this.parsedUnit = getCompilationUnitDeclaration();
@@ -543,6 +584,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeKey() {
 		if (this.typeBinding != null) {
 			this.typeBinding = getArrayBinding(this.dimension, this.typeBinding);
@@ -550,6 +592,7 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeTypeVariable(char[] position, char[] typeVariableName) {
 		if (position.length > 0) {
 			if (this.typeBinding == null)
@@ -577,15 +620,18 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
 	public void consumeTypeWithCapture() {
 		BindingKeyResolver resolver = (BindingKeyResolver) this.types.get(0);
 		this.typeBinding =(TypeBinding) resolver.compilerBinding;
 	}
 
+	@Override
 	public void consumeWildcardRank(int aRank) {
 		this.wildcardRank = aRank;
 	}
 	
+	@Override
 	public void consumeWildCard(int kind) {
 		switch (kind) {
 			case Wildcard.EXTENDS:
@@ -606,6 +652,12 @@ public class BindingKeyResolver extends BindingKeyParser {
 		}
 	}
 
+	@Override
+	public void consumeModule(char[] aModuleName) {
+		this.moduleName = aModuleName;
+		this.compilerBinding = this.environment.getModule(aModuleName);
+	}
+
 	public AnnotationBinding getAnnotationBinding() {
 		return this.annotationBinding;
 	}
@@ -734,14 +786,17 @@ public class BindingKeyResolver extends BindingKeyParser {
 		return arguments;
 	}
 
+	@Override
 	public void malformedKey() {
 		this.compoundName = CharOperation.NO_CHAR_CHAR;
 	}
 
+	@Override
 	public BindingKeyParser newParser() {
 		return new BindingKeyResolver(this, this.compiler, this.environment, this.outerMostParsedUnit == null ? this.parsedUnit : this.outerMostParsedUnit, this.resolvedUnits);
 	}
 
+	@Override
 	public String toString() {
 		return getKey();
 	}
