@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -40,6 +43,7 @@ public final class Messages extends NLS {
 	public static String element_nullName;
 	public static String element_nullType;
 	public static String element_illegalParent;
+	public static String element_moduleInfoNotSupported;
 	public static String javamodel_initialization;
 	public static String javamodel_initializing_delta_state;
 	public static String javamodel_building_after_upgrade;
@@ -185,6 +189,9 @@ public final class Messages extends NLS {
 	public static String classpath_incompatibleLibraryJDKLevelInContainer;
 	public static String classpath_duplicateEntryExtraAttribute;
 	public static String classpath_deprecated_variable;
+	public static String classpath_invalidExternalAnnotationPath;
+	public static String classpath_testSourceRequiresSeparateOutputFolder;
+	public static String classpath_testOutputFolderMustBeSeparateFromMainOutputFolders;
 	public static String file_notFound;
 	public static String file_badFormat;
 	public static String path_nullPath;
@@ -211,6 +218,13 @@ public final class Messages extends NLS {
 	public static String convention_package_nameWithBlanks;
 	public static String convention_package_consecutiveDotsName;
 	public static String convention_package_uppercaseName;
+	public static String convention_module_nullName;
+	public static String convention_module_emptyName;
+	public static String convention_module_dotName;
+	public static String convention_module_nameWithBlanks;
+	public static String convention_module_consecutiveDotsName;
+	public static String convention_module_uppercaseName;
+	public static String convention_module_javaName;
 	public static String dom_cannotDetail;
 	public static String dom_nullTypeParameter;
 	public static String dom_nullNameParameter;
@@ -263,6 +277,8 @@ public final class Messages extends NLS {
 	public static String disassembler_outer_class_info_name;
 	public static String disassembler_inner_name;
 	public static String disassembler_inner_accessflags;
+	public static String disassembler_nesthost;
+	public static String disassembler_nestmembers;
 	public static String disassembler_genericattributeheader;
 	public static String disassembler_signatureattributeheader;
 	public static String disassembler_bootstrapmethodattributesheader;
@@ -290,6 +306,7 @@ public final class Messages extends NLS {
 	public static String disassembler_constantpool_methodhandle;
 	public static String disassembler_constantpool_methodtype;
 	public static String disassembler_constantpool_invokedynamic;
+	public static String disassembler_constantpool_dynamic;
 	public static String disassembler_annotationdefaultheader;
 	public static String disassembler_annotationdefaultvalue;
 	public static String disassembler_annotationenumvalue;
@@ -322,6 +339,10 @@ public final class Messages extends NLS {
 	public static String disassembler_extendedannotation_typepath_typeargument;
 	public static String disassembler_extendedannotation_typepath_innertype;
 	public static String disassembler_localvariabletargetheader;
+	public static String disassembler_module_version;
+	public static String disassembler_module_version_none;
+	public static String disassembler_modulepackagesattributeheader;
+	public static String disassembler_modulemainclassattributeheader;
 
 	public static String disassembler_runtimevisibleannotationsattributeheader;
 	public static String disassembler_runtimeinvisibleannotationsattributeheader;
@@ -345,6 +366,9 @@ public final class Messages extends NLS {
 	public static String classformat_checkcast;
 	public static String classformat_instanceof;
 	public static String classformat_ldc_w_class;
+	public static String classformat_ldc_w_methodtype;
+	public static String classformat_ldc_w_methodhandle;
+	public static String classformat_ldc_w_dynamic;
 	public static String classformat_ldc_w_float;
 	public static String classformat_ldc_w_integer;
 	public static String classformat_ldc_w_string;
@@ -385,7 +409,7 @@ public final class Messages extends NLS {
 	public static String disassembler_frame_full_frame;
 	public static String disassembler_frame_same_frame;
 	public static String disassembler_frame_same_locals_1_stack_item;
-	public static String code_assist_internal_error;
+	public static String internal_error;
 	
 	public static String disassembler_method_type_ref_getfield;
 	public static String disassembler_method_type_ref_putfield;
