@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2013 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -40,6 +43,7 @@ public CodeSnippetParser(ProblemReporter problemReporter, EvaluationContext eval
 	this.reportOnlyOneSyntaxError = true;
 	this.javadocParser.checkDocComment = false;
 }
+@Override
 protected void classInstanceCreation(boolean alwaysQualified) {
 	// ClassInstanceCreationExpression ::= 'new' ClassType '(' ArgumentListopt ')' ClassBodyopt
 
@@ -86,6 +90,7 @@ protected void classInstanceCreation(boolean alwaysQualified) {
 		this.astLengthPtr--;
 	}
 }
+@Override
 protected void consumeClassInstanceCreationExpressionWithTypeArguments() {
 	// ClassInstanceCreationExpression ::= 'new' TypeArguments ClassType '(' ArgumentListopt ')' ClassBodyopt
 	AllocationExpression alloc;
@@ -139,11 +144,13 @@ protected void consumeClassInstanceCreationExpressionWithTypeArguments() {
 		}
 	}
 }
+@Override
 protected void consumeClassDeclaration() {
 	super.consumeClassDeclaration();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeClassHeaderName1() {
 	// ClassHeaderName ::= Modifiersopt 'class' 'Identifier'
 	TypeDeclaration typeDecl;
@@ -192,21 +199,25 @@ protected void consumeClassHeaderName1() {
 	typeDecl.javadoc = this.javadoc;
 	this.javadoc = null;
 }
+@Override
 protected void consumeEmptyStatement() {
 	super.consumeEmptyStatement();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeEnhancedForStatement() {
 	super.consumeEnhancedForStatement();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeExpressionStatement() {
 	super.consumeExpressionStatement();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeFieldAccess(boolean isSuperAccess) {
 	// FieldAccess ::= Primary '.' 'Identifier'
 	// FieldAccess ::= 'super' '.' 'Identifier'
@@ -232,11 +243,13 @@ protected void consumeFieldAccess(boolean isSuperAccess) {
 		this.expressionStack[this.expressionPtr] = fr;
 	}
 }
+@Override
 protected void consumeInternalCompilationUnit() {
 	// InternalCompilationUnit ::= PackageDeclaration
 	// InternalCompilationUnit ::= PackageDeclaration ImportDeclarations ReduceImports
 	// InternalCompilationUnit ::= ImportDeclarations ReduceImports
 }
+@Override
 protected void consumeInternalCompilationUnitWithTypes() {
 	// InternalCompilationUnit ::= PackageDeclaration ImportDeclarations ReduceImports TypeDeclarations
 	// InternalCompilationUnit ::= PackageDeclaration TypeDeclarations
@@ -250,6 +263,7 @@ protected void consumeInternalCompilationUnitWithTypes() {
 		System.arraycopy(this.astStack, this.astPtr + 1, this.compilationUnit.types, 0, length);
 	}
 }
+@Override
 protected void consumeLocalVariableDeclarationStatement() {
 	super.consumeLocalVariableDeclarationStatement();
 	/* recovery */
@@ -262,6 +276,7 @@ protected void consumeLocalVariableDeclarationStatement() {
  * The CSToCuMapper could not be used, since it could have interfered with
  * the syntax recovery specific to code snippets.
  */
+@Override
 protected void consumeMethodDeclaration(boolean isNotAbstract, boolean isDefaultMethod) {
 	// MethodDeclaration ::= MethodHeader MethodBody
 	// AbstractMethodDeclaration ::= MethodHeader ';'
@@ -351,6 +366,7 @@ protected void consumeMethodDeclaration(boolean isNotAbstract, boolean isDefault
 	}
 }
 
+@Override
 protected void consumeMethodInvocationName() {
 	// MethodInvocation ::= Name '(' ArgumentListopt ')'
 
@@ -389,6 +405,7 @@ protected void consumeMethodInvocationName() {
 		super.consumeMethodInvocationName();
 	}
 }
+@Override
 protected void consumeMethodInvocationNameWithTypeArguments() {
 	// MethodInvocation ::= Name '.' TypeArguments 'Identifier' '(' ArgumentListopt ')'
 
@@ -418,6 +435,7 @@ protected void consumeMethodInvocationNameWithTypeArguments() {
 		super.consumeMethodInvocationNameWithTypeArguments();
 	}
 }
+@Override
 protected void consumeMethodInvocationSuper() {
 	// MethodInvocation ::= 'super' '.' 'Identifier' '(' ArgumentListopt ')'
 
@@ -430,6 +448,7 @@ protected void consumeMethodInvocationSuper() {
 	m.receiver = new CodeSnippetSuperReference(m.sourceStart, this.endPosition);
 	pushOnExpressionStack(m);
 }
+@Override
 protected void consumeMethodInvocationSuperWithTypeArguments() {
 	// MethodInvocation ::= 'super' '.' TypeArguments 'Identifier' '(' ArgumentListopt ')'
 
@@ -449,6 +468,7 @@ protected void consumeMethodInvocationSuperWithTypeArguments() {
 	m.receiver = new CodeSnippetSuperReference(m.sourceStart, this.endPosition);
 	pushOnExpressionStack(m);
 }
+@Override
 protected void consumePrimaryNoNewArrayThis() {
 	// PrimaryNoNewArray ::= 'this'
 
@@ -461,56 +481,67 @@ protected void consumePrimaryNoNewArrayThis() {
 		super.consumePrimaryNoNewArrayThis();
 	}
 }
+@Override
 protected void consumeStatementBreak() {
 	super.consumeStatementBreak();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementBreakWithLabel() {
 	super.consumeStatementBreakWithLabel();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementCatch() {
 	super.consumeStatementCatch();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementContinue() {
 	super.consumeStatementContinue();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementContinueWithLabel() {
 	super.consumeStatementContinueWithLabel();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementDo() {
 	super.consumeStatementDo();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementFor() {
 	super.consumeStatementFor();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementIfNoElse() {
 	super.consumeStatementIfNoElse();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementIfWithElse() {
 	super.consumeStatementIfWithElse();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementLabel() {
 	super.consumeStatementLabel();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementReturn() {
 	// ReturnStatement ::= 'return' Expressionopt ';'
 
@@ -533,31 +564,37 @@ protected void consumeStatementReturn() {
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementSwitch() {
 	super.consumeStatementSwitch();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementSynchronized() {
 	super.consumeStatementSynchronized();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementThrow() {
 	super.consumeStatementThrow();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementTry(boolean arg_0, boolean arg_1) {
 	super.consumeStatementTry(arg_0, arg_1);
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected void consumeStatementWhile() {
 	super.consumeStatementWhile();
 	/* recovery */
 	recordLastStatementIfNeeded();
 }
+@Override
 protected CompilationUnitDeclaration endParse(int act) {
 	if (this.hasRecoveredOnExpression) {
 		CompilationResult unitResult = this.compilationUnit.compilationResult;
@@ -636,6 +673,7 @@ protected CompilationUnitDeclaration endParse(int act) {
 	}
 	return super.endParse(act);
 }
+@Override
 protected NameReference getUnspecifiedReference(boolean rejectTypeAnnotations) {
 	/* build a (unspecified) NameReference which may be qualified*/
 	if (rejectTypeAnnotations) {
@@ -672,6 +710,7 @@ protected NameReference getUnspecifiedReference(boolean rejectTypeAnnotations) {
 		return super.getUnspecifiedReference(rejectTypeAnnotations);
 	}
 }
+@Override
 protected NameReference getUnspecifiedReferenceOptimized() {
 	/* build a (unspecified) NameReference which may be qualified
 	The optimization occurs for qualified reference while we are
@@ -721,6 +760,7 @@ protected NameReference getUnspecifiedReferenceOptimized() {
 		return super.getUnspecifiedReferenceOptimized();
 	}
 }
+@Override
 protected void ignoreExpressionAssignment() {
 	super.ignoreExpressionAssignment();
 	/* recovery */
@@ -732,6 +772,7 @@ protected void ignoreExpressionAssignment() {
 private boolean isTopLevelType() {
 	return this.nestedType == (this.diet ? 0 : 1);
 }
+@Override
 protected MessageSend newMessageSend() {
 	// '(' ArgumentListopt ')'
 	// the arguments are on the expression stack
@@ -749,6 +790,7 @@ protected MessageSend newMessageSend() {
 	}
 	return m;
 }
+@Override
 protected MessageSend newMessageSendWithTypeArguments() {
 	// '(' ArgumentListopt ')'
 	// the arguments are on the expression stack
@@ -774,6 +816,7 @@ private void recordLastStatementIfNeeded() {
 	}
 }
 
+@Override
 protected void reportSyntaxErrors(boolean isDietParse, int oldFirstToken) {
 	if (!isDietParse) {
 		this.scanner.initialPosition = this.lastStatement;
@@ -786,6 +829,7 @@ protected void reportSyntaxErrors(boolean isDietParse, int oldFirstToken) {
  * A syntax error was detected. If a method is being parsed, records the number of errors and
  * attempts to restart from the last statement by going for an expression.
  */
+@Override
 protected int resumeOnSyntaxError() {
 	if (this.diet || this.hasRecoveredOnExpression) { // no reentering inside expression recovery
 		return HALT;
