@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -27,6 +30,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryTypeAnnotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.ITypeAnnotationWalker;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BinaryTypeBinding.ExternalAnnotationStatus;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
 
 /**
@@ -48,45 +52,59 @@ public class CodeSnippetSkeleton implements IBinaryType, EvaluationConstants {
 			this.exceptionTypeNames = exceptionTypeNames;
 			this.isConstructor = isConstructor;
 		}
+		@Override
 		public IBinaryAnnotation[] getAnnotations() {
 			return null;
 		}
+		@Override
 		public char[][] getArgumentNames() {
 			return null;
 		}
+		@Override
 		public Object getDefaultValue() {
 			return null;
 		}
+		@Override
 		public char[][] getExceptionTypeNames() {
 			return this.exceptionTypeNames;
 		}
+		@Override
 		public char[] getGenericSignature() {
 			return null;
 		}
+		@Override
 		public char[] getMethodDescriptor() {
 			return this.methodDescriptor;
 		}
+		@Override
 		public int getModifiers() {
 			return ClassFileConstants.AccPublic;
 		}
-		public IBinaryAnnotation[] getParameterAnnotations(int index) {
+		@Override
+		public IBinaryAnnotation[] getParameterAnnotations(int index, char[] classFileName) {
 			return null;
 		}
+		@Override
 		public int getAnnotatedParametersCount() {
 			return 0;
 		}
+		@Override
 		public IBinaryTypeAnnotation[] getTypeAnnotations() {
 			return null; 
 		}
+		@Override
 		public char[] getSelector() {
 			return this.selector;
 		}
+		@Override
 		public long getTagBits() {
 			return 0;
 		}
+		@Override
 		public boolean isClinit() {
 			return false;
 		}
+		@Override
 		public boolean isConstructor() {
 			return this.isConstructor;
 		}
@@ -119,30 +137,38 @@ public class CodeSnippetSkeleton implements IBinaryType, EvaluationConstants {
 public CodeSnippetSkeleton() {
 	super();
 }
+@Override
 public IBinaryAnnotation[] getAnnotations() {
 	return null;
 }
+@Override
 public IBinaryTypeAnnotation[] getTypeAnnotations() {
 	return null; 
 }
+@Override
 public char[] getEnclosingMethod() {
 	return null;
 }
+@Override
 public char[] getEnclosingTypeName() {
 	return null;
 }
+@Override
 public IBinaryField[] getFields() {
 	return null;
 }
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.compiler.env.IDependent#getFileName()
  */
+@Override
 public char[] getFileName() {
 	return CharOperation.concat(CODE_SNIPPET_NAME, Util.defaultJavaExtension().toCharArray());
 }
+@Override
 public char[] getGenericSignature() {
 	return null;
 }
+@Override
 public char[][] getInterfaceNames() {
 	return null;
 }
@@ -152,49 +178,72 @@ public String getJavadocContents() {
 public String getJavadocContents(IProgressMonitor monitor, String defaultEncoding) throws JavaModelException {
 	return null;
 }
+@Override
 public IBinaryNestedType[] getMemberTypes() {
 	return null;
 }
+@Override
 public IBinaryMethod[] getMethods() {
 	return this.methods;
 }
+@Override
 public int getModifiers() {
 	return ClassFileConstants.AccPublic;
 }
+@Override
 public char[][][] getMissingTypeNames() {
 	return null;
 }
+@Override
 public char[] getName() {
 	return CODE_SNIPPET_NAME;
 }
+@Override
 public char[] getSourceName() {
 	return ROOT_CLASS_NAME;
 }
+@Override
 public char[] getSuperclassName() {
 	return null;
 }
+@Override
 public long getTagBits() {
 	return 0;
 }
 public String getURLContents(String docUrlValue, String defaultEncoding) {
 	return null;
 }
+@Override
 public boolean isAnonymous() {
 	return false;
 }
+@Override
 public boolean isBinaryType() {
 	return true;
 }
+@Override
 public boolean isLocal() {
 	return false;
 }
+@Override
 public boolean isMember() {
 	return false;
 }
+@Override
 public char[] sourceFileName() {
 	return null;
 }
+@Override
 public ITypeAnnotationWalker enrichWithExternalAnnotationsFor(ITypeAnnotationWalker walker, Object member, LookupEnvironment environment) {
 	return walker;
 }
+@Override
+public char[] getModule() {
+	// TODO Java 9 Auto-generated method stub
+	return null;
+}
+@Override
+public ExternalAnnotationStatus getExternalAnnotationStatus() {
+	return ExternalAnnotationStatus.NOT_EEA_CONFIGURED;
+}
 }
