@@ -1,13 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2015 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2018 Mateusz Matela and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] IndexOutOfBoundsException in TokenManager - https://bugs.eclipse.org/462945
+ *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Bad line breaking in Eclipse javadoc comments - https://bugs.eclipse.org/348338
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.formatter;
 
@@ -36,8 +42,8 @@ import org.aspectj.org.eclipse.jdt.core.dom.LineComment;
 import org.aspectj.org.eclipse.jdt.core.dom.MemberRef;
 import org.aspectj.org.eclipse.jdt.core.dom.MethodRef;
 import org.aspectj.org.eclipse.jdt.core.dom.QualifiedName;
+import org.aspectj.org.eclipse.jdt.core.dom.SimpleName;
 import org.aspectj.org.eclipse.jdt.core.dom.TagElement;
-import org.aspectj.org.eclipse.jdt.core.formatter.CodeFormatter;
 import org.aspectj.org.eclipse.jdt.core.formatter.DefaultCodeFormatterConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
@@ -57,11 +63,11 @@ public class CommentsPreparator extends ASTVisitor {
 		String separateLineTags = "(dl|hr|nl|p|ul|ol|table|tr)"; //$NON-NLS-1$
 		String breakBeforeTags = "(dd|dt|li|td|th|h1|h2|h3|h4|h5|h6|q)"; //$NON-NLS-1$
 		String breakAfterTags = "(br)"; //$NON-NLS-1$
-		String noFormatTags = "(code|em|tt)"; //$NON-NLS-1$
-		String otherTags = "([^<>&&\\S]++)"; //$NON-NLS-1$
+		String noFormatTags = "(code|tt)"; //$NON-NLS-1$
+		String otherTags = "([\\S&&[^<>]]++)"; //$NON-NLS-1$
 		String ws = "(?>[ \\t]++|[\\r\\n]++[ \\t]*+\\*?)"; // whitespace or line break with optional asterisk //$NON-NLS-1$
-		String attributeValue = "(?>\"[^\"]*\")|(?>\'[^\']*\')|[^/>\"\'&&\\S]++"; //$NON-NLS-1$
-		String attribute = "(?>" + ws + "+[^=&&\\S]+" + ws + "*(=)" + ws + "*(?>" + attributeValue  + "))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
+		String attributeValue = "(?>\"[^\"]*\")|(?>\'[^\']*\')|[\\S&&[^/>\"\']]++"; //$NON-NLS-1$
+		String attribute = "(?>" + ws + "+[\\S&&[^=]]+" + ws + "*(=)" + ws + "*(?>" + attributeValue  + "))"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
 		HTML_TAG_PATTERN = Pattern.compile("<(/)?+(?:" //$NON-NLS-1$
 				+ formatCodeTags + '|' + separateLineTags + '|' + breakBeforeTags + '|' + breakAfterTags + '|' + noFormatTags + '|' + otherTags + ')'
 				+ "(" + attribute + "*)" + ws + "*/?>", Pattern.CASE_INSENSITIVE); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
@@ -99,7 +105,7 @@ public class CommentsPreparator extends ASTVisitor {
 	private List<Token> commentStructure;
 	private int commentIndent;
 	/** Index: position within current comment; Value: whether wrapping on special characters is allowed */
-	private boolean[] noSubstituteWrapping;
+	private boolean[] allowSubstituteWrapping;
 
 	private int noFormatTagOpenStart = -1;
 	private int formatCodeTagOpenEnd = -1;
@@ -141,12 +147,12 @@ public class CommentsPreparator extends ASTVisitor {
 					&& !this.options.comment_format_line_comment_starting_on_first_column) {
 				this.lastLineComment = null;
 				commentToken.setIndent(0);
-				commentToken.setWrapPolicy(null);
+				commentToken.setWrapPolicy(WrapPolicy.FORCE_FIRST_COLUMN);
 				return;
 			}
 			if (this.options.never_indent_line_comments_on_first_column) {
 				commentToken.setIndent(0);
-				commentToken.setWrapPolicy(null);
+				commentToken.setWrapPolicy(WrapPolicy.FORCE_FIRST_COLUMN);
 			}
 		}
 
@@ -193,14 +199,15 @@ public class CommentsPreparator extends ASTVisitor {
 			this.tm.remove(commentIndex);
 			List<Token> lastStructure = this.lastLineComment.getInternalStructure();
 			lastStructure.addAll(structure);
-			structure = lastStructure;
-			commentToken = merged;
+			merged.setInternalStructure(lastStructure);
+			this.lastLineComment = merged;
 		} else {
+			commentToken.setInternalStructure(structure);
+			handleCompilerTags(commentToken, commentIndex);
+			preserveWhitespace(commentToken, commentIndex);
+			this.lastLineComment = commentToken;
 			this.lastLineCommentPosition = positionInLine;
 		}
-		commentToken.setInternalStructure(structure);
-		preserveWhitespace(commentToken, commentIndex);
-		this.lastLineComment = commentToken;
 	}
 
 	private void preserveWhitespace(Token commentToken, int commentIndex) {
@@ -216,7 +223,7 @@ public class CommentsPreparator extends ASTVisitor {
 			if (previous.originalEnd + 1 >= commentToken.originalStart)
 				return;
 			if (structure == null || structure.isEmpty()) {
-				structure = new ArrayList<Token>();
+				structure = new ArrayList<>();
 				structure.add(new Token(previous.originalEnd + 1, commentToken.originalEnd, TokenNameCOMMENT_LINE));
 				commentToken.setInternalStructure(structure);
 			} else {
@@ -252,7 +259,7 @@ public class CommentsPreparator extends ASTVisitor {
 		if (stringLiterals.isEmpty())
 			return;
 
-		List<Token> commentFragments = new ArrayList<Token>();
+		List<Token> commentFragments = new ArrayList<>();
 		Matcher matcher = NLS_TAG_PATTERN.matcher(this.tm.toString(comment));
 		int previousMatcherEnd = 0;
 		boolean nlsFound = false;
@@ -283,8 +290,30 @@ public class CommentsPreparator extends ASTVisitor {
 		}
 	}
 
+	private void handleCompilerTags(Token commentToken, int commentIndex) {
+		final String commentText = this.tm.toString(commentToken);
+		final List<Token> structure = commentToken.getInternalStructure();
+		if (commentText.startsWith("//$FALL-THROUGH$") //$NON-NLS-1$
+				|| commentText.startsWith("//$IDENTITY-COMPARISON$")) { //$NON-NLS-1$
+			structure.get(1).clearSpaceBefore();
+		}
+		if (commentText.contains("//$IDENTITY-COMPARISON$")) { //$NON-NLS-1$
+			// make sure the whole line is not broken
+			Token token = commentToken;
+			for (int i = commentIndex; i > 0; i--) {
+				Token left = this.tm.get(i - 1);
+				if (this.tm.countLineBreaksBetween(left, token) > 0)
+					break;
+				token.clearLineBreaksBefore();
+				left.clearLineBreaksAfter();
+				token.setWrapPolicy(WrapPolicy.DISABLE_WRAP);
+				token = left;
+			}
+		}
+	}
+
 	private List<Token> findStringLiteralsInLine(int lastTokenIndex) {
-		List<Token> stringLiterals = new ArrayList<Token>();
+		List<Token> stringLiterals = new ArrayList<>();
 		Token previous = this.tm.get(lastTokenIndex);
 		for (int i = lastTokenIndex - 1; i >= 0; i--) {
 			Token token = this.tm.get(i);
@@ -303,7 +332,7 @@ public class CommentsPreparator extends ASTVisitor {
 		if (fragments == null) {
 			fragments = Arrays.asList(commentToken);
 		}
-		ArrayList<Token> result = new ArrayList<Token>();
+		ArrayList<Token> result = new ArrayList<>();
 		for (int i = 0; i < fragments.size(); i++) {
 			Token token = fragments.get(i);
 			if (token.hasNLSTag()) {
@@ -333,11 +362,6 @@ public class CommentsPreparator extends ASTVisitor {
 				sourcePosition++;
 			}
 		}
-
-		if (this.tm.getSource().startsWith("$FALL-THROUGH$", result.get(0).originalEnd + 1)) { //$NON-NLS-1$
-			result.get(1).clearSpaceBefore();
-		}
-
 		return result;
 	}
 
@@ -371,7 +395,7 @@ public class CommentsPreparator extends ASTVisitor {
 
 		if (this.options.never_indent_block_comments_on_first_column && isFirstColumn) {
 			commentToken.setIndent(0);
-			commentToken.setWrapPolicy(null);
+			commentToken.setWrapPolicy(WrapPolicy.FORCE_FIRST_COLUMN);
 		}
 	}
 
@@ -406,7 +430,8 @@ public class CommentsPreparator extends ASTVisitor {
 
 		if (existingBreaksBefore <= 1
 				&& (previous.tokenType == TokenNameCOMMENT_LINE || previous.tokenType == TokenNameCOMMENT_BLOCK)) {
-			commentToken.setWrapPolicy(previous.getWrapPolicy());
+			if (previous.getWrapPolicy() != WrapPolicy.FORCE_FIRST_COLUMN)
+				commentToken.setWrapPolicy(previous.getWrapPolicy());
 		} else {
 			int i = commentIndex + 2;
 			while (existingBreaksAfter <= 1 && i < this.tm.size()
@@ -431,7 +456,7 @@ public class CommentsPreparator extends ASTVisitor {
 	}
 
 	private List<Token> commentToLines(Token commentToken, int commentStartPositionInLine) {
-		List<Token> lines = new ArrayList<Token>();
+		List<Token> lines = new ArrayList<>();
 
 		int tab = this.options.tab_size;
 		String commentText = this.tm.toString(commentToken);
@@ -528,6 +553,9 @@ public class CommentsPreparator extends ASTVisitor {
 		this.commentStructure = commentToken.getInternalStructure();
 		this.commentIndent = this.tm.toIndent(commentToken.getIndent(), true);
 		this.ctm = new TokenManager(commentToken.getInternalStructure(), this.tm);
+
+		handleJavadocTagAlignment(node);
+
 		return true;
 	}
 
@@ -564,22 +592,16 @@ public class CommentsPreparator extends ASTVisitor {
 				Token token = this.ctm.get(startIndex + 2);
 				token.breakBefore();
 			}
-
-			if (this.options.comment_indent_root_tags) {
-				int indent = this.ctm.getLength(this.ctm.get(startIndex), 0) + 1;
-				if (isParamTag && this.options.comment_indent_parameter_description)
-					indent += this.options.indentation_size;
-				for (int i = startIndex + 1; i <= endIndex; i++) {
+			if (isParamTag && this.options.comment_indent_parameter_description) {
+				for (int i = startIndex + 2; i <= endIndex; i++) {
 					Token token = this.ctm.get(i);
-					token.setIndent(indent);
-					// indent is used temporarily, tokens that are actually first in line
-					// will have this changed to align (indent is reserved for code inside <pre> tags)
+					token.setIndent(token.getIndent() + this.options.indentation_size);
 				}
 			}
 
 			Token startTokeen = this.ctm.get(startIndex);
 			if (startIndex > 1)
-			startTokeen.breakBefore();
+				startTokeen.breakBefore();
 			int firstTagIndex;
 			if (this.firstTagToken == null || (firstTagIndex = this.ctm.indexOf(this.firstTagToken)) < 0
 					|| startIndex < firstTagIndex)
@@ -588,14 +610,8 @@ public class CommentsPreparator extends ASTVisitor {
 			handleHtml(node);
 		}
 
-		else if (IMMUTABLE_TAGS.contains(tagName)) {
-			if (startIndex < endIndex)
-				disableFormatting(startIndex, endIndex);
-			noSubstituteWrapping(node.getStartPosition(), nodeEnd);
-		}
-
-		else if (node.isNested()) {
-			noSubstituteWrapping(node.getStartPosition(), nodeEnd);
+		if (node.isNested() && IMMUTABLE_TAGS.contains(tagName) && startIndex < endIndex) {
+			disableFormatting(startIndex, endIndex);
 		}
 		return true;
 	}
@@ -606,7 +622,112 @@ public class CommentsPreparator extends ASTVisitor {
 		if (tagName == null || tagName.length() <= 1)
 			handleHtml(node);
 
-		handleStringLiterals(this.tm.toString(node), node.getStartPosition());
+		if (TagElement.TAG_SEE.equals(tagName))
+			handleStringLiterals(this.tm.toString(node), node.getStartPosition());
+
+		if (PARAM_TAGS.contains(tagName) && this.options.comment_indent_parameter_description) {
+			// tokens in the first line should not be indented
+			int startIndex = this.ctm.findIndex(node.getStartPosition(), -1, false);
+			if (this.ctm.get(startIndex).tokenType != TokenNameNotAToken) {
+				int endIndex = this.ctm.lastIndexIn(node, -1);
+				int range = this.options.indentation_size;
+				for (int i = startIndex + 2; i <= endIndex && range > 0; i++) {
+					Token token = this.ctm.get(i);
+					if (token.getLineBreaksBefore() > 0)
+						break;
+					assert token.getIndent() >= this.options.indentation_size;
+					token.setIndent(token.getIndent() - this.options.indentation_size);
+					if (token.getLineBreaksAfter() > 0)
+						break;
+					range -= this.ctm.getLength(token, 0) + (token.isSpaceBefore() ? 1 : 0);
+				}
+			}
+		}
+	}
+
+	private void handleJavadocTagAlignment(Javadoc node) {
+		// Lists of tag tokens: index 0 for tag name, index 1 for param name (may be null), the rest for description
+		List<List<Token>> javadocRootTags = new ArrayList<>();
+		List<TagElement> tagElements = node.tags();
+		for (TagElement tagElement : tagElements) {
+			String tagName = tagElement.getTagName();
+			if (tagName == null || tagName.length() <= 1)
+				continue;
+			int startIndex = tokenStartingAt(tagElement.getStartPosition());
+			int nodeEnd = tagElement.getStartPosition() + tagElement.getLength() - 1;
+			while (ScannerHelper.isWhitespace(this.ctm.charAt(nodeEnd)))
+				nodeEnd--;
+			int endIndex = tokenEndingAt(nodeEnd);
+
+			List<Token> tagTokens = new ArrayList<>();
+			tagTokens.add(this.ctm.get(startIndex));
+			if (!PARAM_TAGS.contains(tagName) || tagElement.fragments().isEmpty()
+					|| !(tagElement.fragments().get(0) instanceof SimpleName)) {
+				tagTokens.add(null);
+			}
+			for (int i = startIndex + 1; i <= endIndex; i++) {
+				tagTokens.add(this.ctm.get(i));
+			}
+			javadocRootTags.add(tagTokens);
+		}
+		
+		// indent is used temporarily, tokens that are actually first in line
+		// will have this changed to align (indent is reserved for code inside <pre> tags)
+		if (this.options.comment_align_tags_names_descriptions) {
+			int maxTagNameLength = 0;
+			int maxParamNameLength = 0;
+			for (List<Token> tagTokens : javadocRootTags) {
+				Token tagName = tagTokens.get(0);
+				Token paramName = tagTokens.get(1);
+				maxTagNameLength = Math.max(maxTagNameLength, this.tm.getLength(tagName, 0));
+				if (paramName != null)
+					maxParamNameLength = Math.max(maxParamNameLength, this.tm.getLength(paramName, 0));
+			}
+			int paramNameAlign = maxTagNameLength + 1;
+			int descriptionAlign = paramNameAlign;
+			if (maxParamNameLength > 0)
+				descriptionAlign += maxParamNameLength + 1;
+			for (List<Token> tagTokens : javadocRootTags) {
+				if (tagTokens.get(1) != null)
+					tagTokens.get(1).setIndent(paramNameAlign);
+				for (int i = 2; i < tagTokens.size(); i++)
+					tagTokens.get(i).setIndent(descriptionAlign);
+			}
+		} else if (this.options.comment_align_tags_descriptions_grouped) {
+			int groupStart = 0;
+			String groupTagName = null;
+			int descriptionAlign = 0;
+			for (int i = 0; i < javadocRootTags.size(); i++) {
+				List<Token> tagTokens = javadocRootTags.get(i);
+				String tagName = this.ctm.toString(tagTokens.get(0));
+				if (!tagName.equals(groupTagName)) {
+					for (int j = groupStart; j < i; j++) {
+						List<Token> tokens = javadocRootTags.get(j);
+						for (int k = 2; k < tokens.size(); k++)
+							tokens.get(k).setIndent(descriptionAlign);
+					}
+					groupStart = i;
+					groupTagName = tagName;
+					descriptionAlign = 0;
+				}
+				int indent = tagName.length() + 1;
+				if (tagTokens.get(1) != null)
+					indent += 1 + this.ctm.getLength(tagTokens.get(1), 0);
+				descriptionAlign = Math.max(descriptionAlign, indent);
+			}
+			for (int j = groupStart; j < javadocRootTags.size(); j++) {
+				List<Token> tokens = javadocRootTags.get(j);
+				for (int k = 2; k < tokens.size(); k++)
+					tokens.get(k).setIndent(descriptionAlign);
+			}
+		} else if (this.options.comment_indent_root_tags) {
+			for (List<Token> tagTokens : javadocRootTags) {
+				Token tagName = tagTokens.get(0);
+				int indent = this.ctm.getLength(tagName, 0) + 1;
+				for (int i = 2; i < tagTokens.size(); i++)
+					tagTokens.get(i).setIndent(indent);
+			}
+		}
 	}
 
 	private void handleHtml(TagElement node) {
@@ -619,33 +740,23 @@ public class CommentsPreparator extends ASTVisitor {
 			int endPos = matcher.end() - 1 + node.getStartPosition();
 			boolean isOpeningTag = (matcher.start(1) == matcher.end(1));
 
-			int firstTokenIndex = 0, lastTokenIndex = 0;
 			if (this.options.comment_format_html) {
 				// make sure tokens inside the tag are wrapped only as a substitute
-				firstTokenIndex = tokenStartingAt(startPos);
-				lastTokenIndex = tokenEndingAt(endPos);
-				Token startToken = this.ctm.get(firstTokenIndex);
-				if (!isOpeningTag && startToken.getWrapPolicy() == null)
-					startToken.setWrapPolicy(WrapPolicy.SUBSTITUTE_ONLY);
+				int firstTokenIndex = tokenStartingAt(startPos), lastTokenIndex = tokenEndingAt(endPos);
 				for (int i = firstTokenIndex + 1; i <= lastTokenIndex; i++) {
 					Token token = this.ctm.get(i);
 					if (token.getWrapPolicy() == null)
 						token.setWrapPolicy(WrapPolicy.SUBSTITUTE_ONLY);
 				}
-				Token nextToken = this.ctm.get(lastTokenIndex + 1);
-				if (isOpeningTag && nextToken.getWrapPolicy() == null)
-					nextToken.setWrapPolicy(WrapPolicy.SUBSTITUTE_ONLY);
 
-				// never break tags on special characters
-				noSubstituteWrapping(startPos, endPos - 1);
-				// ... except for equals sign in attributes
+				// allow wraps around equals sign in attributes
 				String attributesText = matcher.group(8);
 				Matcher attrMatcher = HTML_ATTRIBUTE_PATTERN.matcher(attributesText);
 				final int commentStart = this.ctm.get(0).originalStart;
 				while (attrMatcher.find()) {
 					int equalPos = node.getStartPosition() + matcher.start(8) + attrMatcher.start(1);
 					assert this.tm.charAt(equalPos) == '=';
-					this.noSubstituteWrapping[equalPos - commentStart] = false;
+					this.allowSubstituteWrapping[equalPos - commentStart] = true;
 				}
 			}
 
@@ -708,7 +819,6 @@ public class CommentsPreparator extends ASTVisitor {
 				token.setWrapPolicy(WrapPolicy.DISABLE_WRAP);
 			}
 		}
-		noSubstituteWrapping(node.getStartPosition(), node.getStartPosition() + node.getLength() - 1);
 	}
 
 	private void handleStringLiterals(String text, int textStartPosition) {
@@ -718,8 +828,11 @@ public class CommentsPreparator extends ASTVisitor {
 			int startIndex = this.ctm.findIndex(startPosition, -1, false);
 			int endPosition = textStartPosition + matcher.end() - 1;
 			int endIndex = this.ctm.findIndex(endPosition, -1, false);
-			if (startIndex != endIndex)
-				disableFormatting(tokenStartingAt(startPosition), tokenEndingAt(endPosition));
+			if (startIndex != endIndex) {
+				startIndex = tokenStartingAt(startPosition);
+				endIndex = tokenEndingAt(endPosition);
+				disableFormatting(startIndex, endIndex);
+			}
 			noSubstituteWrapping(startPosition, endPosition);
 		}
 	}
@@ -762,11 +875,11 @@ public class CommentsPreparator extends ASTVisitor {
 		} else if (this.noFormatTagOpenStart >= 0) {
 			int openingTagIndex = tokenStartingAt(this.noFormatTagOpenStart);
 			int closingTagIndex = tokenEndingAt(end);
-			if (openingTagIndex < closingTagIndex)
+			if (openingTagIndex < closingTagIndex) {
 				disableFormatting(openingTagIndex, closingTagIndex);
-			closingTagIndex = tokenEndingAt(end);
+				closingTagIndex = tokenEndingAt(end);
+			}
 			cleanupHTMLElement(openingTagIndex, closingTagIndex, false);
-			noSubstituteWrapping(this.noFormatTagOpenStart, end);
 			this.noFormatTagOpenStart = -1;
 		}
 	}
@@ -829,13 +942,14 @@ public class CommentsPreparator extends ASTVisitor {
 			tokensToReplace.clear();
 			tokensToReplace.addAll(commentToLines(noFormatToken, commentStart));
 		}
+		Token first = tokensToReplace.get(0), last = tokensToReplace.get(tokensToReplace.size() - 1);
 		if (startToken.isSpaceBefore())
-			tokensToReplace.get(0).spaceBefore();
-		tokensToReplace.get(0).putLineBreaksBefore(startToken.getLineBreaksBefore());
-		Token lastToReplace = tokensToReplace.get(tokensToReplace.size() - 1);
+			first.spaceBefore();
+		first.putLineBreaksBefore(startToken.getLineBreaksBefore());
+		first.setWrapPolicy(startToken.getWrapPolicy());
 		if (endToken.isSpaceAfter())
-			lastToReplace.spaceAfter();
-		lastToReplace.putLineBreaksAfter(endToken.getLineBreaksAfter());
+			last.spaceAfter();
+		last.putLineBreaksAfter(endToken.getLineBreaksAfter());
 		for (Token token : tokensToReplace)
 			if (token.tokenType == TokenNameCOMMENT_JAVADOC)
 				token.setIndent(startToken.getIndent());
@@ -852,7 +966,6 @@ public class CommentsPreparator extends ASTVisitor {
 			List<Token> tokensToReplace = this.commentStructure.subList(openingTagIndex + 1, closingTagIndex);
 			tokensToReplace.clear();
 			tokensToReplace.addAll(lines);
-			noSubstituteWrapping(noFormatStart, noFormatEnd);
 		} else {
 			this.commentStructure.subList(openingTagIndex + 1, closingTagIndex).clear();
 			Token closingTag = this.ctm.get(closingTagIndex);
@@ -919,17 +1032,16 @@ public class CommentsPreparator extends ASTVisitor {
 	}
 
 	private boolean tokenizeMultilineComment(Token commentToken) {
-		if (this.noSubstituteWrapping == null || this.noSubstituteWrapping.length < commentToken.countChars()) {
-			this.noSubstituteWrapping = new boolean[commentToken.countChars()];
-		} else {
-			Arrays.fill(this.noSubstituteWrapping, 0, commentToken.countChars(), false);
+		if (this.allowSubstituteWrapping == null || this.allowSubstituteWrapping.length < commentToken.countChars()) {
+			this.allowSubstituteWrapping = new boolean[commentToken.countChars()];
 		}
+		boolean isJavadoc = commentToken.tokenType == TokenNameCOMMENT_JAVADOC;
+		Arrays.fill(this.allowSubstituteWrapping, 0, commentToken.countChars(), !isJavadoc);
 
-		final boolean cleanBlankLines = commentToken.tokenType == TokenNameCOMMENT_JAVADOC
-				? this.options.comment_clear_blank_lines_in_javadoc_comment
+		final boolean cleanBlankLines = isJavadoc ? this.options.comment_clear_blank_lines_in_javadoc_comment
 				: this.options.comment_clear_blank_lines_in_block_comment;
 
-		List<Token> structure = new ArrayList<Token>();
+		List<Token> structure = new ArrayList<>();
 
 		int firstTokenEnd = commentToken.originalStart + 1;
 		while (firstTokenEnd < commentToken.originalEnd - 1 && this.tm.charAt(firstTokenEnd + 1) == '*')
@@ -1001,11 +1113,14 @@ public class CommentsPreparator extends ASTVisitor {
 		boolean newLinesAtBoundries = commentToken.tokenType == TokenNameCOMMENT_JAVADOC
 				? this.options.comment_new_lines_at_javadoc_boundaries
 				: this.options.comment_new_lines_at_block_boundaries;
-		if (newLinesAtBoundries && this.tm.countLineBreaksBetween(first, last) > 0) {
+		if (!newLinesAtBoundries) {
+			structure.get(1).clearLineBreaksBefore();
+			last.clearLineBreaksBefore();
+		} else if (this.tm.countLineBreaksBetween(first, last) > 0) {
 			first.breakAfter();
 			last.breakBefore();
-			last.setAlign(1);
 		}
+		last.setAlign(1);
 
 		if (structure.size() == 2)
 			return false;
@@ -1016,22 +1131,30 @@ public class CommentsPreparator extends ASTVisitor {
 	private void noSubstituteWrapping(int from, int to) {
 		int commentStart = this.ctm.get(0).originalStart;
 		assert commentStart <= from && from <= to && to <= this.ctm.get(this.ctm.size() - 1).originalEnd;
-		Arrays.fill(this.noSubstituteWrapping, from - commentStart, to - commentStart + 1, true);
+		Arrays.fill(this.allowSubstituteWrapping, from - commentStart, to - commentStart + 1, false);
 	}
 
 	private void addSubstituteWraps() {
-		int commentStart = this.ctm.get(0).originalStart;
+		Token previous = this.ctm.get(0);
+		int commentStart = previous.originalStart;
 		for (int i = 1; i < this.ctm.size() - 1; i++) {
 			Token token = this.ctm.get(i);
+			boolean touchesPrevious = token.originalStart == this.ctm.get(i - 1).originalEnd + 1;
+			if (touchesPrevious && token.getLineBreaksBefore() == 0 && previous.getLineBreaksAfter() == 0
+					&& token.getWrapPolicy() == null) {
+				boolean allowWrap = this.allowSubstituteWrapping[token.originalStart - commentStart];
+				token.setWrapPolicy(allowWrap ? WrapPolicy.SUBSTITUTE_ONLY : WrapPolicy.DISABLE_WRAP);
+			}
 			for (int pos = token.originalStart + 1; pos < token.originalEnd; pos++) {
-				if (this.noSubstituteWrapping[pos - commentStart])
+				if (!this.allowSubstituteWrapping[pos - commentStart])
 					continue;
 				char c = this.ctm.charAt(pos);
-				if (!ScannerHelper.isJavaIdentifierPart(c) && c != '@'/* wrapping on @ would create a javadoc tag */) {
+				if (!ScannerHelper.isJavaIdentifierPart(c)) {
 					this.ctm.get(tokenStartingAt(pos)).setWrapPolicy(WrapPolicy.SUBSTITUTE_ONLY);
 					this.ctm.get(tokenStartingAt(pos + 1)).setWrapPolicy(WrapPolicy.SUBSTITUTE_ONLY);
 				}
 			}
+			previous = token;
 		}
 	}
 
@@ -1046,8 +1169,7 @@ public class CommentsPreparator extends ASTVisitor {
 		// ^ index: original source position (minus startPosition), value: position in code string
 		getCodeToFormat(codeStartPosition, codeEndPosition, codeBuilder, positionMapping);
 
-		List<Token> formattedTokens = getCommentCodeFormatter().prepareFormattedCode(codeBuilder.toString(),
-				CodeFormatter.K_UNKNOWN);
+		List<Token> formattedTokens = getCommentCodeFormatter().prepareFormattedCode(codeBuilder.toString());
 
 		if (formattedTokens == null) {
 			disableFormattingExclusively(openingTagLastIndex, closingTagFirstIndex);
@@ -1069,7 +1191,6 @@ public class CommentsPreparator extends ASTVisitor {
 		tokensToReplace.clear();
 		tokensToReplace.addAll(formattedTokens);
 		cleanupHTMLElement(openingTagLastIndex, openingTagLastIndex + formattedTokens.size() + 1, true);
-		noSubstituteWrapping(codeStartPosition, codeEndPosition);
 	}
 
 	private DefaultCodeFormatter getCommentCodeFormatter() {
@@ -1184,7 +1305,7 @@ public class CommentsPreparator extends ASTVisitor {
 	private List<Token> translateFormattedTokens(int startPosition, List<Token> formattedTokens, int[] positionMapping,
 			HashMap<Token, Token> translationMap) {
 		int previousLineBreaks = 0;
-		List<Token> result = new ArrayList<Token>();
+		List<Token> result = new ArrayList<>();
 		for (Token token : formattedTokens) {
 			int newStart = Arrays.binarySearch(positionMapping, token.originalStart);
 			while (newStart > 0 && positionMapping[newStart - 1] == token.originalStart)
@@ -1195,14 +1316,22 @@ public class CommentsPreparator extends ASTVisitor {
 			Token translated = new Token(token, newStart + startPosition, newEnd + startPosition, token.tokenType);
 			if (translated.getWrapPolicy() == null)
 				translated.setWrapPolicy(WrapPolicy.DISABLE_WRAP);
-			if (token.hasNLSTag())
-				translationMap.put(token, translated);
+
+			if (token.hasNLSTag()) {
+				if (translationMap == null)
+					translationMap = new HashMap<>();
+				Token translatedNLS = translationMap.get(token.getNLSTag());
+				if (translatedNLS != null) {
+					translatedNLS.setNLSTag(translated);
+					translated.setNLSTag(translatedNLS);
+				} else {
+					translationMap.put(token, translated);
+				}
+			}
 
 			int lineBreaks = Math.max(previousLineBreaks, token.getLineBreaksBefore());
 			List<Token> structure = token.getInternalStructure();
 			if (structure != null && !structure.isEmpty()) {
-				if (translationMap == null)
-					translationMap = new HashMap<Token, Token>();
 				translated.setInternalStructure(translateFormattedTokens(startPosition, structure, positionMapping,
 						translationMap));
 			}
@@ -1212,14 +1341,6 @@ public class CommentsPreparator extends ASTVisitor {
 		}
 		result.get(result.size() - 1).putLineBreaksAfter(previousLineBreaks);
 
-		for (Token translated : result) {
-			if (translated.getNLSTag() != null) {
-				Token nlsTagToken = translationMap.get(translated.getNLSTag());
-				translated.setNLSTag(nlsTagToken);
-				nlsTagToken.setNLSTag(translated);
-				assert translated.getNLSTag() != null;
-			}
-		}
 		return result;
 	}
 
