@@ -1,9 +1,12 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2016 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2018 Mateusz Matela and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
@@ -35,7 +38,6 @@ import org.eclipse.text.edits.TextEdit;
 public class TextEditsBuilder extends TokenTraverser {
 
 	private final String source;
-	private List<IRegion> regions;
 	private TokenManager tm;
 	private final DefaultCodeFormatterOptions options;
 	private final StringBuilder buffer;
@@ -43,6 +45,7 @@ public class TextEditsBuilder extends TokenTraverser {
 	private final List<Token> stringLiteralsInLine = new ArrayList<Token>();
 	private final List<TextEdit> edits = new ArrayList<TextEdit>();
 
+	private final List<IRegion> regions;
 	private int currentRegion = 0;
 
 	private TextEditsBuilder childBuilder;
@@ -51,14 +54,14 @@ public class TextEditsBuilder extends TokenTraverser {
 	private int sourceLimit;
 	private int parentTokenIndex;
 
-	public TextEditsBuilder(String source, IRegion[] regions, TokenManager tokenManager,
+	public TextEditsBuilder(String source, List<IRegion> regions, TokenManager tokenManager,
 			DefaultCodeFormatterOptions options) {
 		this.source = source;
 		this.tm = tokenManager;
 		this.options = options;
 		this.regions = adaptRegions(regions);
 
-		this.alignChar = this.options.tab_char;
+		this.alignChar = this.options.align_with_spaces ? DefaultCodeFormatterOptions.SPACE : this.options.tab_char;
 		this.sourceLimit = source.length();
 		this.parent = null;
 
@@ -74,9 +77,7 @@ public class TextEditsBuilder extends TokenTraverser {
 		this.alignChar = DefaultCodeFormatterOptions.SPACE;
 	}
 
-	private List<IRegion> adaptRegions(IRegion[] givenRegions) {
-		if (givenRegions == null)
-			return null;
+	private List<IRegion> adaptRegions(List<IRegion> givenRegions) {
 		// make sure regions don't begin or end inside multiline comments
 		ArrayList<IRegion> result = new ArrayList<IRegion>();
 		IRegion previous = null;
@@ -157,9 +158,9 @@ public class TextEditsBuilder extends TokenTraverser {
 					if (index == 0) {
 						indentToken = token;
 					} else {
-						boolean isForced = token.getWrapPolicy() != null
-								&& token.getWrapPolicy().wrapMode == WrapMode.FORCED;
-						Token previous = this.tm.get(this.tm.findFirstTokenInLine(index - 1, true, !isForced));
+						boolean isBlockIndent = token.getWrapPolicy() != null
+								&& token.getWrapPolicy().wrapMode == WrapMode.BLOCK_INDENT;
+						Token previous = this.tm.get(this.tm.findFirstTokenInLine(index - 1, true, !isBlockIndent));
 						indentToken = (token.getIndent() > previous.getIndent()) ? token : previous;
 					}
 				}
@@ -175,8 +176,7 @@ public class TextEditsBuilder extends TokenTraverser {
 		} else if (index == 0 && this.parent == null) {
 			bufferIndent(token, index);
 		} else {
-			bufferAlign(token, index);
-			if (isSpaceBefore() && token.getAlign() == 0)
+			if (!bufferAlign(token, index) && isSpaceBefore())
 				this.buffer.append(' ');
 		}
 	}
@@ -189,10 +189,9 @@ public class TextEditsBuilder extends TokenTraverser {
 
 		this.parent.counter = this.counter;
 		this.parent.bufferLineSeparator(null, false);
+		this.parent.bufferIndent(this.parent.tm.get(this.parentTokenIndex), this.parentTokenIndex);
 		this.counter = this.parent.counter;
 
-		bufferIndent(this.parent.tm.get(this.parentTokenIndex), -1);
-
 		if (token != null && token.tokenType == TokenNameNotAToken)
 			return; // this is an unformatted block comment, don't force asterisk
 
@@ -224,36 +223,32 @@ public class TextEditsBuilder extends TokenTraverser {
 
 	private void bufferIndent(Token token, int index) {
 		int indent = token.getIndent();
-		int additionalSpaces = 0;
-		if (this.options.use_tabs_only_for_leading_indentations) {
-			// use indentation of wrap-line start token and add spaces to match current token
+		if (getCurrent() != null && getCurrent() != token)
+			indent += getCurrent().getEmptyLineIndentAdjustment();
+		int spaces = 0;
+		if (this.options.use_tabs_only_for_leading_indentations
+				&& this.options.tab_char != DefaultCodeFormatterOptions.SPACE) {
 			WrapPolicy wrapPolicy = token.getWrapPolicy();
-			int wrapRootIndent = indent;
-			if (index == -1) { // this means we print a line separator in a multi-line comment
-				TokenManager tm2 = this.parent.tm;
-				wrapRootIndent = tm2.get(tm2.findFirstTokenInLine(this.parentTokenIndex, true, true)).getIndent();
-			} else if (wrapPolicy != null) {
-				wrapRootIndent = this.tm.get(this.tm.findFirstTokenInLine(index, true, true)).getIndent();
+			boolean isWrappedBlockComment = this.childBuilder != null && this.childBuilder.parentTokenIndex == index;
+			if (isWrappedBlockComment) {
+				Token lineStart = this.tm.get(this.tm.findFirstTokenInLine(index));
+				spaces = token.getIndent() - lineStart.getIndent();
+				token = lineStart;
+				wrapPolicy = token.getWrapPolicy();
 			}
-			additionalSpaces = indent - wrapRootIndent;
-			indent = wrapRootIndent;
-
-			if (wrapPolicy != null && wrapPolicy.wrapMode == WrapMode.FORCED) {
-				int parentIndex = wrapPolicy.wrapParentIndex;
-				int parentAlign = 0;
-				int lineStart = this.tm.findFirstTokenInLine(parentIndex);
-				for (int i = parentIndex; i >= lineStart; i--) {
-					parentAlign = this.tm.get(i).getAlign();
-					if (parentAlign > 0)
-						break;
+			while (wrapPolicy != null) {
+				Token parentLineStart = this.tm.get(this.tm.findFirstTokenInLine(wrapPolicy.wrapParentIndex));
+				if (wrapPolicy.wrapMode != WrapMode.BLOCK_INDENT)
+					spaces += token.getIndent() - parentLineStart.getIndent();
+				token = parentLineStart;
+				if (wrapPolicy == token.getWrapPolicy()) {
+					assert wrapPolicy == WrapPolicy.FORCE_FIRST_COLUMN || wrapPolicy == WrapPolicy.DISABLE_WRAP;
+					break;
 				}
-
-				int extraIndent = parentAlign == 0 ? wrapPolicy.extraIndent : (token.getIndent() - parentAlign);
-				additionalSpaces -= extraIndent;
-				indent += extraIndent;
+				wrapPolicy = token.getWrapPolicy();
 			}
 		}
-		appendIndentationString(this.buffer, this.options.tab_char, this.options.tab_size, indent, additionalSpaces);
+		appendIndentationString(this.buffer, this.options.tab_char, this.options.tab_size, indent - spaces, spaces);
 	}
 
 	public static void appendIndentationString(StringBuilder target, int tabChar, int tabSize, int indent,
@@ -289,10 +284,16 @@ public class TextEditsBuilder extends TokenTraverser {
 		target.append(indentChars);
 	}
 
-	private void bufferAlign(Token token, int index) {
+	private boolean bufferAlign(Token token, int index) {
 		int align = token.getAlign();
+		int alignmentChar = this.alignChar;
+		if (align == 0 && getLineBreaksBefore() == 0 && this.parent != null) {
+			align = token.getIndent();
+			token.setAlign(align);
+			alignmentChar = DefaultCodeFormatterOptions.SPACE;
+		}
 		if (align == 0)
-			return;
+			return false;
 
 		int currentPositionInLine = 0;
 		if (getLineBreaksBefore() > 0) {
@@ -302,9 +303,11 @@ public class TextEditsBuilder extends TokenTraverser {
 			currentPositionInLine = this.tm.getPositionInLine(index - 1);
 			currentPositionInLine += this.tm.getLength(this.tm.get(index - 1), currentPositionInLine);
 		}
+		if (currentPositionInLine >= align)
+			return false;
 
 		final int tabSize = this.options.tab_size;
-		switch (this.alignChar) {
+		switch (alignmentChar) {
 			case DefaultCodeFormatterOptions.SPACE:
 				while (currentPositionInLine++ < align) {
 					this.buffer.append(' ');
@@ -326,8 +329,9 @@ public class TextEditsBuilder extends TokenTraverser {
 				}
 				break;
 			default:
-				throw new IllegalStateException("Unrecognized align char: " + this.alignChar); //$NON-NLS-1$
+				throw new IllegalStateException("Unrecognized align char: " + alignmentChar); //$NON-NLS-1$
 		}
+		return true;
 	}
 
 	private void flushBuffer(int currentPosition) {
@@ -342,12 +346,12 @@ public class TextEditsBuilder extends TokenTraverser {
 			if (this.counter >= regionEnd) {
 				this.currentRegion++;
 				continue;
-		}
+			}
 			if (this.currentRegion == this.regions.size() - 1
 					|| this.regions.get(this.currentRegion + 1).getOffset() > currentPosition) {
 				this.edits.add(getReplaceEdit(this.counter, currentPosition, buffered, region));
 				break;
-	}
+			}
 
 			// this edit will span more than one region, split it
 			IRegion nextRegion = this.regions.get(this.currentRegion + 1);
@@ -439,8 +443,8 @@ public class TextEditsBuilder extends TokenTraverser {
 			} else if (c1 == '\t' && c2 == ' ') {
 				for (i = 0; i < this.options.tab_size; i++) {
 					sourcePos += direction;
-					if (i < this.options.tab_size - 1
-							&& (sourcePos < 0 || sourcePos >= this.source.length() || this.source.charAt(sourcePos) != ' '))
+					if (i < this.options.tab_size - 1 && (sourcePos < 0 || sourcePos >= this.source.length()
+							|| this.source.charAt(sourcePos) != ' '))
 						continue theLoop;
 				}
 				textPos -= direction;
