@@ -1,9 +1,12 @@
 /*******************************************************************************
  * Copyright (c) 2014, 2015 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
@@ -35,16 +38,18 @@ public class Token {
 		WHERE_NECESSARY,
 		/** Wrap mode for the "Wrap all elements" policies. */
 		TOP_PRIORITY,
+		/** Wrap mode for tokens that must be wrapped due to "Force wrap" setting. */
+		FORCE,
 		/**
-		 * Wrap mode for tokens that are already in new line before wrapping, but their indentation should be adjusted
-		 * in similar way to wrapping. Used for anonymous class body, lambda body and comments inside code.
+		 * Wrap mode used for lines in anonymous class and lambda body. Means that tokens that are already in new line
+		 * before wrapping, but their indentation should be adjusted in similar way to wrapping.
 		 */
-		FORCED
+		BLOCK_INDENT
 	}
 
 	public static class WrapPolicy {
 
-		/** Policy used for internal structure of multiline comments to mark tokens that should never be wrapped */
+		/** Policy used to mark tokens that should never be wrapped */
 		public final static WrapPolicy DISABLE_WRAP = new WrapPolicy(WrapMode.DISABLED, 0, 0);
 
 		/**
@@ -53,6 +58,9 @@ public class Token {
 		 */
 		public final static WrapPolicy SUBSTITUTE_ONLY = new WrapPolicy(WrapMode.DISABLED, 0, 0);
 
+		/** Policy used to mark comments on first column that should not be indented. */
+		public final static WrapPolicy FORCE_FIRST_COLUMN = new WrapPolicy(WrapMode.DISABLED, 0, 0);
+
 		public final WrapMode wrapMode;
 		public final int wrapParentIndex;
 		public final int groupEndIndex;
@@ -79,10 +87,7 @@ public class Token {
 		public WrapPolicy(WrapMode wrapMode, int wrapParentIndex, int extraIndent) {
 			this(wrapMode, wrapParentIndex, -1, extraIndent, 0, 1, false, false);
 		}
-		}
-
-	/** Special token type used to mark tokens that store empty line indentation */
-	public static final int TokenNameEMPTY_LINE = 10000;
+	}
 
 	/** Position in source of the first character. */
 	public final int originalStart;
@@ -92,7 +97,9 @@ public class Token {
 	public final int tokenType;
 	private boolean spaceBefore, spaceAfter;
 	private int lineBreaksBefore, lineBreaksAfter;
+	private boolean wrapped;
 	private int indent;
+	private int emptyLineIndentAdjustment;
 	private int align;
 	private boolean toEscape;
 
@@ -182,7 +189,12 @@ public class Token {
 	}
 
 	public int getLineBreaksBefore() {
-		return this.lineBreaksBefore;
+		return this.wrapped ? 1 : this.lineBreaksBefore;
+	}
+
+	/** Can be used to temporarily force preceding line break without losing the original number of line breaks. */
+	public void setWrapped(boolean wrapped) {
+		this.wrapped = wrapped;
 	}
 
 	public void clearLineBreaksBefore() {
@@ -216,6 +228,7 @@ public class Token {
 	}
 
 	public void setIndent(int indent) {
+		assert indent >= 0;
 		this.indent = indent;
 	}
 
@@ -223,6 +236,14 @@ public class Token {
 		return this.indent;
 	}
 
+	public void setEmptyLineIndentAdjustment(int adjustment) {
+		this.emptyLineIndentAdjustment = adjustment;
+	}
+
+	public int getEmptyLineIndentAdjustment() {
+		return this.emptyLineIndentAdjustment;
+	}
+
 	public void setAlign(int align) {
 		this.align = align;
 	}
@@ -257,7 +278,7 @@ public class Token {
 
 	public boolean isWrappable() {
 		WrapPolicy wp = this.wrapPolicy;
-		return wp != null && wp.wrapMode != WrapMode.DISABLED && wp.wrapMode != WrapMode.FORCED;
+		return wp != null && wp.wrapMode != WrapMode.DISABLED && wp.wrapMode != WrapMode.BLOCK_INDENT;
 	}
 
 	public void setNLSTag(Token nlsTagToken) {
@@ -291,8 +312,6 @@ public class Token {
 	}
 
 	public String toString(String source) {
-		if (this.tokenType == TokenNameEMPTY_LINE)
-			return ""; //$NON-NLS-1$
 		return source.substring(this.originalStart, this.originalEnd + 1);
 	}
 
@@ -308,6 +327,7 @@ public class Token {
 	 */
 //	public static String source;
 
+	@Override
 	public String toString() {
 //		if (source != null)  // see comment above
 //			return toString(source);
