@@ -1,12 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2015 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2018 Mateusz Matela and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.formatter.linewrap;
 
@@ -31,12 +36,13 @@ public class CommentWrapExecutor extends TokenTraverser {
 	private final TokenManager tm;
 	private final DefaultCodeFormatterOptions options;
 
-	private final ArrayList<Token> nlsTags = new ArrayList<Token>();
+	private final ArrayList<Token> nlsTags = new ArrayList<>();
 
 	private int lineStartPosition;
-	private List<Token> blockStructure;
+	private int lineLimit;
 	private boolean simulation;
 	private boolean wrapDisabled;
+	private boolean newLinesAtBoundries;
 
 	private Token potentialWrapToken, potentialWrapTokenSubstitute;
 	private int counterIfWrapped, counterIfWrappedSubstitute;
@@ -59,40 +65,27 @@ public class CommentWrapExecutor extends TokenTraverser {
 	public int wrapMultiLineComment(Token commentToken, int startPosition, boolean simulate, boolean noWrap) {
 		this.lineCounter = 1;
 		this.counter = startPosition;
+		commentToken.setIndent(this.tm.toIndent(startPosition, true));
+		this.lineStartPosition = commentToken.getIndent();
+		this.lineLimit = getLineLimit(startPosition);
+		this.simulation = simulate;
+		this.wrapDisabled = noWrap;
+		this.potentialWrapToken = this.potentialWrapTokenSubstitute = null;
+		this.newLinesAtBoundries = commentToken.tokenType == TokenNameCOMMENT_JAVADOC
+				? this.options.comment_new_lines_at_javadoc_boundaries
+				: this.options.comment_new_lines_at_block_boundaries;
 
 		List<Token> structure = commentToken.getInternalStructure();
 		if (structure == null || structure.isEmpty())
 			return startPosition + this.tm.getLength(commentToken, startPosition);
 
-		commentToken.setIndent(this.tm.toIndent(startPosition, true));
-		this.lineStartPosition = commentToken.getIndent();
-		this.simulation = simulate;
-		this.wrapDisabled = noWrap;
-		this.potentialWrapToken = this.potentialWrapTokenSubstitute = null;
-		this.blockStructure = structure;
+		int position = tryToFitInOneLine(structure, startPosition, noWrap);
+		if (position > 0)
+			return position;
+
 		traverse(structure, 0);
-
-		boolean newLinesAtBoundries = commentToken.tokenType == TokenNameCOMMENT_JAVADOC
-				? this.options.comment_new_lines_at_javadoc_boundaries
-				: this.options.comment_new_lines_at_block_boundaries;
-		if (this.lineCounter > 1 && newLinesAtBoundries) {
-			Token endingToken = structure.get(structure.size() - 1);
-			if (!simulate && endingToken.tokenType != TokenNameNotAToken) {
-				structure.get(0).breakAfter();
-				endingToken.breakBefore();
-				endingToken.setAlign(1);
-			}
-			return startPosition + this.tm.getLength(endingToken, startPosition);
-		} else if (this.lineCounter > 1 && !newLinesAtBoundries) {
-			// the rest of this code assumes that newLinesAtBoundries==true, so now subtract the additional lines
-			this.lineCounter -= 2;
-
-			this.lineCounter -= structure.get(1).getLineBreaksBefore();
-			structure.get(1).clearLineBreaksBefore();
-			Token last = structure.get(structure.size() - 1);
-			this.lineCounter -= last.getLineBreaksBefore();
-			last.clearLineBreaksBefore();
-		}
+		if (this.newLinesAtBoundries)
+			return this.lineStartPosition + 1 + this.tm.getLength(structure.get(structure.size() - 1), 0);
 		return this.counter;
 	}
 
@@ -100,19 +93,59 @@ public class CommentWrapExecutor extends TokenTraverser {
 		return this.lineCounter;
 	}
 
+	private int tryToFitInOneLine(List<Token> structure, int startPosition, boolean noWrap) {
+		int position = startPosition;
+		boolean hasWrapPotential = false;
+		boolean wasSpaceAfter = false;
+		for (int i = 0; i < structure.size(); i++) {
+			Token token = structure.get(i);
+			if (token.getLineBreaksBefore() > 0 || token.getLineBreaksAfter() > 0) {
+				assert !noWrap; // comment already wrapped
+				return -1;
+			}
+			if (!wasSpaceAfter && token.isSpaceBefore())
+				position++;
+			position += this.tm.getLength(token, position);
+			wasSpaceAfter = token.isSpaceAfter();
+			if (wasSpaceAfter)
+				position++;
+
+			WrapPolicy policy = token.getWrapPolicy();
+			if (i > 1 && (policy == null || policy == WrapPolicy.SUBSTITUTE_ONLY))
+				hasWrapPotential = true;
+		}
+		if (position <= this.lineLimit || noWrap || !hasWrapPotential)
+			return position;
+		return -1;
+	}
+
+	private int getStartingPosition(Token token) {
+		int position = this.lineStartPosition + token.getAlign() + token.getIndent();
+		if (token.tokenType != TokenNameNotAToken)
+			position += COMMENT_LINE_SEPARATOR_LENGTH;
+		return position;
+	}
+
 	@Override
 	protected boolean token(Token token, int index) {
-		int positionIfNewLine = this.lineStartPosition + token.getAlign() + token.getIndent();
-		if (token.tokenType != TokenNameNotAToken)
-			positionIfNewLine += COMMENT_LINE_SEPARATOR_LENGTH;
-		if (getLineBreaksBefore() > 0) {
-			this.lineCounter = Math.max(this.lineCounter + getLineBreaksBefore(), 4);
+		final int positionIfNewLine = getStartingPosition(token);
+
+		int lineBreaksBefore = getLineBreaksBefore();
+		if ((index == 1 || getNext() == null) && this.newLinesAtBoundries && lineBreaksBefore == 0) {
+			if (!this.simulation)
+				token.breakBefore();
+			lineBreaksBefore = 1;
+		}
+
+		if (lineBreaksBefore > 0) {
+			this.lineCounter += lineBreaksBefore;
 			this.counter = positionIfNewLine;
 			this.potentialWrapToken = this.potentialWrapTokenSubstitute = null;
+			this.lineLimit = getLineLimit(this.lineStartPosition);
 
 			boolean isFormattedCode = token.getWrapPolicy() != null
 					&& token.getWrapPolicy() != WrapPolicy.SUBSTITUTE_ONLY;
-			if (!isFormattedCode && token.getAlign() == 0) {
+			if (!isFormattedCode && token.getAlign() == 0 && !this.simulation) {
 				// Indents are reserved for code inside <pre>.
 				// Indentation of javadoc tags can be achieved with align
 				token.setAlign(token.getIndent());
@@ -120,8 +153,7 @@ public class CommentWrapExecutor extends TokenTraverser {
 			}
 		}
 
-		boolean canWrap = getNext() != null && getLineBreaksBefore() == 0 && index > 1
-				&& positionIfNewLine < this.counter;
+		boolean canWrap = getNext() != null && lineBreaksBefore == 0 && index > 1 && positionIfNewLine < this.counter;
 		if (canWrap) {
 			if (token.getWrapPolicy() == null) {
 				this.potentialWrapToken = token;
@@ -132,6 +164,8 @@ public class CommentWrapExecutor extends TokenTraverser {
 			}
 		}
 
+		if (index > 1 && getNext() != null && (token.getAlign() + token.getIndent()) > 0)
+			this.counter = Math.max(this.counter, getStartingPosition(token));
 		this.counter += this.tm.getLength(token, this.counter);
 		this.counterIfWrapped += this.tm.getLength(token, this.counterIfWrapped);
 		this.counterIfWrappedSubstitute += this.tm.getLength(token, this.counterIfWrappedSubstitute);
@@ -141,7 +175,6 @@ public class CommentWrapExecutor extends TokenTraverser {
 				this.potentialWrapToken = this.potentialWrapTokenSubstitute;
 				this.counterIfWrapped = this.counterIfWrappedSubstitute;
 			}
-			this.counter = this.counterIfWrapped;
 			if (!this.simulation) {
 				this.potentialWrapToken.breakBefore();
 				// Indents are reserved for code inside <pre>.
@@ -149,8 +182,10 @@ public class CommentWrapExecutor extends TokenTraverser {
 				this.potentialWrapToken.setAlign(this.potentialWrapToken.getIndent());
 				this.potentialWrapToken.setIndent(0);
 			}
-			this.lineCounter = Math.max(this.lineCounter + 1, 4);
+			this.counter = this.counterIfWrapped;
+			this.lineCounter++;
 			this.potentialWrapToken = this.potentialWrapTokenSubstitute = null;
+			this.lineLimit = getLineLimit(this.lineStartPosition);
 		}
 
 		if (isSpaceAfter()) {
@@ -162,40 +197,22 @@ public class CommentWrapExecutor extends TokenTraverser {
 	}
 
 	private boolean shouldWrap() {
-		int lineLenght = this.options.comment_line_length;
-		if (this.wrapDisabled || this.counter <= lineLenght)
+		if (this.wrapDisabled || this.counter <= this.lineLimit)
 			return false;
+		if (getLineBreaksAfter() == 0 && getNext() != null && getNext().getWrapPolicy() == WrapPolicy.DISABLE_WRAP) {
+			// The next token cannot be wrapped, so there's no need to wrap now.
+			// Let's wait and decide when there's more information available.
+			return false;
+		}
 		if (this.potentialWrapToken != null && this.potentialWrapTokenSubstitute != null
-				&& this.counterIfWrapped > lineLenght && this.counterIfWrappedSubstitute < this.counterIfWrapped) {
+				&& this.counterIfWrapped > this.lineLimit && this.counterIfWrappedSubstitute < this.counterIfWrapped) {
 			// there is a normal token to wrap, but the line would overflow anyway - better use substitute
 			this.potentialWrapToken = null;
 		}
 		if (this.potentialWrapToken == null && this.potentialWrapTokenSubstitute == null) {
-			boolean isFormattingEnabled = this.blockStructure.size() > 1
-					&& this.blockStructure.get(1).tokenType == TokenNameNotAToken;
-			if (isFormattingEnabled) {
-				// can't wrap, but the long comment cannot stay in one line
-				this.lineCounter = Math.max(this.lineCounter, 3);
-			}
 			return false;
 		}
 
-		if (this.options.comment_new_lines_at_javadoc_boundaries) {
-			if (getNext() == null) { // the closing token will go to the next line anyway
-				this.lineCounter = Math.max(this.lineCounter, 3);
-				return false;
-			}
-			if (this.lineCounter == 1) {
-				// when wrapping the first line of javadoc (more asterisks in opening token), the line will
-				// move to the left so it may not need wrapping in the end
-				int openingTokenLength = this.tm.getLength(this.blockStructure.get(0), 0);
-				if (this.counter - (openingTokenLength - 2) <= lineLenght) {
-					this.counter -= (openingTokenLength - 2);
-					this.lineCounter = Math.max(this.lineCounter, 3);
-					return false;
-				}
-			}
-		}
 		return true;
 	}
 
@@ -213,6 +230,7 @@ public class CommentWrapExecutor extends TokenTraverser {
 		int position = startPosition;
 		startPosition = this.tm.toIndent(startPosition, true);
 		int indent = startPosition;
+		int limit = getLineLimit(position);
 
 		for (Token token : structure) {
 			if (token.hasNLSTag()) {
@@ -252,6 +270,7 @@ public class CommentWrapExecutor extends TokenTraverser {
 				position++;
 			if (token.getLineBreaksBefore() > 0) {
 				position = startPosition;
+				limit = getLineLimit(position);
 				lineStartIndex = whitespace == null ? i : i + 1;
 				if (whitespace != null && token != whitespace) {
 					token.clearLineBreaksBefore();
@@ -260,7 +279,9 @@ public class CommentWrapExecutor extends TokenTraverser {
 				}
 			}
 			position += this.tm.getLength(token, position);
-			if (position > this.options.comment_line_length && i > lineStartIndex + 1) {
+			if (token.tokenType == TokenNameWHITESPACE)
+				limit = getLineLimit(position);
+			if (position > limit && i > lineStartIndex + 1) {
 				structure.add(i, prefix);
 				if (whitespace != null)
 					structure.add(i, whitespace);
@@ -273,4 +294,15 @@ public class CommentWrapExecutor extends TokenTraverser {
 		}
 		this.nlsTags.clear();
 	}
+
+	private int getLineLimit(int startPosition) {
+		final int commentLength = this.options.comment_line_length;
+		if (!this.options.comment_count_line_length_from_starting_position)
+			return commentLength;
+		final int pageWidth = this.options.page_width;
+		int lineLength = startPosition + commentLength;
+		if (lineLength > pageWidth && commentLength <= pageWidth)
+			lineLength = pageWidth;
+		return lineLength;
+	}
 }
