@@ -1,12 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2014, 2015 Mateusz Matela and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2014, 2016 Mateusz Matela and others.
+ *
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *     Mateusz Matela <mateusz.matela@gmail.com> - [formatter] Formatter does not format Java code correctly, especially when max line width is set - https://bugs.eclipse.org/303519
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.formatter.linewrap;
 
@@ -22,6 +26,7 @@ import org.aspectj.org.eclipse.jdt.core.dom.BodyDeclaration;
 import org.aspectj.org.eclipse.jdt.core.dom.FieldDeclaration;
 import org.aspectj.org.eclipse.jdt.core.dom.SimpleName;
 import org.aspectj.org.eclipse.jdt.core.dom.VariableDeclarationFragment;
+import org.aspectj.org.eclipse.jdt.internal.formatter.DefaultCodeFormatterOptions;
 import org.aspectj.org.eclipse.jdt.internal.formatter.Token;
 import org.aspectj.org.eclipse.jdt.internal.formatter.TokenManager;
 import org.aspectj.org.eclipse.jdt.internal.formatter.TokenTraverser;
@@ -62,25 +67,49 @@ public class FieldAligner {
 
 	private final List<List<FieldDeclaration>> fieldAlignGroups = new ArrayList<>();
 
+	private final DefaultCodeFormatterOptions options;
+
 	final TokenManager tm;
 
-	public FieldAligner(TokenManager tokenManager) {
+	public FieldAligner(TokenManager tokenManager, DefaultCodeFormatterOptions options) {
 		this.tm = tokenManager;
+		this.options = options;
 	}
 
-	public void prepareAlign(List<FieldDeclaration> bodyDeclarations) {
+	public void handleAlign(List<FieldDeclaration> bodyDeclarations) {
+		if (!this.options.align_type_members_on_columns)
+			return;
 		ArrayList<FieldDeclaration> alignGroup = new ArrayList<>();
+		BodyDeclaration previous = null;
 		for (BodyDeclaration declaration : bodyDeclarations) {
-			if ((declaration instanceof FieldDeclaration)) {
-					alignGroup.add((FieldDeclaration) declaration);
-				} else {
+			if (declaration instanceof FieldDeclaration) {
+				if (isNewGroup(declaration, previous)) {
 					alignFields(alignGroup);
-				alignGroup = new ArrayList<>();
-			}
+					alignGroup = new ArrayList<>();
+				}
+				alignGroup.add((FieldDeclaration) declaration);
 			}
+			previous = declaration;
+		}
 		alignFields(alignGroup);
 	}
 
+	private boolean isNewGroup(BodyDeclaration declaration, BodyDeclaration previousDeclaration) {
+		if (!(previousDeclaration instanceof FieldDeclaration))
+			return true;
+		int lineBreaks = 0;
+		int from = this.tm.lastIndexIn(previousDeclaration, -1);
+		int to = this.tm.firstIndexIn(declaration, -1);
+		Token previous = this.tm.get(from);
+		for (int i = from + 1; i <= to; i++) {
+			Token token = this.tm.get(i);
+			lineBreaks += Math.min(this.tm.countLineBreaksBetween(previous, token),
+					this.options.number_of_empty_lines_to_preserve + 1);
+			previous = token;
+		}
+		return lineBreaks > this.options.align_fields_grouping_blank_lines;
+	}
+
 	private void alignFields(ArrayList<FieldDeclaration> alignGroup) {
 		if (alignGroup.size() < 2)
 			return;
@@ -94,7 +123,7 @@ public class FieldAligner {
 			int positionInLine = this.tm.getPositionInLine(nameIndex);
 			maxNameAlign = Math.max(maxNameAlign, positionInLine);
 		}
-		maxNameAlign = this.tm.toIndent(maxNameAlign, false);
+		maxNameAlign = normalizedAlign(maxNameAlign);
 
 		int maxAssignAlign = 0;
 		for (FieldDeclaration declaration : alignGroup) {
@@ -111,7 +140,7 @@ public class FieldAligner {
 				maxAssignAlign = Math.max(maxAssignAlign, positionInLine);
 			}
 		}
-		maxAssignAlign = this.tm.toIndent(maxAssignAlign, false);
+		maxAssignAlign = normalizedAlign(maxAssignAlign);
 
 		for (FieldDeclaration declaration : alignGroup) {
 			List<VariableDeclarationFragment> fragments = declaration.fragments();
@@ -120,13 +149,14 @@ public class FieldAligner {
 				int assingIndex = this.tm.firstIndexAfter(fragment.getName(), TokenNameEQUAL);
 				Token assignToken = this.tm.get(assingIndex);
 				assignToken.setAlign(maxAssignAlign);
-				}
 			}
 		}
+	}
 
 	public void alignComments() {
 		if (this.fieldAlignGroups.isEmpty())
 			return;
+		boolean alignLineComments = !this.options.comment_preserve_white_space_between_code_and_line_comments;
 		PositionCounter positionCounter = new PositionCounter();
 		// align comments after field declarations
 		for (List<FieldDeclaration> alignGroup : this.fieldAlignGroups) {
@@ -138,7 +168,7 @@ public class FieldAligner {
 				maxCommentAlign = Math.max(maxCommentAlign,
 						positionCounter.findMaxPosition(firstIndexInLine, lastIndex));
 			}
-			maxCommentAlign = this.tm.toIndent(maxCommentAlign, false);
+			maxCommentAlign = normalizedAlign(maxCommentAlign);
 
 			for (FieldDeclaration declaration : alignGroup) {
 				int typeIndex = this.tm.firstIndexIn(declaration.getType(), -1);
@@ -152,10 +182,10 @@ public class FieldAligner {
 					if (lineBreak) {
 						if (token.tokenType == TokenNameCOMMENT_BLOCK) {
 							token.setAlign(maxCommentAlign);
-						} else {
+						} else if (alignLineComments) {
 							this.tm.addNLSAlignIndex(i, maxCommentAlign);
 						}
-					} else if (next.tokenType == TokenNameCOMMENT_LINE
+					} else if (next.tokenType == TokenNameCOMMENT_LINE && alignLineComments
 							|| (next.tokenType == TokenNameCOMMENT_BLOCK && i == lastIndex)) {
 						next.setAlign(maxCommentAlign);
 					}
@@ -163,4 +193,10 @@ public class FieldAligner {
 			}
 		}
 	}
+
+	private int normalizedAlign(int desiredAlign) {
+		if (this.options.align_with_spaces)
+			return desiredAlign;
+		return this.tm.toIndent(desiredAlign, false);
+	}
 }
