@@ -44,7 +44,6 @@ import org.aspectj.bridge.MessageUtil;
 import org.aspectj.bridge.WeaveMessage;
 import org.aspectj.bridge.context.CompilationAndWeavingContext;
 import org.aspectj.bridge.context.ContextToken;
-import org.aspectj.weaver.AjAttribute;
 import org.aspectj.weaver.AjcMemberMaker;
 import org.aspectj.weaver.AnnotationAJ;
 import org.aspectj.weaver.AnnotationOnTypeMunger;
@@ -353,8 +352,21 @@ public class BcelTypeMunger extends ConcreteTypeMunger {
 									&& targetMethod.getBackingGenericMember().getParameterSignature().equals(newParentMethodSig)) {
 								discoveredImpl = targetMethod;
 							} else if (newParentMethod.hasBackingGenericMember()) {
-								if (newParentMethod.getBackingGenericMember().getParameterSignature().equals(targetMethodSignature)) {
+ 								if (newParentMethod.getBackingGenericMember().getParameterSignature().equals(targetMethodSignature)) { // newParentMethod.getBackingGenericMember().getParameterSignature gives: (Pjava/util/List<TI;>;)  targetMethodSignature= (Ljava/util/List;)
 									discoveredImpl = targetMethod;
+								} else if (targetMethod instanceof BcelMethod) {
+									// BcelMethod does not have backing generic member set (need to investigate why). For now, special case here:
+									UnresolvedType[] targetMethodGenericParameterTypes = targetMethod.getGenericParameterTypes();
+									if (targetMethodGenericParameterTypes !=null) {
+										StringBuilder b = new StringBuilder("(");
+										for (UnresolvedType p: targetMethodGenericParameterTypes) {
+											b.append(p.getSignature());
+										}
+										b.append(')');
+										if (b.toString().equals(newParentMethodSig)) {
+											discoveredImpl = targetMethod;
+										}
+									}
 								}
 							}
 						}
@@ -817,7 +829,9 @@ public class BcelTypeMunger extends ConcreteTypeMunger {
 		// e.g ajc$com_blah_SecurityAspect$ptwAspectInstance
 		FieldGen fg = makeFieldGen(gen, AjcMemberMaker.perTypeWithinField(gen.getType(), aspectType));
 		gen.addField(fg, getSourceLocation());
-
+		if (!gen.getType().canBeSeenBy(aspectType) && aspectType.isPrivilegedAspect()) {
+			gen.forcePublic();
+		}
 		// Add an accessor for this new field, the
 		// ajc$<aspectname>$localAspectOf() method
 		// e.g.
